/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.controller;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.TreeSet;
import org.cobi.igg3.Constants;
import org.cobi.igg3.GlobalVariables;
import org.cobi.igg3.gui.IGG3View;

/**
 *
 * @author mxli
 */
public class ABAlleleConvertor implements Constants {

    String chipType;
    Map<String, char[]> ABSSNPMap;

    public ABAlleleConvertor(String chipType) {
        this.chipType = chipType;
    }

    public void loadABNames() throws Exception {
        if (chipType.startsWith("Affy")) {
            loadAffyABNames();
        } else if (chipType.startsWith("Illu")) {
            loadIllumiABNames();
        }
    }

    private void loadIllumiABNames() throws Exception {
        if (ABSSNPMap == null) {
            ABSSNPMap = new HashMap<String, char[]>();
        } else {
            ABSSNPMap.clear();
        }
        BufferedReader br = new BufferedReader(new FileReader(IGG_RESOURCE_CHIP_FOLDER + chipType + ".txt"), FILE_READER_BUFFER_SIZE);
        String line = null;
        String delmilit = "\t";
        br.readLine();
        String snpid;
        StringBuilder tmpBuffer = new StringBuilder();
        while ((line = br.readLine()) != null) {
            StringTokenizer tokenizer = new StringTokenizer(line, delmilit);
            tokenizer.nextToken();
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            snpid = tmpBuffer.toString();
            for (int i = 2; i < 7; i++) {
                tokenizer.nextToken();
            }
            char[] abAllele = new char[2];
            abAllele[0] = tokenizer.nextToken().charAt(0);
            abAllele[1] = tokenizer.nextToken().charAt(0);
            ABSSNPMap.put(snpid, abAllele);
        }
        br.close();
    }

    private void loadAffyABNames() throws Exception {
        if (ABSSNPMap == null) {
            ABSSNPMap = new HashMap<String, char[]>();
        } else {
            ABSSNPMap.clear();
        }
        BufferedReader br = new BufferedReader(new FileReader(IGG_RESOURCE_CHIP_FOLDER + chipType + ".txt"), FILE_READER_BUFFER_SIZE);
        String line = null;
        String delmilit = "\t";
        br.readLine();
        String snpid;
        StringBuilder tmpBuffer = new StringBuilder();
        while ((line = br.readLine()) != null) {
            StringTokenizer tokenizer = new StringTokenizer(line, delmilit);
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            snpid = tmpBuffer.toString();
            tokenizer.nextToken();
            for (int i = 2; i < 7; i++) {
                tokenizer.nextToken();
            }
            char[] abAllele = new char[2];
            abAllele[0] = tokenizer.nextToken().charAt(0);
            abAllele[1] = tokenizer.nextToken().charAt(0);
            ABSSNPMap.put(snpid, abAllele);
        }
        br.close();
    }

    public String convertChipGty(String origPath, String targPath) throws Exception {
        if (chipType.startsWith("Affy")) {
            return convertAffyChip(origPath, targPath);
        } else if (chipType.startsWith("Illu")) {
            return convertIllumiChip(origPath, targPath);
        }
        return "";
    }

    private String convertIllumiChip(String origPath, String targPath) throws Exception {
        File file = new File(origPath);
        File cvtFile = new File(targPath);
        TreeSet<String> ingoredSNP = new TreeSet<String>();
        BufferedReader br = new BufferedReader(new FileReader(file), FILE_READER_BUFFER_SIZE);
        BufferedWriter bw = new BufferedWriter(new FileWriter(cvtFile), FILE_READER_BUFFER_SIZE);
        String line = null;
        boolean starConvert = false;
        String delmilit = "\t ,";
        String snpID;
        int count;
        String indivID;
        String tmpStr;
        int ignoredGty = 0;
        StringBuilder tmpBuffer = new StringBuilder();
        while ((line = br.readLine()) != null) {
            if (!starConvert) {
                if (line.startsWith("rs")) {
                    starConvert = true;
                }
            }
            if (starConvert) {
                //System.out.println(line);

                StringTokenizer tokenizer = new StringTokenizer(line, delmilit);
                count = 2;
                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                snpID = tmpBuffer.toString();
                char[] abAlleles = ABSSNPMap.get(snpID);
                if (abAlleles == null) {
                    ignoredGty++;
                    GlobalVariables.addInforLog(line + " ignored!");
                    ingoredSNP.add(snpID);
                    continue;
                }

                bw.write(snpID);
                bw.write(" ");
                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                indivID = tmpBuffer.toString();
                bw.write(indivID);
                bw.write(" ");

                while (tokenizer.hasMoreTokens()) {
                    bw.write(" ");
                    tmpBuffer.delete(0, tmpBuffer.length());
                    tmpBuffer.append(tokenizer.nextToken().trim());
                    tmpStr = tmpBuffer.toString();
                    if (count == 2 || count == 3) {
                        if (tmpStr.charAt(0) == abAlleles[0]) {
                            bw.write("A");
                        } else if (tmpStr.charAt(0) == abAlleles[1]) {
                            bw.write("B");
                        } else if (tmpStr.charAt(0) == '-') {
                            bw.write("*");
                        } else {
                            // throw new Exception("Unexpected genotype " + tmpStr + " at " + snpID + " for " + indivID+ " in "+file.getName());
                            String unexpectInfo = "Unexpected genotype " + tmpStr + " at " + snpID + " for " + indivID + " in " + file.getName();
                            //GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                            GlobalVariables.addInforLog(unexpectInfo);
                        }
                    } else {
                        bw.write(tmpStr);
                    }
                    count++;
                }
                bw.write("\n");
            } else {
                bw.write(line);
                bw.write("\n");
            }
        }
        bw.flush();
        bw.close();
        br.close();

        String info = ingoredSNP.size() + "SNP's genotypes are ignored because their SNPs have not information in our annotation file (detailed in the log info.)!";
        GlobalVariables.mainView.setBriefRunningInfor(info);
        GlobalVariables.addInforLog(info);
        GlobalVariables.mainView.setBriefRunningInfor(ingoredSNP.toString() + "\n");
        GlobalVariables.addInforLog(ingoredSNP.toString());

        return cvtFile.getCanonicalPath();
    }

    private String convertAffyChip(String origPath, String targPath) throws Exception {
        File file = new File(origPath);
        File cvtFile = new File(targPath);
        TreeSet<String> ingoredSNP = new TreeSet<String>();
        BufferedReader br = new BufferedReader(new FileReader(file), FILE_READER_BUFFER_SIZE);
        BufferedWriter bw = new BufferedWriter(new FileWriter(cvtFile), FILE_READER_BUFFER_SIZE);
        String line = null;
        boolean starConvert = false;
        String delmilit = "\t ,";
        String snpID;

        String gty;
        int ignoredGty = 0;
        StringBuilder tmpBuffer = new StringBuilder();
        while ((line = br.readLine()) != null) {
            if (!starConvert) {
                if (line.startsWith("SNP_")) {
                    starConvert = true;
                }
            }
            if (starConvert) {
                //System.out.println(line);
                StringTokenizer tokenizer = new StringTokenizer(line, delmilit);
                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                snpID = tmpBuffer.toString();

                char[] abAlleles = ABSSNPMap.get(snpID);
                if (abAlleles == null) {
                    ignoredGty++;
                    GlobalVariables.addInforLog(line + " ignored!");
                    ingoredSNP.add(snpID);
                    continue;
                }
                bw.write(snpID);
                bw.write(" ");


                while (tokenizer.hasMoreTokens()) {
                    bw.write(" ");
                    tmpBuffer.delete(0, tmpBuffer.length());
                    tmpBuffer.append(tokenizer.nextToken().trim());
                    gty = tmpBuffer.toString();

                    if (gty.charAt(0) == abAlleles[0]) {
                        bw.write("A");
                    } else if (gty.charAt(0) == abAlleles[1]) {
                        bw.write("B");
                    } else if (gty.charAt(0) == '-') {
                        bw.write("*");
                    } else {
                        // throw new Exception("Unexpected genotype " + tmpStr + " at " + snpID + " for " + indivID+ " in "+file.getName());
                        String unexpectInfo = "Unexpected genotype " + gty.charAt(0) + " at " + snpID + " in " + file.getName();
                        //GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                        GlobalVariables.addInforLog(unexpectInfo);
                    }

                    if (gty.charAt(1) == abAlleles[0]) {
                        bw.write("A");
                    } else if (gty.charAt(1) == abAlleles[1]) {
                        bw.write("B");
                    } else if (gty.charAt(1) == '-') {
                        bw.write("*");
                    } else {
                        // throw new Exception("Unexpected genotype " + tmpStr + " at " + snpID + " for " + indivID+ " in "+file.getName());
                        String unexpectInfo = "Unexpected genotype " + gty.charAt(1) + " at " + snpID + " in " + file.getName();
                        //GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                        GlobalVariables.addInforLog(unexpectInfo);
                    }
                }
                bw.write("\n");
            } else {
                bw.write(line);
                bw.write("\n");
            }
        }
        bw.flush();
        bw.close();
        br.close();

        String info = ingoredSNP.size() + "SNP's genotypes are ignored because their SNPs have not information in our annotation file (detailed in the log info.)!";
        GlobalVariables.mainView.setBriefRunningInfor(info);
        GlobalVariables.addInforLog(info);
        GlobalVariables.mainView.setBriefRunningInfor(ingoredSNP.toString() + "\n");
        GlobalVariables.addInforLog(ingoredSNP.toString());

        return cvtFile.getCanonicalPath();
    }
}
