// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.
// Permission is granted for you to use this file to compile IGG.
// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.controller;

import cern.colt.list.LongArrayList;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import net.sf.picard.liftover.LiftOver;
import net.sf.picard.util.Interval;
import org.cobi.igg3.Constants;
import org.cobi.igg3.entity.AnnotSNP;
import org.cobi.igg3.entity.ExportSetting;
import org.cobi.igg3.entity.Individual;
import org.cobi.igg3.entity.Integration;
import org.cobi.igg3.entity.PedFile;
import org.cobi.igg3.entity.StatusGtySet;
import org.cobi.igg3.gui.IGG3View;
import org.cobi.igg3.GlobalVariables;
import org.cobi.igg3.util.LocalString;

/**
 *
 * @author mxli
 */
public class BasicGenetExporter implements Constants {

    protected Integration integration;
    protected ExportSetting exportSetting;
    int exportGenomeVersion = 36;
    LiftOver liftOver;
    protected String geneticMapFile;

    public String getGeneticMapFile() {
        return geneticMapFile;
    }

    public void setGeneticMapFile(String geneticMapFile) {
        this.geneticMapFile = geneticMapFile;
    }
    

    public int getExportGenomeVersion() {
        return exportGenomeVersion;
    }

    public void setExportGenomeVersion(int exportGenomeVersion) {
        this.exportGenomeVersion = exportGenomeVersion;
        File CHAIN_FILE = null;
        if (exportGenomeVersion == 37) {
            CHAIN_FILE = new File(RESOURCE_PATH + "chip_annotation/hg18ToHg19.over.chain");
        } else if (exportGenomeVersion == 35) {
            CHAIN_FILE = new File(RESOURCE_PATH + "chip_annotation/hg18ToHg17.over.chain");
        }
        if (CHAIN_FILE != null) {
            liftOver = new LiftOver(CHAIN_FILE);
        }
    }

    public ExportSetting getExportSetting() {
        return exportSetting;
    }

    public void setExportSetting(ExportSetting exportSetting) {
        this.exportSetting = exportSetting;
    }

    public Integration getIntegrator() {
        return integration;
    }

    public void setIntegrator(Integration integrator) {
        this.integration = integrator;
    }

    public void preprocessSNPGty(List<AnnotSNP> mainSnpMap, Map<String, StatusGtySet> indivGtyMap, String chrom) throws Exception {
        String msg = "";
        int snpSize = mainSnpMap.size();
        if (snpSize == 0) {
            msg = "No SNPs on chromosome " + chrom;
            GlobalVariables.mainView.setBriefRunningInfor(msg);
            GlobalVariables.addInforLog(msg);
            return;
        }
        GenotypeQC genotypeQC = new GenotypeQC();
        if (exportSetting.isRemoveMissingAll()) {
            msg = genotypeQC.removeAllMissing(mainSnpMap, indivGtyMap);
            if (msg.length() > 0) {
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);
            }
        }
        snpSize = mainSnpMap.size();
        if (snpSize == 0) {
            msg = "No SNPs on chromosome " + chrom;
            GlobalVariables.mainView.setBriefRunningInfor(msg);
            GlobalVariables.addInforLog(msg);
            return;
        }

        if (exportSetting.getTrimmingInterval() > 0) {
            msg = genotypeQC.trimCompiledGtySet(mainSnpMap, indivGtyMap,
                    exportSetting.getTrimmingInterval(), exportSetting.getTrimingType());
            GlobalVariables.mainView.setBriefRunningInfor(msg);
            GlobalVariables.addInforLog(msg);
        }

        if (exportSetting.isRemoveBadMendel()) {
            msg = genotypeQC.removeBadMendel(mainSnpMap, indivGtyMap);
            GlobalVariables.mainView.setBriefRunningInfor(msg);
            GlobalVariables.addInforLog(msg);
        }

        if (exportSetting.isCorrectAnnoFreq()) {
            genotypeQC.correctFrequency(mainSnpMap, indivGtyMap);
            msg = "Incosistant allele frequencies in our annoation are re-estimated according to observed sample.";
            GlobalVariables.mainView.setBriefRunningInfor(msg);
            GlobalVariables.addInforLog(msg);
        }

        //convert refrence genome
        if (exportGenomeVersion == 37 || exportGenomeVersion == 35) {
            Interval int1 = new Interval("chr" + chrom, 0, 0);
            snpSize = mainSnpMap.size();
            for (int i = 0; i < snpSize; i++) {
                AnnotSNP snp = mainSnpMap.get(i);
                int1.setPoint((int) snp.getPhysicalPosition());
                Interval int2 = liftOver.liftOver(int1);
                //System.out.println(int2.toString());
                if (int2 != null) {
                    snp.setPhysicalPosition(int2.getStart());
                    int2 = null;
                } else {
                    snp.setPhysicalPosition(-1);
                }
            }
        }

        snpSize = mainSnpMap.size();
        if (exportSetting.getStrandOut() == '+') {
            for (int i = 0; i < snpSize; i++) {
                AnnotSNP snp = mainSnpMap.get(i);
                if (snp.getStrand() == '-') {
                    //note: I do not alter the flanking sequence here because it will not be used
                    // although it should be altered correspondingly
                    snp.setAAllele(LocalString.getComplementalChar(snp.getAAllele()));
                    snp.setBAllele(LocalString.getComplementalChar(snp.getBAllele()));
                    snp.setStrand('+');
                }
            }
        } else if (exportSetting.getStrandOut() == '-') {
            for (int i = 0; i < snpSize; i++) {
                AnnotSNP snp = mainSnpMap.get(i);
                if (snp.getStrand() == '+') {
                    //note: I do not alter the flanking sequence here because it will not be used
                    // although it should be altered correspondingly
                    snp.setAAllele(LocalString.getComplementalChar(snp.getAAllele()));
                    snp.setBAllele(LocalString.getComplementalChar(snp.getBAllele()));
                    snp.setStrand('-');
                }
            }
        }

    }

    protected void generalLinkPedGenotypeCharAllele(List<AnnotSNP> entries, Map<String, StatusGtySet> indivGtyMap,
            String chrom) throws Exception {
        String missingGty = exportSetting.getMissingGty();
        int snpSize = entries.size();
        int[] orders = new int[snpSize];
        char[] AAlleles = new char[snpSize];
        char[] BAlleles = new char[snpSize];

        for (int i = 0; i < snpSize; i++) {
            AnnotSNP snp = entries.get(i);
            orders[i] = snp.order;
            AAlleles[i] = snp.getAAllele();
            BAlleles[i] = snp.getBAllele();
        }

        BufferedWriter bw = new BufferedWriter(new FileWriter(exportSetting.getExportPath() + chrom + ".ped.tmp"));
        List<String> prjIndiLabels = GlobalVariables.currentPedFile.getIndividualLabel();

        int prjIndiviSize = prjIndiLabels.size();
        for (int i = 0; i < prjIndiviSize; i++) {         
            StatusGtySet indivGty = indivGtyMap.get(prjIndiLabels.get(i));
            if (indivGty != null) {
                int gtySize = indivGty.existence.size();
                for (int j = 0; j < snpSize; j++) {
                    bw.write(" ");
                    if (gtySize > orders[j] && indivGty.existence.getQuick(orders[j])) {
                        if (indivGty.paternalChrom.getQuick(orders[j])) {
                            bw.write(String.valueOf(BAlleles[j]));
                        } else {
                            bw.write(String.valueOf(AAlleles[j]));
                        }
                        bw.write(" ");
                        if (indivGty.maternalChrom.getQuick(orders[j])) {
                            bw.write(String.valueOf(BAlleles[j]));
                        } else {
                            bw.write(String.valueOf(AAlleles[j]));
                        }
                    } else {
                        bw.write(missingGty);
                    }
                }
            } else {
                for (int j = 0; j < snpSize; j++) {
                    bw.write(" ");
                    bw.write(missingGty);
                }
            }
            bw.write("\n");
        }

        Iterator hapIndiLabelIter = integration.getHapIndivLabels().iterator();
        String hapmapLabel = null;
        while (hapIndiLabelIter.hasNext()) {
            hapmapLabel = hapIndiLabelIter.next().toString();
            StatusGtySet indivGty = indivGtyMap.get(hapmapLabel);

            if (indivGty != null) {
                int gtySize = indivGty.existence.size();
                for (int j = 0; j < snpSize; j++) {
                    bw.write(" ");
                    if (gtySize > orders[j] && indivGty.existence.getQuick(orders[j])) {
                        if (indivGty.paternalChrom.getQuick(orders[j])) {
                            bw.write(String.valueOf(BAlleles[j]));
                        } else {
                            bw.write(String.valueOf(AAlleles[j]));
                        }
                        bw.write(" ");
                        if (indivGty.maternalChrom.getQuick(orders[j])) {
                            bw.write(String.valueOf(BAlleles[j]));
                        } else {
                            bw.write(String.valueOf(AAlleles[j]));
                        }
                    } else {
                        bw.write(missingGty);
                    }
                }
            } else {
                for (int j = 0; j < snpSize; j++) {
                    bw.write(" ");
                    bw.write(missingGty);
                }
            }
            bw.write("\n");
        }
        bw.close();
    }

    
    protected void generalLinkPedGenotypeNumberAllele(BufferedWriter bwA2N, List<AnnotSNP> entries, Map<String, StatusGtySet> indivGtyMap,
            String chrom) throws Exception {
        int snpSize = entries.size();
        int[] orders = new int[snpSize];
        String missingGty = exportSetting.getMissingGty();

        for (int i = 0; i < snpSize; i++) {
            orders[i] = entries.get(i).order;
            AnnotSNP snp = entries.get(i);
            bwA2N.write(String.valueOf(snp.getChromosomeNum()));
            bwA2N.write(" ");
            bwA2N.write(String.valueOf(snp.getPhysicalPosition()));
            bwA2N.write(" ");
            bwA2N.write(snp.getRSID());
            bwA2N.write(" ");
            bwA2N.write(String.valueOf(snp.getAAllele()));
            bwA2N.write(" ");
            bwA2N.write(String.valueOf(snp.getBAllele()));
            bwA2N.newLine();
        }

        BufferedWriter bw = new BufferedWriter(new FileWriter(exportSetting.getExportPath() + chrom + ".ped.tmp"));
        List<String> prjIndiLabels = GlobalVariables.currentPedFile.getIndividualLabel();
        int prjIndiviSize = prjIndiLabels.size();
        for (int i = 0; i < prjIndiviSize; i++) {
            StatusGtySet indivGty = indivGtyMap.get(prjIndiLabels.get(i));
            if (indivGty != null) {
                int gtySize = indivGty.existence.size();
                for (int j = 0; j < snpSize; j++) {
                    bw.write(" ");
                    if (gtySize > orders[j] && indivGty.existence.getQuick(orders[j])) {
                        if (indivGty.paternalChrom.getQuick(orders[j])) {
                            bw.write("2");
                        } else {
                            bw.write("1");
                        }
                        bw.write(" ");
                        if (indivGty.maternalChrom.getQuick(orders[j])) {
                            bw.write("2");
                        } else {
                            bw.write("1");
                        }
                    } else {
                        bw.write(missingGty);
                    }
                }
            } else {
                for (int j = 0; j < snpSize; j++) {
                    bw.write(" ");
                    bw.write(missingGty);
                }
            }
            bw.write("\n");
        }

        Iterator<String> hapIndiLabelIter = integration.getHapIndivLabels().iterator();
        while (hapIndiLabelIter.hasNext()) {
            StatusGtySet indivGty = indivGtyMap.get(hapIndiLabelIter.next());
            if (indivGty != null) {
                int gtySize = indivGty.existence.size();
                for (int j = 0; j < snpSize; j++) {
                    bw.write(" ");
                    if (gtySize > orders[j] && indivGty.existence.getQuick(orders[j])) {
                        if (indivGty.paternalChrom.getQuick(orders[j])) {
                            bw.write("2");
                        } else {
                            bw.write("1");
                        }
                        bw.write(" ");
                        if (indivGty.maternalChrom.getQuick(orders[j])) {
                            bw.write("2");
                        } else {
                            bw.write("1");
                        }
                    } else {
                        bw.write(missingGty);
                    }
                }
            } else {
                for (int j = 0; j < snpSize; j++) {
                    bw.write(" ");
                    bw.write(missingGty);
                }
            }
            bw.write("\n");
        }
        bw.close();
    }

    public boolean mergeGeneralLinkPedGenotypeFiles(List<String> chroms, String writeFileName, PedFile hapPed) throws
            Exception {
        BufferedWriter bw = new BufferedWriter(new FileWriter(writeFileName));
        String line = null;
        Map<String, Individual> prjIndivMap = GlobalVariables.currentPedFile.getAllIndividuals();
        List<String> prjIndiLabels = GlobalVariables.currentPedFile.getIndividualLabel();
        List<String[]> outTraits = exportSetting.getTraitProper();
        List<String> avaiTrais = GlobalVariables.currentPedFile.getTraitNames();
        int traitSize = outTraits.size();
        int[] traitIndex = new int[traitSize];
        int allTraitSize = avaiTrais.size();
        String[] cell;
        traitSize--;
        //determine the output trait
        for (int i = 0; i <= traitSize; i++) {
            cell = outTraits.get(i);
            for (int j = 0; j < allTraitSize; j++) {
                if (cell[0].equals(avaiTrais.get(j))) {
                    traitIndex[i] = j;
                    break;
                }
            }
        }
        BufferedReader[] gtyBr = new BufferedReader[chroms.size()];
        for (int j = 0; j < chroms.size(); j++) {
            gtyBr[j] = new BufferedReader(new FileReader(exportSetting.getExportPath() + chroms.get(j) + ".ped.tmp"), FILE_READER_BUFFER_SIZE);
            if (gtyBr[j] == null) {
                throw new Exception("Cannot merge genotypes for SNPs on chromosome " + chroms.get(j));
            }
        }
        int indiviSize = prjIndiLabels.size();
        for (int i = 0; i < indiviSize; i++) {
            Individual indiv = prjIndivMap.get(prjIndiLabels.get(i));
            bw.write(indiv.getFamilyID());
            bw.write(" ");
            bw.write(indiv.getIndividualID());
            bw.write(" ");
            bw.write(indiv.getDadID());
            bw.write(" ");
            bw.write(indiv.getMomID());
            bw.write(" ");
            bw.write(String.valueOf(indiv.getGender()));
            if (traitSize >= 0) {
                bw.write(" ");
                //write trait value
                for (int j = 0; j < traitSize; j++) {
                    bw.write(indiv.getTraits().get(traitIndex[j]));
                    bw.write(" ");
                }
                bw.write(indiv.getTraits().get(traitIndex[traitSize]));
            }

            for (int j = 0; j < chroms.size(); j++) {
                if ((line = gtyBr[j].readLine()) != null) {
                    bw.write(line);
                }
            }
            bw.write("\n");
        }
        if (hapPed != null) {
            Iterator hapIndiLabelIter = integration.getHapIndivLabels().iterator();
            Map<String, Individual> hapIndivMap = hapPed.getAllIndividuals();
            String hapIndiv;
            while (hapIndiLabelIter.hasNext()) {
                //this is awful format of hapmap like NA18594.dup
                hapIndiv = (String) hapIndiLabelIter.next();
                int awfulIndex = hapIndiv.indexOf(".");
                if (awfulIndex >= 0) {
                    hapIndiv = hapIndiv.substring(0, hapIndiv.length() - 4);
                    GlobalVariables.mainView.setBriefRunningInfor("Warning!!! HapMap sample " + hapIndiv + " has duplicate genotypes", true);
                    GlobalVariables.addInforLog("Warning!!! HapMap sample " + hapIndiv + " has duplicate genotypes");
                }
                Individual indiv = hapIndivMap.get(hapIndiv);

                bw.write(indiv.getFamilyID());
                bw.write(" ");
                bw.write(indiv.getIndividualID());

                bw.write(" ");
                bw.write(indiv.getDadID());
                bw.write(" ");
                bw.write(indiv.getMomID());
                bw.write(" ");
                bw.write(String.valueOf(indiv.getGender()));
                if (traitSize >= 0) {
                    bw.write(" ");
                    //write trait value
                    for (int j = 0; j < traitSize; j++) {
                        bw.write(outTraits.get(j)[2]);
                        bw.write(" ");
                    }
                    bw.write(outTraits.get(traitSize)[2]);
                }

                for (int j = 0; j < chroms.size(); j++) {
                    if ((line = gtyBr[j].readLine()) != null) {
                        bw.write(line);
                        bw.write(" ");
                    }
                }
                bw.write("\n");
            }
        }

        bw.close();
        //remove the temparary files
        for (int j = 0; j < chroms.size(); j++) {
            gtyBr[j].close();
            File file = new File(exportSetting.getExportPath() + chroms.get(j) + ".ped.tmp");
            if (file.exists()) {
                file.delete();
            }
        }
        return true;
    }

    protected void generalLinkPedCharAllele(List<AnnotSNP> entries, Map<String, StatusGtySet> indivGtyMap,
            String chrom, PedFile hapPed) throws Exception {
        String missingGty = exportSetting.getMissingGty();
        int snpSize = entries.size();
        int[] orders = new int[snpSize];
        char[] AAlleles = new char[snpSize];
        char[] BAlleles = new char[snpSize];

        for (int i = 0; i < snpSize; i++) {
            AnnotSNP snp = entries.get(i);
            orders[i] = snp.order;
            AAlleles[i] = snp.getAAllele();
            BAlleles[i] = snp.getBAllele();

        }

        Map<String, Individual> prjIndivMap = GlobalVariables.currentPedFile.getAllIndividuals();
        List<String> prjIndiLabels = GlobalVariables.currentPedFile.getIndividualLabel();
        List<String[]> outTraits = exportSetting.getTraitProper();
        List<String> avaiTrais = GlobalVariables.currentPedFile.getTraitNames();
        int traitSize = outTraits.size();
        int[] traitIndex = new int[traitSize];
        int allTraitSize = avaiTrais.size();
        String[] cell;
        traitSize--;
        //determine the output trait
        for (int i = 0; i <= traitSize; i++) {
            cell = outTraits.get(i);
            for (int j = 0; j < allTraitSize; j++) {
                if (cell[0].equals(avaiTrais.get(j))) {
                    traitIndex[i] = j;
                    break;
                }
            }
        }

        BufferedWriter bw = new BufferedWriter(new FileWriter(exportSetting.getExportPath() + File.separator + exportSetting.getPrefixFileName() + "." + chrom + ".ped"));
        int indiviSize = prjIndiLabels.size();
        for (int k = 0; k < indiviSize; k++) {
            Individual indiv = prjIndivMap.get(prjIndiLabels.get(k));
            bw.write(indiv.getFamilyID());
            bw.write(" ");
            bw.write(indiv.getIndividualID());
            bw.write(" ");
            bw.write(indiv.getDadID());
            bw.write(" ");
            bw.write(indiv.getMomID());
            bw.write(" ");
            bw.write(String.valueOf(indiv.getGender()));
            if (traitSize >= 0) {
                bw.write(" ");
                //write trait value
                for (int j = 0; j < traitSize; j++) {
                    bw.write(indiv.getTraits().get(traitIndex[j]));
                    bw.write(" ");
                }
                bw.write(indiv.getTraits().get(traitIndex[traitSize]));
            }
            StatusGtySet indivGty = indivGtyMap.get(prjIndiLabels.get(k));
            if (indivGty != null) {
                int gtySize = indivGty.existence.size();
                for (int j = 0; j < snpSize; j++) {
                    bw.write(" ");
                    if (gtySize > orders[j] && indivGty.existence.getQuick(orders[j])) {
                        if (indivGty.paternalChrom.getQuick(orders[j])) {
                            bw.write(String.valueOf(BAlleles[j]));
                        } else {
                            bw.write(String.valueOf(AAlleles[j]));
                        }
                        bw.write(" ");
                        if (indivGty.maternalChrom.getQuick(orders[j])) {
                            bw.write(String.valueOf(BAlleles[j]));
                        } else {
                            bw.write(String.valueOf(AAlleles[j]));
                        }
                    } else {
                        bw.write(missingGty);
                    }
                }
            } else {
                for (int j = 0; j < snpSize; j++) {
                    bw.write(" ");
                    bw.write(missingGty);
                }
            }
            bw.write("\n");
        }

        if (hapPed != null) {
            Iterator<String> hapIndiLabelIter = integration.getHapIndivLabels().iterator();
            Map<String, Individual> hapIndivMap = hapPed.getAllIndividuals();
            while (hapIndiLabelIter.hasNext()) {
                Individual indiv = hapIndivMap.get(hapIndiLabelIter.next());
                bw.write(indiv.getFamilyID());
                bw.write(" ");
                bw.write(indiv.getIndividualID());

                bw.write(" ");
                bw.write(indiv.getDadID());
                bw.write(" ");
                bw.write(indiv.getMomID());
                bw.write(" ");
                bw.write(String.valueOf(indiv.getGender()));
                if (traitSize >= 0) {
                    bw.write(" ");
                    //write trait value
                    for (int j = 0; j < traitSize; j++) {
                        bw.write(outTraits.get(j)[2]);
                        bw.write(" ");
                    }
                    bw.write(outTraits.get(traitSize)[2]);
                }

                StatusGtySet indivGty = indivGtyMap.get(indiv.getLabelInChip());
                if (indivGty != null) {
                    int gtySize = indivGty.existence.size();
                    for (int j = 0; j < snpSize; j++) {
                        bw.write(" ");
                        if (gtySize > orders[j] && indivGty.existence.getQuick(orders[j])) {
                            if (indivGty.paternalChrom.getQuick(orders[j])) {
                                bw.write(String.valueOf(BAlleles[j]));
                            } else {
                                bw.write(String.valueOf(AAlleles[j]));
                            }
                            bw.write(" ");
                            if (indivGty.maternalChrom.getQuick(orders[j])) {
                                bw.write(String.valueOf(BAlleles[j]));
                            } else {
                                bw.write(String.valueOf(AAlleles[j]));
                            }
                        } else {
                            bw.write(missingGty);
                        }
                    }
                } else {
                    for (int j = 0; j < snpSize; j++) {
                        bw.write(" ");
                        bw.write(missingGty);
                    }
                }
                bw.write("\n");
            }
        }
        bw.close();
    }

    protected void generalLinkPedNumberAllele(BufferedWriter bwA2N, List<AnnotSNP> entries, Map<String, StatusGtySet> indivGtyMap,
            String chrom, PedFile hapPed) throws Exception {
        int snpSize = entries.size();
        int[] orders = new int[snpSize];
        String missingGty = exportSetting.getMissingGty();
        for (int i = 0; i < snpSize; i++) {
            orders[i] = entries.get(i).order;

            AnnotSNP snp = entries.get(i);
            bwA2N.write(String.valueOf(snp.getChromosomeNum()));
            bwA2N.write(" ");
            bwA2N.write(String.valueOf(snp.getPhysicalPosition()));
            bwA2N.write(" ");
            bwA2N.write(snp.getRSID());
            bwA2N.write(" ");
            bwA2N.write(String.valueOf(snp.getAAllele()));
            bwA2N.write(" ");
            bwA2N.write(String.valueOf(snp.getBAllele()));
            bwA2N.newLine();
        }

        Map<String, Individual> prjIndivMap = GlobalVariables.currentPedFile.getAllIndividuals();
        List<String> prjIndiLabels = GlobalVariables.currentPedFile.getIndividualLabel();
        List<String[]> outTraits = exportSetting.getTraitProper();
        List<String> avaiTrais = GlobalVariables.currentPedFile.getTraitNames();
        int traitSize = outTraits.size();
        int[] traitIndex = new int[traitSize];
        int allTraitSize = avaiTrais.size();
        String[] cell;
        traitSize--;
        //determine the output trait
        for (int i = 0; i <= traitSize; i++) {
            cell = outTraits.get(i);
            for (int j = 0; j < allTraitSize; j++) {
                if (cell[0].equals(avaiTrais.get(j))) {
                    traitIndex[i] = j;
                    break;
                }
            }
        }

        BufferedWriter bw = new BufferedWriter(new FileWriter(exportSetting.getExportPath() + File.separator + exportSetting.getPrefixFileName() + "." + chrom + ".ped"));
        int indiviSize = prjIndiLabels.size();
        for (int k = 0; k < indiviSize; k++) {
            Individual indiv = prjIndivMap.get(prjIndiLabels.get(k));
            bw.write(indiv.getFamilyID());
            bw.write(" ");
            bw.write(indiv.getIndividualID());
            bw.write(" ");
            bw.write(indiv.getDadID());
            bw.write(" ");
            bw.write(indiv.getMomID());
            bw.write(" ");
            bw.write(String.valueOf(indiv.getGender()));
            if (traitSize >= 0) {
                bw.write(" ");
                //write trait value
                for (int j = 0; j < traitSize; j++) {
                    bw.write(indiv.getTraits().get(traitIndex[j]));
                    bw.write(" ");
                }
                bw.write(indiv.getTraits().get(traitIndex[traitSize]));
            }
            StatusGtySet indivGty = indivGtyMap.get(prjIndiLabels.get(k));
            if (indivGty != null) {
                int gtySize = indivGty.existence.size();
                for (int j = 0; j < snpSize; j++) {
                    bw.write(" ");
                    if (gtySize > orders[j] && indivGty.existence.getQuick(orders[j])) {
                        if (indivGty.paternalChrom.getQuick(orders[j])) {
                            bw.write("2");
                        } else {
                            bw.write("1");
                        }
                        bw.write(" ");
                        if (indivGty.maternalChrom.getQuick(orders[j])) {
                            bw.write("2");
                        } else {
                            bw.write("1");
                        }
                    } else {
                        bw.write(missingGty);
                    }
                }
            } else {
                for (int j = 0; j < snpSize; j++) {
                    bw.write(" ");
                    bw.write(missingGty);
                }
            }
            bw.write("\n");
        }

        if (hapPed != null) {
            Iterator hapIndiLabelIter = integration.getHapIndivLabels().iterator();
            Map<String, Individual> hapIndivMap = hapPed.getAllIndividuals();
            String hapIndiv = null;
            while (hapIndiLabelIter.hasNext()) {
                //this is awful format of hapmap like NA18594.dup
                hapIndiv = (String) hapIndiLabelIter.next();
                int awfulIndex = hapIndiv.indexOf(".");
                if (awfulIndex >= 0) {
                    hapIndiv = hapIndiv.substring(0, hapIndiv.length() - 4);
                    GlobalVariables.mainView.setBriefRunningInfor("Warning!!! HapMap sample " + hapIndiv + " has duplicate genotypes", true);
                    GlobalVariables.addInforLog("Warning!!! HapMap sample " + hapIndiv + " has duplicate genotypes");
                }
                Individual indiv = hapIndivMap.get(hapIndiv);
                bw.write(indiv.getFamilyID());
                bw.write(" ");
                bw.write(indiv.getIndividualID());
                bw.write(" ");
                bw.write(indiv.getDadID());
                bw.write(" ");
                bw.write(indiv.getMomID());
                bw.write(" ");
                bw.write(String.valueOf(indiv.getGender()));
                //write trait value
                if (traitSize >= 0) {
                    bw.write(" ");
                    for (int j = 0; j < traitSize; j++) {
                        bw.write(outTraits.get(j)[2]);
                        bw.write(" ");
                    }
                    bw.write(outTraits.get(traitSize)[2]);
                }

                StatusGtySet indivGty = indivGtyMap.get(indiv.getLabelInChip());
                if (indivGty != null) {
                    int gtySize = indivGty.existence.size();
                    for (int j = 0; j < snpSize; j++) {
                        bw.write(" ");
                        if (gtySize > orders[j] && indivGty.existence.getQuick(orders[j])) {
                            if (indivGty.paternalChrom.getQuick(orders[j])) {
                                bw.write("2");
                            } else {
                                bw.write("1");
                            }
                            bw.write(" ");
                            if (indivGty.maternalChrom.getQuick(orders[j])) {
                                bw.write("2");
                            } else {
                                bw.write("1");
                            }
                        } else {
                            bw.write(missingGty);
                        }
                    }
                } else {
                    for (int j = 0; j < snpSize; j++) {
                        bw.write(" ");
                        bw.write(missingGty);
                    }
                }
                bw.write("\n");
            }
        }
        bw.close();
    }

    public void filterSNPsbyRegion(List<AnnotSNP> mainSnpMap, LongArrayList regions) throws Exception {
        List<AnnotSNP> tmpSnpMap = new ArrayList<AnnotSNP>();
        tmpSnpMap.addAll(mainSnpMap);
        mainSnpMap.clear();
        int regionNum = regions.size() / 2;
        boolean isWithin;
        long pos = -1;

        int size = tmpSnpMap.size();
        for (int k = 0; k < size; k++) {
            isWithin = false;
            pos = tmpSnpMap.get(k).getPhysicalPosition();
            for (int i = 0; i < regionNum; i++) {
                if (pos >= regions.get(i * 2) && pos <= regions.get(i * 2 + 1)) {
                    isWithin = true;
                    break;
                }
            }
            if (isWithin) {
                mainSnpMap.add(tmpSnpMap.get(k));
            }
        }
    }
}
