// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.
// Permission is granted for you to use this file to compile IGG.
// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.controller;

import cern.colt.bitvector.BitVector;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.TreeSet;
import org.cobi.igg3.Constants;
import org.cobi.igg3.entity.AnnotSNP;
import org.cobi.igg3.entity.MergedRsIDSet;
import org.cobi.igg3.entity.StatusGtySet;
import org.cobi.igg3.GlobalVariables;
import org.cobi.igg3.util.LocalString;

/**
 *
 * @author Miaoxin Li
 */
public class BasicIntegrator implements Constants {

    static int ANNO_COL_NUM = 13;
    static int COMPARE_LEN = 4;

    public String mergeSNPGtybyList(List<AnnotSNP> snpList1, List<AnnotSNP> snpList2,
            Map<String, StatusGtySet> gtyList1, Map<String, StatusGtySet> gtyList2) {
        int n1 = snpList1.size();
        int n2 = snpList2.size();
        int curMainSNPNum = n1;
        List<AnnotSNP> tmpSnpList = new ArrayList<AnnotSNP>();

        int copied1 = 0; // Number copied from the first list  of data
        int copied2 = 0; // Number copied from the second list  of data

        long position1 = 0;
        long position2 = 0;
        boolean isReversed = false;

        AnnotSNP snpInSub, tmpMainSNP;

        char mainAlleleALabel, subAlleleALabel, mainAlleleBLabel, subAlleleBLabel;
        int notMergedSNPNum = 0;
        int subSNPOrder, subSNPPos, newPos;
        StringBuilder runningInfo = new StringBuilder();
        int newSNPNum = 0;
        char sameStrand = '0';
        while ((copied1 < n1) && (copied2 < n2)) {
            position1 = snpList1.get(copied1).getPhysicalPosition();
            position2 = snpList2.get(copied2).getPhysicalPosition();

            if (position1 < position2) {
                copied1++;
            } else if (position1 == position2) {
                copied1++;
                copied2++;
            } else {
                copied2++;
                newSNPNum++;
            }
        }
        newSNPNum = newSNPNum + n2 - copied2;
        //runningInfo.append("Overlapped SNP: ");
        //runningInfo.append(n2 - newSNPNum);

        int finalMainSNPNum = (curMainSNPNum + newSNPNum);
        // System.out.println(curMainSNPNum + " " + newSNPNum + " " + finalMainSNPNum);
        if (newSNPNum > 0) {
            //resize genotype bitvectors of individuals
            for (Map.Entry<String, StatusGtySet> m : gtyList1.entrySet()) {
                m.getValue().paternalChrom.setSize(finalMainSNPNum);
                m.getValue().maternalChrom.setSize(finalMainSNPNum);
                m.getValue().existence.setSize(finalMainSNPNum);
            }
        }
        System.gc();
        // add new subjects into the mainIndivGty
        for (Map.Entry<String, StatusGtySet> m1 : gtyList2.entrySet()) {
            StatusGtySet invid2 = gtyList1.get(m1.getKey());
            if (invid2 == null) {
                invid2 = new StatusGtySet();
                invid2.paternalChrom = new BitVector(finalMainSNPNum);
                invid2.maternalChrom = new BitVector(finalMainSNPNum);
                invid2.existence = new BitVector(finalMainSNPNum);
                gtyList1.put(m1.getKey(), invid2);
            }
        }
        copied1 = 0;
        copied2 = 0;
        // Merge elements, copying from two halves of data to the temporary array.
        while ((copied1 < n1) && (copied2 < n2)) {
            position1 = snpList1.get(copied1).getPhysicalPosition();
            position2 = snpList2.get(copied2).getPhysicalPosition();

            if (position1 < position2) {
                tmpSnpList.add(snpList1.get(copied1));
                copied1++;
            } else if (position1 == position2) {
                tmpMainSNP = snpList1.get(copied1);
                snpInSub = snpList2.get(copied2);
                subSNPOrder = snpInSub.order;
                subSNPPos = subSNPOrder;

                copied1++;
                copied2++;

                mainAlleleALabel = tmpMainSNP.getAAllele();
                subAlleleALabel = snpInSub.getAAllele();
                mainAlleleBLabel = tmpMainSNP.getBAllele();
                subAlleleBLabel = snpInSub.getBAllele();
                //use one allele names to infer the missing
                if (mainAlleleBLabel == MISSING_ALLELE_NAME && subAlleleALabel != MISSING_ALLELE_NAME && subAlleleBLabel != MISSING_ALLELE_NAME) {
                    if (!LocalString.isComplementalChar(subAlleleALabel, subAlleleBLabel)) {
                        if (mainAlleleALabel == subAlleleALabel) {
                            mainAlleleBLabel = subAlleleBLabel;
                            tmpMainSNP.setBAllele(mainAlleleBLabel);
                        } else if (mainAlleleALabel == subAlleleBLabel) {
                            mainAlleleBLabel = subAlleleALabel;
                            tmpMainSNP.setBAllele(mainAlleleBLabel);
                        } else if (LocalString.isComplementalChar(mainAlleleALabel, subAlleleALabel)) {
                            mainAlleleBLabel = LocalString.getComplementalChar(subAlleleBLabel);
                            tmpMainSNP.setBAllele(mainAlleleBLabel);
                        } else if (LocalString.isComplementalChar(mainAlleleALabel, subAlleleBLabel)) {
                            mainAlleleBLabel = LocalString.getComplementalChar(subAlleleALabel);
                            tmpMainSNP.setBAllele(mainAlleleBLabel);
                        } else {
                            notMergedSNPNum++;
                            GlobalVariables.addInforLog("Allele Designation is inconsistant for " + tmpMainSNP.getRSID());
                            continue;
                        }
                    } else {
                        sameStrand = checkStrandAndFlankSequence(tmpMainSNP, snpInSub);
                        if (sameStrand == '+') {
                            if (mainAlleleALabel == subAlleleALabel) {
                                mainAlleleBLabel = subAlleleBLabel;
                                tmpMainSNP.setBAllele(mainAlleleBLabel);
                            } else if (mainAlleleALabel == subAlleleBLabel) {
                                mainAlleleBLabel = subAlleleALabel;
                                tmpMainSNP.setBAllele(mainAlleleBLabel);
                            } else {
                                notMergedSNPNum++;
                                GlobalVariables.addInforLog("Allele Designation is inconsistant for " + tmpMainSNP.getRSID());
                                continue;
                            }
                        } else if (sameStrand == '-') {
                            if (LocalString.isComplementalChar(mainAlleleALabel, subAlleleALabel)) {
                                mainAlleleBLabel = LocalString.getComplementalChar(subAlleleBLabel);
                                tmpMainSNP.setBAllele(mainAlleleBLabel);
                            } else if (LocalString.isComplementalChar(mainAlleleALabel, subAlleleBLabel)) {
                                mainAlleleBLabel = LocalString.getComplementalChar(subAlleleALabel);
                                tmpMainSNP.setBAllele(mainAlleleBLabel);
                            } else {
                                notMergedSNPNum++;
                                GlobalVariables.addInforLog("Allele Designation is inconsistant for " + tmpMainSNP.getRSID());
                                continue;
                            }
                        } else {
                            notMergedSNPNum++;
                            GlobalVariables.addInforLog("Allele Designation is inconsistant for " + tmpMainSNP.getRSID());
                            continue;
                        }
                    }
                } else if (subAlleleBLabel == MISSING_ALLELE_NAME && mainAlleleALabel != MISSING_ALLELE_NAME && mainAlleleBLabel != MISSING_ALLELE_NAME) {
                    if (!LocalString.isComplementalChar(mainAlleleALabel, mainAlleleBLabel)) {
                        if (subAlleleALabel == mainAlleleALabel) {
                            subAlleleBLabel = mainAlleleBLabel;
                            snpInSub.setBAllele(subAlleleBLabel);
                        } else if (subAlleleALabel == mainAlleleBLabel) {
                            subAlleleBLabel = mainAlleleALabel;
                            snpInSub.setBAllele(subAlleleBLabel);
                        } else if (LocalString.isComplementalChar(subAlleleALabel, mainAlleleALabel)) {
                            subAlleleBLabel = LocalString.getComplementalChar(mainAlleleBLabel);
                            snpInSub.setBAllele(subAlleleBLabel);
                        } else if (LocalString.isComplementalChar(subAlleleALabel, mainAlleleBLabel)) {
                            subAlleleBLabel = LocalString.getComplementalChar(mainAlleleALabel);
                            snpInSub.setBAllele(subAlleleBLabel);
                        } else {
                            notMergedSNPNum++;
                            GlobalVariables.addInforLog("Allele Designation is inconsistant for " + snpInSub.getRSID());
                            continue;
                        }
                    } else {
                        sameStrand = checkStrandAndFlankSequence(snpInSub, tmpMainSNP);
                        if (sameStrand == '+') {
                            if (subAlleleALabel == mainAlleleALabel) {
                                subAlleleBLabel = mainAlleleBLabel;
                                snpInSub.setBAllele(subAlleleBLabel);
                            } else if (subAlleleALabel == mainAlleleBLabel) {
                                subAlleleBLabel = mainAlleleALabel;
                                snpInSub.setBAllele(subAlleleBLabel);
                            } else {
                                notMergedSNPNum++;
                                GlobalVariables.addInforLog("Allele Designation is inconsistant for " + snpInSub.getRSID());
                                continue;
                            }
                        } else if (sameStrand == '-') {
                            if (LocalString.isComplementalChar(subAlleleALabel, mainAlleleALabel)) {
                                subAlleleBLabel = LocalString.getComplementalChar(mainAlleleBLabel);
                                snpInSub.setBAllele(subAlleleBLabel);
                            } else if (LocalString.isComplementalChar(subAlleleALabel, mainAlleleBLabel)) {
                                subAlleleBLabel = LocalString.getComplementalChar(mainAlleleALabel);
                                snpInSub.setBAllele(subAlleleBLabel);
                            } else {
                                notMergedSNPNum++;
                                GlobalVariables.addInforLog("Allele Designation is inconsistant for " + snpInSub.getRSID());
                                continue;
                            }
                        } else {
                            notMergedSNPNum++;
                            GlobalVariables.addInforLog("Allele Designation is inconsistant for " + snpInSub.getRSID());
                            continue;
                        }
                    }
                } else if (subAlleleALabel != MISSING_ALLELE_NAME && mainAlleleALabel != MISSING_ALLELE_NAME && subAlleleBLabel == MISSING_ALLELE_NAME && mainAlleleBLabel == MISSING_ALLELE_NAME) {
                    sameStrand = checkStrandAndFlankSequence(tmpMainSNP, snpInSub);
                    if (sameStrand == '+') {
                        if (mainAlleleALabel != subAlleleALabel) {
                            mainAlleleBLabel = subAlleleALabel;
                            tmpMainSNP.setBAllele(mainAlleleBLabel);
                        }
                    } else if (sameStrand == '-') {
                        if (!LocalString.isComplementalChar(mainAlleleALabel, subAlleleALabel)) {
                            mainAlleleBLabel = LocalString.getComplementalChar(subAlleleALabel);
                            tmpMainSNP.setBAllele(mainAlleleBLabel);
                        }
                    }
                }

                tmpSnpList.add(tmpMainSNP);

                //only complementary allele names need the flanking sequenceis
                if (LocalString.isComplementalChar(mainAlleleALabel, mainAlleleBLabel)
                        && LocalString.isComplementalChar(subAlleleALabel, subAlleleBLabel)) {
                    sameStrand = checkStrandAndFlankSequence(tmpMainSNP, snpInSub);
                    //if none of them is true, there must be something wrong in the annoation
                    if (sameStrand == '0') {
                        notMergedSNPNum++;
                        GlobalVariables.addInforLog("Flanking Sequences are inconsistant for " + tmpMainSNP.getRSID() + " This item are ignored");
                        continue;
                    }

                    if (sameStrand == '+') {
                        if (mainAlleleALabel == subAlleleALabel
                                && mainAlleleBLabel == subAlleleBLabel) {
                            isReversed = false;
                        } else if (mainAlleleALabel == subAlleleBLabel
                                && mainAlleleBLabel == subAlleleALabel) {
                            isReversed = true;
                        } else {
                            notMergedSNPNum++;
                            GlobalVariables.addInforLog("Allele Designation is inconsistant for " + tmpMainSNP.getRSID());
                            continue;
                        }
                    } else if (sameStrand == '-') {
                        if (LocalString.isComplementalChar(mainAlleleALabel, subAlleleALabel)
                                && LocalString.isComplementalChar(mainAlleleBLabel, subAlleleBLabel)) {
                            isReversed = false;
                        } else if (LocalString.isComplementalChar(mainAlleleALabel, subAlleleBLabel)
                                && LocalString.isComplementalChar(mainAlleleBLabel, subAlleleALabel)) {
                            isReversed = true;
                        } else {
                            notMergedSNPNum++;
                            GlobalVariables.addInforLog("Allele Designation is inconsistant for " + tmpMainSNP.getRSID());
                            continue;
                        }
                    }
                } else if (mainAlleleALabel == subAlleleALabel && mainAlleleBLabel == subAlleleBLabel) {
                    isReversed = false;
                } else if (mainAlleleALabel == subAlleleBLabel && mainAlleleBLabel == subAlleleALabel) {
                    isReversed = true;
                } else if (LocalString.isComplementalChar(mainAlleleALabel, subAlleleALabel)
                        && LocalString.isComplementalChar(mainAlleleBLabel, subAlleleBLabel)) {
                    isReversed = false;
                } else if (LocalString.isComplementalChar(mainAlleleALabel, subAlleleBLabel)
                        && LocalString.isComplementalChar(mainAlleleBLabel, subAlleleALabel)) {
                    isReversed = true;
                } else {
                    notMergedSNPNum++;
                    GlobalVariables.addInforLog("Allele Designation is inconsistant for " + tmpMainSNP.getRSID());
                    continue;
                }

                int mainAlleleCount = 0, subAlleleCount = 0;
                newPos = tmpMainSNP.order;

                if (isReversed) {
                    for (Map.Entry<String, StatusGtySet> m1 : gtyList2.entrySet()) {
                        StatusGtySet subInvid = m1.getValue();
                        StatusGtySet mainInvid = gtyList1.get(m1.getKey());
                        if (subInvid.existence.getQuick(subSNPPos) && mainInvid.existence.getQuick(newPos)) {
                            mainAlleleCount = 0;
                            subAlleleCount = 0;
                            if (mainInvid.paternalChrom.getQuick(newPos)) {
                                mainAlleleCount++;
                            }
                            if (mainInvid.maternalChrom.getQuick(newPos)) {
                                mainAlleleCount++;
                            }
                            if (!subInvid.paternalChrom.getQuick(subSNPPos)) {
                                subAlleleCount++;
                            }
                            if (!subInvid.maternalChrom.getQuick(subSNPPos)) {
                                subAlleleCount++;
                            }
                            if (mainAlleleCount != subAlleleCount) {
                                runningInfo.append("\nGenotpye  of ");
                                runningInfo.append(m1.getKey());
                                runningInfo.append(" is conflicting for SNP ");
                                runningInfo.append(snpInSub.getRSID());
                                runningInfo.append(".");
                                runningInfo.append(". Neither genotype is used. You are suggested to check them in original files!\n");
                                mainInvid.existence.putQuick(newPos, false);
                            }
                        } else if (subInvid.existence.getQuick(subSNPPos)) {
                            mainInvid.paternalChrom.putQuick(newPos, !subInvid.paternalChrom.getQuick(subSNPPos));
                            mainInvid.maternalChrom.putQuick(newPos, !subInvid.maternalChrom.getQuick(subSNPPos));
                            mainInvid.existence.putQuick(newPos, true);
                        }
                    }
                } else {
                    for (Map.Entry<String, StatusGtySet> m1 : gtyList2.entrySet()) {
                        StatusGtySet subInvid = m1.getValue();
                        StatusGtySet mainInvid = gtyList1.get(m1.getKey());
                        subInvid.existence.getQuick(subSNPPos);
                        mainInvid.existence.getQuick(newPos);

                        if (subInvid.existence.getQuick(subSNPPos) && mainInvid.existence.getQuick(newPos)) {
                            mainAlleleCount = 0;
                            subAlleleCount = 0;
                            if (mainInvid.paternalChrom.getQuick(newPos)) {
                                mainAlleleCount++;
                            }
                            if (mainInvid.maternalChrom.getQuick(newPos)) {
                                mainAlleleCount++;
                            }
                            if (subInvid.paternalChrom.getQuick(subSNPPos)) {
                                subAlleleCount++;
                            }
                            if (subInvid.maternalChrom.getQuick(subSNPPos)) {
                                subAlleleCount++;
                            }
                            if (mainAlleleCount != subAlleleCount) {
                                runningInfo.append("Genotpye  of ");
                                runningInfo.append(m1.getKey());
                                runningInfo.append(" is conflicting for SNP ");
                                runningInfo.append(snpInSub.getRSID());
                                runningInfo.append(".");
                                runningInfo.append(". Neither genotype is used. You are suggested to check them in original files!\n");
                                mainInvid.existence.putQuick(newPos, false);
                            }
                        } else if (subInvid.existence.getQuick(subSNPPos)) {
                            mainInvid.paternalChrom.putQuick(newPos, subInvid.paternalChrom.getQuick(subSNPPos));
                            mainInvid.maternalChrom.putQuick(newPos, subInvid.maternalChrom.getQuick(subSNPPos));
                            mainInvid.existence.putQuick(newPos, true);
                        }
                    }
                }
            } else {
                snpInSub = snpList2.get(copied2);
                subSNPOrder = snpInSub.order;
                subSNPPos = subSNPOrder;
                newPos = curMainSNPNum;

                for (Map.Entry<String, StatusGtySet> m1 : gtyList2.entrySet()) {
                    StatusGtySet invid = m1.getValue();
                    if (invid.existence.getQuick(subSNPPos)) {
                        StatusGtySet invid2 = gtyList1.get(m1.getKey());
                        invid2.paternalChrom.putQuick(newPos, invid.paternalChrom.getQuick(subSNPPos));
                        invid2.existence.putQuick(newPos, true);
                        invid2.maternalChrom.putQuick(newPos, invid.maternalChrom.getQuick(subSNPPos));
                    }
                }

                copied2++;
                snpInSub.order = newPos;
                tmpSnpList.add(snpInSub);
                curMainSNPNum++;
            }
        }

// Copy any remaining entries in the left and right subarrays.
        if (copied1 < n1) {
            snpList2.clear();
            while (copied1 < n1) {
                tmpSnpList.add(snpList1.get(copied1));
                copied1++;
            }
        }
        snpList1.clear();
        snpList1.addAll(tmpSnpList);
        tmpSnpList.clear();
        while (copied2 < n2) {
            snpInSub = snpList2.get(copied2);
            subSNPOrder = snpInSub.order;
            subSNPPos = subSNPOrder;


            newPos = curMainSNPNum;

            for (Map.Entry<String, StatusGtySet> m1 : gtyList2.entrySet()) {
                StatusGtySet invid = m1.getValue();
                if (invid.existence.getQuick(subSNPPos)) {
                    StatusGtySet invid2 = gtyList1.get(m1.getKey());
                    invid2.paternalChrom.putQuick(newPos, invid.paternalChrom.getQuick(subSNPPos));
                    invid2.paternalChrom.putQuick(newPos, invid.paternalChrom.getQuick(subSNPPos));
                }
            }

            copied2++;
            snpInSub.order = newPos;
            snpList1.add(snpInSub);
            curMainSNPNum++;
        }
        snpList2.clear();

        if (notMergedSNPNum > 0) {
            runningInfo.append(notMergedSNPNum);
            runningInfo.append(" SNPs have were inconsistent flanking sequences!");
        }

        return runningInfo.toString();
    }

    private char checkStrandAndFlankSequence(AnnotSNP snp1, AnnotSNP snp2) {
        //'0': uknown '+' same '-' contraversary
        char sameStrand = '0';
        String mainLeftFlankSeq = null, subLeftFlankSeq = null, mainRightFlankSeq = null, subRightFlankSeq = null;
        boolean isForwardSame = false;
        boolean isBackwardSame = false;
        String tmpStr;
        StringBuilder reverse = new StringBuilder();

        //use the strand information decide first
        if (snp2.getStrand() == snp1.getStrand() && snp1.getStrand() != MISSING_STRAND_NAME) {
            sameStrand = '+';
        } else if (snp2.getStrand() != snp1.getStrand() && snp1.getStrand() != MISSING_STRAND_NAME && snp2.getStrand() != MISSING_STRAND_NAME) {
            sameStrand = '-';
        } else {
            //consider the flanking sequences
            //finally we have to use the flanking sequence to decide
            //if either flanking sequence is null this cannot be down
            if (subLeftFlankSeq == null || mainLeftFlankSeq == null) {
                return sameStrand;
            }
            int endNum1 = mainLeftFlankSeq.indexOf('[');
            int endNum2 = subLeftFlankSeq.indexOf('[');
            mainLeftFlankSeq = mainLeftFlankSeq.substring(0, endNum1);
            subLeftFlankSeq = subLeftFlankSeq.substring(0, endNum2);

            endNum1 = mainRightFlankSeq.lastIndexOf(']');
            endNum2 = subRightFlankSeq.lastIndexOf(']');
            mainRightFlankSeq = mainRightFlankSeq.substring(endNum1 + 1);
            subRightFlankSeq = subRightFlankSeq.substring(endNum2 + 1);

            //check whether the flanking sequences are the same in the same order.
            isForwardSame = LocalString.isRightSideSameSequences(mainLeftFlankSeq, subLeftFlankSeq, COMPARE_LEN) && LocalString.isLeftSideSameSequences(mainRightFlankSeq, subRightFlankSeq, COMPARE_LEN);
            if (isForwardSame) {
                sameStrand = '+';
            } else if (LocalString.isRightSideComplementalSquences(mainLeftFlankSeq, subLeftFlankSeq, COMPARE_LEN) && LocalString.isLeftSideComplementalSquences(mainRightFlankSeq, subRightFlankSeq, COMPARE_LEN)) {
                sameStrand = '-';
            } else {
                tmpStr = subLeftFlankSeq;
                reverse.delete(0, reverse.length());
                reverse.append(subRightFlankSeq);
                subLeftFlankSeq = reverse.reverse().toString();
                reverse.delete(0, reverse.length());
                reverse.append(tmpStr);
                subRightFlankSeq = reverse.reverse().toString();

                //check whether the flanking sequences are the same in the same order.
                isBackwardSame = LocalString.isRightSideSameSequences(mainLeftFlankSeq, subLeftFlankSeq, COMPARE_LEN) && LocalString.isLeftSideSameSequences(mainRightFlankSeq, subRightFlankSeq, COMPARE_LEN);
                if (isBackwardSame) {
                    sameStrand = '+';
                } else if (LocalString.isRightSideComplementalSquences(mainLeftFlankSeq, subLeftFlankSeq, COMPARE_LEN) && LocalString.isLeftSideComplementalSquences(mainRightFlankSeq, subRightFlankSeq, COMPARE_LEN)) {
                    sameStrand = '-';
                } else {
                    return sameStrand;
                }
            }
        }
        return sameStrand;
    }

    /**
     * Merge genotypes in two different genotype datasets, merged data will be stored in snpList1 and gtyList1
     * Data in snpList2 and gtyList2 will be removedFF
     *
     * @param snpList1 one snp list ordered by physical positions
     * @param snpList2 the other snp list ordered by physical positions
     * @param gtyList1  genotypes of snps in snpList1
     * @param gtyList2  genotypes of snps in snpList2
     */
    public String mergeSNPGtybyListNoGtyExtend(List<AnnotSNP> snpList1, List<AnnotSNP> snpList2,
            Map<String, StatusGtySet> gtyList1, Map<String, StatusGtySet> gtyList2) {
        int n1 = snpList1.size();
        int n2 = snpList2.size();
        int curMainSNPNum = n1;
        List<AnnotSNP> mergedSnpList = new ArrayList<AnnotSNP>();

        int copied1 = 0; // Number copied from the first list  of data
        int copied2 = 0; // Number copied from the second list  of data

        long position1 = 0;
        long position2 = 0;
        boolean isReversed = false;

        List<String> noFlunkSequence = new ArrayList<String>();
        AnnotSNP snpInSub, tmpMainSNP;

        char mainAlleleALabel, subAlleleALabel, mainAlleleBLabel, subAlleleBLabel;
        int notMergedSNPNum = 0;


        int subSNPOrder, subSNPPos, newPos;
        StringBuilder runningInfo = new StringBuilder();
        int newSNPNum = 0;
        while ((copied1 < n1) && (copied2 < n2)) {
            position1 = snpList1.get(copied1).getPhysicalPosition();
            position2 = snpList2.get(copied2).getPhysicalPosition();

            if (position1 < position2) {
                copied1++;
            } else if (position1 == position2) {
                copied1++;
                copied2++;
            } else {
                copied2++;
                newSNPNum++;
            }
        }
        newSNPNum = newSNPNum + n2 - copied2;
        //runningInfo.append("Overlapped SNP: ");
        //runningInfo.append(n2 - newSNPNum);

        int finalMainSNPNum = (curMainSNPNum + newSNPNum);
        char sameStrand = '0';
        // add new subjects into the mainIndivGty
        for (Map.Entry<String, StatusGtySet> m1 : gtyList2.entrySet()) {
            StatusGtySet invid2 = gtyList1.get(m1.getKey());
            if (invid2 == null) {
                invid2 = new StatusGtySet();
                invid2.paternalChrom = new BitVector(finalMainSNPNum);
                invid2.maternalChrom = new BitVector(finalMainSNPNum);
                invid2.existence = new BitVector(finalMainSNPNum);
                gtyList1.put(m1.getKey(), invid2);
            } else {
                invid2.paternalChrom.setSize(finalMainSNPNum);
                invid2.maternalChrom.setSize(finalMainSNPNum);
                invid2.existence.setSize(finalMainSNPNum);
            }
        }
        copied1 = 0;
        copied2 = 0;
        // Merge elements, copying from two halves of data to the temporary array.
        while ((copied1 < n1) && (copied2 < n2)) {
            position1 = snpList1.get(copied1).getPhysicalPosition();
            position2 = snpList2.get(copied2).getPhysicalPosition();


            if (position1 < position2) {
                mergedSnpList.add(snpList1.get(copied1));
                copied1++;
            } else if (position1 == position2) {
                tmpMainSNP = snpList1.get(copied1);
                snpInSub = snpList2.get(copied2);
                subSNPOrder = snpInSub.order;
                subSNPPos = subSNPOrder;

                copied1++;
                copied2++;
                mainAlleleALabel = tmpMainSNP.getAAllele();
                subAlleleALabel = snpInSub.getAAllele();
                mainAlleleBLabel = tmpMainSNP.getBAllele();
                subAlleleBLabel = snpInSub.getBAllele();
                //use one allele names to infer the missing
                if (mainAlleleBLabel == MISSING_ALLELE_NAME && subAlleleALabel != MISSING_ALLELE_NAME && subAlleleBLabel != MISSING_ALLELE_NAME) {
                    if (!LocalString.isComplementalChar(subAlleleALabel, subAlleleBLabel)) {
                        if (mainAlleleALabel == subAlleleALabel) {
                            mainAlleleBLabel = subAlleleBLabel;
                            tmpMainSNP.setBAllele(mainAlleleBLabel);
                        } else if (mainAlleleALabel == subAlleleBLabel) {
                            mainAlleleBLabel = subAlleleALabel;
                            tmpMainSNP.setBAllele(mainAlleleBLabel);
                        } else if (LocalString.isComplementalChar(mainAlleleALabel, subAlleleALabel)) {
                            mainAlleleBLabel = LocalString.getComplementalChar(subAlleleBLabel);
                            tmpMainSNP.setBAllele(mainAlleleBLabel);
                        } else if (LocalString.isComplementalChar(mainAlleleALabel, subAlleleBLabel)) {
                            mainAlleleBLabel = LocalString.getComplementalChar(subAlleleALabel);
                            tmpMainSNP.setBAllele(mainAlleleBLabel);
                        } else {
                            notMergedSNPNum++;
                            GlobalVariables.addInforLog("Allele Designation is inconsistant for " + tmpMainSNP.getRSID());
                            continue;
                        }
                    } else {
                        sameStrand = checkStrandAndFlankSequence(tmpMainSNP, snpInSub);
                        if (sameStrand == '+') {
                            if (mainAlleleALabel == subAlleleALabel) {
                                mainAlleleBLabel = subAlleleBLabel;
                                tmpMainSNP.setBAllele(mainAlleleBLabel);
                            } else if (mainAlleleALabel == subAlleleBLabel) {
                                mainAlleleBLabel = subAlleleALabel;
                                tmpMainSNP.setBAllele(mainAlleleBLabel);
                            } else {
                                notMergedSNPNum++;
                                GlobalVariables.addInforLog("Allele Designation is inconsistant for " + tmpMainSNP.getRSID());
                                continue;
                            }
                        } else if (sameStrand == '-') {
                            if (LocalString.isComplementalChar(mainAlleleALabel, subAlleleALabel)) {
                                mainAlleleBLabel = LocalString.getComplementalChar(subAlleleBLabel);
                                tmpMainSNP.setBAllele(mainAlleleBLabel);
                            } else if (LocalString.isComplementalChar(mainAlleleALabel, subAlleleBLabel)) {
                                mainAlleleBLabel = LocalString.getComplementalChar(subAlleleALabel);
                                tmpMainSNP.setBAllele(mainAlleleBLabel);
                            } else {
                                notMergedSNPNum++;
                                GlobalVariables.addInforLog("Allele Designation is inconsistant for " + tmpMainSNP.getRSID());
                                continue;
                            }
                        } else {
                            notMergedSNPNum++;
                            GlobalVariables.addInforLog("Allele Designation is inconsistant for " + tmpMainSNP.getRSID());
                            continue;
                        }
                    }
                } else if (subAlleleBLabel == MISSING_ALLELE_NAME && mainAlleleALabel != MISSING_ALLELE_NAME && mainAlleleBLabel != MISSING_ALLELE_NAME) {
                    if (!LocalString.isComplementalChar(mainAlleleALabel, mainAlleleBLabel)) {
                        if (subAlleleALabel == mainAlleleALabel) {
                            subAlleleBLabel = mainAlleleBLabel;
                            snpInSub.setBAllele(subAlleleBLabel);
                        } else if (subAlleleALabel == mainAlleleBLabel) {
                            subAlleleBLabel = mainAlleleALabel;
                            snpInSub.setBAllele(subAlleleBLabel);
                        } else if (LocalString.isComplementalChar(subAlleleALabel, mainAlleleALabel)) {
                            subAlleleBLabel = LocalString.getComplementalChar(mainAlleleBLabel);
                            snpInSub.setBAllele(subAlleleBLabel);
                        } else if (LocalString.isComplementalChar(subAlleleALabel, mainAlleleBLabel)) {
                            subAlleleBLabel = LocalString.getComplementalChar(mainAlleleALabel);
                            snpInSub.setBAllele(subAlleleBLabel);
                        } else {
                            notMergedSNPNum++;
                            GlobalVariables.addInforLog("Allele Designation is inconsistant for " + snpInSub.getRSID());
                            continue;
                        }
                    } else {
                        sameStrand = checkStrandAndFlankSequence(snpInSub, tmpMainSNP);
                        if (sameStrand == '+') {
                            if (subAlleleALabel == mainAlleleALabel) {
                                subAlleleBLabel = mainAlleleBLabel;
                                snpInSub.setBAllele(subAlleleBLabel);
                            } else if (subAlleleALabel == mainAlleleBLabel) {
                                subAlleleBLabel = mainAlleleALabel;
                                snpInSub.setBAllele(subAlleleBLabel);
                            } else {
                                notMergedSNPNum++;
                                GlobalVariables.addInforLog("Allele Designation is inconsistant for " + snpInSub.getRSID());
                                continue;
                            }
                        } else if (sameStrand == '-') {
                            if (LocalString.isComplementalChar(subAlleleALabel, mainAlleleALabel)) {
                                subAlleleBLabel = LocalString.getComplementalChar(mainAlleleBLabel);
                                snpInSub.setBAllele(subAlleleBLabel);
                            } else if (LocalString.isComplementalChar(subAlleleALabel, mainAlleleBLabel)) {
                                subAlleleBLabel = LocalString.getComplementalChar(mainAlleleALabel);
                                snpInSub.setBAllele(subAlleleBLabel);
                            } else {
                                notMergedSNPNum++;
                                GlobalVariables.addInforLog("Allele Designation is inconsistant for " + snpInSub.getRSID());
                                continue;
                            }
                        } else {
                            notMergedSNPNum++;
                            GlobalVariables.addInforLog("Allele Designation is inconsistant for " + snpInSub.getRSID());
                            continue;
                        }
                    }
                } else if (subAlleleALabel != MISSING_ALLELE_NAME && mainAlleleALabel != MISSING_ALLELE_NAME && subAlleleBLabel == MISSING_ALLELE_NAME && mainAlleleBLabel == MISSING_ALLELE_NAME) {
                    sameStrand = checkStrandAndFlankSequence(tmpMainSNP, snpInSub);
                    if (sameStrand == '+') {
                        if (mainAlleleALabel != subAlleleALabel) {
                            mainAlleleBLabel = subAlleleALabel;
                            tmpMainSNP.setBAllele(mainAlleleBLabel);
                        }
                    } else if (sameStrand == '-') {
                        if (!LocalString.isComplementalChar(mainAlleleALabel, subAlleleALabel)) {
                            mainAlleleBLabel = LocalString.getComplementalChar(subAlleleALabel);
                            tmpMainSNP.setBAllele(mainAlleleBLabel);
                        }
                    }
                }
                mergedSnpList.add(tmpMainSNP);


                //only complementary allele names need the flanking sequenceis
                if (LocalString.isComplementalChar(mainAlleleALabel, mainAlleleBLabel)
                        && LocalString.isComplementalChar(subAlleleALabel, subAlleleBLabel)) {
                    sameStrand = checkStrandAndFlankSequence(tmpMainSNP, snpInSub);

                    //if none of them is true, there must be something wrong in the annoation
                    if (sameStrand == '0') {
                        notMergedSNPNum++;
                        GlobalVariables.addInforLog("Flanking Sequences are inconsistant for " + tmpMainSNP.getRSID() + " This item are ignored");
                        continue;
                    }

                    if (sameStrand == '+') {
                        if (mainAlleleALabel == subAlleleALabel
                                && mainAlleleBLabel == subAlleleBLabel) {
                            isReversed = false;
                        } else if (mainAlleleALabel == subAlleleBLabel
                                && mainAlleleBLabel == subAlleleALabel) {
                            isReversed = true;
                        } else {
                            notMergedSNPNum++;
                            GlobalVariables.addInforLog("Allele Designation is inconsistant for " + tmpMainSNP.getRSID());
                            continue;
                        }
                    } else if (sameStrand == '-') {
                        if (LocalString.isComplementalChar(mainAlleleALabel, subAlleleALabel)
                                && LocalString.isComplementalChar(mainAlleleBLabel, subAlleleBLabel)) {
                            isReversed = false;
                        } else if (LocalString.isComplementalChar(mainAlleleALabel, subAlleleBLabel)
                                && LocalString.isComplementalChar(mainAlleleBLabel, subAlleleALabel)) {
                            isReversed = true;
                        } else {
                            notMergedSNPNum++;
                            GlobalVariables.addInforLog("Allele Designation is inconsistant for " + tmpMainSNP.getRSID());
                            continue;
                        }
                    }
                } else if (mainAlleleALabel == subAlleleALabel && mainAlleleBLabel == subAlleleBLabel) {
                    isReversed = false;
                } else if (mainAlleleALabel == subAlleleBLabel && mainAlleleBLabel == subAlleleALabel) {
                    isReversed = true;
                } else if (LocalString.isComplementalChar(mainAlleleALabel, subAlleleALabel)
                        && LocalString.isComplementalChar(mainAlleleBLabel, subAlleleBLabel)) {
                    isReversed = false;
                } else if (LocalString.isComplementalChar(mainAlleleALabel, subAlleleBLabel)
                        && LocalString.isComplementalChar(mainAlleleBLabel, subAlleleALabel)) {
                    isReversed = true;
                } else {
                    notMergedSNPNum++;
                    GlobalVariables.addInforLog("Allele Designation is inconsistant for " + tmpMainSNP.getRSID());
                    continue;
                }

                int mainAlleleCount = 0, subAlleleCount = 0;
                newPos = tmpMainSNP.order;

                if (isReversed) {
                    for (Map.Entry<String, StatusGtySet> m1 : gtyList2.entrySet()) {
                        StatusGtySet subInvid = m1.getValue();
                        if (subSNPPos >= subInvid.existence.size()) {
                            continue;
                        }
                        StatusGtySet mainInvid = gtyList1.get(m1.getKey());
                        if (subInvid.existence.getQuick(subSNPPos) && mainInvid.existence.getQuick(newPos)) {
                            mainAlleleCount = 0;
                            subAlleleCount = 0;
                            if (mainInvid.paternalChrom.getQuick(newPos)) {
                                mainAlleleCount++;
                            }
                            if (mainInvid.maternalChrom.getQuick(newPos)) {
                                mainAlleleCount++;
                            }
                            if (!subInvid.paternalChrom.getQuick(subSNPPos)) {
                                subAlleleCount++;
                            }
                            if (!subInvid.maternalChrom.getQuick(subSNPPos)) {
                                subAlleleCount++;
                            }
                            if (mainAlleleCount != subAlleleCount) {
                                runningInfo.append("\nGenotpye  of ");
                                runningInfo.append(m1.getKey());
                                runningInfo.append(" is conflicting for SNP ");
                                runningInfo.append(snpInSub.getRSID());
                                runningInfo.append(".");
                                runningInfo.append(". Neither genotype is used. You are suggested to check them in original files!\n");
                                mainInvid.existence.putQuick(newPos, false);
                            }
                        } else if (subInvid.existence.getQuick(subSNPPos)) {
                            mainInvid.paternalChrom.putQuick(newPos, !subInvid.paternalChrom.getQuick(subSNPPos));
                            mainInvid.maternalChrom.putQuick(newPos, !subInvid.maternalChrom.getQuick(subSNPPos));
                            mainInvid.existence.putQuick(newPos, true);
                        }
                    }
                } else {
                    for (Map.Entry<String, StatusGtySet> m1 : gtyList2.entrySet()) {
                        StatusGtySet subInvid = m1.getValue();
                        if (subSNPPos >= subInvid.existence.size()) {
                            continue;
                        }
                        StatusGtySet mainInvid = gtyList1.get(m1.getKey());

                        if (subInvid.existence.getQuick(subSNPPos) && mainInvid.existence.getQuick(newPos)) {
                            mainAlleleCount = 0;
                            subAlleleCount = 0;
                            if (mainInvid.paternalChrom.getQuick(newPos)) {
                                mainAlleleCount++;
                            }
                            if (mainInvid.maternalChrom.getQuick(newPos)) {
                                mainAlleleCount++;
                            }
                            if (subInvid.paternalChrom.getQuick(subSNPPos)) {
                                subAlleleCount++;
                            }
                            if (subInvid.maternalChrom.getQuick(subSNPPos)) {
                                subAlleleCount++;
                            }
                            if (mainAlleleCount != subAlleleCount) {
                                runningInfo.append("Genotpye  of ");
                                runningInfo.append(m1.getKey());
                                runningInfo.append(" is conflicting for SNP ");
                                runningInfo.append(snpInSub.getRSID());
                                runningInfo.append(".");
                                runningInfo.append(". Neither genotype is used. You are suggested to check them in original files!\n");
                                mainInvid.existence.putQuick(newPos, false);
                            }
                        } else if (subInvid.existence.getQuick(subSNPPos)) {
                            mainInvid.paternalChrom.putQuick(newPos, subInvid.paternalChrom.getQuick(subSNPPos));
                            mainInvid.maternalChrom.putQuick(newPos, subInvid.maternalChrom.getQuick(subSNPPos));
                            mainInvid.existence.putQuick(newPos, true);
                        }
                    }
                }
            } else {
                snpInSub = snpList2.get(copied2);
                subSNPOrder = snpInSub.order;
                subSNPPos = subSNPOrder;
                newPos = curMainSNPNum;

                for (Map.Entry<String, StatusGtySet> m1 : gtyList2.entrySet()) {
                    StatusGtySet invid = m1.getValue();
                    if (subSNPPos < invid.existence.size() && invid.existence.getQuick(subSNPPos)) {
                        StatusGtySet invid2 = gtyList1.get(m1.getKey());
                        invid2.paternalChrom.putQuick(newPos, invid.paternalChrom.getQuick(subSNPPos));
                        invid2.maternalChrom.putQuick(newPos, invid.maternalChrom.getQuick(subSNPPos));
                        invid2.existence.putQuick(newPos, true);
                    }
                }

                copied2++;
                snpInSub.order = newPos;
                mergedSnpList.add(snpInSub);
                curMainSNPNum++;
            }
        }

// Copy any remaining entries in the left and right subarrays.
        if (copied1 < n1) {
            snpList2.clear();
            while (copied1 < n1) {
                mergedSnpList.add(snpList1.get(copied1));
                copied1++;
            }
        }
        snpList1.clear();
        snpList1.addAll(mergedSnpList);
        mergedSnpList.clear();
        while (copied2 < n2) {
            snpInSub = snpList2.get(copied2);
            subSNPOrder = snpInSub.order;
            subSNPPos = subSNPOrder;

            newPos = curMainSNPNum;

            for (Map.Entry<String, StatusGtySet> m1 : gtyList2.entrySet()) {
                StatusGtySet invid = m1.getValue();
                if (subSNPPos < invid.existence.size() && invid.existence.getQuick(subSNPPos)) {
                    StatusGtySet invid2 = gtyList1.get(m1.getKey());
                    invid2.paternalChrom.putQuick(newPos, invid.paternalChrom.getQuick(subSNPPos));
                    invid2.maternalChrom.putQuick(newPos, invid.maternalChrom.getQuick(subSNPPos));
                    invid2.existence.putQuick(newPos, true);
                }
            }

            copied2++;
            snpInSub.order = newPos;
            snpList1.add(snpInSub);
            curMainSNPNum++;
        }
        snpList2.clear();
        gtyList2.clear();

        if (notMergedSNPNum > 0) {
            runningInfo.append(notMergedSNPNum);
            runningInfo.append(" SNPs have were inconsistent flanking sequences!");
        }
        if (noFlunkSequence.size() > 0) {
            runningInfo.append("\n\n");
            runningInfo.append(noFlunkSequence);
            runningInfo.append(" SNPs in the other set have no flanking sequences\n!");
            for (int i = 0; i < noFlunkSequence.size(); i++) {
                runningInfo.append(noFlunkSequence.get(i));
                runningInfo.append(", ");
            }
        }
        return runningInfo.toString();
    }

    static public void appendMissingGty(int snpSize, Map<String, StatusGtySet> gtyList1) {
        for (Map.Entry<String, StatusGtySet> m1 : gtyList1.entrySet()) {
            StatusGtySet invid = m1.getValue();
            if (invid.paternalChrom.size() < snpSize) {
                invid.paternalChrom.setSize(snpSize);
                invid.maternalChrom.setSize(snpSize);
                invid.existence.setSize(snpSize);
            }
        }
    }

    /**
     * retrieve SNP information from the resource files
     * @param FileName
     * @param indices
     * @param arry
     * @param genetRegions pair of physical regions
     * @throws java.lang.Exception
     * @return
     */
    protected boolean retrieveChipSNPAnnotationByPysic(String fileName, Map<String, AnnotSNP> snpMapList, int keyIndex, int popuID, String chromosome,
            long[] physicalRegions) throws Exception {
        File annotationFile = new File(IGG_RESOURCE_CHIP_FOLDER + fileName + ".txt");
        BufferedReader br = new BufferedReader(new FileReader(annotationFile), FILE_READER_BUFFER_SIZE);
        String line = null;
        String[] linecells = null;
        String delmilit = "\t";

        boolean checkRegion = false;
        int regionNum = physicalRegions.length / 2;

        int j = 0;
        StringBuilder strRegion = new StringBuilder();
        for (j = 0; j < regionNum; j++) {
            if (physicalRegions[j * 2] != -9 || physicalRegions[j * 2 + 1] != -9) {
                checkRegion = true;
                if (physicalRegions[j * 2 + 1] == -9) {
                    physicalRegions[1] = Long.MAX_VALUE;
                }
                strRegion.append(" [");
                strRegion.append(physicalRegions[j * 2]);
                strRegion.append(",");
                strRegion.append(physicalRegions[j * 2 + 1]);
                strRegion.append("]bp");
            }
        }

        double freqA = 0.0;
        int counter = 0;
        int chroNum;
        if (chromosome.equalsIgnoreCase("X")) {
            chroNum = 23;
        } else if (chromosome.equalsIgnoreCase("Y")) {
            chroNum = 24;
        } else if (chromosome.equalsIgnoreCase("MT")) {
            chroNum = 26;
        } else if (chromosome.equalsIgnoreCase("XY")) {
            chroNum = 25;
        } else {
            chroNum = Integer.parseInt(chromosome);
        }
        if (checkRegion) {
            regionNum = physicalRegions.length / 2;
        }
        boolean invalid = false;
        long pos = 0;

        //skip the head line
        br.readLine();
        linecells = new String[ANNO_COL_NUM];
        boolean diffChrom = false;


        int index, posIndex;
        boolean toStop = false;
        Map<Long, Integer> physicalPos = new HashMap<Long, Integer>(10000, 0.9f);
        double decodeGenet = -9;
        StringBuilder tmpBuffer = new StringBuilder();
        boolean isInRegion = false;
        try {
            while ((line = br.readLine()) != null) {
                StringTokenizer tokenizer = new StringTokenizer(line, delmilit);
                index = 0;
                diffChrom = false;
                invalid = false;
                isInRegion = false;
                while (tokenizer.hasMoreTokens()) {
                    tmpBuffer.delete(0, tmpBuffer.length());
                    tmpBuffer.append(tokenizer.nextToken().trim());
                    linecells[index] = tmpBuffer.toString();
                    //filter other chromosomes, which is ordered in the annotation file
                    if (index == 2) {
                        if (linecells[index].compareTo(chromosome) < 0) {
                            diffChrom = true;
                            break;
                        } else if (linecells[index].compareTo(chromosome) > 0) {
                            toStop = true;
                            break;
                        }
                    } else if (index == 3) {
                        //filter items without physical postion
                        if (linecells[3].startsWith("-")) {
                            invalid = true;
                            break;
                        }

                        pos = Long.parseLong(linecells[3]);
                        if (physicalPos.get(pos) != null) {
                            invalid = true;
                            break;
                        }

                        //filter physical region                        
                        if (checkRegion) {
                            for (int i = 0; i < regionNum; i++) {
                                if (pos >= physicalRegions[i * 2] && pos > physicalRegions[i * 2 + 1]) {
                                    isInRegion = true;
                                    break;
                                }
                            }
                        }
                    }
                    index++;
                }
                if (toStop) {
                    break;
                }
                if (diffChrom) {
                    continue;
                }
                if (invalid) {
                    continue;
                }
                if (checkRegion && !isInRegion) {
                    continue;
                }
                //split is much slower than Tokenizer
                // linecells = line.split(delmilit);
                /*
                if (!LocalString.isNumeric(linecells[3])) {
                //System.markerOut.println(line);
                //continue;
                linecells[3] = "-1";
                }*/


                if (!linecells[1].startsWith("rs")) {
                    linecells[1] = "Chr" + chromosome + "Pos" + linecells[3];
                    //continue;
                }
                physicalPos.put(pos, 0);
                if (linecells[popuID].equals("-//-")) {
                    freqA = -0.5;
                } else {
                    posIndex = linecells[popuID].indexOf("//");
                    if (posIndex >= 0) {
                        freqA = Double.parseDouble(linecells[popuID].substring(0, posIndex).trim());
                    } else {
                        freqA = -0.5;
                    }
                }
                decodeGenet = -9;
                if (!linecells[4].startsWith("-")) {
                    decodeGenet = Double.parseDouble(linecells[4]);
                }
                // System.out.println(linecells[1]);

                //"Probe ID", "dbSNP ID", "Chromosome", "Physical Position", "Genetic Map",  "Allele Frequencies", "Flanking", "Allele_A", "Allele_B"
                //AnnotSNP(String RSID, int chromosomeNum, long physicalPosition, double decodeGeneticMap, String probeSetID, String flankingSequence, char aAllele, char bAllele, double aAlleleFreq, double bAlleleFreq)
                AnnotSNP aSNP = new AnnotSNP(linecells[1],
                        chroNum, pos,
                        decodeGenet, 0, linecells[6],
                        linecells[7].charAt(0), linecells[8].charAt(0), freqA,
                        1 - freqA, counter, linecells[12].charAt(0));

                counter++;
                snpMapList.put(linecells[keyIndex], aSNP);
            }
            StringBuilder runningInfo = new StringBuilder();
            runningInfo.append("The number of SNPs on chromosome ");
            runningInfo.append(chromosome);
            runningInfo.append(" ");
            runningInfo.append(strRegion);
            runningInfo.append(" in chip ");
            runningInfo.append(fileName);
            runningInfo.append(" is ");
            runningInfo.append(counter);
            runningInfo.append(".");
            GlobalVariables.mainView.setBriefRunningInfor(runningInfo.toString(), true);
            GlobalVariables.addInforLog(runningInfo.toString());
        } finally {
            br.close();
            return true;
        }

    }

    /**
     * retrieve SNP information from the resource files
     * @param FileName
     * @param indices
     * @param arry
     * @param genetRegions pair of physical regions
     * @throws java.lang.Exception
     * @return
     */
    protected boolean retrieveChipSNPAnnotationByGenet(String fileName, Map<String, AnnotSNP> snpMapList, int keyIndex, int popuID, String chromosome,
            double[] genetRegions) throws Exception {
        File annotationFile = new File(IGG_RESOURCE_CHIP_FOLDER + fileName + ".txt");
        BufferedReader br = new BufferedReader(new FileReader(annotationFile), FILE_READER_BUFFER_SIZE);
        String line = null;
        String[] linecells = null;
        String delmilit = "\t";

        boolean checkRegion = false;
        int regionNum = genetRegions.length / 2;

        int j = 0;
        StringBuilder strRegion = new StringBuilder();
        for (j = 0; j < regionNum; j++) {
            if (genetRegions[j * 2] != -9 || genetRegions[j * 2 + 1] != -9) {
                checkRegion = true;
                if (genetRegions[j * 2 + 1] == -9) {
                    genetRegions[1] = Long.MAX_VALUE;
                }
                strRegion.append(" [");
                strRegion.append(genetRegions[0]);
                strRegion.append(",");
                strRegion.append(genetRegions[1]);
                strRegion.append("]cM");
            }
        }

        double freqA = 0.0;
        int counter = 0;
        int chroNum;
        if (chromosome.equalsIgnoreCase("X")) {
            chroNum = 23;
        } else if (chromosome.equalsIgnoreCase("Y")) {
            chroNum = 24;
        } else if (chromosome.equalsIgnoreCase("MT")) {
            chroNum = 26;
        } else if (chromosome.equalsIgnoreCase("XY")) {
            chroNum = 25;
        } else {
            chroNum = Integer.parseInt(chromosome);
        }
        if (checkRegion) {
            regionNum = genetRegions.length / 2;
        }
        boolean invalid = false;
        //skip the head line
        br.readLine();
        linecells = new String[ANNO_COL_NUM];
        boolean diffChrom = false;

        int index, posIndex;
        Map<Long, Integer> physicalPos = new HashMap<Long, Integer>();
        double pos = 0.0;
        long posPhy = 0;
        StringBuilder tmpBuffer = new StringBuilder();

        boolean toStop = false;
        try {
            while ((line = br.readLine()) != null) {
                StringTokenizer tokenizer = new StringTokenizer(line, delmilit);
                index = 0;
                diffChrom = false;
                invalid = false;
                while (tokenizer.hasMoreTokens()) {
                    tmpBuffer.delete(0, tmpBuffer.length());
                    tmpBuffer.append(tokenizer.nextToken().trim());
                    linecells[index] = tmpBuffer.toString();
                    //filter other chromosomes
                    if (index == 2) {
                        if (linecells[index].compareTo(chromosome) < 0) {
                            diffChrom = true;
                            break;
                        } else if (linecells[index].compareTo(chromosome) > 0) {
                            toStop = true;
                            break;
                        }
                    } else if (index == 3) {
                        //filter items without physical postion
                        if (linecells[3].startsWith("-")) {
                            invalid = true;
                            break;
                        }

                        posPhy = Long.parseLong(linecells[3]);
                        if (physicalPos.containsKey(posPhy)) {
                            invalid = true;
                            break;
                        }
                    } else if (index == 4) {
                        //filter items without genetic postion
                        if (linecells[4].startsWith("-")) {
                            continue;
                        }
                        pos = Double.parseDouble(linecells[4]);
                        //filter genetic region                        
                        if (checkRegion) {
                            for (int i = 0; i < regionNum; i++) {
                                if (pos < genetRegions[i * 2] || pos > genetRegions[i * 2 + 1]) {
                                    invalid = true;
                                    break;
                                }
                            }
                        }
                    }
                    index++;
                }
                if (diffChrom) {
                    continue;
                }
                if (toStop) {
                    break;
                }
                //split is much slower than Tokenizer
                // linecells = line.split(delmilit);
                /*
                if (!LocalString.isNumeric(linecells[4])) {
                //System.markerOut.println(line);
                //continue;
                linecells[4] = "-1";
                }*/
                if (invalid) {
                    continue;
                }

                if (!linecells[1].startsWith("rs")) {
                    linecells[1] = "Chr" + chromosome + "Pos" + linecells[3];
                    //continue;
                }
                physicalPos.put(posPhy, 0);
                if (linecells[popuID].equals("-//-")) {
                    freqA = -0.5;
                } else {
                    posIndex = linecells[popuID].indexOf("//");
                    if (posIndex >= 0) {
                        freqA = Double.parseDouble(linecells[popuID].substring(0, posIndex).trim());
                    } else {
                        freqA = -0.5;
                    }
                }

                //"Probe ID", "dbSNP ID", "Chromosome", "Physical Position", "Genetic Map",  "Allele Frequencies", "Flanking", "Allele_A", "Allele_B"
                //AnnotSNP(String RSID, int chromosomeNum, long physicalPosition, double decodeGeneticMap, String probeSetID, String flankingSequence, char aAllele, char bAllele, double aAlleleFreq, double bAlleleFreq)
                AnnotSNP aSNP = new AnnotSNP(linecells[1],
                        chroNum, posPhy, pos, 0, linecells[6],
                        linecells[7].charAt(0), linecells[8].charAt(0),
                        freqA, 1 - freqA, counter, linecells[12].charAt(0));
                counter++;
                snpMapList.put(linecells[keyIndex], aSNP);
            }
            StringBuilder runningInfo = new StringBuilder();
            runningInfo.append("The number of SNPs on chromosome ");
            runningInfo.append(chromosome);
            runningInfo.append(" ");
            runningInfo.append(strRegion);
            runningInfo.append(" in chip ");
            runningInfo.append(fileName);
            runningInfo.append(" is ");
            runningInfo.append(counter);
            runningInfo.append(".");
            GlobalVariables.mainView.setBriefRunningInfor(runningInfo.toString(), true);
            GlobalVariables.addInforLog(runningInfo.toString());
        } finally {
            br.close();
            return true;
        }
    }

    public void addDbSNPFlankingSequenceInSNPMap(Map<Long, AnnotSNP> hapSnpMap, String chrom) throws Exception {
        File anaFile = new File("resources/hapmap/" + chrom);
        String line;
        String delmilit = "\t";
        StringTokenizer tokenizer = null;
        String rsID;
        String flankSeq;
        long pos;
        MergedRsIDSet mergedRsIDSet = new MergedRsIDSet();
        mergedRsIDSet.readRsMergeMap("resources" + File.separator + "hapmap" + File.separator + "mergedRSID" + File.separator + chrom + ".meg");
        boolean consistent;

        char aAllele, bAllele;
        char strand;
        StringBuilder discordantPosSNP = new StringBuilder();
        BufferedReader br = new BufferedReader(new FileReader(anaFile.getCanonicalPath()), FILE_READER_BUFFER_SIZE);
        //skip the title line
        br.readLine();

        TreeSet<String> misMatchedAlleleNames = new TreeSet<String>();
        TreeSet<String> misMatchedPhysicalPostions = new TreeSet<String>();
        StringBuilder tmpBuffer = new StringBuilder();

        while ((line = br.readLine()) != null) {
            tokenizer = new StringTokenizer(line, delmilit);
            tokenizer.nextToken();
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            rsID = tmpBuffer.toString();
            tokenizer.nextToken();
            pos = Long.parseLong(tokenizer.nextToken().trim());
            AnnotSNP snp = hapSnpMap.get(pos);
            if (snp != null) {
                consistent = false;
                if (snp.getRSID().equals(rsID)) {
                    consistent = true;
                } else {
                    if (mergedRsIDSet.rsMergeMap != null && !mergedRsIDSet.rsMergeMap.isEmpty()) {
                        TreeSet<String> synonymousSNPID = mergedRsIDSet.rsMergeMap.get(rsID);
                        if (synonymousSNPID != null && synonymousSNPID.contains(snp.getRSID())) {
                            //System.out.println(rsID+" <-> "+snp.getRSID());
                            consistent = true;
                        }
                    }
                }

                if (consistent) {
                    tokenizer.nextToken();
                    tokenizer.nextToken();
                    tmpBuffer.delete(0, tmpBuffer.length());
                    tmpBuffer.append(tokenizer.nextToken().trim());
                    flankSeq = tmpBuffer.toString();

                    aAllele = tokenizer.nextToken().trim().charAt(0);
                    bAllele = tokenizer.nextToken().trim().charAt(0);
                    tokenizer.nextToken();
                    tokenizer.nextToken();
                    tokenizer.nextToken();
                    tokenizer.nextToken();
                    strand = snp.getStrand();
                    //note: we assume all strands in our dbSNP anntation set are forward.
                    if (snp.getBAllele() != MISSING_ALLELE_NAME) {
                        if (strand == '+') {
                            boolean consistant = (aAllele == snp.getAAllele() && bAllele == snp.getBAllele()) || (bAllele == snp.getAAllele() && aAllele == snp.getBAllele());
                            if (consistant) {
                                snp.setFlankingSequence(flankSeq);
                            } else {
                                misMatchedAlleleNames.add(snp.getRSID());
                            }
                        } else if (strand == '-') {
                            boolean consistant = LocalString.isComplementalChar(aAllele, snp.getAAllele()) && LocalString.isComplementalChar(bAllele, snp.getBAllele());
                            consistant = consistant || (LocalString.isComplementalChar(aAllele, snp.getBAllele()) && LocalString.isComplementalChar(bAllele, snp.getAAllele()));
                            if (consistant) {
                                flankSeq = LocalString.getReverseComplementalSquences(flankSeq);
                                snp.setFlankingSequence(flankSeq);
                                /*
                                correctedStandInf.append(rsID);
                                correctedStandNum++;
                                correctedStandInf.append(" ");
                                 */
                            } else {
                                misMatchedAlleleNames.add(snp.getRSID());
                            }
                        }
                    } else {
                        //if the snp has no B alleles, denoted by MISSING_ALLELE_NAME
                        if (strand == '+') {
                            if (aAllele == snp.getAAllele()) {
                                snp.setBAllele(bAllele);
                                snp.setFlankingSequence(flankSeq);
                            } else if (bAllele == snp.getAAllele()) {
                                snp.setBAllele(aAllele);
                                snp.setFlankingSequence(flankSeq);
                            } else {
                                misMatchedAlleleNames.add(snp.getRSID());
                            }
                        } else if (strand == '-') {
                            if (LocalString.isComplementalChar(aAllele, snp.getAAllele())) {
                                snp.setBAllele(LocalString.getComplementalChar(bAllele));
                                flankSeq = LocalString.getReverseComplementalSquences(flankSeq);
                                snp.setFlankingSequence(flankSeq);
                            } else if (LocalString.isComplementalChar(bAllele, snp.getAAllele())) {
                                snp.setBAllele(LocalString.getComplementalChar(aAllele));
                                flankSeq = LocalString.getReverseComplementalSquences(flankSeq);
                                snp.setFlankingSequence(flankSeq);
                            } else {
                                misMatchedAlleleNames.add(snp.getRSID());
                            }
                        }
                    }
                } else {
                    misMatchedPhysicalPostions.add(snp.getRSID());
                    discordantPosSNP.append(pos);
                    discordantPosSNP.append('\t');
                    discordantPosSNP.append(snp.getRSID());
                    discordantPosSNP.append('\t');
                    discordantPosSNP.append(rsID);
                    discordantPosSNP.append("\n");
                }
            }
        }
        br.close();

        String info = "\nWhen retrieving the flanking sequence from our dbSNP annotation set:";
        GlobalVariables.mainView.setBriefRunningInfor(info);
        GlobalVariables.addInforLog(info);

        if (misMatchedPhysicalPostions.size() > 0) {
            discordantPosSNP.insert(0, misMatchedPhysicalPostions.size() + " physical positions mapped onto two different SNPs:\nPhysical_position\tHapMap SNPs\tdbSNP\n");
            GlobalVariables.mainView.setBriefRunningInfor(discordantPosSNP.toString(), true);
            GlobalVariables.addInforLog(discordantPosSNP.toString());
        }

        if (misMatchedAlleleNames.size() > 0) {
            GlobalVariables.mainView.setBriefRunningInfor(misMatchedAlleleNames.size() + " local SNPs have wrong allele names in the gentoype files! (Details in the log file)");
            GlobalVariables.addInforLog(misMatchedAlleleNames.size() + " local SNPs have wrong allele names in the gentoype files:\n"
                    + misMatchedAlleleNames.toString());
        }


        List<Long> nullFlankS = new ArrayList<Long>();
        for (Map.Entry<Long, AnnotSNP> m1 : hapSnpMap.entrySet()) {
            if (m1.getValue().getFlankingSequence() == null) {
                nullFlankS.add(m1.getKey());
            }
        }
        StringBuilder nullInfor = new StringBuilder();
        if (nullFlankS.size() > 0) {
            GlobalVariables.mainView.setBriefRunningInfor((nullFlankS.size() - misMatchedPhysicalPostions.size() - misMatchedAlleleNames.size()) + " local SNPs have no flanking sequence in our annotation files.  (Details in the log file)");
            nullInfor.append("\n").append(nullFlankS.size() - misMatchedPhysicalPostions.size() - misMatchedAlleleNames.size()).append(" local SNPs have no flanking sequence in our annotation files.\n");
            for (int i = 0; i < nullFlankS.size(); i++) {
                String tmpSNPID = hapSnpMap.remove(nullFlankS.get(i)).getRSID();
                if (!misMatchedAlleleNames.contains(tmpSNPID) && !misMatchedPhysicalPostions.contains(tmpSNPID)) {
                    nullInfor.append(tmpSNPID);
                    nullInfor.append(' ');
                }
            }
            GlobalVariables.addInforLog(nullInfor.toString());
        }
    }

    public void addDbSNPFlankingSequenceInComplemenetaryHapMapSNP(Map<Long, AnnotSNP> hapSnpMap, String chrom) throws Exception {
        File anaFile = new File("resources/hapmap/" + chrom);
        String line;
        String delmilit = "\t";
        StringTokenizer tokenizer = null;
        String rsID;
        String flankSeq;
        long pos;
        MergedRsIDSet mergedRsIDSet = new MergedRsIDSet();
        mergedRsIDSet.readRsMergeMap("resources" + File.separator + "hapmap" + File.separator + "mergedRSID" + File.separator + chrom + ".meg");
        boolean consistent;

        char aAllele, bAllele;
        char strand;
        StringBuilder discordantPosSNP = new StringBuilder();
        BufferedReader br = new BufferedReader(new FileReader(anaFile.getCanonicalPath()), FILE_READER_BUFFER_SIZE);
        //skip the title line
        br.readLine();

        TreeSet<String> misMatchedAlleleNames = new TreeSet<String>();
        TreeSet<String> misMatchedPhysicalPostions = new TreeSet<String>();
        StringBuilder tmpBuffer = new StringBuilder();

        while ((line = br.readLine()) != null) {
            tokenizer = new StringTokenizer(line, delmilit);
            tokenizer.nextToken();
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            rsID = tmpBuffer.toString();

            tokenizer.nextToken();
            pos = Long.parseLong(tokenizer.nextToken().trim());
            AnnotSNP snp = hapSnpMap.get(pos);
            if (snp != null) {
                consistent = false;
                if (snp.getRSID().equals(rsID)) {
                    consistent = true;
                } else {
                    if (mergedRsIDSet.rsMergeMap != null && !mergedRsIDSet.rsMergeMap.isEmpty()) {
                        TreeSet<String> synonymousSNPID = mergedRsIDSet.rsMergeMap.get(rsID);
                        if (synonymousSNPID != null && synonymousSNPID.contains(snp.getRSID())) {
                            //System.out.println(rsID+" <-> "+snp.getRSID());
                            consistent = true;
                        }
                    }
                }

                //Note consistent=true could be very dangerous because sometimes a physical position is wrongly mapped onto two different SNPs:
                consistent = true;
                if (consistent) {
                    //we need to get the second Allele
                    if (snp.getBAllele() == MISSING_ALLELE_NAME) {
                        tokenizer.nextToken();
                        tokenizer.nextToken();
                        tmpBuffer.delete(0, tmpBuffer.length());
                        tmpBuffer.append(tokenizer.nextToken().trim());
                        flankSeq = tmpBuffer.toString();


                        aAllele = tokenizer.nextToken().trim().charAt(0);
                        bAllele = tokenizer.nextToken().trim().charAt(0);
                        tokenizer.nextToken();
                        tokenizer.nextToken();
                        tokenizer.nextToken();
                        tokenizer.nextToken();
                        strand = snp.getStrand();
                        //if the snp has no B alleles, denoted by MISSING_ALLELE_NAME
                        // assign flanking sequences for this rare cases regardless of whether the allele names are complementary or not
                        if (strand == '+') {
                            if (aAllele == snp.getAAllele()) {
                                snp.setBAllele(bAllele);
                                snp.setFlankingSequence(flankSeq);
                            } else if (bAllele == snp.getAAllele()) {
                                snp.setBAllele(aAllele);
                                snp.setFlankingSequence(flankSeq);
                            } else {
                                misMatchedAlleleNames.add(snp.getRSID());
                                //remove SNPs with misMatchedAlleleNames
                                hapSnpMap.remove(pos);
                            }
                        } else if (strand == '-') {
                            if (LocalString.isComplementalChar(aAllele, snp.getAAllele())) {
                                snp.setBAllele(LocalString.getComplementalChar(bAllele));
                                flankSeq = LocalString.getReverseComplementalSquences(flankSeq);
                                snp.setFlankingSequence(flankSeq);
                            } else if (LocalString.isComplementalChar(bAllele, snp.getAAllele())) {
                                snp.setBAllele(LocalString.getComplementalChar(aAllele));
                                flankSeq = LocalString.getReverseComplementalSquences(flankSeq);
                                snp.setFlankingSequence(flankSeq);
                            } else {
                                misMatchedAlleleNames.add(snp.getRSID());
                                //remove SNPs with misMatchedAlleleNames
                                hapSnpMap.remove(pos);
                            }
                        }
                    } else if (LocalString.isComplementalChar(snp.getAAllele(), snp.getBAllele())) {
                        //only assigne flanking sequences for SNPs of complementary allele names so that it can be integrated with SNP having no strand information (Some illumina SNPs have no strand information)
                        tokenizer.nextToken();
                        tokenizer.nextToken();
                        tmpBuffer.delete(0, tmpBuffer.length());
                        tmpBuffer.append(tokenizer.nextToken().trim());
                        flankSeq = tmpBuffer.toString();

                        aAllele = tokenizer.nextToken().trim().charAt(0);
                        bAllele = tokenizer.nextToken().trim().charAt(0);
                        tokenizer.nextToken();
                        tokenizer.nextToken();
                        tokenizer.nextToken();
                        tokenizer.nextToken();
                        strand = snp.getStrand();
                        //note: we assume all strands in our dbSNP anntation set are forward.
                        if (strand == '+') {
                            // -	rs9358505	6	22296399	-	-	AATGTGACAT[C/G/T]AAATAGGCCT	C	T
                            // it seems not good to strictly match the allele names because the SNPs may have free allele names
                            snp.setFlankingSequence(flankSeq);
                            /*
                            //boolean consistant = (aAllele == snp.getAAllele() && bAllele == snp.getBAllele()) || (bAllele == snp.getAAllele() && aAllele == snp.getBAllele());
                            if (consistant) {
                            snp.setFlankingSequence(flankSeq);
                            } else {
                            misMatchedAlleleNames.add(snp.getRSID());
                            //remove SNPs with misMatchedAlleleNames
                            hapSnpMap.remove(pos);
                            }*/
                        } else if (strand == '-') {
                            flankSeq = LocalString.getReverseComplementalSquences(flankSeq);
                            snp.setFlankingSequence(flankSeq);
                            // -	rs9358505	6	22296399	-	-	AATGTGACAT[C/G/T]AAATAGGCCT	C	T
                            // it seems not good to strictly match the allele names because the SNPs may have free allele names
                        /*
                            boolean consistant = LocalString.isComplementalChar(aAllele, snp.getAAllele()) && LocalString.isComplementalChar(bAllele, snp.getBAllele());
                            consistant = consistant || (LocalString.isComplementalChar(aAllele, snp.getBAllele()) && LocalString.isComplementalChar(bAllele, snp.getAAllele()));
                            if (consistant) {
                            flankSeq = LocalString.getReverseComplementalSquences(flankSeq);
                            snp.setFlankingSequence(flankSeq);
                            } else {
                            misMatchedAlleleNames.add(snp.getRSID());
                            //remove SNPs with misMatchedAlleleNames
                            hapSnpMap.remove(pos);
                            }
                             */
                        }
                    }

                } else {
                    misMatchedPhysicalPostions.add(snp.getRSID());
                    //remove SNPs with misMathcedPhysicalPostions
                    hapSnpMap.remove(pos);
                    discordantPosSNP.append(pos);
                    discordantPosSNP.append('\t');
                    discordantPosSNP.append(snp.getRSID());
                    discordantPosSNP.append('\t');
                    discordantPosSNP.append(rsID);
                    discordantPosSNP.append("\n");
                }
            }
        }
        br.close();

        String info = "\nRetrieving the flanking sequence from our dbSNP annotation set...";
        GlobalVariables.mainView.setBriefRunningInfor(info);
        GlobalVariables.addInforLog(info);

        if (misMatchedPhysicalPostions.size() > 0) {
            discordantPosSNP.insert(0, misMatchedPhysicalPostions.size() + " physical positions mapped onto two different SNPs:\nPhysical_position\tHapMap SNPs\tdbSNP\n");
            GlobalVariables.mainView.setBriefRunningInfor(discordantPosSNP.toString(), true);
            GlobalVariables.addInforLog(discordantPosSNP.toString());
        }

        if (misMatchedAlleleNames.size() > 0) {
            GlobalVariables.mainView.setBriefRunningInfor(misMatchedAlleleNames.size() + " local SNPs have wrong allele names in the gentoype files! (Details in the log file)");
            GlobalVariables.addInforLog(misMatchedAlleleNames.size() + " HapMap SNPs have wrong allele names in the gentoype files:\n"
                    + misMatchedAlleleNames.toString());
        }

    }

    public void removeDbSNPNullFlankingSequences(List<AnnotSNP> AnnotSNPList) throws Exception {
        int size = AnnotSNPList.size();
        List<AnnotSNP> annotSNPList1 = new ArrayList<AnnotSNP>();
        StringBuilder excludedSNP = new StringBuilder();
        int count = 0;
        for (int i = 0; i < size; i++) {
            AnnotSNP snp = AnnotSNPList.get(i);
            if (snp.getFlankingSequence() != null) {
                annotSNPList1.add(snp);
            } else {
                excludedSNP.append(snp.getRSID());
                excludedSNP.append(" ");
                count++;
            }
        }
        /*
        if (count > 0) {
        GlobalVariables.mainView.setBriefRunningInfor(count + " local SNPs have no flanking sequence in our annotation files.  (Details in the log file)");
        excludedSNP.insert(0, count + " local SNPs have no flanking sequence in our annotation files:\n");
        GlobalVariables.addInforLog(excludedSNP.toString());
        }
         */
        AnnotSNPList.clear();
        AnnotSNPList.addAll(annotSNPList1);
        annotSNPList1 = null;
    }

    // a new version of addDbSNPFlankingSequences when the items of dbSNP are sorted according to physical positions
    public void addDbSNPFlankingSequences(List<AnnotSNP> annotSNPList, String chrom, String mapFileName) throws Exception {
        File resourceFile = new File("resources" + File.separator + "hapmap" + File.separator + chrom);
        if (!resourceFile.exists()) {
            return;
        }
        String line;
        String delmilit = "\t";
        StringTokenizer tokenizer = null;
        String rsID;
        String flankSeq;
        long posFile;
        MergedRsIDSet mergedRsIDSet = new MergedRsIDSet();
        mergedRsIDSet.readRsMergeMap("resources" + File.separator + "hapmap" + File.separator + "mergedRSID" + File.separator + chrom + ".meg");

        char aAllele, bAllele;
        int listSNPSize = annotSNPList.size();
        if (listSNPSize == 0) {
            return;
        }
        StringBuilder discordantPosSNP = new StringBuilder();
        BufferedReader br = new BufferedReader(new FileReader(resourceFile.getCanonicalPath()), FILE_READER_BUFFER_SIZE);
        //skip the title line
        br.readLine();

        List<String> missMatchedSNP = new ArrayList<String>();
        TreeSet<String> noFlankingSequence = new TreeSet<String>();
        int listIndex = 0;
        int discordantPosSNPNum = 0;

        AnnotSNP snp = annotSNPList.get(listIndex);
        long posList = snp.getPhysicalPosition();

        boolean isSameSNP;
        boolean endList = false;
        StringBuilder tmpBuffer = new StringBuilder();
        while ((line = br.readLine()) != null) {
            tokenizer = new StringTokenizer(line, delmilit);

            tokenizer.nextToken();
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            rsID = tmpBuffer.toString();

            tokenizer.nextToken();
            posFile = Long.parseLong(tokenizer.nextToken().trim());

            while (posList < posFile) {
                //noFlankingSequence.add(snp.getRSID());
                listIndex++;
                if (listIndex >= listSNPSize) {
                    endList = true;
                    break;
                }
                snp = annotSNPList.get(listIndex);
                posList = snp.getPhysicalPosition();
            }
            if (endList) {
                break;
            }

            if (posList > posFile) {
                //snpNumMissingFlank++;
                continue;
            }

            isSameSNP = false;
            // otherwise posList == posFile
            if (!snp.getRSID().startsWith("rs")) {
                snp.setRSID(rsID);
                isSameSNP = true;
            } else if (!snp.getRSID().equals(rsID)) {
                if (mergedRsIDSet.rsMergeMap != null && !mergedRsIDSet.rsMergeMap.isEmpty()) {
                    TreeSet<String> synonymousSNPID = mergedRsIDSet.rsMergeMap.get(rsID);
                    if (synonymousSNPID != null && synonymousSNPID.contains(snp.getRSID())) {
                        isSameSNP = true;
                    } else {
                        discordantPosSNP.append(posFile);
                        discordantPosSNP.append('\t');
                        discordantPosSNP.append(snp.getRSID());
                        discordantPosSNP.append('\t');
                        discordantPosSNP.append(rsID);
                        discordantPosSNP.append("\n");
                        discordantPosSNPNum++;
                    }
                }
            } else {
                isSameSNP = true;
            }

            if (isSameSNP) {
                tokenizer.nextToken();
                tokenizer.nextToken();
                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                flankSeq = tmpBuffer.toString();

                aAllele = tokenizer.nextToken().trim().charAt(0);
                bAllele = tokenizer.nextToken().trim().charAt(0);

                //assume the  annotation are possitive
                if (snp.getStrand() == '+') {
                    //use the order
                    if (snp.getAAllele() == MISSING_ALLELE_NAME && snp.getBAllele() == MISSING_ALLELE_NAME) {
                        snp.setAAllele(aAllele);
                        snp.setBAllele(bAllele);
                    } else {
                        if ((snp.getAAllele() == aAllele && snp.getBAllele() == bAllele) || (snp.getAAllele() == bAllele && snp.getBAllele() == aAllele)) {
                            //it is correct, do nothing
                        } else {
                            missMatchedSNP.add(snp.getRSID());
                        }
                    }

                    //default strand is + in prepared dbSNP
                    snp.setFlankingSequence(flankSeq);
                } else if (snp.getStrand() == '-') {
                    if (snp.getAAllele() == MISSING_ALLELE_NAME && snp.getBAllele() == MISSING_ALLELE_NAME) {
                        snp.setAAllele(LocalString.getComplementalChar(aAllele));
                        snp.setBAllele(LocalString.getComplementalChar(bAllele));
                    } else {
                        if ((snp.getAAllele() == LocalString.getComplementalChar(aAllele) && snp.getBAllele() == LocalString.getComplementalChar(bAllele))
                                || (snp.getAAllele() == LocalString.getComplementalChar(bAllele) && snp.getBAllele() == LocalString.getComplementalChar(aAllele))) {
                            //it is correct, do nothing
                        } else {
                            missMatchedSNP.add(snp.getRSID());
                        }
                    }
                    snp.setFlankingSequence(LocalString.getReverseComplementalSquences(flankSeq));
                }
            }

            listIndex++;
            if (listIndex >= listSNPSize) {
                break;
            }
            snp = annotSNPList.get(listIndex);
            posList = snp.getPhysicalPosition();
        }
        br.close();
        if (missMatchedSNP.size() > 0) {
            StringBuilder missedInfo = new StringBuilder("\nWhen retrieving the flanking sequence from our dbSNP annotation set:");
            missedInfo.append(missMatchedSNP.size());
            missedInfo.append(" have inconsistent allele names although they have the strand information!\n");
            int misSize = missMatchedSNP.size() - 1;
            for (int k = 0; k < misSize; k++) {
                missedInfo.append(missMatchedSNP.get(k));
                missedInfo.append(", ");
            }
            missedInfo.append(missMatchedSNP.get(misSize));

            GlobalVariables.mainView.setBriefRunningInfor(missedInfo.toString());
            GlobalVariables.addInforLog(missedInfo.toString());
        }

        if (discordantPosSNPNum > 0) {
            String info = "\nWhen retrieving the flanking sequence from our dbSNP annotation set:";
            GlobalVariables.mainView.setBriefRunningInfor(info);
            GlobalVariables.addInforLog(info);
            discordantPosSNP.insert(0, discordantPosSNPNum + " physical positions mapped onto two different SNPs:\nPhysical_position\t" + mapFileName + "\tdbSNP\n");
            GlobalVariables.mainView.setBriefRunningInfor(discordantPosSNP.toString(), true);
            GlobalVariables.addInforLog(discordantPosSNP.toString());
        }

        /*
        if (noFlankingSequence.size() > 0) {
        GlobalVariables.mainView.setBriefRunningInfor(noFlankingSequence.size() + " local SNPs have no flanking sequence in our annotation files.  (Details in the log file)");
        GlobalVariables.addInforLog("\n" + noFlankingSequence.size() + " local SNPs have no flanking sequence in our annotation files.\n");
        GlobalVariables.addInforLog(noFlankingSequence.toString());
        }*/
    }
}
