/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.controller;

import cern.colt.bitvector.BitVector;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import java.util.TreeSet;
import org.cobi.igg3.entity.ChromSNPGtySet;
import org.cobi.igg3.entity.CommonGenetFileSet;
import org.cobi.igg3.entity.Individual;
import org.cobi.igg3.entity.Integration;
import org.cobi.igg3.entity.AnnotSNP;
import org.cobi.igg3.entity.Project;

import org.cobi.igg3.entity.SNPChromosomeComparator;
import org.cobi.igg3.entity.SNPFileIndexComparator;
import org.cobi.igg3.entity.SNPPosiComparator;
import org.cobi.igg3.entity.StatusGtySet;
import org.cobi.igg3.gui.IGG3View;
import org.cobi.igg3.GlobalVariables;

/**
 *
 * @author MX Li
 */
public class CommonGenetDatasetIntegrator extends BasicIntegrator {

    public boolean integrateCommonGenetDataset(Integration intgration) throws Exception {
        boolean succeed = false;
        Project prg = GlobalVariables.currentProject;
        List<CommonGenetFileSet> setList = prg.getCommonGenetFileSetList();


        String msg = "Integrating common genetic datasets...";
        GlobalVariables.mainView.setBriefRunningInfor("\n" + msg, true);
        GlobalVariables.addInforLog(msg);

        List<Individual> indList = new ArrayList<Individual>();
        int chromIncLen = 5;
        int startChrom, endChrom;
        boolean[] hasAddPedigree = new boolean[1];
        for (int j = 0; j < setList.size(); j++) {
            startChrom = 0;
            endChrom = startChrom + chromIncLen;
            hasAddPedigree[0] = false;
            do {
                if (endChrom > CHROM_NAME.length) {
                    endChrom = CHROM_NAME.length;
                }

                integrateCommonPedOnChromosomes(intgration, setList.get(j), indList, startChrom, endChrom, hasAddPedigree);

                startChrom = endChrom;
                endChrom = startChrom + chromIncLen;
            } while (startChrom < CHROM_NAME.length);
        }

        //save pedigree information
        File chekPath = new File(intgration.getResultPath());
        if (!chekPath.exists()) {
            chekPath.mkdir();
        }

        FileOutputStream objFOut = new FileOutputStream(chekPath.getCanonicalPath() + File.separator + "ped");
        BufferedOutputStream objOBfs = new BufferedOutputStream(objFOut);
        ObjectOutputStream localObjOut = new ObjectOutputStream(objOBfs);

        localObjOut.writeObject(indList);

        localObjOut.flush();
        localObjOut.close();
        objOBfs.flush();
        objOBfs.close();
        objFOut.close();
        return succeed;
    }

    protected void integrateCommonPedOnChromosomes(Integration intgration, CommonGenetFileSet fileSet, List<Individual> indList, int startChom, int endChrom, boolean[] hasAddPedigree) throws Exception {
        String msg = "";
        ChromSNPGtySet chromSet = new ChromSNPGtySet();
        Map<String, StatusGtySet> indivGtyMap = new HashMap<String, StatusGtySet>();

        Map<Integer, Integer> snpNumPerChrom = new HashMap<Integer, Integer>();
        List<AnnotSNP> snpList = new ArrayList<AnnotSNP>();


        for (int i = startChom; i < endChrom; i++) {
            String chrom = CHROM_NAME[i];
            if (intgration.getPhysicalRegionMap().get(chrom) == null && intgration.getGenetRegionMap().get(chrom) == null) {
                continue;
            }
            List<AnnotSNP> tmpSnpList = new ArrayList<AnnotSNP>();
            if (fileSet.getPlinkBinaryFileName() == null) {
                readSNPsinMapFileByPysic(fileSet, tmpSnpList, CHROM_NAME[i],
                        intgration.getPhysicalRegionMap().get(CHROM_NAME[i]));
            } else {
                //it must be plink binary
                readSNPsinPlinkBinaryMapFileByPysic(fileSet, tmpSnpList, CHROM_NAME[i],
                        intgration.getPhysicalRegionMap().get(CHROM_NAME[i]));
            }

            if (tmpSnpList.isEmpty()) {
                continue;
            }
            Collections.sort(tmpSnpList, new SNPPosiComparator());
            addDbSNPFlankingSequences(tmpSnpList, CHROM_NAME[i], fileSet.getMapFileName());
            //no need to remove SNPs without flanking sequencies because I can match them by allele names which are distinct
            //removeDbSNPNullFlankingSequences(tmpSnpList);
            snpNumPerChrom.put(STANDARD_CHROM_NUM[i], tmpSnpList.size());
            snpList.addAll(tmpSnpList);
            tmpSnpList = null;
        }
        if (snpList.isEmpty()) {
            return;
        }

        Collections.sort(snpList, new SNPFileIndexComparator());
        List<Individual> indListTmp = new ArrayList<Individual>();
        if (fileSet.getPlinkBinaryFileName() == null) {
            readGenotypeinPedigreeFile(fileSet, indListTmp, snpList, indivGtyMap);
        } else {
            //it must be plink binary
            readPlinkBinaryGenotypeinPedigreeFile(fileSet, indListTmp, snpList, indivGtyMap);
        }
        if (!hasAddPedigree[0]) {
            indList.addAll(indListTmp);
            hasAddPedigree[0] = true;
        }
        Collections.sort(snpList, new SNPChromosomeComparator());
        List<AnnotSNP> tmpSnpList = new ArrayList<AnnotSNP>();
        //devide SNPs and genotypes into file according to SNP.
        int chNum;
        int snpIndex = snpList.size() - 1;
        int newSNPOrder = 0;
        chNum = snpList.get(snpIndex).getChromosomeNum();
        AnnotSNP snp;
        Map<String, StatusGtySet> tmpIndivGtyMap = new HashMap<String, StatusGtySet>();

        msg = "Merging genotypes from " + fileSet.getPedigreeFileName();
        GlobalVariables.mainView.setBriefRunningInfor(msg, true);
        GlobalVariables.addInforLog(msg);

        do {
            tmpIndivGtyMap.clear();
            tmpSnpList.clear();


            //speparate the SNPs chromosome by chromsome
            List<StatusGtySet> tmpListToSpeedGT = new ArrayList<StatusGtySet>();
            List<StatusGtySet> tmpListToSpeedGTOrg = new ArrayList<StatusGtySet>();
            List<String> tmpListToSpeedIDOrg = new ArrayList<String>();

            for (Map.Entry<String, StatusGtySet> m1 : indivGtyMap.entrySet()) {
                tmpListToSpeedIDOrg.add(m1.getKey());
                tmpListToSpeedGTOrg.add(m1.getValue());
                StatusGtySet gtySet = new StatusGtySet();
                gtySet.existence = new BitVector(snpNumPerChrom.get(chNum));
                gtySet.paternalChrom = new BitVector(snpNumPerChrom.get(chNum));
                gtySet.maternalChrom = new BitVector(snpNumPerChrom.get(chNum));
                tmpListToSpeedGT.add(gtySet);
            }

            newSNPOrder = 0;
            //Later I do not know why I reversed the order the genotypes; So I will change it  because it is time-consuming
            int indivNum = tmpListToSpeedIDOrg.size();
            while (snpIndex >= 0 && chNum == snpList.get(snpIndex).getChromosomeNum()) {
                snp = snpList.remove(snpIndex);

                for (int i = 0; i < indivNum; i++) {
                    StatusGtySet tmpGtySet = tmpListToSpeedGT.get(i);
                    StatusGtySet origGtySet = tmpListToSpeedGTOrg.get(i);

                    tmpGtySet.existence.putQuick(newSNPOrder, origGtySet.existence.getQuick(snp.order));
                    tmpGtySet.paternalChrom.putQuick(newSNPOrder, origGtySet.paternalChrom.getQuick(snp.order));
                    tmpGtySet.maternalChrom.putQuick(newSNPOrder, origGtySet.maternalChrom.getQuick(snp.order));
                }
                snp.order = newSNPOrder;
                tmpSnpList.add(snp);
                newSNPOrder++;
                snpIndex--;
            }
            for (int i = 0; i < indivNum; i++) {
                tmpIndivGtyMap.put(tmpListToSpeedIDOrg.get(i), tmpListToSpeedGT.get(i));
            }
            Collections.sort(tmpSnpList, new SNPPosiComparator());
            msg = "Genotypes on chromsome  " + CHROM_NAME[chNum - 1] + " have been selected to merge.";
            GlobalVariables.mainView.setBriefRunningInfor(msg, true);
            GlobalVariables.addInforLog(msg);

            //tempararily
            chromSet.setChromName(CHROM_NAME[chNum - 1]);
            chromSet.readAllObjects(intgration.getResultPath());
            List<AnnotSNP> availableSNPList = chromSet.getSnpInfoList();
            Map<String, StatusGtySet> availableGtyMap = chromSet.getIndiviGty();


            String infor = mergeSNPGtybyListNoGtyExtend(availableSNPList, tmpSnpList, availableGtyMap, tmpIndivGtyMap);
            intgration.updateChromResult(CHROM_NAME[chNum - 1], availableSNPList.size());
            chromSet.saveAllObjectsAndClear(intgration.getResultPath(), chromSet.getTotalGtyBlockNum());
            msg = "Genotypes on chromsome  " + CHROM_NAME[chNum - 1] + " have been encoded and merged.";
            GlobalVariables.mainView.setBriefRunningInfor(msg, true);
            GlobalVariables.addInforLog(msg);
            if (snpIndex >= 0) {
                chNum = snpList.get(snpIndex).getChromosomeNum();
            } else {
                break;
            }
        } while (true);
    }

    protected boolean readGenotypeinPedigreeFile(CommonGenetFileSet fileSet, List<Individual> indList, List<AnnotSNP> snpList,
            Map<String, StatusGtySet> indivGtyMap) throws Exception {
        String pedFileName = fileSet.getPedigreeFileName();
        BufferedReader br = new BufferedReader(new FileReader(pedFileName));
        TreeSet<String> unexpectedSNPSet = new TreeSet<String>();
        String line;
        String delimiter = "\t\" \",/";
        int totalSNPSize = snpList.size();
        int gtyStaringCol = -1;

        String indivLabel;

        String sigGty1;
        String sigGty2;
        int unexpectedGtyNum = 0;

        String info = "Encoding Pedigree File " + pedFileName + " ...";
        GlobalVariables.mainView.setBriefRunningInfor(info);
        GlobalVariables.addInforLog(info);

        List<String> traitNames = GlobalVariables.currentPedFile.getTraitNames();
        //guess genotype starting column
        gtyStaringCol = 5 + traitNames.size();
        int snpOrder = 0;
        char aAllele, bAllele;
        boolean[] needAlleleNames = new boolean[totalSNPSize];
        for (int i = 0; i < totalSNPSize; i++) {
            AnnotSNP snp = snpList.get(i);
            if (snp.getAAllele() == MISSING_ALLELE_NAME || snp.getBAllele() == MISSING_ALLELE_NAME) {
                needAlleleNames[i] = true;
            } else {
                needAlleleNames[i] = false;
            }
        }


        StringBuilder tmpBuffer = new StringBuilder();
        //long t3 = System.currentTimeMillis();
        while ((line = br.readLine()) != null) {
            line = line.toUpperCase();
            StringTokenizer tokenizer = new StringTokenizer(line, delimiter);

            Individual indiv = new Individual();
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            indiv.setFamilyID(tmpBuffer.toString());
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            indiv.setIndividualID(tmpBuffer.toString());
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            indiv.setDadID(tmpBuffer.toString());
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            indiv.setMomID(tmpBuffer.toString());
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            indiv.setGender(Integer.valueOf(tmpBuffer.toString()));
            indiv.setLabelInChip(indiv.getFamilyID() + "@*@" + indiv.getIndividualID());
            for (int i = 5; i < gtyStaringCol; i++) {
                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                indiv.addTrait(tmpBuffer.toString());
            }
            //System.out.println(indiv.getLabelInChip());
            indList.add(indiv);

            indivLabel = indiv.getLabelInChip();
            StatusGtySet sGty = indivGtyMap.get(indivLabel);
            if (sGty == null) {
                sGty = new StatusGtySet();
                sGty.paternalChrom = new BitVector(totalSNPSize);
                sGty.maternalChrom = new BitVector(totalSNPSize);
                sGty.existence = new BitVector(totalSNPSize);
                indivGtyMap.put(indivLabel, sGty);
            } else {
                String unexpectInfo = "Duplicated Individuals  in " + pedFileName + " for PedID " + indiv.getFamilyID() + " IndivID " + indiv.getIndividualID();
                GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                GlobalVariables.addInforLog(unexpectInfo);
            }
            int j = -1;
            int halfJ = 0;
            sigGty1 = null;
            snpOrder = 0;
//System.out.println(indiv.getLabelInChip());
            for (int i = 0; i < totalSNPSize; i++) {
                AnnotSNP snp = snpList.get(i);
                do {
                    j++;
                    halfJ = j / 2;
                    if (snp.probeSetID == halfJ) {
                        tmpBuffer.delete(0, tmpBuffer.length());
                        tmpBuffer.append(tokenizer.nextToken().trim());
                        sigGty1 = tmpBuffer.toString();
                    } else {
                        tokenizer.nextToken();
                    }
                } while (snp.probeSetID > halfJ);

                snp.order = snpOrder;
                aAllele = snp.getAAllele();
                bAllele = snp.getBAllele();
                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                sigGty2 = tmpBuffer.toString();

                if (needAlleleNames[i]) {
                    //decide the allele names
                    if (aAllele != MISSING_ALLELE_NAME && bAllele == MISSING_ALLELE_NAME) {
                        //decide the allele names
                        if (sigGty1.charAt(0) != aAllele && sigGty1.charAt(0) != DEFAULT_MISSING_GTY_NAME) {
                            bAllele = sigGty1.charAt(0);
                            snp.setBAllele(bAllele);
                            needAlleleNames[i] = false;
                        } else if (sigGty2.charAt(0) != aAllele && sigGty2.charAt(0) != DEFAULT_MISSING_GTY_NAME) {
                            bAllele = sigGty2.charAt(0);
                            snp.setBAllele(bAllele);
                            needAlleleNames[i] = false;
                        }
                    } else if (aAllele == MISSING_ALLELE_NAME && bAllele == MISSING_ALLELE_NAME) {
                        //decide the allele names
                        if (sigGty1.charAt(0) != DEFAULT_MISSING_GTY_NAME) {
                            aAllele = sigGty1.charAt(0);
                            snp.setAAllele(aAllele);
                            if (sigGty2.charAt(0) != DEFAULT_MISSING_GTY_NAME && sigGty2.charAt(0) != aAllele) {
                                bAllele = sigGty2.charAt(0);
                                snp.setBAllele(bAllele);
                                needAlleleNames[i] = false;
                            }
                        } else if (sigGty2.charAt(0) != DEFAULT_MISSING_GTY_NAME) {
                            aAllele = sigGty2.charAt(0);
                            snp.setAAllele(aAllele);
                        }
                    } else if (aAllele == MISSING_ALLELE_NAME && bAllele != MISSING_ALLELE_NAME) {
                        //decide the allele names
                        if (sigGty1.charAt(0) != bAllele && sigGty1.charAt(0) != DEFAULT_MISSING_GTY_NAME) {
                            aAllele = sigGty1.charAt(0);
                            snp.setAAllele(aAllele);
                            needAlleleNames[i] = false;
                        } else if (sigGty2.charAt(0) != bAllele && sigGty2.charAt(0) != DEFAULT_MISSING_GTY_NAME) {
                            aAllele = sigGty2.charAt(0);
                            snp.setAAllele(aAllele);
                            needAlleleNames[i] = false;
                        }
                    }
                }

                // anyhow the  aAllele and bAllele must be known
                //re-code: A->0; B->1;
                if (sigGty1.charAt(0) == bAllele) {
                    sGty.existence.putQuick(snpOrder, true);
                    sGty.paternalChrom.putQuick(snpOrder, true);
                } else if (sigGty1.charAt(0) == aAllele) {
                    sGty.existence.putQuick(snpOrder, true);
                } else if (sigGty1.charAt(0) != DEFAULT_MISSING_GTY_NAME) {
                    //throw new Exception("Unexpected genotype in " + filePath + " for " + probID);
                    String unexpectInfo = "Unexpected genotype " + sigGty1 + " in " + pedFileName + " for " + snp.getRSID() + " of PedID " + indiv.getFamilyID() + " IndivID " + indiv.getIndividualID();
                    unexpectedSNPSet.add(snp.getRSID());
                    //GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                    GlobalVariables.addInforLog(unexpectInfo);
                    unexpectedGtyNum++;
                }

                j++;

                if (sigGty2.charAt(0) == bAllele) {
                    sGty.maternalChrom.putQuick(snpOrder, true);
                } else if (sigGty2.charAt(0) != aAllele && sigGty2.charAt(0) != DEFAULT_MISSING_GTY_NAME) {
                    //throw new Exception("Unexpected genotype in " + filePath + " for " + probID);
                    String unexpectInfo = "Unexpected genotype " + sigGty2 + " in " + pedFileName + " for " + snp.getRSID() + " of PedID " + indiv.getFamilyID() + " IndivID " + indiv.getIndividualID();
                    unexpectedSNPSet.add(snp.getRSID());
                    //GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                    GlobalVariables.addInforLog(unexpectInfo);
                    unexpectedGtyNum++;
                }
                snpOrder++;
            }
            tokenizer = null;
            line = null;
        }
        br.close();
        if (unexpectedGtyNum > 0) {
            GlobalVariables.mainView.setBriefRunningInfor((unexpectedGtyNum / 2) + " Unexpected genotypes in " + pedFileName + " for " + unexpectedSNPSet.size() + " SNPs (detailed in the log).");
            GlobalVariables.addInforLog(unexpectedSNPSet.size() + " SNPs have unexpected genotypes in " + pedFileName + ":\n" + unexpectedSNPSet.toString());

            GlobalVariables.mainView.setBriefRunningInfor(unexpectedSNPSet.toString());
            unexpectedSNPSet.clear();
        }
        return false;
    }

    protected boolean readPlinkBinaryGenotypeinPedigreeFile(CommonGenetFileSet fileSet, List<Individual> indList, List<AnnotSNP> snpList,
            Map<String, StatusGtySet> indivGtyMap) throws Exception {
        String pedFileName = fileSet.getPedigreeFileName();
        BufferedReader br = new BufferedReader(new FileReader(pedFileName));
        TreeSet<String> unexpectedSNPSet = new TreeSet<String>();
        String line;
        String delimiter = "\t\" \",/";
        int totalSNPSize = snpList.size();
        int gtyStaringCol = -1;

        String indivLabel;

        int unexpectedGtyNum = 0;
        List<String> indiviIDInPed = new ArrayList<String>();


        List<String> traitNames = GlobalVariables.currentPedFile.getTraitNames();
        //guess genotype starting column
        gtyStaringCol = 5 + traitNames.size();
        int snpOrder = 0;

        StringBuilder tmpBuffer = new StringBuilder();
        //long t3 = System.currentTimeMillis();
        while ((line = br.readLine()) != null) {
            line = line.toUpperCase();
            StringTokenizer tokenizer = new StringTokenizer(line, delimiter);

            Individual indiv = new Individual();
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            indiv.setFamilyID(tmpBuffer.toString());
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            indiv.setIndividualID(tmpBuffer.toString());
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            indiv.setDadID(tmpBuffer.toString());
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            indiv.setMomID(tmpBuffer.toString());
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            indiv.setGender(Integer.valueOf(tmpBuffer.toString()));
            indiv.setLabelInChip(indiv.getFamilyID() + "@*@" + indiv.getIndividualID());
            for (int i = 5; i < gtyStaringCol; i++) {
                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                indiv.addTrait(tmpBuffer.toString());
            }
            //System.out.println(indiv.getLabelInChip());
            indList.add(indiv);

            indivLabel = indiv.getLabelInChip();
            StatusGtySet sGty = indivGtyMap.get(indivLabel);
            if (sGty == null) {
                sGty = new StatusGtySet();
                //////////////////////////////
                // Allocate space for SNPs
                sGty.paternalChrom = new BitVector(totalSNPSize);
                sGty.maternalChrom = new BitVector(totalSNPSize);
                sGty.existence = new BitVector(totalSNPSize);
                indivGtyMap.put(indivLabel, sGty);
                indiviIDInPed.add(indivLabel);
            } else {
                String unexpectInfo = "Duplicated Individuals  in " + pedFileName + " for PedID " + indiv.getFamilyID() + " IndivID " + indiv.getIndividualID();
                GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                GlobalVariables.addInforLog(unexpectInfo);
            }
            tokenizer = null;
            line = null;
        }
        br.close();

        //start to read binary genotypes

        DataInputStream in = new DataInputStream(new BufferedInputStream(new FileInputStream(fileSet.getPlinkBinaryFileName())));
        boolean bfile_SNP_major = false;



        //openPlinkPedFormat
        in = new DataInputStream(new BufferedInputStream(new FileInputStream(fileSet.getPlinkBinaryFileName())));
        StringBuilder logInfor = new StringBuilder();
        byte bt = in.readByte();

        boolean v1_bfile = true;
        boolean[] b = new boolean[8];
        for (int i = 7; i >= 0; i--) {
            if (((1 << i) & bt) != 0) {
                b[i] = true;
            } else {
                b[i] = false;
            }
        }

        // printBitSet(b);
        // If v1.00 file format
        // Magic numbers for .bed file: 00110110 11011000 = v1.00 bed file
        if ((b[2] && b[3] && b[5] && b[6]) && !(b[0] || b[1] || b[4] || b[7])) {
            // Next number
            bt = in.readByte();
            for (int i = 7; i >= 0; i--) {
                if (((1 << i) & bt) != 0) {
                    b[i] = true;
                } else {
                    b[i] = false;
                }
            }
            // printBitSet(b);
            if ((b[0] && b[1] && b[3] && b[4]) && !(b[2] || b[5] || b[6] || b[7])) {
                // Read SNP/Ind major coding
                bt = in.readByte();
                for (int i = 7; i >= 0; i--) {
                    if (((1 << i) & bt) != 0) {
                        b[i] = true;
                    } else {
                        b[i] = false;
                    }
                }
                if (b[0]) {
                    bfile_SNP_major = true;
                } else {
                    bfile_SNP_major = false;
                }

                if (bfile_SNP_major) {
                    logInfor.append("Detected that binary PED file is v1.00 SNP-major mode\n");
                } else {
                    logInfor.append("Detected that binary PED file is v1.00 individual-major mode\n");
                }

            } else {
                v1_bfile = false;
            }

        } else {
            v1_bfile = false;
        }


        // Reset file if < v1
        if (!bfile_SNP_major) {
            logInfor.append("Warning, old BED file <v1.00 : will try to recover...\n");
            logInfor.append("  bs.get(t you should --make-bs.get(d from PED )\n");
            in.close();
            in = new DataInputStream(new BufferedInputStream(new FileInputStream(fileSet.getPlinkBinaryFileName())));
            bt = in.readByte();
            for (int i = 7; i >= 0; i--) {
                if (((1 << i) & bt) != 0) {
                    b[i] = true;
                } else {
                    b[i] = false;
                }
            }
        }

        // If 0.99 file format
        if ((!v1_bfile) && (b[1] || b[2] || b[3] || b[4] || b[5] || b[6] || b[7])) {
            logInfor.append("\n *** Possibs.get(e probs.get(em: guessing that BED is < v0.99      *** \n");
            logInfor.append(" *** High chance of data corruption, spurious results    *** \n");
            logInfor.append(" *** Unles you are _sure_ this really is an old BED file *** \n");
            logInfor.append(" *** you should recreate PED -> BED                      *** \n\n");

            bfile_SNP_major = false;
            in.close();
            in = new DataInputStream(new BufferedInputStream(new FileInputStream(fileSet.getPlinkBinaryFileName())));
        } else if (!v1_bfile) {
            if (b[0]) {
                bfile_SNP_major = true;
            } else {
                bfile_SNP_major = false;
            }

            logInfor.append("Binary PED file is v0.99\n");
            if (bfile_SNP_major) {
                logInfor.append("Detected that binary PED file is in SNP-major mode\n");
            } else {
                logInfor.append("Detected that binary PED file is in individual-major mode\n");
            }
        }
        //System.out.println(logInfor);
        GlobalVariables.mainView.setBriefRunningInfor(logInfor.toString());
        GlobalVariables.addInforLog(logInfor.toString());


//http://pngu.mgh.harvard.edu/~purcell/plink/binary.shtml
        String info = ("Reading genotype bit-file from [ " + fileSet.getPlinkBinaryFileName() + " ] \n");
        GlobalVariables.mainView.setBriefRunningInfor(info);
        GlobalVariables.addInforLog(info);

        int indiviNum = indiviIDInPed.size();

        ///////////////////////////
        // SNP-major mode
        if (bfile_SNP_major) {
            snpOrder = 0;
            int byteNumPerSNP = indiviNum / 4;
            if (indiviNum % 4 > 0) {
                byteNumPerSNP++;
            }
            int currentBytePoint = 0;
            int shouldBytePoint = 0;
            for (int i = 0; i < totalSNPSize; i++) {
                AnnotSNP snp = snpList.get(i);
                shouldBytePoint = snp.probeSetID * byteNumPerSNP;
                if (currentBytePoint < shouldBytePoint) {
                    in.skipBytes(shouldBytePoint - currentBytePoint);
                    currentBytePoint = shouldBytePoint;
                }
                snp.order = snpOrder;
                int k = 0;
                while (k < indiviNum) {
                    bt = in.readByte();
                    currentBytePoint++;
                    for (int s = 7; s >= 0; s--) {
                        if (((1 << s) & bt) != 0) {
                            b[s] = true;
                        } else {
                            b[s] = false;
                        }
                    }

                    // printBitSet(b);
                    for (int s = 0; s < 8; s += 2) {
                        StatusGtySet sGty = indivGtyMap.get(indiviIDInPed.get(k));
                        if (!b[s] && b[s + 1]) {
                            sGty.existence.putQuick(i, true);
                            sGty.paternalChrom.putQuick(i, true);
                        } else if (!b[s] && !b[s + 1]) {
                            sGty.existence.putQuick(i, true);
                        } else if (b[s] && b[s + 1]) {
                            sGty.existence.putQuick(i, true);
                            sGty.maternalChrom.putQuick(i, true);
                            sGty.paternalChrom.putQuick(i, true);
                        }
                        k++;
                        if (k == indiviNum) {
                            break;
                        }
                    }
                }
                snpOrder++;
            }
        } else {
            int j = -1;
            int currentSNPNum = 0;

            snpOrder = 0;
            int s = 0;
            //This code has not been tested yet because the latest version generate SNP-major model
            ///////////////////////////
            // individual-major mode
            for (int k = 0; k < indiviNum; k++) {
                currentSNPNum = 0;
                StatusGtySet sGty = indivGtyMap.get(indiviIDInPed.get(k));
                for (int i = 0; i < totalSNPSize; i++) {
                    AnnotSNP snp = snpList.get(i);
                    while (currentSNPNum < snp.probeSetID) {
                        bt = in.readByte();
                        for (int t = 7; t >= 0; t--) {
                            if (((1 << t) & bt) != 0) {
                                b[t] = true;
                            } else {
                                b[t] = false;
                            }
                        }
                        currentSNPNum += 4;
                    }
                    s = (snp.probeSetID % 4) * 2;
                    if (!b[s] && b[s + 1]) {
                        sGty.existence.putQuick(i, true);
                        sGty.paternalChrom.putQuick(i, true);
                    } else if (!b[s] && !b[s + 1]) {
                        sGty.existence.putQuick(i, true);
                    } else if (b[s] && b[s + 1]) {
                        sGty.existence.putQuick(i, true);
                        sGty.maternalChrom.putQuick(i, true);
                        sGty.paternalChrom.putQuick(i, true);
                    }
                    snp.order = snpOrder;
                    snpOrder++;
                }
            }
        }

        in.close();
        if (unexpectedGtyNum > 0) {
            GlobalVariables.mainView.setBriefRunningInfor((unexpectedGtyNum / 2) + " Unexpected genotypes in " + pedFileName + " for " + unexpectedSNPSet.size() + " SNPs (detailed in the log).");
            GlobalVariables.addInforLog(unexpectedSNPSet.size() + " SNPs have unexpected genotypes in " + pedFileName + ":\n" + unexpectedSNPSet.toString());

            GlobalVariables.mainView.setBriefRunningInfor(unexpectedSNPSet.toString());
            unexpectedSNPSet.clear();
        }
        return false;
    }

    void printBitSet(boolean[] b) {
        System.out.println("bits: " + b);
        String bbits = new String();
        for (int j = b.length - 1; j >= 0; j--) {
            bbits += (b[j] ? "1" : "0");
        }
        System.out.println("bit pattern: " + bbits);
    }

    /**
     * retrieve SNP information from the Map files
     * @param FileName
     * @param indices
     * @param arry
     * @param genetRegions pair of physical regions
     * @throws java.lang.Exception
     * @return
     */
    protected int readSNPsinMapFileByPysic(CommonGenetFileSet fileSet, List<AnnotSNP> snpList,
            String chromosome, long[] physicalRegions) throws Exception {
        File mapFile = new File(fileSet.getMapFileName());
        BufferedReader br = new BufferedReader(new FileReader(mapFile), FILE_READER_BUFFER_SIZE);
        String line = null;

        String delmilit = ", \t";
        int chromIndex = fileSet.getChromIndex();
        int posIndex = fileSet.getPosIndex();
        int rsIndex = fileSet.getRsIndex();
        int strandIndex = fileSet.getStrand();
        int genetMapIndex = 2;
        int maxIndex = Math.max(chromIndex, posIndex);

        maxIndex = Math.max(maxIndex, rsIndex);
        maxIndex = Math.max(maxIndex, strandIndex);
        maxIndex = Math.max(maxIndex, genetMapIndex);
        boolean checkRegion = false;
        int regionNum = physicalRegions.length / 2;

        int j = 0;
        StringBuilder strRegion = new StringBuilder();
        for (j = 0; j < regionNum; j++) {
            if (physicalRegions[j * 2] != -9 || physicalRegions[j * 2 + 1] != -9) {
                checkRegion = true;
                if (physicalRegions[j * 2 + 1] == -9) {
                    physicalRegions[1] = Long.MAX_VALUE;
                }
                strRegion.append(" [");
                strRegion.append(physicalRegions[j * 2]);
                strRegion.append(",");
                strRegion.append(physicalRegions[j * 2 + 1]);
                strRegion.append("]bp");
            }
        }

        int lineCounter = -1;

        int chroNum;
        if (chromosome.equalsIgnoreCase("X")) {
            chroNum = 23;
        } else if (chromosome.equalsIgnoreCase("Y")) {
            chroNum = 24;
        } else if (chromosome.equalsIgnoreCase("MT")) {
            chroNum = 26;
        } else if (chromosome.equalsIgnoreCase("XY")) {
            chroNum = 25;
        } else {
            chroNum = Integer.parseInt(chromosome);
        }

        if (checkRegion) {
            regionNum = physicalRegions.length / 2;
        }
        boolean invalid = false;
        long pos = 0;

        //skip the head line
        br.readLine();

        boolean diffChrom = false;
        String tmpStr;
        String rsID = null;
        StringBuilder tmpBuffer = new StringBuilder();
        int index;
        Map<Long, Integer> physicalPos = new HashMap<Long, Integer>(10000, 0.9f);
        double decodeGenet = -9;
        char strand = MISSING_STRAND_NAME;
        boolean isInRegion = false;
        try {
            while ((line = br.readLine()) != null) {
                if (line.trim().length() == 0) {
                    System.err.println(line);
                    continue;
                }

                lineCounter++;
                StringTokenizer tokenizer = new StringTokenizer(line, delmilit);
                index = 0;
                diffChrom = false;
                invalid = false;
                isInRegion = false;
                decodeGenet = -9;
                while (tokenizer.hasMoreTokens()) {
                    tmpBuffer.delete(0, tmpBuffer.length());
                    tmpBuffer.append(tokenizer.nextToken().trim());
                    tmpStr = tmpBuffer.toString();
                    //filter other chromosomes, which is ordered in the annotation file
                    if (index == chromIndex) {
                        if (tmpStr.equalsIgnoreCase("23")) {
                            tmpStr = "X";
                        } else if (chromosome.equalsIgnoreCase("24")) {
                            tmpStr = "Y";
                        } else if (chromosome.equalsIgnoreCase("26")) {
                            tmpStr = "MT";
                        } else if (chromosome.equalsIgnoreCase("25")) {
                            tmpStr = "XY";
                        }
                        if (tmpStr.compareTo(chromosome) != 0) {
                            diffChrom = true;
                            break;
                        }
                    } else if (index == rsIndex) {
                        rsID = tmpStr;
                    } else if (index == posIndex) {
                        //filter items without physical postion
                        if (tmpStr.startsWith("-")) {
                            invalid = true;
                            break;
                        }

                        pos = Long.parseLong(tmpStr);
                        if (physicalPos.get(pos) != null) {
                            invalid = true;
                            break;
                        }

                        //filter physical region
                        if (checkRegion) {
                            for (int i = 0; i < regionNum; i++) {
                                if (pos >= physicalRegions[i * 2] && pos <= physicalRegions[i * 2 + 1]) {
                                    isInRegion = true;
                                    break;
                                }
                            }
                        }
                    } else if (index == strandIndex) {
                        strand = tmpStr.charAt(0);
                    } else if (index == genetMapIndex) {
                        decodeGenet = Double.parseDouble(tmpStr);
                    }


                    if (index == maxIndex) {
                        break;
                    }
                    index++;
                }

                if (diffChrom) {
                    System.err.println(line);
                    continue;
                }
                if (invalid) {
                    System.err.println(line);
                    continue;
                }
                if (checkRegion && !isInRegion) {
                    continue;
                }

                //split is much slower than Tokenizer
                // linecells = line.split(delmilit);
                /*
                if (!LocalString.isNumeric(linecells[3])) {
                //System.markerOut.println(line);
                //continue;
                linecells[3] = "-1";
                }*/

                if (!rsID.startsWith("rs")) {
                    rsID = "Chr" + chromosome + "Pos" + pos;
                    //continue;
                }
                physicalPos.put(pos, 0);

                // System.out.println(linecells[1]); 
                AnnotSNP aSNP = new AnnotSNP(rsID, chroNum, pos, decodeGenet);

                aSNP.probeSetID = lineCounter;
                aSNP.setStrand(strand);

                //MISSING_STRAND_NAME indicates unknown strand information
                strand = MISSING_STRAND_NAME;
                snpList.add(aSNP);
            }


            // change the strand information
            if (strandIndex == -3) {
                for (int i = snpList.size() - 1; i >= 0; i--) {
                    snpList.get(i).setStrand('+');
                }
            } else if (strandIndex == -2) {
                for (int i = snpList.size() - 1; i >= 0; i--) {
                    snpList.get(i).setStrand('-');
                }
            }

            StringBuilder runningInfo = new StringBuilder();
            runningInfo.append("The number of SNPs on chromosome ");
            runningInfo.append(chromosome);
            runningInfo.append(" ");
            runningInfo.append(strRegion);
            runningInfo.append(" in map file ");
            runningInfo.append(mapFile.getName());
            runningInfo.append(" is ");
            runningInfo.append(snpList.size());
            runningInfo.append(".");
            GlobalVariables.mainView.setBriefRunningInfor(runningInfo.toString(), true);
            GlobalVariables.addInforLog(runningInfo.toString());
            return (lineCounter);
        } finally {
            br.close();
        }
    }

    /**
     * retrieve SNP information from the Map files
     * @param FileName
     * @param indices
     * @param arry
     * @param genetRegions pair of physical regions
     * @throws java.lang.Exception
     * @return
     */
    protected int readSNPsinPlinkBinaryMapFileByPysic(CommonGenetFileSet fileSet, List<AnnotSNP> snpList,
            String chromosome, long[] physicalRegions) throws Exception {
        File mapFile = new File(fileSet.getMapFileName());
        BufferedReader br = new BufferedReader(new FileReader(mapFile), FILE_READER_BUFFER_SIZE);
        String line = null;

        String delmilit = ", \t";
        int chromIndex = fileSet.getChromIndex();
        int posIndex = fileSet.getPosIndex();
        int rsIndex = fileSet.getRsIndex();
        int strandIndex = fileSet.getStrand();
        int genetMapIndex = 2;
        int maxIndex = Math.max(chromIndex, posIndex);

        maxIndex = Math.max(maxIndex, rsIndex);
        maxIndex = Math.max(maxIndex, strandIndex);
        maxIndex = Math.max(maxIndex, genetMapIndex);
        boolean checkRegion = false;
        int regionNum = physicalRegions.length / 2;

        int j = 0;
        StringBuilder strRegion = new StringBuilder();
        for (j = 0; j < regionNum; j++) {
            if (physicalRegions[j * 2] != -9 || physicalRegions[j * 2 + 1] != -9) {
                checkRegion = true;
                if (physicalRegions[j * 2 + 1] == -9) {
                    physicalRegions[1] = Long.MAX_VALUE;
                }
                strRegion.append(" [");
                strRegion.append(physicalRegions[j * 2]);
                strRegion.append(",");
                strRegion.append(physicalRegions[j * 2 + 1]);
                strRegion.append("]bp");
            }
        }

        int lineCounter = -1;

        int chroNum;
        if (chromosome.equalsIgnoreCase("X")) {
            chroNum = 23;
        } else if (chromosome.equalsIgnoreCase("Y")) {
            chroNum = 24;
        } else if (chromosome.equalsIgnoreCase("MT")) {
            chroNum = 26;
        } else if (chromosome.equalsIgnoreCase("XY")) {
            chroNum = 25;
        } else {
            chroNum = Integer.parseInt(chromosome);
        }

        if (checkRegion) {
            regionNum = physicalRegions.length / 2;
        }
        boolean invalid = false;
        long pos = 0;

        boolean diffChrom = false;
        boolean isInRegion = false;
        String tmpStr;
        String rsID = null;
        StringBuilder tmpBuffer = new StringBuilder();
        int index;
        Map<Long, Integer> physicalPos = new HashMap<Long, Integer>(10000, 0.9f);
        double decodeGenet = -9;
        char strand = MISSING_STRAND_NAME;
        int alleleAIndex = 4;
        int alleleBIndex = 5;
        maxIndex = Math.max(maxIndex, alleleAIndex);
        maxIndex = Math.max(maxIndex, alleleBIndex);
        String decodeChromNum = String.valueOf(chroNum);
        char alleleA = MISSING_ALLELE_NAME, alleleB = MISSING_ALLELE_NAME;
        try {
            while ((line = br.readLine()) != null) {
                if (line.trim().length() == 0) {
                    continue;
                }
                lineCounter++;
                StringTokenizer tokenizer = new StringTokenizer(line, delmilit);
                index = 0;
                diffChrom = false;
                invalid = false;
                isInRegion = false;
                alleleA = MISSING_ALLELE_NAME;
                alleleB = MISSING_ALLELE_NAME;
                while (tokenizer.hasMoreTokens()) {
                    tmpBuffer.delete(0, tmpBuffer.length());
                    tmpBuffer.append(tokenizer.nextToken().trim());
                    tmpStr = tmpBuffer.toString();
                    if (index == chromIndex) {
                        if (tmpStr.compareTo(decodeChromNum) != 0) {
                            diffChrom = true;
                            break;
                        }
                    } else if (index == rsIndex) {
                        rsID = tmpStr;
                        if (rsID.equals("rs1107060")){
                            int sss=0;
                        }
                    } else if (index == posIndex) {
                        //filter items without physical postion
                        if (tmpStr.startsWith("-")) {
                            invalid = true;
                            break;
                        }

                        pos = Long.parseLong(tmpStr);
                        if (physicalPos.get(pos) != null) {
                            invalid = true;
                            break;
                        }

                        //filter physical region
                        if (checkRegion) {
                            for (int i = 0; i < regionNum; i++) {
                                if (pos >= physicalRegions[i * 2] && pos <= physicalRegions[i * 2 + 1]) {
                                    isInRegion = true;
                                    break;
                                }
                            }
                        }
                    } else if (index == strandIndex) {
                        strand = tmpStr.charAt(0);
                    } else if (index == alleleAIndex) {
                        alleleA = tmpStr.charAt(0);
                    } else if (index == alleleBIndex) {
                        alleleB = tmpStr.charAt(0);
                    } else if (index == genetMapIndex) {
                        decodeGenet = Double.parseDouble(tmpStr);
                    }


                    if (index == maxIndex) {
                        break;
                    }
                    index++;
                }

                if (diffChrom) {
                    continue;
                }
                if (invalid) {
                    continue;
                }

                if (checkRegion && !isInRegion) {
                    continue;
                }

                //split is much slower than Tokenizer
                // linecells = line.split(delmilit);
                /*
                if (!LocalString.isNumeric(linecells[3])) {
                //System.markerOut.println(line);
                //continue;
                linecells[3] = "-1";
                }*/

                if (!rsID.startsWith("rs")) {
                    rsID = "Chr" + chromosome + "Pos" + pos;
                    //continue;
                }
                physicalPos.put(pos, 0);

                // System.out.println(linecells[1]);
                AnnotSNP aSNP = new AnnotSNP(rsID, chroNum, pos, decodeGenet);
                aSNP.setAAllele(alleleA);
                aSNP.setBAllele(alleleB);
                aSNP.probeSetID = lineCounter;
                aSNP.setStrand(strand);
                //MISSING_STRAND_NAME indicates unknown strand information
                strand = MISSING_STRAND_NAME;
                snpList.add(aSNP);

            }


            // change the strand information
            if (strandIndex == -3) {
                for (int i = snpList.size() - 1; i >= 0; i--) {
                    snpList.get(i).setStrand('+');
                }
            } else if (strandIndex == -2) {
                for (int i = snpList.size() - 1; i >= 0; i--) {
                    snpList.get(i).setStrand('-');
                }
            }

            StringBuilder runningInfo = new StringBuilder();
            runningInfo.append("The number of SNPs on chromosome ");
            runningInfo.append(chromosome);
            runningInfo.append(" ");
            runningInfo.append(strRegion);
            runningInfo.append(" in map file ");
            runningInfo.append(mapFile.getName());
            runningInfo.append(" is ");
            runningInfo.append(snpList.size());
            runningInfo.append(".");
            GlobalVariables.mainView.setBriefRunningInfor(runningInfo.toString(), true);
            GlobalVariables.addInforLog(runningInfo.toString());
            return (lineCounter);
        } finally {
            br.close();
        }
    }
}
