// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.
// Permission is granted for you to use this file to compile IGG.
// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.controller;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.cobi.igg3.entity.StatusGtySet;
import org.cobi.igg3.entity.AnnotSNP;
import org.cobi.igg3.entity.Individual;
import org.cobi.igg3.GlobalVariables;

/**
 *
 * @author mxli
 */
public class GenotypeQC {

    public String removeAllMissing(List<AnnotSNP> mainSnpMap, Map<String, StatusGtySet> indivGtyMap) throws Exception {
        StringBuilder msg = new StringBuilder();
        int snpPos;
        boolean allMiss;
        List<AnnotSNP> tmpSNPMap = new ArrayList<AnnotSNP>();
        tmpSNPMap.addAll(mainSnpMap);
        mainSnpMap.clear();
        int counter = 0;
        int listSize = tmpSNPMap.size();
        for (int i = 0; i < listSize; i++) {
            AnnotSNP snp = tmpSNPMap.get(i);
            snpPos = snp.order;
            allMiss = true;

            for (Map.Entry<String, StatusGtySet> mIndivi : indivGtyMap.entrySet()) {
                if (mIndivi.getValue().existence.size() <= snpPos) {
                    continue;
                }

                if (mIndivi.getValue().existence.getQuick(snpPos)) {
                    allMiss = false;
                    break;
                }
            }
            if (allMiss) {
                msg.append(snp.getRSID());
                msg.append(" ");
                counter++;
            } else {
                mainSnpMap.add(snp);
            }
        }
        tmpSNPMap = null;
        if (msg.length() > 0) {
            return counter + " SNP are excluded due to no genotypes:\n" + msg.toString();
        } else {
            return "";
        }
    }

    public String removeOnceMissing(List<AnnotSNP> mainSnpMap, Map<String, StatusGtySet> indivGtyMap) throws Exception {
        StringBuilder msg = new StringBuilder();
        int snpPos;
        boolean onceMiss;
        List<AnnotSNP> tmpSNPMap = new ArrayList<AnnotSNP>();
        tmpSNPMap.addAll(mainSnpMap);
        mainSnpMap.clear();
        int counter = 0;
        int listSize = tmpSNPMap.size();
        for (int i = 0; i < listSize; i++) {
            AnnotSNP snp = tmpSNPMap.get(i);

            snpPos = snp.order;

            onceMiss = false;
            for (Map.Entry<String, StatusGtySet> mIndivi : indivGtyMap.entrySet()) {
                if (mIndivi.getValue().existence.size() <= snpPos) {
                    continue;
                }
                if (!mIndivi.getValue().existence.getQuick(snpPos)) {
                    onceMiss = true;
                    break;
                }
            }
            if (onceMiss) {
                msg.append(snp.getRSID());
                msg.append(" ");
                counter++;
            } else {
                mainSnpMap.add(snp);
            }
        }
        tmpSNPMap = null;
        if (msg.length() > 0) {
            return counter + " SNP are excluded due to having missing genotypes:\n" + msg.toString();
        } else {
            return "";
        }
    }

    private String getUniqueChipLabe(String famID, String indivID) throws Exception {
        Map<String, Individual> allIndiv = GlobalVariables.currentPedFile.getAllIndividuals();
        for (Map.Entry<String, Individual> mIndiv : allIndiv.entrySet()) {
            Individual indiv = mIndiv.getValue();
            if (indiv.getFamilyID().equals(famID) && indiv.getIndividualID().equals(indivID)) {
                return mIndiv.getKey();
            }
        }
        return null;
    }

    public String removeBadMendel(List<AnnotSNP> mainSnpMap, Map<String, StatusGtySet> indivGtyMap) throws Exception {
        StringBuilder msg = new StringBuilder();
        Map<String, Individual> allIndiv = GlobalVariables.currentPedFile.getAllIndividuals();

        for (Map.Entry<String, Individual> mIndiv : allIndiv.entrySet()) {
            if (!mIndiv.getKey().startsWith("-")) {
                Individual indiv = mIndiv.getValue();
                StatusGtySet indivGty = indivGtyMap.get(mIndiv.getKey());
                String faID = indiv.getDadID();
                String moID = indiv.getMomID();
                String famID = indiv.getFamilyID();
                StatusGtySet faGty = null;
                StatusGtySet moGty = null;

                if (!faID.equals("0") || !moID.equals("0")) {
                    String tmp = getUniqueChipLabe(famID, faID);
                    faGty = indivGtyMap.get(tmp);
                    tmp = getUniqueChipLabe(famID, moID);
                    moGty = indivGtyMap.get(tmp);
                    StringBuilder medmsg = new StringBuilder();

                    mendelCheck1(mainSnpMap, indivGty, faGty, moGty, medmsg);
                    if (medmsg.length() > 0) {
                        msg.append("Family: ");
                        msg.append(famID);
                        msg.append(" Father: ");
                        msg.append(faID);
                        msg.append(" Mother: ");
                        msg.append(moID);
                        msg.append(" Child: ");
                        msg.append(indiv.getIndividualID());
                        msg.append(" (Child's Genotypes are masked at SNPs with Mendelian Violation.)\n");
                        msg.append(medmsg);
                    }
                }
            }
        }
        if (msg.length() > 0) {
            return "Violation of Mendelian Inheritance\n" + msg.toString();
        } else {
            return "No Violation of Mendelian Inheritance!";
        }
    }

    public String trimCompiledGtySet(List<AnnotSNP> mainSnpMap, Map<String, StatusGtySet> indivGtyMap,
            double thresholdDistance, String type) throws Exception {
        List<AnnotSNP> snpList = new ArrayList<AnnotSNP>(mainSnpMap);

        int orgGenotypeInfoSize = snpList.size();
        int trimmedGenotypeInfoSize = 0;
        StringBuilder info = new StringBuilder();
        if (thresholdDistance != 0) {
            if (type.equals("Physical Map")) {
                thresholdDistance = thresholdDistance * 1000;
                trimDatasetbyPhyiscalDistance(snpList, indivGtyMap, (long) thresholdDistance);
            } else if (type.equals("Genetic Map")) {
                trimDatasetbyGeneticDistance(snpList, indivGtyMap, thresholdDistance);
            }
            mainSnpMap.clear();
            trimmedGenotypeInfoSize = snpList.size();
            for (int i = 0; i < trimmedGenotypeInfoSize; i++) {
                mainSnpMap.add(snpList.get(i));
            }
            double propotion = trimmedGenotypeInfoSize * 1.0 / orgGenotypeInfoSize;
            propotion = (1 - propotion) * 100;
            DecimalFormat df = new DecimalFormat("0.0");

            info.append(df.format(propotion));
            info.append("% of SNPs are filtered out by the trimming criterion.");
        }
        return info.toString();
    }

    private void trimDatasetbyPhyiscalDistance(List<AnnotSNP> snpInfo, Map<String, StatusGtySet> indivGtyMap, long threshold) throws
            Exception {
        int genotypeInfoSize = snpInfo.size();
        List<AnnotSNP> tempSNPList = new ArrayList<AnnotSNP>();
        tempSNPList.addAll(snpInfo);
        snpInfo.clear();
        long startPosition = 0;
        if (genotypeInfoSize > 0) {
            startPosition = tempSNPList.get(0).getPhysicalPosition();
        }
        long accumulateDistance = 0;
        int maxHeteroNumber = 0;
        int maxHeteroIndex = 0;
        int counter = 0;
        int pos;
        //select heterozygous within a region
        for (int j = 0; j < genotypeInfoSize; j++) {
            accumulateDistance = tempSNPList.get(j).getPhysicalPosition() - startPosition;
            if (accumulateDistance < threshold) {
                pos = tempSNPList.get(j).order;

                counter = 0;
                //count the number of heterozygous
                for (Map.Entry<String, StatusGtySet> mIndiv : indivGtyMap.entrySet()) {
                    if (mIndiv.getValue().existence.size() < pos) {
                        continue;
                    }
                    if (mIndiv.getValue().existence.getQuick(pos) && mIndiv.getValue().paternalChrom.getQuick(pos) && mIndiv.getValue().maternalChrom.getQuick(pos)) {
                        counter++;
                    }
                }

                if (counter > maxHeteroNumber) {
                    maxHeteroNumber = counter;
                    maxHeteroIndex = j;
                }
            } else {
                snpInfo.add(tempSNPList.get(maxHeteroIndex));
                maxHeteroIndex = j;
                startPosition = tempSNPList.get(j).getPhysicalPosition();
                accumulateDistance = 0;
                maxHeteroNumber = counter;
            }
        }
    }

    private void trimDatasetbyGeneticDistance(List<AnnotSNP> snpInfo, Map<String, StatusGtySet> indivGtyMap, double threshold) throws
            Exception {
        int genotypeInfoSize = snpInfo.size();
        List<AnnotSNP> tempSNPList = new ArrayList<AnnotSNP>();
        tempSNPList.addAll(snpInfo);
        snpInfo.clear();
        double startPosition = 0;
        if (genotypeInfoSize > 0) {
            startPosition = tempSNPList.get(0).getDecodeGeneticMap();
        }
        double accumulateDistance = 0;
        int maxHeteroNumber = 0;
        int maxHeteroIndex = 0;
        int counter = 0;


        int pos;
        //select heterozygous within a region
        for (int j = 0; j < genotypeInfoSize; j++) {
            accumulateDistance = tempSNPList.get(j).getDecodeGeneticMap() - startPosition;
            if (accumulateDistance < threshold) {
                pos = tempSNPList.get(j).order;

                counter = 0;
                //count the number of heterozygous
                for (Map.Entry<String, StatusGtySet> mIndiv : indivGtyMap.entrySet()) {
                    if (mIndiv.getValue().existence.size() < pos) {
                        continue;
                    }
                    if (mIndiv.getValue().existence.getQuick(pos) && mIndiv.getValue().paternalChrom.getQuick(pos) && mIndiv.getValue().maternalChrom.getQuick(pos)) {
                        counter++;
                    }
                }

                if (counter > maxHeteroNumber) {
                    maxHeteroNumber = counter;
                    maxHeteroIndex = j;
                }
            } else {
                snpInfo.add(tempSNPList.get(maxHeteroIndex));
                maxHeteroIndex = j;
                startPosition = tempSNPList.get(j).getDecodeGeneticMap();
                accumulateDistance = 0;
                maxHeteroNumber = counter;
            }
        }
    }

    /**
     * Correct zeor  allele frequencies for linkage analysis
     */
    public void reEstimateFrequency(Map<Long, AnnotSNP> mainSnpMap, Map<String, StatusGtySet> indivGtyMap) throws Exception {
        double alleleAFreq = 0.0;


        int counter0, counter1;

        int pos;
        //select heterozygous within a region
        for (Map.Entry<Long, AnnotSNP> mSNP : mainSnpMap.entrySet()) {
            pos = mSNP.getValue().order;

            counter0 = counter1 = 0;
            //count the number of heterozygous
            for (Map.Entry<String, StatusGtySet> mIndiv : indivGtyMap.entrySet()) {
                if (mIndiv.getValue().existence.size() < pos) {
                    continue;
                }
                if (mIndiv.getValue().existence.getQuick(pos)) {
                    if (mIndiv.getValue().paternalChrom.getQuick(pos)) {
                        counter1++;
                    } else {
                        counter0++;
                    }
                    if (mIndiv.getValue().maternalChrom.getQuick(pos)) {
                        counter1++;
                    } else {
                        counter0++;
                    }
                }
            }
            alleleAFreq = counter0 * 1.0 / (counter1 + counter0);
            mSNP.getValue().setAAlleleFreq(alleleAFreq);
        }
    }

    /**
     * Correct zeor  allele frequencies for linkage analysis
     */
    public void correctFrequency(List<AnnotSNP> mainSnpMap, Map<String, StatusGtySet> indivGtyMap) throws Exception {
        double alleleAFreq = 0.0;
        int counter0, counter1;
        int pos;
        int listSize = mainSnpMap.size();
        AnnotSNP snp;
        //select heterozygous within a region
        for (int i = 0; i < listSize; i++) {
            snp = mainSnpMap.get(i);
            pos = snp.order;

            counter0 = counter1 = 0;

            //count the number of heterozygous
            for (Map.Entry<String, StatusGtySet> mIndiv : indivGtyMap.entrySet()) {
                if (mIndiv.getValue().existence.size() < pos) {
                    continue;
                }
                if (mIndiv.getValue().existence.getQuick(pos)) {
                    if (mIndiv.getValue().paternalChrom.getQuick(pos)) {
                        counter1++;
                    } else {
                        counter0++;
                    }
                    if (mIndiv.getValue().maternalChrom.getQuick(pos)) {
                        counter1++;
                    } else {
                        counter0++;
                    }
                }
            }

            alleleAFreq = counter0 * 1.0 / (counter1 + counter0);
            if (alleleAFreq > 0 && snp.getAAlleleFreq() == 0) {
                snp.setAAlleleFreq(alleleAFreq);
            } else if (alleleAFreq < 1 && snp.getAAlleleFreq() == 1) {
                snp.setAAlleleFreq(alleleAFreq);
            }
        }
    }

    private void mendelCheck(List<AnnotSNP> mainSnpMap, StatusGtySet indivGty, StatusGtySet faGty, StatusGtySet moGty, StringBuilder msg) throws Exception {
        Map<Integer, Integer> orderList = new HashMap<Integer, Integer>();
        int pos = 0;
        byte[] chGy = new byte[2];
        byte[] faGy = new byte[2];
        byte[] moGy = new byte[2];

        if (indivGty != null && faGty != null && moGty != null) {
            int size = indivGty.existence.size();
            for (int i = 0; i < size; i++) {
                pos = i;

                if (indivGty.existence.getQuick(pos)) {
                    if (indivGty.paternalChrom.getQuick(pos)) {
                        chGy[0] = 2;
                    } else {
                        chGy[0] = 1;
                    }
                    if (indivGty.maternalChrom.getQuick(pos)) {
                        chGy[1] = 2;
                    } else {
                        chGy[1] = 1;
                    }

                    if (faGty.existence.size() > pos && faGty.existence.getQuick(pos)) {
                        if (faGty.paternalChrom.getQuick(pos)) {
                            faGy[0] = 2;
                        } else {
                            faGy[0] = 1;
                        }
                        if (faGty.maternalChrom.getQuick(pos)) {
                            faGy[1] = 2;
                        } else {
                            faGy[1] = 1;
                        }
                    } else {
                        faGy[0] = 0;
                        faGy[1] = 0;
                    }

                    if (moGty.existence.size() > pos && moGty.existence.getQuick(pos)) {
                        if (moGty.paternalChrom.getQuick(pos)) {
                            moGy[0] = 2;
                        } else {
                            moGy[0] = 1;
                        }
                        if (moGty.maternalChrom.getQuick(pos)) {
                            moGy[1] = 2;
                        } else {
                            moGy[1] = 1;
                        }
                    } else {
                        moGy[0] = 0;
                        moGy[1] = 0;
                    }

                    if (!isConsistent(chGy, faGy, moGy)) {
                        orderList.put(i - 1, 0);
                    }
                }
            }
        } else if (indivGty != null && faGty != null) {
            int size = indivGty.existence.size();
            for (int i = 0; i < size; i++) {
                pos = i;


                if (indivGty.existence.getQuick(pos)) {
                    if (indivGty.paternalChrom.getQuick(pos)) {
                        chGy[0] = 2;
                    } else {
                        chGy[0] = 1;
                    }
                    if (indivGty.maternalChrom.getQuick(pos)) {
                        chGy[1] = 2;
                    } else {
                        chGy[1] = 1;
                    }

                    if (faGty.existence.size() > pos && faGty.existence.getQuick(pos)) {
                        if (faGty.paternalChrom.getQuick(pos)) {
                            faGy[0] = 2;
                        } else {
                            faGy[0] = 1;
                        }
                        if (faGty.maternalChrom.getQuick(pos)) {
                            faGy[1] = 2;
                        } else {
                            faGy[1] = 1;
                        }
                    } else {
                        faGy[0] = 0;
                        faGy[1] = 0;
                    }
                    moGy[0] = 0;
                    moGy[1] = 0;
                    if (!isConsistent(chGy, faGy, moGy)) {
                        orderList.put(i - 1, 0);
                    }
                }
            }
        } else if (indivGty != null && moGty != null) {
            int size = indivGty.existence.size();
            for (int i = 0; i < size; i++) {
                pos = i;

                if (indivGty.existence.getQuick(pos)) {
                    if (indivGty.paternalChrom.getQuick(pos)) {
                        chGy[0] = 2;
                    } else {
                        chGy[0] = 1;
                    }
                    if (indivGty.maternalChrom.getQuick(pos)) {
                        chGy[1] = 2;
                    } else {
                        chGy[1] = 1;
                    }
                    faGy[0] = 0;
                    faGy[1] = 0;
                    if (moGty.existence.size() > pos && moGty.existence.getQuick(pos)) {
                        if (moGty.paternalChrom.getQuick(pos)) {
                            moGy[0] = 2;
                        } else {
                            moGy[0] = 1;
                        }
                        if (moGty.maternalChrom.getQuick(pos)) {
                            moGy[1] = 2;
                        } else {
                            moGy[1] = 1;
                        }
                    } else {
                        moGy[0] = 0;
                        moGy[1] = 0;
                    }

                    if (!isConsistent(chGy, faGy, moGy)) {
                        orderList.put(i - 1, 0);
                    }
                }
            }
        }
        int i = 0, listSize = mainSnpMap.size();

        if (orderList.size() > 0) {
            for (i = 0; i < listSize; i++) {
                if (orderList.get(mainSnpMap.get(i).order) != null) {
                    msg.append(mainSnpMap.get(i).getRSID());
                    msg.append(": ");
                    pos = mainSnpMap.get(i).order;
                    if (faGty != null && faGty.existence.getQuick(pos)) {
                        if (faGty.paternalChrom.getQuick(pos)) {
                            msg.append(mainSnpMap.get(i).getBAllele());
                        } else {
                            msg.append(mainSnpMap.get(i).getAAllele());
                        }
                        if (faGty.maternalChrom.getQuick(pos)) {
                            msg.append(mainSnpMap.get(i).getBAllele());
                        } else {
                            msg.append(mainSnpMap.get(i).getAAllele());
                        }
                    } else {
                        msg.append("00");
                    }
                    msg.append(" + ");
                    if (moGty != null && moGty.existence.getQuick(pos)) {
                        if (moGty.paternalChrom.getQuick(pos)) {
                            msg.append(mainSnpMap.get(i).getBAllele());
                        } else {
                            msg.append(mainSnpMap.get(i).getAAllele());
                        }
                        if (moGty.maternalChrom.getQuick(pos)) {
                            msg.append(mainSnpMap.get(i).getBAllele());
                        } else {
                            msg.append(mainSnpMap.get(i).getAAllele());
                        }
                    } else {
                        msg.append("00");
                    }
                    msg.append(" -> ");
                    if (indivGty.existence.getQuick(pos)) {
                        if (indivGty.paternalChrom.getQuick(pos)) {
                            msg.append(mainSnpMap.get(i).getBAllele());
                        } else {
                            msg.append(mainSnpMap.get(i).getAAllele());
                        }
                        if (indivGty.maternalChrom.getQuick(pos)) {
                            msg.append(mainSnpMap.get(i).getBAllele());
                        } else {
                            msg.append(mainSnpMap.get(i).getAAllele());
                        }
                        indivGty.existence.putQuick(pos, false);
                    } else {
                        msg.append("00");
                    }
                    msg.append("\n");
                }
            }
        }

    }

    private void mendelCheck1(List<AnnotSNP> mainSnpMap, StatusGtySet indivGty, StatusGtySet faGty, StatusGtySet moGty, StringBuilder msg) throws Exception {
        int pos = 0;
        byte[] chGy = new byte[2];
        byte[] faGy = new byte[2];
        byte[] moGy = new byte[2];
        int size = mainSnpMap.size();
        boolean isViolated = false;
        for (int i = 0; i < size; i++) {
            pos = mainSnpMap.get(i).order;
            isViolated = false;
            if (indivGty != null && faGty != null && moGty != null) {
                if (indivGty.existence.getQuick(pos)) {
                    if (indivGty.paternalChrom.getQuick(pos)) {
                        chGy[0] = 2;
                    } else {
                        chGy[0] = 1;
                    }
                    if (indivGty.maternalChrom.getQuick(pos)) {
                        chGy[1] = 2;
                    } else {
                        chGy[1] = 1;
                    }

                    if (faGty.existence.size() > pos && faGty.existence.getQuick(pos)) {
                        if (faGty.paternalChrom.getQuick(pos)) {
                            faGy[0] = 2;
                        } else {
                            faGy[0] = 1;
                        }
                        if (faGty.maternalChrom.getQuick(pos)) {
                            faGy[1] = 2;
                        } else {
                            faGy[1] = 1;
                        }
                    } else {
                        faGy[0] = 0;
                        faGy[1] = 0;
                    }

                    if (moGty.existence.size() > pos && moGty.existence.getQuick(pos)) {
                        if (moGty.paternalChrom.getQuick(pos)) {
                            moGy[0] = 2;
                        } else {
                            moGy[0] = 1;
                        }
                        if (moGty.maternalChrom.getQuick(pos)) {
                            moGy[1] = 2;
                        } else {
                            moGy[1] = 1;
                        }
                    } else {
                        moGy[0] = 0;
                        moGy[1] = 0;
                    }
                    if (!isConsistent(chGy, faGy, moGy)) {
                        isViolated = true;
                    }
                }

            } else if (indivGty != null && faGty != null) {
                if (indivGty.existence.getQuick(pos)) {
                    if (indivGty.paternalChrom.getQuick(pos)) {
                        chGy[0] = 2;
                    } else {
                        chGy[0] = 1;
                    }
                    if (indivGty.maternalChrom.getQuick(pos)) {
                        chGy[1] = 2;
                    } else {
                        chGy[1] = 1;
                    }

                    if (faGty.existence.size() > pos && faGty.existence.getQuick(pos)) {
                        if (faGty.paternalChrom.getQuick(pos)) {
                            faGy[0] = 2;
                        } else {
                            faGy[0] = 1;
                        }
                        if (faGty.maternalChrom.getQuick(pos)) {
                            faGy[1] = 2;
                        } else {
                            faGy[1] = 1;
                        }
                    } else {
                        faGy[0] = 0;
                        faGy[1] = 0;
                    }
                    moGy[0] = 0;
                    moGy[1] = 0;
                    if (!isConsistent(chGy, faGy, moGy)) {
                        isViolated = true;
                    }
                }
            } else if (indivGty != null && moGty != null) {
                if (indivGty.existence.getQuick(pos)) {
                    if (indivGty.paternalChrom.getQuick(pos)) {
                        chGy[0] = 2;
                    } else {
                        chGy[0] = 1;
                    }
                    if (indivGty.maternalChrom.getQuick(pos)) {
                        chGy[1] = 2;
                    } else {
                        chGy[1] = 1;
                    }
                    faGy[0] = 0;
                    faGy[1] = 0;
                    if (moGty.existence.size() > pos && moGty.existence.getQuick(pos)) {
                        if (moGty.paternalChrom.getQuick(pos)) {
                            moGy[0] = 2;
                        } else {
                            moGy[0] = 1;
                        }
                        if (moGty.maternalChrom.getQuick(pos)) {
                            moGy[1] = 2;
                        } else {
                            moGy[1] = 1;
                        }
                    } else {
                        moGy[0] = 0;
                        moGy[1] = 0;
                    }
                    if (!isConsistent(chGy, faGy, moGy)) {
                        isViolated = true;
                    }
                }
            }
            if (isViolated) {
                msg.append(mainSnpMap.get(i).getRSID());
                msg.append(": ");
                pos = mainSnpMap.get(i).order;
                if (faGty != null && faGty.existence.getQuick(pos)) {
                    if (faGty.paternalChrom.getQuick(pos)) {
                        msg.append(mainSnpMap.get(i).getBAllele());
                    } else {
                        msg.append(mainSnpMap.get(i).getAAllele());
                    }
                    if (faGty.maternalChrom.getQuick(pos)) {
                        msg.append(mainSnpMap.get(i).getBAllele());
                    } else {
                        msg.append(mainSnpMap.get(i).getAAllele());
                    }
                    faGty.existence.putQuick(pos, false);
                } else {
                    msg.append("00");
                }
                msg.append(" + ");
                if (moGty != null && moGty.existence.getQuick(pos)) {
                    if (moGty.paternalChrom.getQuick(pos)) {
                        msg.append(mainSnpMap.get(i).getBAllele());
                    } else {
                        msg.append(mainSnpMap.get(i).getAAllele());
                    }
                    if (moGty.maternalChrom.getQuick(pos)) {
                        msg.append(mainSnpMap.get(i).getBAllele());
                    } else {
                        msg.append(mainSnpMap.get(i).getAAllele());
                    }
                    moGty.existence.putQuick(pos, false);
                } else {
                    msg.append("00");
                }
                msg.append(" -> ");
                if (indivGty.existence.getQuick(pos)) {
                    if (indivGty.paternalChrom.getQuick(pos)) {
                        msg.append(mainSnpMap.get(i).getBAllele());
                    } else {
                        msg.append(mainSnpMap.get(i).getAAllele());
                    }
                    if (indivGty.maternalChrom.getQuick(pos)) {
                        msg.append(mainSnpMap.get(i).getBAllele());
                    } else {
                        msg.append(mainSnpMap.get(i).getAAllele());
                    }
                    indivGty.existence.putQuick(pos, false);
                } else {
                    msg.append("00");
                }
                msg.append("\n");
            }
        }

    }

    /**
     * check the mendelian inheritance 
     * @param chGy genotype of child 0 indicating missing
     * @param chGy genotype of father 0 indicating missing
     * @param chGy genotype of mother 0 indicating missing
     * @return whether the inheritance is held
     */
    static private boolean isConsistent(byte[] chGy, byte[] faGY, byte[] moGt) {
        byte tmp;
        if (chGy[0] > chGy[1]) {
            tmp = chGy[1];
            chGy[1] = chGy[0];
            chGy[0] = tmp;
        }
        if (chGy[0] == 0) {
            return true;
        }
        if ((faGY[0] == 0 && faGY[1] == 0) && (moGt[0] == 0 && moGt[1] == 0)) {
            return true;
        } else if ((faGY[0] == 0 && faGY[1] == 0) && (moGt[0] != 0 && moGt[1] != 0)) {
            if (chGy[0] == moGt[0] || chGy[0] == moGt[1] || chGy[1] == moGt[0] || chGy[1] == moGt[1]) {
                return true;
            } else {
                return false;
            }

        } else if ((faGY[0] != 0 && faGY[1] != 0) && (moGt[0] == 0 && moGt[1] == 0)) {
            if (chGy[0] == faGY[0] || chGy[0] == faGY[1] || chGy[1] == faGY[0] || chGy[1] == faGY[1]) {
                return true;
            } else {
                return false;
            }
        }
        if (chGy[0] == chGy[1]) {
            tmp = 0;
            if (chGy[0] == faGY[0]) {
                tmp++;
            } else if (chGy[0] == faGY[1]) {
                tmp++;
            }
            if (chGy[0] == moGt[0]) {
                tmp++;
            } else if (chGy[0] == moGt[1]) {
                tmp++;
            }
            if (tmp == 2) {
                return true;
            } else {
                return false;
            }
        } else {
            boolean consistant = ((chGy[0] == faGY[0] || chGy[0] == faGY[1])
                    && (chGy[1] == moGt[0] || chGy[1] == moGt[1]))
                    || ((chGy[1] == faGY[0] || chGy[1] == faGY[1]) && (chGy[0] == moGt[0] || chGy[0] == moGt[1]));
            if (consistant) {
                return true;
            } else {
                return false;
            }
        }
    }
}
