// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.
// Permission is granted for you to use this file to compile IGG.
// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.controller;

import cern.colt.bitvector.BitVector;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.TreeSet;
import net.sf.picard.liftover.LiftOver;
import net.sf.picard.util.Interval;
import org.cobi.igg3.entity.AnnotSNP;
import org.cobi.igg3.entity.ChromSNPGtySet;
import org.cobi.igg3.entity.Integration;
import org.cobi.igg3.entity.SNPPosiComparator;
import org.cobi.igg3.entity.StatusGtySet;
import org.cobi.igg3.GlobalVariables;

/**
 *
 * @author mxli
 */
public class HapMapGenotypeIntegrator extends BasicIntegrator {

    Integration hapIntgration;
    Integration oldIntgration;
    HashMap<String, List<String>> chromFiles;
    HashMap<String, List<String>> chromPopus;
    int referenceGenomeVersion = 36;
    LiftOver liftOver = null;

    public HapMapGenotypeIntegrator(Integration hapIntgration, Integration oldIntgration) {
        this.hapIntgration = hapIntgration;
        this.oldIntgration = oldIntgration;
        this.chromFiles = new HashMap<String, List<String>>();
        this.chromPopus = new HashMap<String, List<String>>();
    }

    public HapMapGenotypeIntegrator() {
    }

    public int getReferenceGenomeVersion() {
        return referenceGenomeVersion;
    }

    public void setReferenceGenomeVersion(int referenceGenomeVersion) {
        this.referenceGenomeVersion = referenceGenomeVersion;
        File CHAIN_FILE = null;
        if (referenceGenomeVersion == 37) {
            //lazy!
            CHAIN_FILE = new File(RESOURCE_PATH + "chip_annotation/hg19ToHg18.over.chain");
        } else if (referenceGenomeVersion == 35) {
            CHAIN_FILE = new File(RESOURCE_PATH + "chip_annotation/hg17ToHg18.over.chain");
        }
        if (CHAIN_FILE != null) {
            liftOver = new LiftOver(CHAIN_FILE);
        }
    }

    public boolean checkGenotyeFileCompletion(List<String> fileList, StringBuilder missingFileInfor, List<String> chroms) throws Exception {
        Map<String, long[]> regions = hapIntgration.getPhysicalRegionMap();
        if (regions == null) {
            throw new Exception("No Available Physical Region");
        }
        this.chromFiles.clear();
        this.chromPopus.clear();
        boolean pass = true;

        List<String> fileNames = new ArrayList<String>();
        int fileListSize = fileList.size();
        String tmpStr;
        String fileSperator = File.separator;
        for (int i = 0; i < fileListSize; i++) {
            tmpStr = fileList.get(i);
            fileNames.add(tmpStr.substring(tmpStr.lastIndexOf(fileSperator) + 1));
        }

        //check whether the hapmap genotype files are complete
        for (Map.Entry<String, long[]> m : regions.entrySet()) {
            tmpStr = m.getKey();
            if (tmpStr.equals("MT")) {
                tmpStr = "chrMT_";
            } else {
                tmpStr = "chr" + tmpStr + "_";
            }
            List<String> filePath = new ArrayList<String>();
            List<String> popus = new ArrayList<String>();
            for (int j = 0; j < HAPMAP_POPULATIONS.length; j++) {
                for (int k = 0; k < fileListSize; k++) {
                    String fileName = fileNames.get(k);
                    if (fileName.indexOf("JPT+CHB") >= 0 && !HAPMAP_POPULATIONS[j].equals("JPT+CHB")) {
                        continue;
                    }
                    if (fileName.indexOf(tmpStr) >= 0
                            && fileName.indexOf(HAPMAP_POPULATIONS[j]) >= 0) {
                        filePath.add(fileList.get(k));
                        popus.add(HAPMAP_POPULATIONS[j]);
                        break;
                    }
                }
            }
            tmpStr = tmpStr.substring(0, tmpStr.length() - 1);
            if (filePath.isEmpty()) {
                missingFileInfor.append("HapMap genotype files cannot be found for ");
                missingFileInfor.append(tmpStr);
                missingFileInfor.append("\n");
                pass = false;
            } else {
                chromFiles.put(tmpStr, filePath);
                chromPopus.put(tmpStr, popus);
                chroms.add(m.getKey());
            }
        }

        missingFileInfor.append("HapMap Genotype files are found for these chromosome(s) and popualtion(s):\n");
        for (Map.Entry<String, List<String>> m : chromPopus.entrySet()) {
            String chrom = m.getKey();
            missingFileInfor.append(chrom);
            missingFileInfor.append(":");
            List<String> popus = m.getValue();
            for (int i = 0; i < popus.size(); i++) {
                missingFileInfor.append(" ");
                missingFileInfor.append(popus.get(i));
            }
            missingFileInfor.append("\n");
        }
        return pass;
    }

    public void integrate(boolean matchRegion, boolean isForward) throws Exception {
        Map<Long, AnnotSNP> hapSnpMap = new HashMap<Long, AnnotSNP>();
        Map<String, StatusGtySet> hapIndivGty = new HashMap<String, StatusGtySet>();
        List<AnnotSNP> projSnpList;
        Map<String, StatusGtySet> projIndivGty;
        ChromSNPGtySet chromSet = new ChromSNPGtySet();
        List<AnnotSNP> hapSnpList = new ArrayList<AnnotSNP>();
        String runningInfo = "";
        TreeSet<String> enteredHapIndiv = new TreeSet<String>();
        for (int i = 0; i < CHROM_NAME.length; i++) {
            List<String> files = chromFiles.get("chr" + CHROM_NAME[i]);
            if (files == null) {
                continue;
            }
            hapSnpMap.clear();
            hapIndivGty.clear();
            File resultPath = new File(hapIntgration.getResultPath() + File.separator);
            if (!resultPath.exists()) {
                resultPath.mkdir();
            }
            chromSet.setChromName(CHROM_NAME[i]);

            if (matchRegion) {
                readHapMapGenotypebyChromosomeRegion(CHROM_NAME[i], files, hapSnpMap, hapIndivGty, enteredHapIndiv, isForward);
                addDbSNPFlankingSequenceInComplemenetaryHapMapSNP(hapSnpMap, CHROM_NAME[i]);

                chromSet.readChromSNPInfo(oldIntgration.getResultPath());
                chromSet.readNextGtyBlock(oldIntgration.getResultPath());
                projSnpList = chromSet.getSnpInfoList();
            } else {
                chromSet.readChromSNPInfo(oldIntgration.getResultPath());
                chromSet.readNextGtyBlock(oldIntgration.getResultPath());
                projSnpList = chromSet.getSnpInfoList();
                Map<Long, Boolean> projSnpMap = new HashMap<Long, Boolean>(projSnpList.size(), 0.9f);
                TreeSet<String> projSNPID = new TreeSet<String>();
                //record the snp position a
                for (int j = projSnpList.size() - 1; j >= 0; j--) {
                    projSnpMap.put(projSnpList.get(j).getPhysicalPosition(), true);
                    projSNPID.add(projSnpList.get(j).getRSID().toLowerCase());
                }
                readHapMapGenotypebySNP(CHROM_NAME[i], files, projSNPID, projSnpMap, hapSnpMap, hapIndivGty, enteredHapIndiv, isForward);
                projSnpMap.clear();
                StringBuilder info = new StringBuilder();
                info.append("The following ");
                info.append(projSNPID.size());
                info.append(" SNPs have no genotypes in HapMap:\n");
                info.append(projSNPID.toString());
                projSNPID.clear();


                GlobalVariables.mainView.setBriefRunningInfor(info.toString(), true);
                GlobalVariables.addInforLog(info.toString());
                addDbSNPFlankingSequenceInComplemenetaryHapMapSNP(hapSnpMap, CHROM_NAME[i]);
            }

            hapSnpList.clear();
            projIndivGty = chromSet.getIndiviGty();
            hapSnpList.addAll(hapSnpMap.values());
            hapSnpMap.clear();
            Collections.sort(hapSnpList, new SNPPosiComparator());

            runningInfo = mergeSNPGtybyListNoGtyExtend(projSnpList, hapSnpList, projIndivGty, hapIndivGty);
            hapSnpList.clear();
            hapIndivGty.clear();
            hapIntgration.addAChromResult(CHROM_NAME[i] + " SNP#" + projSnpList.size());

            chromSet.saveChromSNPInforAndClear(resultPath.getCanonicalPath());
            projSnpList.clear();
            chromSet.saveGtyBlock(resultPath.getCanonicalPath());
            projIndivGty.clear();

            // save other geneotype while assuming the hapmap individual labels do not exsit in the project ones
            while (chromSet.hasMoreGtyBlock()) {
                chromSet.readNextGtyBlock(oldIntgration.getResultPath());
                chromSet.saveGtyBlock(resultPath.getCanonicalPath());
                chromSet.setIndiviGty(null);
            }

            GlobalVariables.mainView.setBriefRunningInfor(runningInfo, true);
            GlobalVariables.addInforLog(runningInfo);
        }
        hapIntgration.setHapIndivLabels(enteredHapIndiv);

        //directly copy the common pedigree structure into the folder with hapmap data
        File commPedFile = new File(oldIntgration.getResultPath() + File.separator + "ped");
        if (commPedFile.exists()) {
            // Create channel on the source
            FileChannel srcChannel = new FileInputStream(oldIntgration.getResultPath() + File.separator + "ped").getChannel();
            // Create channel on the destination
            FileChannel dstChannel = new FileOutputStream(hapIntgration.getResultPath() + File.separator + "ped").getChannel();
            // Copy file contents from source to destination
            dstChannel.transferFrom(srcChannel, 0, srcChannel.size());
            // Close the channels
            srcChannel.close();
            dstChannel.close();

        }
    }

    public void readHapMapGenotypebyChromosomeRegion(String chroStr, List<String> files, Map<Long, AnnotSNP> snpMapList,
            Map<String, StatusGtySet> allIndivGty, TreeSet<String> enteredHapIndiv, boolean isForward) throws Exception {
        int chroNum;
        StringBuilder strRegion = new StringBuilder();
        String line = null;
        StringTokenizer tokenizer = null;
        String delmilit = "\t ,";

        StringBuilder runningInfo = new StringBuilder();
        String sigGty;
        int totalColNum = -1;
        int startGtyCol = -1;
        int rsIndex = -1;
        int alleleIndex = -1;
        int posIndex = -1;

        TreeSet<String> unexpectedSNPSet = new TreeSet<String>();

        int availabeSNPSpace = 100000;
        List<String> indiviChipLabelList = new ArrayList<String>();
        String tmpStr;
        char strandLabel = '+';
        if (!isForward) {
            strandLabel = '-';
        }

        if (chroStr.equalsIgnoreCase("X")) {
            chroNum = 23;
        } else if (chroStr.equalsIgnoreCase("Y")) {
            chroNum = 24;
        } else if (chroStr.equalsIgnoreCase("MT")) {
            chroNum = 26;
        } else if (chroStr.equalsIgnoreCase("XY")) {
            chroNum = 25;
        } else {
            chroNum = Integer.parseInt(chroStr);
        }

        boolean needConvert = false;
        if (referenceGenomeVersion == 37) {
            needConvert = true;
        } else if (referenceGenomeVersion == 35) {
            needConvert = true;
        }
        Interval int1 = new Interval("chr" + chroStr, 0, 0);

        long[] physicalRegions = hapIntgration.getPhysicalRegionMap().get(chroStr);
        int regionNum = physicalRegions.length / 2;
        boolean checkRegion = false;
        int index = 0;
        long pos = 0;
        int validSNPNum = 0;
        boolean invalid = true;
        int curSNPNum = 0;
boolean isInRegion=false;
        char aAllele, bAllele;
        for (int j = 0; j < regionNum; j++) {
            if (physicalRegions[j * 2] != -9 || physicalRegions[j * 2 + 1] != -9) {
                checkRegion = true;
                if (physicalRegions[j * 2 + 1] == -9) {
                    physicalRegions[1] = Long.MAX_VALUE;
                }
                strRegion.append(" [");
                strRegion.append(physicalRegions[j * 2]);
                strRegion.append(",");
                strRegion.append(physicalRegions[j * 2+1]);
                strRegion.append("]bp");
            }
        }
        StringBuilder tmpBuffer = new StringBuilder();
        for (int j = 0; j < files.size(); j++) {
            BufferedReader br = new BufferedReader(new FileReader(files.get(j)), FILE_READER_BUFFER_SIZE);
            String info = "Encoding HapMap File " + files.get(j) + " ...";
            GlobalVariables.mainView.setBriefRunningInfor(info);
            GlobalVariables.addInforLog(info);
            validSNPNum = 0;
            //read individual label 
            if ((line = br.readLine()) != null) {
                tokenizer = new StringTokenizer(line, delmilit);
                totalColNum = 0;
                startGtyCol = -1;
                rsIndex = -1;
                alleleIndex = -1;
                posIndex = -1;
                indiviChipLabelList.clear();
                while (tokenizer.hasMoreTokens()) {
                    tmpBuffer.delete(0, tmpBuffer.length());
                    tmpBuffer.append(tokenizer.nextToken().trim());
                    tmpStr = tmpBuffer.toString();

                    if (tmpStr.startsWith("NA")) {
                        if (startGtyCol < 0) {
                            startGtyCol = totalColNum;
                        }
                        //some HapMap file has duplicated sample
                        int awfulIndex = tmpStr.indexOf(".");
                        if (awfulIndex >= 0) {
                            tmpStr = tmpStr.substring(0, awfulIndex);
                        }
                        if (allIndivGty.get(tmpStr) == null) {
                            StatusGtySet genotypes = new StatusGtySet();
                            genotypes.existence = new BitVector(availabeSNPSpace);
                            genotypes.paternalChrom = new BitVector(availabeSNPSpace);
                            genotypes.maternalChrom = new BitVector(availabeSNPSpace);
                            allIndivGty.put(tmpStr, genotypes);
                        }
                        indiviChipLabelList.add(tmpStr);
                        if (!enteredHapIndiv.contains(tmpStr)) {
                            enteredHapIndiv.add(tmpStr);
                        }
                    } else if (tmpStr.startsWith("rs#")) {
                        rsIndex = totalColNum;
                    } else if (tmpStr.equals("alleles") || tmpStr.equals("SNPalleles")) {
                        alleleIndex = totalColNum;
                    } else if (tmpStr.startsWith("pos")) {
                        posIndex = totalColNum;
                    }
                    totalColNum++;
                }
            }
            String[] cells = new String[totalColNum];
            int redundantSNPcounter = 0;
            //start to read genotypes
            while ((line = br.readLine()) != null) {
                line = line.toUpperCase();
                index = 0;
                tokenizer = new StringTokenizer(line, delmilit);
                invalid = false;
                isInRegion=false;
                while (tokenizer.hasMoreTokens()) {
                    tmpBuffer.delete(0, tmpBuffer.length());
                    tmpBuffer.append(tokenizer.nextToken().trim());
                    cells[index] = tmpBuffer.toString();
                    if (index == posIndex) {
                        //filter physical region

                        pos = Long.parseLong(cells[posIndex]);
                        // convert the current corordinates to be build   36
                        if (needConvert) {
                            int1.setPoint((int) pos);
                            Interval int2 = liftOver.liftOver(int1);
                            //System.out.println(int2.toString());
                            if (int2 == null) {
                                invalid = true;
                                break;
                            }
                            pos = int2.getStart();
                            int2 = null;
                        }
                        if (checkRegion) {
                            for (int k = 0; k < regionNum; k++) {
                                if (pos >= physicalRegions[k * 2] && pos <= physicalRegions[k * 2 + 1]) {
                                    isInRegion = true;
                                    break;
                                }
                            }
                        }
                    }
                    index++;
                }
                if (invalid) {
                    continue;
                }

                 if (checkRegion && !isInRegion) {
                    continue;
                }
                 
                validSNPNum++;
                AnnotSNP snp = snpMapList.get(pos);
                if (snp == null) {
                    snp = new AnnotSNP("rs" + cells[rsIndex].substring(2), chroNum, pos, 0.0,
                            cells[alleleIndex].charAt(0), cells[alleleIndex].charAt(2));
                    snp.order = curSNPNum;
                    snp.setStrand(strandLabel);
                    curSNPNum += 2;
                    if (curSNPNum > availabeSNPSpace) {
                        //adjust the size of bitvectors
                        availabeSNPSpace += HAP_GENOTYPE_LOAD_FACTOR;
                        for (Map.Entry<String, StatusGtySet> m1 : allIndivGty.entrySet()) {
                            m1.getValue().paternalChrom.setSize(availabeSNPSpace);
                            m1.getValue().maternalChrom.setSize(availabeSNPSpace);
                            m1.getValue().existence.setSize(availabeSNPSpace);
                        }
                    }
                    snpMapList.put(pos, snp);
                } else {
                    redundantSNPcounter++;
                }


                aAllele = snp.getAAllele();
                bAllele = snp.getBAllele();
                for (int k = startGtyCol; k < totalColNum; k++) {
                    sigGty = cells[k];
                    // make sure the genotype is made up of two alleles. otherwise regarded it as a missing data
                    if (sigGty.length() != 2) {
                        continue;
                    }
                    String indivLabel = indiviChipLabelList.get(k - startGtyCol);
                    StatusGtySet gty = allIndivGty.get(indivLabel);

                    //re-code: A->0; B->1;
                    if (sigGty.charAt(0) == bAllele) {
                        gty.paternalChrom.putQuick(snp.order, true);
                        gty.existence.putQuick(snp.order, true);
                    } else if (sigGty.charAt(0) == aAllele) {
                        gty.existence.putQuick(snp.order, true);
                    } else if (sigGty.charAt(0) != 'N') {
                        String unexpectInfo = "Unexpected genotype " + sigGty + " at " + snp.getRSID() + " for " + indivLabel;
                        //GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                        unexpectedSNPSet.add(snp.getRSID());
                        GlobalVariables.addInforLog(unexpectInfo);
                        // throw new Exception("Unexpected genotype " + sigGty + " at " + snp.getRSID() + " for " + indivLabel);
                    }
                    if (sigGty.charAt(1) == bAllele) {
                        gty.maternalChrom.putQuick(snp.order, true);
                    } else if (sigGty.charAt(1) != aAllele && sigGty.charAt(1) != 'N') {
                        //throw new Exception("Unexpected genotype " + sigGty + " at " + snp.getRSID() + " for " + indivLabel);
                        String unexpectInfo = "Unexpected genotype " + sigGty + " at " + snp.getRSID() + " for " + indivLabel;
                        //GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                        GlobalVariables.addInforLog(unexpectInfo);
                        unexpectedSNPSet.add(snp.getRSID());
                    }
                }
            }
            br.close();
            if (unexpectedSNPSet.size() > 0) {
                GlobalVariables.mainView.setBriefRunningInfor(unexpectedSNPSet.size() + " SNPs have unexpected genotypes in " + files.get(j));
                GlobalVariables.addInforLog(unexpectedSNPSet.size() + " SNPs have unexpected genotypes in " + files.get(j) + ":\n" + unexpectedSNPSet.toString());
                GlobalVariables.mainView.setBriefRunningInfor(unexpectedSNPSet.toString());
                unexpectedSNPSet.clear();
            }
            runningInfo.delete(0, runningInfo.length());
            runningInfo.append(files.get(j)).append(" encoded!\n");
            runningInfo.append("The number of SNPs on chromosome ");
            runningInfo.append(chroStr);
            runningInfo.append(" ");
            runningInfo.append(strRegion);
            runningInfo.append(" is ");
            runningInfo.append(validSNPNum);
            runningInfo.append(".");
            //runningInfo.append(redundantSNPcounter);
            //runningInfo.append(" redundant).");
            GlobalVariables.mainView.setBriefRunningInfor(runningInfo.toString(), true);
            GlobalVariables.addInforLog(runningInfo.toString());
        }

    }

    public void readHapMapGenotypebySNP(String chroStr, List<String> files, TreeSet<String> projSNPID, Map<Long, Boolean> projSnpMap,
            Map<Long, AnnotSNP> snpMapList, Map<String, StatusGtySet> allIndivGty, TreeSet<String> enteredHapIndiv, boolean isForward) throws Exception {
        int chroNum;
        StringBuilder strRegion = new StringBuilder();
        String line = null;
        StringTokenizer tokenizer = null;
        String delmilit = "\t ,";

        StringBuilder runningInfo = new StringBuilder();
        String sigGty;
        int totalColNum = -1;
        int startGtyCol = -1;
        int rsIndex = -1;
        int alleleIndex = -1;
        int posIndex = -1;

        int availabeSNPSpace = 100000;
        List<String> indiviChipLabelList = new ArrayList<String>();
        String tmpStr;

        char strandLabel = '+';
        if (!isForward) {
            strandLabel = '-';
        }
        if (chroStr.equalsIgnoreCase("X")) {
            chroNum = 23;
        } else if (chroStr.equalsIgnoreCase("Y")) {
            chroNum = 24;
        } else if (chroStr.equalsIgnoreCase("MT")) {
            chroNum = 26;
        } else if (chroStr.equalsIgnoreCase("XY")) {
            chroNum = 25;
        } else {
            chroNum = Integer.parseInt(chroStr);
        }
        boolean needConvert = false;
        if (referenceGenomeVersion == 37 || referenceGenomeVersion == 35) {
            needConvert = true;
        }
        Interval int1 = new Interval("chr" + chroStr, 0, 0);
        long[] physicalRegions = hapIntgration.getPhysicalRegionMap().get(chroStr);
        int regionNum = physicalRegions.length / 2;
        int index = 0;
        long pos = 0;

        boolean invalid = true;
        int curSNPNum = 0;
        TreeSet<String> unexpectedSNPSet = new TreeSet<String>();

        char aAllele, bAllele;
        for (int j = 0; j < regionNum; j++) {
            if (physicalRegions[j * 2] != -9 || physicalRegions[j * 2 + 1] != -9) {
                if (physicalRegions[j * 2 + 1] == -9) {
                    physicalRegions[1] = Long.MAX_VALUE;
                }
                strRegion.append(" [");
                strRegion.append(physicalRegions[j * 2]);
                strRegion.append(",");
                strRegion.append(physicalRegions[j * 2+1]);
                strRegion.append("]bp");
            }
        }
        StringBuilder tmpBuffer = new StringBuilder();
        for (int j = 0; j < files.size(); j++) {
            String info = "Encoding HapMap File " + files.get(j) + " ...";
            GlobalVariables.mainView.setBriefRunningInfor(info);
            GlobalVariables.addInforLog(info);

            BufferedReader br = new BufferedReader(new FileReader(files.get(j)), FILE_READER_BUFFER_SIZE);
            //read individual label 
            if ((line = br.readLine()) != null) {
                tokenizer = new StringTokenizer(line, delmilit);
                totalColNum = 0;
                startGtyCol = -1;
                rsIndex = -1;
                alleleIndex = -1;
                posIndex = -1;
                indiviChipLabelList.clear();
                while (tokenizer.hasMoreTokens()) {
                    tmpBuffer.delete(0, tmpBuffer.length());
                    tmpBuffer.append(tokenizer.nextToken().trim());
                    tmpStr = tmpBuffer.toString();
                    if (tmpStr.startsWith("NA")) {
                        if (startGtyCol < 0) {
                            startGtyCol = totalColNum;
                        }
                        //some HapMap file has duplicated sample
                        int awfulIndex = tmpStr.indexOf(".");
                        if (awfulIndex >= 0) {
                            tmpStr = tmpStr.substring(0, awfulIndex);
                        }
                        if (allIndivGty.get(tmpStr) == null) {
                            StatusGtySet genotypes = new StatusGtySet();
                            genotypes.existence = new BitVector(availabeSNPSpace);
                            genotypes.paternalChrom = new BitVector(availabeSNPSpace);
                            genotypes.maternalChrom = new BitVector(availabeSNPSpace);
                            allIndivGty.put(tmpStr, genotypes);
                        }

                        indiviChipLabelList.add(tmpStr);
                        if (!enteredHapIndiv.contains(tmpStr)) {
                            enteredHapIndiv.add(tmpStr);
                        }
                    } else if (tmpStr.equals("rs#")) {
                        rsIndex = totalColNum;
                    } else if (tmpStr.equals("alleles")) {
                        alleleIndex = totalColNum;
                    } else if (tmpStr.equals("pos")) {
                        posIndex = totalColNum;
                    }
                    totalColNum++;
                }
            }
            String[] cells = new String[totalColNum];
            int counter = 0;
            //start to read genotypes
            while ((line = br.readLine()) != null) {
                line = line.toUpperCase();
                index = 0;
                tokenizer = new StringTokenizer(line, delmilit);
                invalid = false;
                while (tokenizer.hasMoreTokens()) {
                    tmpBuffer.delete(0, tmpBuffer.length());
                    tmpBuffer.append(tokenizer.nextToken().trim());
                    cells[index] = tmpBuffer.toString();
                    if (index == posIndex) {
                        //filter physical region 
                        pos = Long.parseLong(cells[posIndex]);
                        if (needConvert) {
                            int1.setPoint((int) pos);
                            Interval int2 = liftOver.liftOver(int1);
                            //System.out.println(int2.toString());
                            if (int2 == null) {
                                invalid = true;
                                break;
                            }
                            pos = int2.getStart();
                            int2 = null;
                        }
                        if (projSnpMap.get(pos) == null) {
                            invalid = true;
                            break;
                        }
                    }
                    index++;
                }
                if (invalid) {
                    continue;
                }

                counter++;
                projSNPID.remove(cells[rsIndex].toLowerCase());
                AnnotSNP snp = snpMapList.get(pos);
                if (snp == null) {
                    cells[alleleIndex] = cells[alleleIndex];
                    snp = new AnnotSNP("rs" + cells[rsIndex].substring(2), chroNum, pos, 0.0,
                            cells[alleleIndex].charAt(0), cells[alleleIndex].charAt(2));
                    snp.order = curSNPNum;
                    snp.setStrand(strandLabel);
                    curSNPNum += 2;
                    if (curSNPNum > availabeSNPSpace) {
                        //adjust the size of bitvectors
                        availabeSNPSpace += HAP_GENOTYPE_LOAD_FACTOR;

                        for (Map.Entry<String, StatusGtySet> m1 : allIndivGty.entrySet()) {
                            m1.getValue().paternalChrom.setSize(availabeSNPSpace);
                            m1.getValue().maternalChrom.setSize(availabeSNPSpace);
                            m1.getValue().existence.setSize(availabeSNPSpace);
                        }
                    }
                    snpMapList.put(pos, snp);
                }


                aAllele = snp.getAAllele();
                bAllele = snp.getBAllele();
                for (int k = startGtyCol; k < totalColNum; k++) {
                    sigGty = cells[k];
                    // make sure the genotype is made up of two alleles. otherwise regarded it as a missing data
                    if (sigGty.length() != 2) {
                        continue;
                    }
                    String indivLabel = indiviChipLabelList.get(k - startGtyCol);
                    StatusGtySet gty = allIndivGty.get(indivLabel);

                    //re-code: A->0; B->1;
                    if (sigGty.charAt(0) == bAllele) {
                        gty.paternalChrom.putQuick(snp.order, true);
                        gty.existence.putQuick(snp.order, true);
                    } else if (sigGty.charAt(0) == aAllele) {
                        gty.existence.putQuick(snp.order, true);
                    } else if (sigGty.charAt(0) != 'N') {
                        //throw new Exception("Unexpected genotype " + sigGty + " at " + snp.getRSID() + " for " + indivLabel);
                        String unexpectInfo = "Unexpected genotype " + sigGty + " at " + snp.getRSID() + " for " + indivLabel;
                        //GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                        unexpectedSNPSet.add(snp.getRSID());
                        GlobalVariables.addInforLog(unexpectInfo);
                    }
                    if (sigGty.charAt(1) == bAllele) {
                        gty.maternalChrom.putQuick(snp.order, true);
                    } else if (sigGty.charAt(1) != aAllele && sigGty.charAt(1) != 'N') {
                        //throw new Exception("Unexpected genotype " + sigGty + " at " + snp.getRSID() + " for " + indivLabel);
                        //throw new Exception("Unexpected genotype " + sigGty + " at " + snp.getRSID() + " for " + indivLabel);
                        String unexpectInfo = "Unexpected genotype " + sigGty + " at " + snp.getRSID() + " for " + indivLabel;
                        //GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                        unexpectedSNPSet.add(snp.getRSID());
                        GlobalVariables.addInforLog(unexpectInfo);
                    }
                }
            }
            br.close();

            if (unexpectedSNPSet.size() > 0) {
                GlobalVariables.mainView.setBriefRunningInfor(unexpectedSNPSet.size() + " SNPs have unexpected genotypes in " + files.get(j));
                GlobalVariables.addInforLog(unexpectedSNPSet.size() + " SNPs have unexpected genotypes in " + files.get(j) + ":\n" + unexpectedSNPSet.toString());

                GlobalVariables.mainView.setBriefRunningInfor(unexpectedSNPSet.toString());
                unexpectedSNPSet.clear();
            }

            runningInfo.delete(0, runningInfo.length());
            runningInfo.append(files.get(j)).append(" encoded!\n");
            runningInfo.append("The number of SNPs on chromosome ");
            runningInfo.append(chroStr);
            runningInfo.append(" ");
            runningInfo.append(strRegion);
            runningInfo.append(" is ");
            runningInfo.append(counter);
            runningInfo.append(".");

            GlobalVariables.mainView.setBriefRunningInfor(runningInfo.toString(), true);
            GlobalVariables.addInforLog(runningInfo.toString());
        }

    }
}
