// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.
// Permission is granted for you to use this file to compile IGG.
// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.controller;

import cern.colt.bitvector.BitVector;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.TreeSet;
import net.sf.picard.liftover.LiftOver;
import net.sf.picard.util.Interval;
import org.cobi.igg3.entity.AnnotSNP;
import org.cobi.igg3.entity.ChromSNPGtySet;
import org.cobi.igg3.entity.Integration;
import org.cobi.igg3.entity.SNPPosiComparator;
import org.cobi.igg3.entity.StatusGtySet;
import org.cobi.igg3.gui.IGG3View;
import org.cobi.igg3.GlobalVariables;

/**
 *
 * @author mxli
 */
public class HapMapPhaseIntegrator extends BasicIntegrator {

    Integration hapIntgration;
    Integration oldIntgration;
    HashMap<String, List<HapPhaseFileSet>> chromSetFiles;
    HashMap<String, List<String>> chromFiles;
    HashMap<String, List<String>> chromPopus;
    int referenceGenomeVersion = 36;
    LiftOver liftOver;

    protected class HapPhaseFileSet {

        String sampleFile;
        String legendFile;
        String phaseFile;

        public HapPhaseFileSet(String sampleFile, String legendFile, String phaseFile) {
            this.sampleFile = sampleFile;
            this.legendFile = legendFile;
            this.phaseFile = phaseFile;
        }

        public HapPhaseFileSet() {
        }

        public String getLegendFile() {
            return legendFile;
        }

        public void setLegendFile(String legendFile) {
            this.legendFile = legendFile;
        }

        public String getPhaseFile() {
            return phaseFile;
        }

        public void setPhaseFile(String phaseFile) {
            this.phaseFile = phaseFile;
        }

        public String getSampleFile() {
            return sampleFile;
        }

        public void setSampleFile(String sampleFile) {
            this.sampleFile = sampleFile;
        }
    }

    public int getReferenceGenomeVersion() {
        return referenceGenomeVersion;
    }

    public void setReferenceGenomeVersion(int referenceGenomeVersion) {
        this.referenceGenomeVersion = referenceGenomeVersion;
        File CHAIN_FILE = null;
        if (referenceGenomeVersion == 37) {
            CHAIN_FILE = new File(RESOURCE_PATH + "chip_annotation/hg19ToHg18.over.chain");
        } else if (referenceGenomeVersion == 35) {
            CHAIN_FILE = new File(RESOURCE_PATH + "chip_annotation/hg17ToHg18.over.chain");
        }
        if (CHAIN_FILE != null) {
            liftOver = new LiftOver(CHAIN_FILE);
        }
    }

    public HapMapPhaseIntegrator(Integration hapIntgration, Integration oldIntgration) {
        this.hapIntgration = hapIntgration;
        this.oldIntgration = oldIntgration;
        this.chromSetFiles = new HashMap<String, List<HapPhaseFileSet>>();
        this.chromPopus = new HashMap<String, List<String>>();
        this.chromFiles = new HashMap<String, List<String>>();
    }

    public boolean checkPhaseSetFileCompletion(List<String> fileList, StringBuilder missingFileInfor, List<String> chroms) throws Exception {
        Map<String, long[]> regions = hapIntgration.getPhysicalRegionMap();
        if (regions == null) {
            throw new Exception("No Available Physical Region");
        }
        this.chromSetFiles.clear();
        this.chromPopus.clear();
        boolean pass = true;
        List<String> fileNames = new ArrayList<String>();
        int fileListSize = fileList.size();
        String tmpStr;
        String fileSperator = File.separator;
        for (int i = 0; i < fileListSize; i++) {
            tmpStr = fileList.get(i);
            fileNames.add(tmpStr.substring(tmpStr.lastIndexOf(fileSperator) + 1));
        }
        String[] feature = {"sample", "legend", "phase"};

        //check whether the hapmap genotype files are complete
        for (Map.Entry<String, long[]> m : regions.entrySet()) {
            tmpStr = m.getKey();
            if (tmpStr.equals("MT")) {
                tmpStr = "chrMT_";
            } else {
                tmpStr = "chr" + tmpStr + "_";
            }

            List<HapPhaseFileSet> filePath = new ArrayList<HapPhaseFileSet>();
            List<String> popus = new ArrayList<String>();
            for (int j = 0; j < HAPMAP_POPULATIONS.length; j++) {
                String[] items = new String[3];
                for (int i = 0; i < feature.length; i++) {
                    for (int k = 0; k < fileListSize; k++) {
                        String fileName = fileNames.get(k);
                        if (fileName.indexOf("JPT+CHB") >= 0 && !HAPMAP_POPULATIONS[j].equals("JPT+CHB")) {
                            continue;
                        }
                        if (fileName.indexOf(tmpStr) >= 0
                                && fileName.indexOf(HAPMAP_POPULATIONS[j]) >= 0
                                && fileName.indexOf(feature[i]) >= 0) {
                            items[i] = fileList.get(k);
                            break;
                        }
                    }
                }
                if (items[0] != null && items[1] != null && items[2] != null) {
                    HapPhaseFileSet hapFile = new HapPhaseFileSet(items[0], items[1], items[2]);
                    filePath.add(hapFile);
                    popus.add(HAPMAP_POPULATIONS[j]);
                } else if (items[0] != null || items[1] != null || items[2] != null) {
                    missingFileInfor.append("HapMap phase file set are incomplete for ").append(HAPMAP_POPULATIONS[j]).append(" on chromosme ");
                    missingFileInfor.append(tmpStr);
                    missingFileInfor.append("\n");
                    pass = false;
                }
            }

            tmpStr = tmpStr.substring(0, tmpStr.length() - 1);
            if (filePath.isEmpty()) {
                missingFileInfor.append("HapMap phase files cannot be found for ");
                missingFileInfor.append(tmpStr);
                missingFileInfor.append("\n");
                pass = false;
            } else {
                chromSetFiles.put(tmpStr, filePath);
                chromPopus.put(tmpStr, popus);
                chroms.add(m.getKey());
            }
        }

        missingFileInfor.append("HapMap phase files are found for these chromosome(s) and popualtion(s):\n");
        for (Map.Entry<String, List<String>> m : chromPopus.entrySet()) {
            String chrom = m.getKey();
            missingFileInfor.append(chrom);
            missingFileInfor.append(":");
            List<String> popus = m.getValue();
            for (int i = 0; i < popus.size(); i++) {
                missingFileInfor.append(" ");
                missingFileInfor.append(popus.get(i));
            }
            missingFileInfor.append("\n");
        }
        return pass;
    }

    public boolean checkPhaseFileCompletion(List<String> fileList, StringBuilder missingFileInfor, List<String> chroms) throws Exception {
        Map<String, long[]> regions = hapIntgration.getPhysicalRegionMap();
        if (regions == null) {
            throw new Exception("No Available Physical Region");
        }
        this.chromFiles.clear();
        this.chromPopus.clear();
        boolean pass = true;
        List<String> fileNames = new ArrayList<String>();
        int fileListSize = fileList.size();
        String tmpStr;
        String fileSperator = File.separator;
        for (int i = 0; i < fileListSize; i++) {
            tmpStr = fileList.get(i);
            fileNames.add(tmpStr.substring(tmpStr.lastIndexOf(fileSperator) + 1));
        }


        //check whether the hapmap genotype files are complete
        for (Map.Entry<String, long[]> m : regions.entrySet()) {
            tmpStr = m.getKey();
            if (tmpStr.equals("MT")) {
                tmpStr = "chrM_";
            } else {
                tmpStr = "chr" + tmpStr + "_";
            }
            tmpStr = tmpStr.toUpperCase();
            List<String> filePath = new ArrayList<String>();
            List<String> popus = new ArrayList<String>();
            for (int j = 0; j < HAPMAP_POPULATIONS.length; j++) {
                boolean found = false;
                int k = 0;
                for (k = 0; k < fileListSize; k++) {
                    String fileName = fileNames.get(k).toUpperCase();
                    if (fileName.indexOf("JPT+CHB") >= 0 && !HAPMAP_POPULATIONS[j].equals("JPT+CHB")) {
                        continue;
                    }
                    //synominous name
                    fileName = fileName.replaceFirst("CHR23_", "CHRX_");
                    if (fileName.indexOf(tmpStr) >= 0
                            && fileName.indexOf(HAPMAP_POPULATIONS[j]) >= 0) {
                        found = true;
                        break;
                    }
                }

                if (found) {
                    filePath.add(fileList.get(k));
                    popus.add(HAPMAP_POPULATIONS[j]);
                }
            }

            tmpStr = tmpStr.substring(0, tmpStr.length() - 1);
            if (filePath.isEmpty()) {
                missingFileInfor.append("HapMap phase files cannot be found for ");
                missingFileInfor.append(tmpStr);
                missingFileInfor.append("\n");
                pass = false;
            } else {
                chromFiles.put(tmpStr, filePath);
                chromPopus.put(tmpStr, popus);
                chroms.add(m.getKey());
            }
        }

        missingFileInfor.append("HapMap phase files are found for these chromosome(s) and popualtion(s):\n");
        for (Map.Entry<String, List<String>> m : chromPopus.entrySet()) {
            String chrom = m.getKey();
            missingFileInfor.append(chrom);
            missingFileInfor.append(":");
            List<String> popus = m.getValue();
            for (int i = 0; i < popus.size(); i++) {
                missingFileInfor.append(" ");
                missingFileInfor.append(popus.get(i));
            }
            missingFileInfor.append("\n");
        }
        return pass;
    }

    public void integrate(int hapV, boolean matchRegion, boolean isForward) throws Exception {
        Map<Long, AnnotSNP> hapSnpMap = new HashMap<Long, AnnotSNP>();
        List<AnnotSNP> hapSnpList = new ArrayList<AnnotSNP>();
        Map<String, StatusGtySet> hapIndivGty = new HashMap<String, StatusGtySet>();
        List<AnnotSNP> projSnpList = null;

        Map<String, StatusGtySet> projIndivGty;
        ChromSNPGtySet chromSet = new ChromSNPGtySet();
        String runningInfo = "";
        TreeSet<String> enteredHapIndiv = new TreeSet<String>();
        for (int i = 0; i < CHROM_NAME.length; i++) {
            //denote version r22 or earlier
            if (hapV <= 22) {
                List<HapPhaseFileSet> files = chromSetFiles.get("chr" + CHROM_NAME[i]);
                if (files == null) {
                    continue;
                }
                hapSnpMap.clear();
                hapIndivGty.clear();

                chromSet.setChromName(CHROM_NAME[i]);
                if (matchRegion) {
                    readHapMapPhaseOldbyChromosomeRegion(CHROM_NAME[i], files, hapSnpMap, hapIndivGty, enteredHapIndiv, isForward);
                    addDbSNPFlankingSequenceInComplemenetaryHapMapSNP(hapSnpMap, CHROM_NAME[i]);
                    chromSet.readChromSNPInfo(oldIntgration.getResultPath());
                    chromSet.readNextGtyBlock(oldIntgration.getResultPath());
                    projSnpList = chromSet.getSnpInfoList();

                } else {
                    chromSet.readChromSNPInfo(oldIntgration.getResultPath());
                    chromSet.readNextGtyBlock(oldIntgration.getResultPath());
                    projSnpList = chromSet.getSnpInfoList();
                    Map<Long, Boolean> projSnpMap = new HashMap<Long, Boolean>(projSnpList.size(), 0.9f);
                    TreeSet<String> projSNPID = new TreeSet<String>();
                    for (int j = projSnpList.size() - 1; j >= 0; j--) {
                        projSnpMap.put(projSnpList.get(j).getPhysicalPosition(), true);
                        projSNPID.add(projSnpList.get(j).getRSID().toLowerCase());
                    }
                    readHapMapPhaseOldbySNP(CHROM_NAME[i], files, projSNPID, projSnpMap, hapSnpMap, hapIndivGty, enteredHapIndiv, isForward);

                    projSnpMap.clear();
                    StringBuilder info = new StringBuilder();
                    info.append("The following ");
                    info.append(projSNPID.size());
                    info.append(" SNPs have no genotypes in HapMap:\n");
                    info.append(projSNPID.toString());
                    projSNPID.clear();
                    GlobalVariables.mainView.setBriefRunningInfor(info.toString(), true);
                    GlobalVariables.addInforLog(info.toString());
                    addDbSNPFlankingSequenceInComplemenetaryHapMapSNP(hapSnpMap, CHROM_NAME[i]);
                }
            } else {
                List<String> files = chromFiles.get("CHR" + CHROM_NAME[i]);
                if (files == null) {
                    continue;
                }
                hapSnpMap.clear();
                hapIndivGty.clear();

                chromSet.setChromName(CHROM_NAME[i]);
                if (matchRegion) {
                    readHapMapPhasedNewbyChromosomeRegion(CHROM_NAME[i], files, hapSnpMap, hapIndivGty, enteredHapIndiv, isForward);
                    addDbSNPFlankingSequenceInComplemenetaryHapMapSNP(hapSnpMap, CHROM_NAME[i]);
                    chromSet.readChromSNPInfo(oldIntgration.getResultPath());
                    chromSet.readNextGtyBlock(oldIntgration.getResultPath());
                    projSnpList = chromSet.getSnpInfoList();
                } else {
                    chromSet.readChromSNPInfo(oldIntgration.getResultPath());
                    chromSet.readNextGtyBlock(oldIntgration.getResultPath());
                    projSnpList = chromSet.getSnpInfoList();
                    Map<Long, Boolean> projSnpMap = new HashMap<Long, Boolean>(projSnpList.size(), 0.9f);
                    TreeSet<String> projSNPID = new TreeSet<String>();
                    for (int j = projSnpList.size() - 1; j >= 0; j--) {
                        projSnpMap.put(projSnpList.get(j).getPhysicalPosition(), true);
                        projSNPID.add(projSnpList.get(j).getRSID().toLowerCase());
                    }
                    readHapMapPhasedNewbySNP(CHROM_NAME[i], files, projSNPID, projSnpMap, hapSnpMap, hapIndivGty, enteredHapIndiv, isForward);
                    projSnpMap.clear();
                    StringBuilder info = new StringBuilder();
                    info.append("The following ");
                    info.append(projSNPID.size());
                    info.append(" SNPs have no genotypes in HapMap:\n");
                    info.append(projSNPID.toString());
                    projSNPID.clear();
                    GlobalVariables.mainView.setBriefRunningInfor(info.toString(), true);
                    GlobalVariables.addInforLog(info.toString());
                    addDbSNPFlankingSequenceInComplemenetaryHapMapSNP(hapSnpMap, CHROM_NAME[i]);
                }
            }


            projIndivGty = chromSet.getIndiviGty();
            hapSnpList.clear();
            hapSnpList.addAll(hapSnpMap.values());
            hapSnpMap.clear();

            Collections.sort(hapSnpList, new SNPPosiComparator());
            System.gc();

            runningInfo = mergeSNPGtybyListNoGtyExtend(projSnpList, hapSnpList, projIndivGty, hapIndivGty);
            hapSnpList.clear();
            hapIndivGty.clear();
            hapIntgration.addAChromResult(CHROM_NAME[i] + " SNP#" + projSnpList.size());

            File resultPath = new File(hapIntgration.getResultPath() + File.separator);
            if (!resultPath.exists()) {
                resultPath.mkdir();
            }

            chromSet.saveChromSNPInforAndClear(resultPath.getCanonicalPath());
            projSnpList.clear();
            chromSet.saveGtyBlock(resultPath.getCanonicalPath());
            projIndivGty.clear();

            // save other geneotype while assuming the hapmap individual labels do not exsit in the project ones
            while (chromSet.hasMoreGtyBlock()) {
                chromSet.readNextGtyBlock(oldIntgration.getResultPath());
                chromSet.saveGtyBlock(resultPath.getCanonicalPath());
                chromSet.setIndiviGty(null);
                System.gc();
                System.runFinalization();
            }

            GlobalVariables.mainView.setBriefRunningInfor(runningInfo, true);
            GlobalVariables.addInforLog(runningInfo);
        }
        hapIntgration.setHapIndivLabels(enteredHapIndiv);

        //directly copy the common pedigree structure into the folder with hapmap data
        File commPedFile = new File(oldIntgration.getResultPath() + File.separator + "ped");
        if (commPedFile.exists()) {
            // Create channel on the source
            FileChannel srcChannel = new FileInputStream(oldIntgration.getResultPath() + File.separator + "ped").getChannel();
            // Create channel on the destination
            FileChannel dstChannel = new FileOutputStream(hapIntgration.getResultPath() + File.separator + "ped").getChannel();
            // Copy file contents from source to destination
            dstChannel.transferFrom(srcChannel, 0, srcChannel.size());
            // Close the channels
            srcChannel.close();
            dstChannel.close();
        }

    }

    public void readHapMapPhasedNewbyChromosomeRegion(String chroStr, List<String> files, Map<Long, AnnotSNP> snpMapList,
            Map<String, StatusGtySet> allIndivGty, TreeSet<String> enteredHapIndiv, boolean isForward) throws Exception {
        int chroNum;
        StringBuilder strRegion = new StringBuilder();
        String line = null;
        StringTokenizer tokenizer = null;
        String delmilit = "\t ,";

        StringBuilder runningInfo = new StringBuilder();
        TreeSet<String> unexpectedSNPSet = new TreeSet<String>();
        int totalColNum = -1;
        int startGtyCol = -1;
        int rsIndex = -1;
        int posIndex = -1;

//sometimes the rsid in the HapMap III has the format like ASNP_A-1809060
        Map<String, String> mappedSNPIDsInHapMap = new HashMap<String, String>();
        mappedSNPIDsInHapMap.put("SNP_A-1809060", "rs41513046");
        mappedSNPIDsInHapMap.put("SNP_A-1856196", "rs56026748");
        mappedSNPIDsInHapMap.put("SNP_A-1864388", "rs55680733");
        mappedSNPIDsInHapMap.put("SNP_A-1965574", "rs41451553");
        mappedSNPIDsInHapMap.put("SNP_A-2074420", "rs56272869");
        mappedSNPIDsInHapMap.put("SNP_A-2097957", "rs760737");
        mappedSNPIDsInHapMap.put("SNP_A-2137777", "rs41450445");
        mappedSNPIDsInHapMap.put("SNP_A-2144818", "rs12635398");
        mappedSNPIDsInHapMap.put("SNP_A-2164680", "rs33943415");
        mappedSNPIDsInHapMap.put("SNP_A-2217336", "rs41413150");
        mappedSNPIDsInHapMap.put("SNP_A-2218168", "rs41346346");
        mappedSNPIDsInHapMap.put("SNP_A-2280133", "rs3092957");
        mappedSNPIDsInHapMap.put("SNP_A-2290639", "rs55704340");


        int availabeSNPSpace = 100000;
        List<String> indiviChipLabelList = new ArrayList<String>();
        String tmpStr;
        StringBuilder tmpBuffer = new StringBuilder();
        char strandLabel = '+';
        if (!isForward) {
            strandLabel = '-';
        }


        if (chroStr.equalsIgnoreCase("X")) {
            chroNum = 23;
        } else if (chroStr.equalsIgnoreCase("Y")) {
            chroNum = 24;
        } else if (chroStr.equalsIgnoreCase("MT")) {
            chroNum = 26;
        } else if (chroStr.equalsIgnoreCase("XY")) {
            chroNum = 25;
        } else {
            chroNum = Integer.parseInt(chroStr);
        }

        boolean needConvert = false;
        if (referenceGenomeVersion == 37 || referenceGenomeVersion == 35) {
            needConvert = true;
        }
        Interval int1 = new Interval("chr" + chroStr, 0, 0);

        long[] physicalRegions = hapIntgration.getPhysicalRegionMap().get(chroStr);
        int regionNum = physicalRegions.length / 2;
        boolean checkRegion = false;
        int index = 0;
        long pos = 0;

        boolean invalid = true;
        int curSNPNum = 0;
        boolean isInRegion = false;
        char aAllele, bAllele;
        boolean noABAllele = false;
        for (int j = 0; j < regionNum; j++) {
            if (physicalRegions[j * 2] != -9 || physicalRegions[j * 2 + 1] != -9) {
                checkRegion = true;
                if (physicalRegions[j * 2 + 1] == -9) {
                    physicalRegions[1] = Long.MAX_VALUE;
                }
                strRegion.append(" [");
                strRegion.append(physicalRegions[j * 2]);
                strRegion.append(",");
                strRegion.append(physicalRegions[j * 2 + 1]);
                strRegion.append("]bp");
            }
        }

        for (int j = 0; j < files.size(); j++) {
            String info = "Encoding HapMap File " + files.get(j) + " ...";
            GlobalVariables.mainView.setBriefRunningInfor(info);
            GlobalVariables.addInforLog(info);
            BufferedReader br = new BufferedReader(new FileReader(files.get(j)), FILE_READER_BUFFER_SIZE);

            //synominous name
            String tmpFileName = files.get(j).toUpperCase();
            tmpFileName = tmpFileName.replaceFirst("CHR23_", "CHRX_");
            boolean isXChrom = tmpFileName.indexOf("CHRX_") >= 0;
            //read individual label 
            if ((line = br.readLine()) != null) {
                tokenizer = new StringTokenizer(line, delmilit);
                totalColNum = 0;
                startGtyCol = -1;
                rsIndex = -1;
                posIndex = -1;
                indiviChipLabelList.clear();
                while (tokenizer.hasMoreTokens()) {
                    tmpBuffer.delete(0, tmpBuffer.length());
                    tmpBuffer.append(tokenizer.nextToken().trim());
                    tmpStr = tmpBuffer.toString();
                    if (tmpStr.startsWith("NA")) {
                        if (startGtyCol < 0) {
                            startGtyCol = totalColNum;
                        }
                        //some HapMap file has duplicated sample
                        int awfulIndex = tmpStr.indexOf(".");
                        if (awfulIndex >= 0) {
                            tmpStr = tmpStr.substring(0, awfulIndex);
                        }
                        tmpStr = tmpStr.substring(0, tmpStr.length() - 2);
                        if (allIndivGty.get(tmpStr) == null) {
                            StatusGtySet genotypes = new StatusGtySet();
                            genotypes.existence = new BitVector(availabeSNPSpace);
                            genotypes.paternalChrom = new BitVector(availabeSNPSpace);
                            genotypes.maternalChrom = new BitVector(availabeSNPSpace);
                            allIndivGty.put(tmpStr, genotypes);
                        }
                        indiviChipLabelList.add(tmpStr);
                        if (!enteredHapIndiv.contains(tmpStr)) {
                            enteredHapIndiv.add(tmpStr);
                        }
                    } else if (tmpStr.equals("rsID")) {
                        rsIndex = totalColNum;
                    } else if (tmpStr.indexOf("position") >= 0) {
                        posIndex = totalColNum;
                    }
                    totalColNum++;
                }
            }
            String[] cells = new String[totalColNum];
            int counter = 0;
            AnnotSNP snp = null;
            //start to read genotypes
            while ((line = br.readLine()) != null) {
                line = line.toUpperCase();
                index = 0;
                tokenizer = new StringTokenizer(line, delmilit);
                invalid = false;
                aAllele = MISSING_ALLELE_NAME;
                bAllele = MISSING_ALLELE_NAME;
                noABAllele = true;
                isInRegion = false;

                //decide allele names
                while (tokenizer.hasMoreTokens()) {
                    tmpBuffer.delete(0, tmpBuffer.length());
                    tmpBuffer.append(tokenizer.nextToken().trim());
                    cells[index] = tmpBuffer.toString();
                    if (index == posIndex) {
                        //filter physical region                       
                        pos = Long.parseLong(cells[posIndex]);
                        if (needConvert) {
                            int1.setPoint((int) pos);
                            Interval int2 = liftOver.liftOver(int1);
                            //System.out.println(int2.toString());
                            if (int2 == null) {
                                invalid = true;
                                break;
                            }
                            pos = int2.getStart();
                            int2 = null;
                        }
                        if (checkRegion) {
                            for (int k = 0; k < regionNum; k++) {
                                if (pos >= physicalRegions[k * 2] && pos <= physicalRegions[k * 2 + 1]) {
                                    isInRegion = true;
                                    break;
                                }
                            }
                        }
                        snp = snpMapList.get(pos);
                        if (snp != null) {
                            aAllele = snp.getAAllele();
                            bAllele = snp.getBAllele();
                            if (aAllele != MISSING_ALLELE_NAME && bAllele != MISSING_ALLELE_NAME) {
                                noABAllele = false;
                            }
                        }
                    }

                    //- denote unavailble alleles in HapMap
                    if (noABAllele && cells[index].charAt(0) != '-') {
                        if (index >= startGtyCol) {
                            if (aAllele == MISSING_ALLELE_NAME) {
                                aAllele = cells[index].charAt(0);
                                if (snp != null) {
                                    snp.setAAllele(aAllele);
                                }
                            } else if (bAllele == MISSING_ALLELE_NAME && aAllele != cells[index].charAt(0)) {
                                bAllele = cells[index].charAt(0);
                                if (snp != null) {
                                    snp.setBAllele(bAllele);
                                }
                                noABAllele = false;
                            }
                        }
                    }
                    index++;
                }
                if (invalid) {
                    continue;
                }

                if (checkRegion && !isInRegion) {
                    continue;
                }

                counter++;
                if (snp == null) {
                    //sometimes the rsid in the HapMap III has the format like AFFX-SNP_9976462__rs2810434
                    String rsIds = cells[rsIndex];
                    if (rsIds.startsWith("AFFX")) {
                        int indexTmp = rsIds.indexOf("RS");
                        if (indexTmp >= 0) {
                            rsIds = rsIds.substring(indexTmp);
                        }
                    } else if (rsIds.startsWith("SNP_A")) {
                        rsIds = mappedSNPIDsInHapMap.get(rsIds);
                        if (rsIds == null) {
                            continue;
                        }
                    }
                    snp = new AnnotSNP("rs" + rsIds.substring(2), chroNum, pos, 0.0, aAllele, bAllele);
                    snp.setStrand(strandLabel);
                    snp.order = curSNPNum;
                    curSNPNum++;
                    if (curSNPNum > availabeSNPSpace) {
                        //adjust the size of bitvectors
                        availabeSNPSpace += HAP_GENOTYPE_LOAD_FACTOR;
                        for (Map.Entry<String, StatusGtySet> m1 : allIndivGty.entrySet()) {
                            m1.getValue().paternalChrom.setSize(availabeSNPSpace);
                            m1.getValue().maternalChrom.setSize(availabeSNPSpace);
                            m1.getValue().existence.setSize(availabeSNPSpace);
                        }
                    }
                    snpMapList.put(pos, snp);
                }


                aAllele = snp.getAAllele();
                bAllele = snp.getBAllele();

                for (int k = startGtyCol; k < totalColNum; k += 2) {
                    // make sure the genotype is made up of two alleles. otherwise regarded it as a missing data                  
                    String indivLabel = indiviChipLabelList.get((k - startGtyCol));
                    StatusGtySet gty = allIndivGty.get(indivLabel);

                    //re-code: A->0; B->1;
                    if (cells[k].charAt(0) == bAllele) {
                        gty.paternalChrom.putQuick(snp.order, true);
                        gty.existence.putQuick(snp.order, true);
                    } else if (cells[k].charAt(0) == aAllele) {
                        gty.existence.putQuick(snp.order, true);
                    } else if (cells[k].charAt(0) != 'N' && cells[k].charAt(0) != '0') {
                        // throw new Exception("Unexpected genotype " + cells[k] + " at " + snp.getRSID() + " for " + indivLabel);
                        String unexpectInfo = "Unexpected genotype " + cells[k] + " at " + snp.getRSID() + " for " + indivLabel;
                        //GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                        GlobalVariables.addInforLog(unexpectInfo);
                        unexpectedSNPSet.add(snp.getRSID());
                    }

                    //must be a male indiviudal; the first alleles are always an allele name and the following one is - for males
                    if (isXChrom && cells[k + 1].charAt(0) == '-') {
                        cells[k + 1] = cells[k];
                    }

                    if (cells[k + 1].charAt(0) == bAllele) {
                        gty.maternalChrom.putQuick(snp.order, true);
                    } else if (cells[k + 1].charAt(0) != aAllele && cells[k + 1].charAt(0) != 'N' && cells[k + 1].charAt(0) != '0') {
                        //throw new Exception("Unexpected genotype " + cells[k + 1] + " at " + snp.getRSID() + " for " + indivLabel);
                        String unexpectInfo = "Unexpected genotype " + cells[k + 1] + " at " + snp.getRSID() + " for " + indivLabel;
                        //GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                        GlobalVariables.addInforLog(unexpectInfo);
                        unexpectedSNPSet.add(snp.getRSID());
                    }
                }
            }
            br.close();
            if (unexpectedSNPSet.size() > 0) {
                GlobalVariables.mainView.setBriefRunningInfor(unexpectedSNPSet.size() + " SNPs have unexpected genotypes in " + files.get(j));
                GlobalVariables.addInforLog(unexpectedSNPSet.size() + " SNPs have unexpected genotypes in " + files.get(j) + ":\n" + unexpectedSNPSet.toString());
                GlobalVariables.mainView.setBriefRunningInfor(unexpectedSNPSet.toString());
                unexpectedSNPSet.clear();
            }
            runningInfo.delete(0, runningInfo.length());
            runningInfo.append(files.get(j)).append(" encoded!\n");
            runningInfo.append("The number of SNPs on chromosome ");
            runningInfo.append(chroStr);
            runningInfo.append(" ");
            runningInfo.append(strRegion);
            runningInfo.append(" is ");
            runningInfo.append(counter);
            runningInfo.append(".");

            GlobalVariables.mainView.setBriefRunningInfor(runningInfo.toString(), true);
            GlobalVariables.addInforLog(runningInfo.toString());
        }

    }

    public void readHapMapPhasedNewbySNP(String chroStr, List<String> files, TreeSet<String> projSNPID, Map<Long, Boolean> projSnpMap, Map<Long, AnnotSNP> snpMapList,
            Map<String, StatusGtySet> allIndivGty, TreeSet<String> enteredHapIndiv, boolean isForward) throws Exception {
        int chroNum;
        StringBuilder strRegion = new StringBuilder();
        String line = null;
        StringTokenizer tokenizer = null;
        String delmilit = "\t ,";

        StringBuilder runningInfo = new StringBuilder();
        int totalColNum = -1;
        int startGtyCol = -1;
        int rsIndex = -1;
        int posIndex = -1;

        TreeSet<String> unexpectedSNPSet = new TreeSet<String>();
        int availabeSNPSpace = 100000;
        List<String> indiviChipLabelList = new ArrayList<String>();
        String tmpStr;

        char strandLabel = '+';
        if (!isForward) {
            strandLabel = '-';
        }

        boolean needConvert = false;
        if (referenceGenomeVersion == 37 || referenceGenomeVersion == 35) {
            needConvert = true;
        }
        Interval int1 = new Interval("chr" + chroStr, 0, 0);
        if (chroStr.equalsIgnoreCase("X")) {
            chroNum = 23;
        } else if (chroStr.equalsIgnoreCase("Y")) {
            chroNum = 24;
        } else if (chroStr.equalsIgnoreCase("MT")) {
            chroNum = 26;
        } else if (chroStr.equalsIgnoreCase("XY")) {
            chroNum = 25;
        } else {
            chroNum = Integer.parseInt(chroStr);
        }
        int index = 0;
        long pos = 0;

        boolean invalid = true;
        int curSNPNum = 0;
//sometimes the rsid in the HapMap III has the format like ASNP_A-1809060
        Map<String, String> mappedSNPIDsInHapMap = new HashMap<String, String>();
        mappedSNPIDsInHapMap.put("SNP_A-1809060", "rs41513046");
        mappedSNPIDsInHapMap.put("SNP_A-1856196", "rs56026748");
        mappedSNPIDsInHapMap.put("SNP_A-1864388", "rs55680733");
        mappedSNPIDsInHapMap.put("SNP_A-1965574", "rs41451553");
        mappedSNPIDsInHapMap.put("SNP_A-2074420", "rs56272869");
        mappedSNPIDsInHapMap.put("SNP_A-2097957", "rs760737");
        mappedSNPIDsInHapMap.put("SNP_A-2137777", "rs41450445");
        mappedSNPIDsInHapMap.put("SNP_A-2144818", "rs12635398");
        mappedSNPIDsInHapMap.put("SNP_A-2164680", "rs33943415");
        mappedSNPIDsInHapMap.put("SNP_A-2217336", "rs41413150");
        mappedSNPIDsInHapMap.put("SNP_A-2218168", "rs41346346");
        mappedSNPIDsInHapMap.put("SNP_A-2280133", "rs3092957");
        mappedSNPIDsInHapMap.put("SNP_A-2290639", "rs55704340");


        char aAllele, bAllele;
        boolean noABAllele = false;
        StringBuilder tmpBuffer = new StringBuilder();

        for (int j = 0; j < files.size(); j++) {
            String info = "Encoding HapMap File " + files.get(j) + " ...";
            GlobalVariables.mainView.setBriefRunningInfor(info);
            GlobalVariables.addInforLog(info);
            BufferedReader br = new BufferedReader(new FileReader(files.get(j)), FILE_READER_BUFFER_SIZE);

            //synominous name
            String tmpFileName = files.get(j).toUpperCase();
            tmpFileName = tmpFileName.replaceFirst("CHR23_", "CHRX_");
            boolean isXChrom = tmpFileName.indexOf("CHRX_") >= 0;


            //read individual label 
            if ((line = br.readLine()) != null) {
                tokenizer = new StringTokenizer(line, delmilit);
                totalColNum = 0;
                startGtyCol = -1;
                rsIndex = -1;
                posIndex = -1;
                indiviChipLabelList.clear();
                while (tokenizer.hasMoreTokens()) {
                    tmpBuffer.delete(0, tmpBuffer.length());
                    tmpBuffer.append(tokenizer.nextToken().trim());
                    tmpStr = tmpBuffer.toString();
                    if (tmpStr.startsWith("NA")) {
                        if (startGtyCol < 0) {
                            startGtyCol = totalColNum;
                        }
                        //some HapMap file has duplicated sample
                        int awfulIndex = tmpStr.indexOf(".");
                        if (awfulIndex >= 0) {
                            tmpStr = tmpStr.substring(0, awfulIndex);
                        }
                        tmpStr = tmpStr.substring(0, tmpStr.length() - 2);
                        if (allIndivGty.get(tmpStr) == null) {
                            StatusGtySet genotypes = new StatusGtySet();
                            genotypes.existence = new BitVector(availabeSNPSpace);
                            genotypes.paternalChrom = new BitVector(availabeSNPSpace);
                            genotypes.maternalChrom = new BitVector(availabeSNPSpace);
                            allIndivGty.put(tmpStr, genotypes);
                        }
                        indiviChipLabelList.add(tmpStr);
                        if (!enteredHapIndiv.contains(tmpStr)) {
                            enteredHapIndiv.add(tmpStr);
                        }
                    } else if (tmpStr.equals("rsID")) {
                        rsIndex = totalColNum;
                    } else if (tmpStr.indexOf("position") >= 0) {
                        posIndex = totalColNum;
                    }
                    totalColNum++;
                }
            }
            String[] cells = new String[totalColNum];
            int counter = 0;
            AnnotSNP snp = null;
            //start to read genotypes
            while ((line = br.readLine()) != null) {
                line = line.toUpperCase();
                index = 0;
                tokenizer = new StringTokenizer(line, delmilit);
                invalid = false;
                aAllele = MISSING_ALLELE_NAME;
                bAllele = MISSING_ALLELE_NAME;
                noABAllele = true;
                while (tokenizer.hasMoreTokens()) {
                    tmpBuffer.delete(0, tmpBuffer.length());
                    tmpBuffer.append(tokenizer.nextToken().trim());
                    cells[index] = tmpBuffer.toString();
                    if (index == posIndex) {
                        //filter physical region                        
                        pos = Long.parseLong(cells[posIndex]);
                        if (needConvert) {
                            int1.setPoint((int) pos);
                            Interval int2 = liftOver.liftOver(int1);
                            //System.out.println(int2.toString());
                            if (int2 == null) {
                                invalid = true;
                                break;
                            }
                            pos = int2.getStart();
                            int2 = null;
                        }

                        if (projSnpMap.get(pos) == null) {
                            invalid = true;
                            break;
                        }
                        snp = snpMapList.get(pos);
                        if (snp != null) {
                            aAllele = snp.getAAllele();
                            bAllele = snp.getBAllele();
                            if (aAllele != MISSING_ALLELE_NAME && bAllele != MISSING_ALLELE_NAME) {
                                noABAllele = false;
                            }
                        }
                    }
                    if (noABAllele && cells[index].charAt(0) != '-') {
                        if (index >= startGtyCol) {
                            if (aAllele == MISSING_ALLELE_NAME) {
                                aAllele = cells[index].charAt(0);
                            } else if (bAllele == MISSING_ALLELE_NAME && aAllele != cells[index].charAt(0)) {
                                bAllele = cells[index].charAt(0);
                                noABAllele = false;
                            }
                        }
                    }
                    index++;
                }
                if (invalid) {
                    continue;
                }
                //sometimes the rsid in the HapMap III has the format like AFFX-SNP_9976462__rs2810434
                String rsIds = cells[rsIndex];
                if (rsIds.startsWith("AFFX")) {
                    int indexTmp = rsIds.indexOf("RS");
                    if (indexTmp >= 0) {
                        rsIds = rsIds.substring(indexTmp);
                    }
                } else if (rsIds.startsWith("SNP_A")) {
                    rsIds = mappedSNPIDsInHapMap.get(rsIds);
                    if (rsIds == null) {
                        continue;
                    }
                }

                projSNPID.remove(rsIds.toLowerCase());
                counter++;
                if (snp == null) {
                    snp = new AnnotSNP("rs" + rsIds.substring(2), chroNum, pos, 0.0, aAllele, bAllele);
                    snp.setStrand(strandLabel);
                    snp.order = curSNPNum;
                    curSNPNum++;
                    if (curSNPNum > availabeSNPSpace) {
                        //adjust the size of bitvectors
                        availabeSNPSpace += HAP_GENOTYPE_LOAD_FACTOR;
                        for (Map.Entry<String, StatusGtySet> m1 : allIndivGty.entrySet()) {
                            m1.getValue().paternalChrom.setSize(availabeSNPSpace);
                            m1.getValue().maternalChrom.setSize(availabeSNPSpace);
                            m1.getValue().existence.setSize(availabeSNPSpace);
                        }
                    }
                    snpMapList.put(pos, snp);
                }


                aAllele = snp.getAAllele();
                bAllele = snp.getBAllele();
                for (int k = startGtyCol; k < totalColNum; k += 2) {
                    // make sure the genotype is made up of two alleles. otherwise regarded it as a missing data                  
                    String indivLabel = indiviChipLabelList.get((k - startGtyCol));
                    StatusGtySet gty = allIndivGty.get(indivLabel);

                    //re-code: A->0; B->1;
                    if (cells[k].charAt(0) == bAllele) {
                        gty.paternalChrom.putQuick(snp.order, true);
                        gty.existence.putQuick(snp.order, true);
                    } else if (cells[k].charAt(0) == aAllele) {
                        gty.existence.putQuick(snp.order, true);
                    } else if (cells[k].charAt(0) != 'N' && cells[k].charAt(0) != '0') {
                        //throw new Exception("Unexpected genotype " + cells[k] + " at " + snp.getRSID() + " for " + indivLabel);
                        String unexpectInfo = "Unexpected genotype " + cells[k] + " at " + snp.getRSID() + " for " + indivLabel;
                        //GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                        GlobalVariables.addInforLog(unexpectInfo);
                        unexpectedSNPSet.add(snp.getRSID());
                    }
                    //must be a male indiviudal; the first alleles are always an allele name and the following one is - for males
                    if (isXChrom && cells[k + 1].charAt(0) == '-') {
                        cells[k + 1] = cells[k];
                    }
                    if (cells[k + 1].charAt(0) == bAllele) {
                        gty.maternalChrom.putQuick(snp.order, true);
                    } else if (cells[k + 1].charAt(0) != aAllele && cells[k + 1].charAt(0) != 'N' && cells[k + 1].charAt(0) != '0') {
                        //throw new Exception("Unexpected genotype " + cells[k + 1] + " at " + snp.getRSID() + " for " + indivLabel);
                        String unexpectInfo = "Unexpected genotype " + cells[k + 1] + " at " + snp.getRSID() + " for " + indivLabel;
                        GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                        GlobalVariables.addInforLog(unexpectInfo);
                        unexpectedSNPSet.add(snp.getRSID());
                    }
                }
            }
            br.close();
            if (unexpectedSNPSet.size() > 0) {
                GlobalVariables.mainView.setBriefRunningInfor(unexpectedSNPSet.size() + " SNPs have unexpected genotypes in " + files.get(j));
                GlobalVariables.addInforLog(unexpectedSNPSet.size() + " SNPs have unexpected genotypes in " + files.get(j) + ":\n" + unexpectedSNPSet.toString());
                GlobalVariables.mainView.setBriefRunningInfor(unexpectedSNPSet.toString());
                unexpectedSNPSet.clear();
            }
            runningInfo.delete(0, runningInfo.length());
            runningInfo.append(files.get(j)).append(" encoded!\n");
            runningInfo.append("The number of SNPs on chromosome ");
            runningInfo.append(chroStr);
            runningInfo.append(" ");
            runningInfo.append(strRegion);
            runningInfo.append(" is ");
            runningInfo.append(counter);
            runningInfo.append(".");

            GlobalVariables.mainView.setBriefRunningInfor(runningInfo.toString(), true);
            GlobalVariables.addInforLog(runningInfo.toString());
        }

    }

    public void readHapMapPhaseOldbyChromosomeRegion(String chroStr, List<HapPhaseFileSet> files, Map<Long, AnnotSNP> snpMapList,
            Map<String, StatusGtySet> allIndivGty, TreeSet<String> enteredHapIndiv, boolean isForward) throws Exception {
        int chroNum;
        StringBuilder strRegion = new StringBuilder();
        String line = null;
        StringTokenizer tokenizer = null;
        String delmilit = "\t ,";

        StringBuilder runningInfo = new StringBuilder();
        int totalColNum = 4;
        String[] cells = new String[totalColNum];
        int rsIndex = 0;
        int alleleAIndex = 2;
        int alleleBIndex = 3;
        int posIndex = 1;

        int availabeSNPSpace = 100000;
        char strandLabel = '+';
        if (!isForward) {
            strandLabel = '-';
        }
        if (chroStr.equalsIgnoreCase("X")) {
            chroNum = 23;
        } else if (chroStr.equalsIgnoreCase("Y")) {
            chroNum = 24;
        } else if (chroStr.equalsIgnoreCase("MT")) {
            chroNum = 26;
        } else if (chroStr.equalsIgnoreCase("XY")) {
            chroNum = 25;
        } else {
            chroNum = Integer.parseInt(chroStr);
        }
        boolean needConvert = false;
        if (referenceGenomeVersion == 37 || referenceGenomeVersion == 35) {
            needConvert = true;
        }
        Interval int1 = new Interval("chr" + chroStr, 0, 0);

        long[] physicalRegions = hapIntgration.getPhysicalRegionMap().get(chroStr);
        int regionNum = physicalRegions.length / 2;
        boolean checkRegion = false;
        int index = 0;
        long pos = 0;

        boolean invalid = true;
        int curSNPNum = 0;
        int lineCounter = -1;
        boolean isInRegion = false;
        Map<Integer, Long> snpPosInFile = new HashMap<Integer, Long>();
        String indivLabel;
        String phase1, phase2;

        for (int j = 0; j < regionNum; j++) {
            if (physicalRegions[j * 2] != -9 || physicalRegions[j * 2 + 1] != -9) {
                checkRegion = true;
                if (physicalRegions[j * 2 + 1] == -9) {
                    physicalRegions[1] = Long.MAX_VALUE;
                }
                strRegion.append(" [");
                strRegion.append(physicalRegions[0]);
                strRegion.append(",");
                strRegion.append(physicalRegions[1]);
                strRegion.append("]bp");
            }
        }
        StringBuilder tmpBuffer = new StringBuilder();
        for (int j = 0; j < files.size(); j++) {
            BufferedReader brLegend = new BufferedReader(new FileReader(files.get(j).getLegendFile()), FILE_READER_BUFFER_SIZE);
            //skip the first line
            brLegend.readLine();
            lineCounter = -1;
            snpPosInFile.clear();
            //start to read genotypes
            while ((line = brLegend.readLine()) != null) {
                line = line.toUpperCase();
                lineCounter++;
                index = 0;
                tokenizer = new StringTokenizer(line, delmilit);
                invalid = false;
                isInRegion = false;
                while (tokenizer.hasMoreTokens()) {
                    tmpBuffer.delete(0, tmpBuffer.length());
                    tmpBuffer.append(tokenizer.nextToken().trim());
                    cells[index] = tmpBuffer.toString();
                    if (index == posIndex) {
                        //filter physical region                        
                        pos = Long.parseLong(cells[posIndex]);
                        if (needConvert) {
                            int1.setPoint((int) pos);
                            Interval int2 = liftOver.liftOver(int1);
                            //System.out.println(int2.toString());
                            if (int2 == null) {
                                invalid = true;
                                break;
                            }
                            pos = int2.getStart();
                            int2 = null;
                        }
                        if (checkRegion) {
                            for (int k = 0; k < regionNum; k++) {
                                if (pos >= physicalRegions[k * 2] && pos <= physicalRegions[k * 2 + 1]) {
                                    isInRegion = true;
                                    break;
                                }
                            }
                        }
                    }
                    index++;
                }
                if (invalid) {
                    continue;
                }

                if (checkRegion && !isInRegion) {
                    continue;
                }

                AnnotSNP snp = snpMapList.get(pos);

                if (snp == null) {
                    snp = new AnnotSNP("rs" + cells[rsIndex].substring(2), chroNum, pos, 0.0,
                            cells[alleleAIndex].charAt(0), cells[alleleBIndex].charAt(0));
                    snp.order = curSNPNum;
                    snp.setStrand(strandLabel);
                    curSNPNum++;
                    snpMapList.put(pos, snp);
                } else {

                    if (cells[alleleAIndex].charAt(0) != snp.getAAllele()
                            || cells[alleleBIndex].charAt(0) != snp.getBAllele()) {
                        throw new Exception("Inconsistant strand for SNP " + snp.getRSID() + " in file ");
                    }

                }
                snpPosInFile.put(lineCounter, pos);
            }
            brLegend.close();

            if (curSNPNum > availabeSNPSpace) {
                //adjust the size of bitvectors
                while (curSNPNum > availabeSNPSpace) {
                    availabeSNPSpace += HAP_GENOTYPE_LOAD_FACTOR;
                }
                for (Map.Entry<String, StatusGtySet> m1 : allIndivGty.entrySet()) {
                    m1.getValue().paternalChrom.setSize(availabeSNPSpace);
                    m1.getValue().maternalChrom.setSize(availabeSNPSpace);
                    m1.getValue().existence.setSize(availabeSNPSpace);
                }
            }

            runningInfo.delete(0, runningInfo.length());
            runningInfo.append(files.get(j).phaseFile).append(" encoded!\n");
            runningInfo.append("The number of SNPs on chromosome ");
            runningInfo.append(chroStr);
            runningInfo.append(" ");
            runningInfo.append(strRegion);
            runningInfo.append(" is ");
            runningInfo.append(curSNPNum);
            runningInfo.append(".");

            //encode genotypes
            BufferedReader brSample = new BufferedReader(new FileReader(files.get(j).getSampleFile()), FILE_READER_BUFFER_SIZE);
            BufferedReader brPhase = new BufferedReader(new FileReader(files.get(j).getPhaseFile()), FILE_READER_BUFFER_SIZE);
            while ((phase1 = brPhase.readLine()) != null) {
                phase2 = brPhase.readLine();
                indivLabel = brSample.readLine();
                indivLabel = indivLabel.substring(0, indivLabel.indexOf(' '));
                StatusGtySet indivGty = allIndivGty.get(indivLabel);
                if (indivGty == null) {
                    indivGty = new StatusGtySet();
                    indivGty.existence = new BitVector(availabeSNPSpace);
                    indivGty.paternalChrom = new BitVector(availabeSNPSpace);
                    indivGty.maternalChrom = new BitVector(availabeSNPSpace);
                    allIndivGty.put(indivLabel, indivGty);
                    if (!enteredHapIndiv.contains(indivLabel)) {
                        enteredHapIndiv.add(indivLabel);
                    }
                }


                for (Map.Entry<Integer, Long> m : snpPosInFile.entrySet()) {
                    index = m.getKey();
                    pos = m.getValue();
                    AnnotSNP snp = snpMapList.get(pos);

                    //re-code: 0->0; 1->1;
                    switch (phase1.charAt(index * 2)) {
                        case '1':
                            indivGty.existence.putQuick(snp.order, true);
                            indivGty.paternalChrom.putQuick(snp.order, true);
                            break;
                        case '0':
                            indivGty.existence.putQuick(snp.order, true);
                            break;
                        default:
                            //throw new Exception("Unexpected genotype in " + files.get(j).getPhaseFile() + " for " + indivLabel);
                            String unexpectInfo = "Unexpected genotype in " + files.get(j).getPhaseFile() + " for " + indivLabel;
                            GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                            GlobalVariables.addInforLog(unexpectInfo);
                    }

                    switch (phase2.charAt(index * 2)) {
                        case '1':
                            indivGty.maternalChrom.putQuick(snp.order, true);
                            break;
                        case '0':
                            break;
                        case '-':
                            indivGty.maternalChrom.putQuick(snp.order, indivGty.paternalChrom.getQuick(snp.order));
                            break;
                        default:
                            //throw new Exception("Unexpected genotype in " + files.get(j).getPhaseFile() + " for " + indivLabel);
                            String unexpectInfo = "Unexpected genotype in " + files.get(j).getPhaseFile() + " for " + indivLabel;
                            GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                            GlobalVariables.addInforLog(unexpectInfo);
                    }
                }
            }
            brSample.close();
            brPhase.close();
            GlobalVariables.mainView.setBriefRunningInfor(runningInfo.toString(), true);
            GlobalVariables.addInforLog(runningInfo.toString());
        }
    }

    public void readHapMapPhaseOldbySNP(String chroStr, List<HapPhaseFileSet> files, TreeSet<String> projSNPID, Map<Long, Boolean> projSnpMap,
            Map<Long, AnnotSNP> snpMapList, Map<String, StatusGtySet> allIndivGty, TreeSet<String> enteredHapIndiv, boolean isForward) throws Exception {
        int chroNum;
        StringBuilder strRegion = new StringBuilder();
        String line = null;
        StringTokenizer tokenizer = null;
        String delmilit = "\t ,";

        StringBuilder runningInfo = new StringBuilder();
        int totalColNum = 4;

        int rsIndex = 0;
        int alleleAIndex = 2;
        int alleleBIndex = 3;
        int posIndex = 1;
        int availabeSNPSpace = 100000;
        char strandLabel = '+';
        if (!isForward) {
            strandLabel = '-';
        }

        if (chroStr.equalsIgnoreCase("X")) {
            chroNum = 23;
        } else if (chroStr.equalsIgnoreCase("Y")) {
            chroNum = 24;
        } else if (chroStr.equalsIgnoreCase("MT")) {
            chroNum = 26;
        } else if (chroStr.equalsIgnoreCase("XY")) {
            chroNum = 25;
        } else {
            chroNum = Integer.parseInt(chroStr);
        }

        int index = 0;
        long pos = 0;

        boolean invalid = true;
        int curSNPNum = 0;
        boolean needConvert = false;
        if (referenceGenomeVersion == 37 || referenceGenomeVersion == 35) {
            needConvert = true;
        }
        Interval int1 = new Interval("chr" + chroStr, 0, 0);

        int lineCounter = -1;
        Map<Integer, Long> snpPosInFile = new HashMap<Integer, Long>();
        String indivLabel;

        String phase1, phase2;
        String[] cells = new String[totalColNum];
        StringBuilder tmpBuffer = new StringBuilder();
        for (int j = 0; j < files.size(); j++) {
            BufferedReader brLegend = new BufferedReader(new FileReader(files.get(j).getLegendFile()), FILE_READER_BUFFER_SIZE);

            //skip the first line
            brLegend.readLine();
            lineCounter = -1;
            snpPosInFile.clear();
            //start to read genotypes
            while ((line = brLegend.readLine()) != null) {
                line = line.toUpperCase();
                lineCounter++;

                index = 0;
                tokenizer = new StringTokenizer(line, delmilit);
                invalid = false;
                while (tokenizer.hasMoreTokens()) {
                    tmpBuffer.delete(0, tmpBuffer.length());
                    tmpBuffer.append(tokenizer.nextToken().trim());
                    cells[index] = tmpBuffer.toString();
                    if (index == posIndex) {
                        pos = Long.parseLong(cells[posIndex]);
                        if (needConvert) {
                            int1.setPoint((int) pos);
                            Interval int2 = liftOver.liftOver(int1);
                            //System.out.println(int2.toString());
                            if (int2 == null) {
                                invalid = true;
                                break;
                            }
                            pos = int2.getStart();
                            int2 = null;
                        }
                        //if the project SNP list has no such a SNP
                        if (projSnpMap.get(pos) == null) {
                            invalid = true;
                            break;
                        }
                    }
                    index++;
                }

                if (invalid) {
                    continue;
                }

                projSNPID.remove(cells[rsIndex].toLowerCase());
                AnnotSNP snp = snpMapList.get(pos);
                if (snp == null) {
                    snp = new AnnotSNP("rs" + cells[rsIndex].substring(2), chroNum, pos, 0.0,
                            cells[alleleAIndex].charAt(0), cells[alleleBIndex].charAt(0));
                    snp.order = curSNPNum;
                    snp.setStrand(strandLabel);
                    curSNPNum += 2;

                    snpMapList.put(pos, snp);
                } else {
                    if (cells[alleleAIndex].charAt(0) != snp.getAAllele()
                            || cells[alleleBIndex].charAt(0) != snp.getBAllele()) {
                        throw new Exception("Inconsistant strand for SNP " + snp.getRSID() + " in file ");
                    }

                }
                snpPosInFile.put(lineCounter, pos);
            }
            brLegend.close();

            if (curSNPNum > availabeSNPSpace) {
                //adjust the size of bitvectors
                while (curSNPNum > availabeSNPSpace) {
                    availabeSNPSpace += HAP_GENOTYPE_LOAD_FACTOR;
                }

                for (Map.Entry<String, StatusGtySet> m1 : allIndivGty.entrySet()) {
                    m1.getValue().paternalChrom.setSize(availabeSNPSpace);
                    m1.getValue().maternalChrom.setSize(availabeSNPSpace);
                    m1.getValue().existence.setSize(availabeSNPSpace);
                }
            }

            runningInfo.delete(0, runningInfo.length());
            runningInfo.append(files.get(j).phaseFile).append(" encoded!\n");
            runningInfo.append("The number of SNPs on chromosome ");
            runningInfo.append(chroStr);
            runningInfo.append(" ");
            runningInfo.append(strRegion);
            runningInfo.append(" is ");
            runningInfo.append(curSNPNum / 2);
            runningInfo.append(".");

            //encode genotypes
            BufferedReader brSample = new BufferedReader(new FileReader(files.get(j).getSampleFile()), FILE_READER_BUFFER_SIZE);
            BufferedReader brPhase = new BufferedReader(new FileReader(files.get(j).getPhaseFile()), FILE_READER_BUFFER_SIZE);

            while ((phase1 = brPhase.readLine()) != null) {
                phase2 = brPhase.readLine();
                indivLabel = brSample.readLine();
                indivLabel = indivLabel.substring(0, indivLabel.indexOf(' '));
                StatusGtySet indivGty = allIndivGty.get(indivLabel);
                if (indivGty == null) {
                    indivGty = new StatusGtySet();
                    indivGty.existence = new BitVector(availabeSNPSpace);
                    indivGty.paternalChrom = new BitVector(availabeSNPSpace);
                    indivGty.maternalChrom = new BitVector(availabeSNPSpace);
                    allIndivGty.put(indivLabel, indivGty);
                    if (!enteredHapIndiv.contains(indivLabel)) {
                        enteredHapIndiv.add(indivLabel);
                    }
                }

                for (Map.Entry<Integer, Long> m : snpPosInFile.entrySet()) {
                    index = m.getKey();
                    pos = m.getValue();
                    AnnotSNP snp = snpMapList.get(pos);

                    //re-code: 0->0; 1->1;
                    switch (phase1.charAt(index * 2)) {
                        case '1':
                            indivGty.existence.putQuick(snp.order, true);
                            indivGty.paternalChrom.putQuick(snp.order, true);
                            break;
                        case '0':
                            indivGty.existence.putQuick(snp.order, true);
                            break;
                        default:
                            //throw new Exception("Unexpected genotype in " + files.get(j).getPhaseFile() + " for " + indivLabel);
                            String unexpectInfo = "Unexpected genotype in " + files.get(j).getPhaseFile() + " for " + indivLabel;
                            GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                            GlobalVariables.addInforLog(unexpectInfo);
                    }

                    //for phase in y chromosome '- ' was used in HapMap. Here we just set the genotype as homozygous
                    switch (phase2.charAt(index * 2)) {
                        case '1':
                            indivGty.maternalChrom.putQuick(snp.order, true);
                            break;
                        case '0':
                            break;
                        case '-':
                            indivGty.maternalChrom.putQuick(snp.order, indivGty.paternalChrom.getQuick(snp.order));
                            break;
                        default:
                            //throw new Exception("Unexpected genotype in " + files.get(j).getPhaseFile() + " for " + indivLabel);
                            String unexpectInfo = "Unexpected genotype in " + files.get(j).getPhaseFile() + " for " + indivLabel;
                            GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                            GlobalVariables.addInforLog(unexpectInfo);
                    }
                }
            }
            brSample.close();
            brPhase.close();
            GlobalVariables.mainView.setBriefRunningInfor(runningInfo.toString(), true);
            GlobalVariables.addInforLog(runningInfo.toString());
        }
    }
}
