// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.
// Permission is granted for you to use this file to compile IGG.
// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.controller;

import cern.colt.bitvector.BitVector;
import java.io.BufferedReader;
import java.io.FileReader;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import org.cobi.igg3.Constants;
import org.cobi.igg3.entity.StatusGtySet;
import org.cobi.igg3.gui.IGG3View;
import org.cobi.igg3.GlobalVariables;
import org.cobi.util.thread.Task;

/**
 *
 * @author mxli
 */
public class ReadChipGenotypesTask implements Task, Constants {

    private static int count = 0;
    private int num = count;
    private List<String> indiviChipLabelList;
    public Map<String, StatusGtySet> indiviGtyList;
    private int maxGtyNum;
    private String localFileAddress;

    public ReadChipGenotypesTask(List<String> indiviChipLabelList, Map<String, StatusGtySet> indiviGtyList,
            int maxGtyNum, String localFileAddress) {
        count++;
        this.indiviChipLabelList = indiviChipLabelList;
        this.indiviGtyList = indiviGtyList;//要同时下载的线程�?
        this.maxGtyNum = maxGtyNum;
        this.localFileAddress = localFileAddress;
    }

    @Override
    public void execute() {
        // System.out.println("[Task " + num + "] start...");
        GlobalVariables.mainView.setBriefRunningInfor("[Task " + num + "] start...");
        try {
            readAffyGenotypesInFile();
        } catch (Exception ie) {
        }

        // System.out.println("[Task " + num + "] done.");
        GlobalVariables.mainView.setBriefRunningInfor("[Task " + num + "] done.");
    }

    private void readAffyGenotypesInFile() throws Exception {
        String line = null;
        StringTokenizer tokenizer = null;
        String probID;
        String delmilit = "\t ,";
        String startLabel1 = "SNP_";
        String startLabel2 = "AX-";
        int gtyP;
        String runningInfo;
        String sigGty;
        BufferedReader br = new BufferedReader(new FileReader(localFileAddress), FILE_READER_BUFFER_SIZE);
        //read chipLabel 
        if ((line = br.readLine()) != null) {
            line = line.split(":")[1].trim();
        }
        if (line.length() == 0) {
            throw new Exception("Cannot read chip label in chip file, " + localFileAddress);
        }
        indiviChipLabelList.add(line);
        gtyP = (indiviChipLabelList.size() - 1);

        while ((line = br.readLine()) != null) {
            if (line.trim().startsWith(startLabel1)||line.trim().startsWith(startLabel2)) {
                break;
            }
        }
        StringBuilder tmpBuffer = new StringBuilder();
        //stat to retreive the data
        do {
            if (line.trim().length() == 0) {
                continue;
            }
            tokenizer = new StringTokenizer(line, delmilit);
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            probID = tmpBuffer.toString();

            StatusGtySet gty = indiviGtyList.get(probID);

            if (gty == null) {
                gty = new StatusGtySet();
                gty.paternalChrom = new BitVector(maxGtyNum);
                gty.maternalChrom = new BitVector(maxGtyNum);
                gty.existence = new BitVector(maxGtyNum);
                indiviGtyList.put(probID, gty);
            }
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            sigGty = tmpBuffer.toString();
            // make sure the genotype is made up of two alleles. otherwise regarded it as a missing data
            if (sigGty.length() != 2) {
                continue;
            }

            //re-code: A->0; B->1;
            switch (sigGty.charAt(0)) {
                case 'B':
                    gty.existence.putQuick(gtyP, true);
                    gty.paternalChrom.putQuick(gtyP, true);
                    break;
                case 'A':
                    gty.existence.putQuick(gtyP, true);
                    break;
                default:
                    //throw new Exception("Unexpected genotype " + sigGty + " in " + localFileAddress + " for " + probID);
                    String unexpectInfo = "Unexpected genotype " + sigGty + " in " + localFileAddress + " for " + probID;
                    GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                    GlobalVariables.addInforLog(unexpectInfo);
            }
            switch (sigGty.charAt(1)) {
                case 'B':
                    gty.maternalChrom.putQuick(gtyP, true);
                    break;
                case 'A':
                    break;
                default:
                    //throw new Exception("Unexpected genotype " + sigGty + " in " + localFileAddress + " for " + probID);
                    String unexpectInfo = "Unexpected genotype " + sigGty + " in " + localFileAddress + " for " + probID;
                    GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                    GlobalVariables.addInforLog(unexpectInfo);
            }
        } while ((line = br.readLine()) != null);
        br.close();

        runningInfo = localFileAddress + " encoded!";
        GlobalVariables.mainView.setBriefRunningInfor(runningInfo, true);
        GlobalVariables.addInforLog(runningInfo);
    }

    private void readAffyGenotypesInFileBySubject() throws Exception {
        String line = null;
        StringTokenizer tokenizer = null;
        String probID;
        String delmilit = "\t ,";
        String startLabel1 = "SNP_";
 String startLabel2 = "AX-";
        int gtyP;
        String runningInfo;
        String sigGty;
        BufferedReader br = new BufferedReader(new FileReader(localFileAddress), FILE_READER_BUFFER_SIZE);
        //read chipLabel 
        if ((line = br.readLine()) != null) {
            line = line.split(":")[1].trim();
        }
        if (line.length() == 0) {
            throw new Exception("Cannot read chip label in chip file, " + localFileAddress);
        }
        indiviChipLabelList.add(line);
        gtyP = (indiviChipLabelList.size() - 1);

        while ((line = br.readLine()) != null) {
            if (line.trim().startsWith(startLabel1)||line.trim().startsWith(startLabel2)) {
                break;
            }
        }
        StringBuilder tmpBuffer = new StringBuilder();
        //stat to retreive the data
        do {
            if (line.trim().length() == 0) {
                continue;
            }
            tokenizer = new StringTokenizer(line, delmilit);
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            probID = tmpBuffer.toString();

            StatusGtySet gty = indiviGtyList.get(probID);

            if (gty == null) {
                gty = new StatusGtySet();
                gty.paternalChrom = new BitVector(maxGtyNum);
                gty.maternalChrom = new BitVector(maxGtyNum);
                gty.existence = new BitVector(maxGtyNum);
                indiviGtyList.put(probID, gty);
            }
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            sigGty = tmpBuffer.toString();

            // make sure the genotype is made up of two alleles. otherwise regarded it as a missing data
            if (sigGty.length() != 2) {
                continue;
            }


            //re-code: A->0; B->1;
            switch (sigGty.charAt(0)) {
                case 'B':
                    gty.existence.putQuick(gtyP, true);
                    gty.paternalChrom.putQuick(gtyP, true);
                    break;
                case 'A':
                    gty.existence.putQuick(gtyP, true);
                    break;
                default:
                    //throw new Exception("Unexpected genotype " + sigGty + " in " + localFileAddress + " for " + probID);
                    String unexpectInfo = "Unexpected genotype " + sigGty + " in " + localFileAddress + " for " + probID;
                    GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                    GlobalVariables.addInforLog(unexpectInfo);
            }
            switch (sigGty.charAt(1)) {
                case 'B':
                    gty.maternalChrom.putQuick(gtyP, true);
                    break;
                case 'A':
                    break;
                default:
                    //throw new Exception("Unexpected genotype " + sigGty + " in " + localFileAddress + " for " + probID);
                    String unexpectInfo = "Unexpected genotype " + sigGty + " in " + localFileAddress + " for " + probID;
                    GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                    GlobalVariables.addInforLog(unexpectInfo);

            }
        } while ((line = br.readLine()) != null);
        br.close();

        runningInfo = localFileAddress + " encoded!";
        GlobalVariables.mainView.setBriefRunningInfor(runningInfo, true);
        GlobalVariables.addInforLog(runningInfo);
    }
}
