// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.
// Permission is granted for you to use this file to compile IGG.
// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.controller;

import cern.colt.bitvector.BitVector;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.TreeSet;
import org.cobi.igg3.entity.AnnotSNP;
import org.cobi.igg3.entity.ChromSNPGtySet;
import org.cobi.igg3.entity.Integration;
import org.cobi.igg3.entity.Project;
import org.cobi.igg3.entity.SNPPosiComparator;
import org.cobi.igg3.entity.StatusGtySet;
import org.cobi.igg3.gui.IGG3View;
import org.cobi.igg3.GlobalVariables;

/**
 *
 * @author Miaoxin Li
 */
public class RegionGenomeChipIntegrator extends BasicIntegrator {

    public boolean integrateChipFiles(Integration intgration) throws Exception {
        boolean succeed = false;
        Project prg = GlobalVariables.currentProject;
        Map<String, StatusGtySet> mergedIndivGty = new HashMap<String, StatusGtySet>();
        Map<String, StatusGtySet> tmpIndivGty = new HashMap<String, StatusGtySet>();

        int populationID = 9;
        if ("Caucasian Population".equals(intgration.getFreqPopulation())) {
            populationID = 11;
        } else if ("African Population".equals(intgration.getFreqPopulation())) {
            populationID = 10;
        }
        String runningInfo;

        Map<String, AnnotSNP> tmpSnpMap1 = new HashMap<String, AnnotSNP>();
        List<AnnotSNP> tmpSnpList = new ArrayList<AnnotSNP>();
        List<AnnotSNP> mergedSnpList = new ArrayList<AnnotSNP>();

        ChromSNPGtySet chromSet = new ChromSNPGtySet();
        File resultPath = new File(intgration.getResultPath() + File.separator);
        if (!resultPath.exists()) {
            resultPath.mkdir();
        }
        for (int i = 0; i < CHROM_NAME.length; i++) {
            String chrom = CHROM_NAME[i];
            if (intgration.getPhysicalRegionMap().get(chrom) == null &&
                    intgration.getGenetRegionMap().get(chrom) == null) {
                continue;
            }
            chromSet.setChromName(chrom);
            for (int j = 0; j < AFFY_CHIPS.length; j++) {
                Collection filePaths = prg.getChipFilePathList().getCollection(AFFY_CHIPS[j]);
                if (filePaths == null) {
                    continue;
                }
                if (intgration.getGenetRegionMap().isEmpty()) {
                    retrieveChipSNPAnnotationByPysic(AFFY_CHIPS[j], tmpSnpMap1, 0,
                            populationID, chrom, intgration.getPhysicalRegionMap().get(chrom));
                } else {
                    retrieveChipSNPAnnotationByGenet(AFFY_CHIPS[j], tmpSnpMap1, 0,
                            populationID, chrom, intgration.getGenetRegionMap().get(chrom));
                }
                if (tmpSnpMap1.isEmpty()) {
                    continue;
                }
                readChipGenotypes(filePaths, true, tmpSnpMap1, tmpIndivGty);
                tmpSnpList.clear();
                tmpSnpList.addAll(tmpSnpMap1.values());
                tmpSnpMap1.clear();
                Collections.sort(tmpSnpList, new SNPPosiComparator());
                if (mergedSnpList.isEmpty()) {
                    mergedSnpList.addAll(tmpSnpList);
                    //copy the genotypes      
                    mergedIndivGty.putAll(tmpIndivGty);
                    tmpSnpList.clear();
                    tmpIndivGty.clear();
                } else {
                    runningInfo = mergeSNPGtybyListNoGtyExtend(mergedSnpList, tmpSnpList, mergedIndivGty, tmpIndivGty);
                    tmpSnpList.clear();
                    tmpIndivGty.clear();
                    GlobalVariables.mainView.setBriefRunningInfor(runningInfo, true);
                    GlobalVariables.addInforLog(runningInfo);
                }
            }

            for (int j = 0; j < ILLUMINA_CHIPS.length; j++) {
                Collection filePaths = prg.getChipFilePathList().getCollection(ILLUMINA_CHIPS[j]);
                if (filePaths == null) {
                    continue;
                }

                if (intgration.getGenetRegionMap().isEmpty()) {
                    retrieveChipSNPAnnotationByPysic(ILLUMINA_CHIPS[j], tmpSnpMap1, 1, populationID, chrom,
                            intgration.getPhysicalRegionMap().get(chrom));
                } else {
                    retrieveChipSNPAnnotationByGenet(ILLUMINA_CHIPS[j], tmpSnpMap1, 1, populationID, chrom,
                            intgration.getGenetRegionMap().get(chrom));
                }
                if (tmpSnpMap1.isEmpty()) {
                    continue;
                }
                readChipGenotypes(filePaths, false, tmpSnpMap1, tmpIndivGty);
                tmpSnpList.clear();
                tmpSnpList.addAll(tmpSnpMap1.values());
                tmpSnpMap1.clear();
                Collections.sort(tmpSnpList, new SNPPosiComparator());
                if (mergedSnpList.isEmpty()) {
                    mergedSnpList.addAll(tmpSnpList);
                    //copy the genotypes       
                    mergedIndivGty.putAll(tmpIndivGty);

                } else {
                    runningInfo = mergeSNPGtybyListNoGtyExtend(mergedSnpList, tmpSnpList, mergedIndivGty, tmpIndivGty);
                    tmpSnpList.clear();
                    tmpIndivGty.clear();
                    GlobalVariables.mainView.setBriefRunningInfor(runningInfo, true);
                    GlobalVariables.addInforLog(runningInfo);
                }
            }
            chromSet.setSnpInfoList(mergedSnpList);
            chromSet.setIndiviGty(mergedIndivGty);
            intgration.addAChromResult(chrom + " SNP#" + mergedSnpList.size());
            chromSet.saveAllObjectsAndClear(resultPath.getCanonicalPath(), DEFAULT_GTY_BLOCKNUM);

            mergedSnpList.clear();
            mergedIndivGty.clear();
        }
        return succeed;
    }

    public void readChipGenotypes(Collection filePaths, boolean isAffy, Map<String, AnnotSNP> snpMapList, Map<String, StatusGtySet> indivGty) throws Exception {
        String chipFile;
        Iterator iter = filePaths.iterator();

        //Clear genotypes
        // assign default values to genotypes to unify the situation
        TreeSet<String> indivChipLabelInPed = GlobalVariables.currentPedFile.extractIndivChipLabels();

        while (iter.hasNext()) {
            chipFile = (String) iter.next();
            if (chipFile.endsWith("*.*")) {
                String filePath = chipFile.substring(0, chipFile.lastIndexOf(File.separator));
                File chipFiles = new File(filePath);
                String[] chipFileList = chipFiles.list();
                for (int i = 0; i < chipFileList.length; i++) {
                    if (isAffy) {
                        readAffyGenotypesInFile(filePath + File.separator + chipFileList[i], snpMapList, indivGty, indivChipLabelInPed);
                    } else {
                        readIllumGenotypesInFile(filePath + File.separator + chipFileList[i], snpMapList, indivGty, indivChipLabelInPed);
                    }
                }
            } else {
                if (isAffy) {
                    readAffyGenotypesInFile(chipFile, snpMapList, indivGty, indivChipLabelInPed);
                } else {
                    readIllumGenotypesInFile(chipFile, snpMapList, indivGty, indivChipLabelInPed);
                }
            }
        }
    }

    private void readIllumGenotypesInFile(String filePath, Map<String, AnnotSNP> snpMapList,
            Map<String, StatusGtySet> indivGty, TreeSet<String> indiviInPed) throws Exception {
        String line = null;
        StringTokenizer tokenizer = null;
        String delmilit = "\t ,";
        String startLabel = "rs";

        String runningInfo;
        int gtyNum = snpMapList.size();

        String indivLabel, sigGty;
        BufferedReader br = new BufferedReader(new FileReader(filePath), FILE_READER_BUFFER_SIZE);

        StringBuilder tmpBuffer = new StringBuilder();

        while ((line = br.readLine()) != null) {
            if (line.trim().startsWith(startLabel)) {
                break;
            }
        }
        //stat to retreive the data
        do {
            if (line.trim().length() == 0) {
                continue;
            }
            tokenizer = new StringTokenizer(line, delmilit);
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());

            AnnotSNP snp = snpMapList.get(tmpBuffer.toString());
            if (snp == null) {
                continue;
            }
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            indivLabel = tmpBuffer.toString();

            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            tmpBuffer.append(tokenizer.nextToken().trim());
            sigGty = tmpBuffer.toString();

            // make sure the genotype is made up of two alleles. otherwise regarded it as a missing data
            if (sigGty.length() != 2) {
                continue;
            }
            if (!indiviInPed.contains(indivLabel)) {
                //  throw new Exception("Pedigree has not such a chip label " + indivLabel + " for file, " + filePath);
                continue;
            }

            //get individuals
            StatusGtySet indi = indivGty.get(indivLabel);
            if (indi == null) {
                indi = new StatusGtySet();
                indi.paternalChrom = new BitVector(gtyNum);
                indi.maternalChrom = new BitVector(gtyNum);
                indi.existence = new BitVector(gtyNum);
                indivGty.put(indivLabel, indi);
            }


            //re-code: A->0; B->1;
            switch (sigGty.charAt(0)) {
                case 'B':
                    indi.existence.putQuick(snp.order, true);
                    indi.paternalChrom.putQuick(snp.order, true);
                    break;
                case 'A':
                    break;
                case '*':
                    break;
                default:
                    //throw new Exception("Unexpected genotype " + sigGty + " in " + filePath + " for " + indivLabel + " at " + snp.getRSID());
                    String unexpectInfo = "Unexpected genotype " + sigGty + " in " + filePath + " for " + indivLabel + " at " + snp.getRSID();
                    GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                    GlobalVariables.addInforLog(unexpectInfo);
            }
            switch (sigGty.charAt(1)) {
                case 'B':
                    indi.maternalChrom.putQuick(snp.order, true);
                    break;
                case 'A':
                    break;
                case '*':
                    break;
                default:
                    //throw new Exception("Unexpected genotype " + sigGty + " in " + filePath + " for " + indivLabel + " at " + snp.getRSID());
                    String unexpectInfo = "Unexpected genotype " + sigGty + " in " + filePath + " for " + indivLabel + " at " + snp.getRSID();
                    GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                    GlobalVariables.addInforLog(unexpectInfo);
            }
        } while ((line = br.readLine()) != null);
        br.close();
        runningInfo = filePath + " encoded!";
        GlobalVariables.mainView.setBriefRunningInfor(runningInfo, true);
        GlobalVariables.addInforLog(runningInfo);
    }

    private void readAffyGenotypesInFile(String filePath, Map<String, AnnotSNP> snpMapList,
            Map<String, StatusGtySet> indivGtyMap, TreeSet<String> indiviInPed) throws Exception {
        String line = null;
        StringTokenizer tokenizer = null;

        String delmilit = "\t ,";
        String startLabel1 = "SNP_";
         String startLabel2 = "AX-";
        int gtyNum = snpMapList.size();
        String indivLabel;

        String runningInfo;
        String sigGty;
        BufferedReader br = new BufferedReader(new FileReader(filePath), FILE_READER_BUFFER_SIZE);
        //read chipLabel 
        if ((line = br.readLine()) != null) {
            line = line.split(":")[1].trim();
        }
        if (line.length() == 0) {
            throw new Exception("Cannot read chip label in chip file, " + filePath);
        }
        if (!indiviInPed.contains(line)) {
            throw new Exception("Pedigree has not such a chip label" + line + " for file, " + filePath);
        }
        indivLabel = line;
        StatusGtySet indi = indivGtyMap.get(indivLabel);
        if (indi == null) {
            indi = new StatusGtySet();
            indi.paternalChrom = new BitVector(gtyNum);
            indi.maternalChrom = new BitVector(gtyNum);
            indi.existence = new BitVector(gtyNum);
            indivGtyMap.put(line, indi);
        }
        while ((line = br.readLine()) != null) {
            if (line.trim().startsWith(startLabel1)||line.trim().startsWith(startLabel2)) {
                break;
            }
        }
        StringBuilder tmpBuffer = new StringBuilder();

        //stat to retreive the data
        do {
            if (line.trim().length() == 0) {
                continue;
            }
            tokenizer = new StringTokenizer(line, delmilit);
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());

            AnnotSNP snp = snpMapList.get(tmpBuffer.toString());
            if (snp == null) {
                continue;
            }
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            sigGty = tmpBuffer.toString();
            // make sure the genotype is made up of two alleles. otherwise regarded it as a missing data
            if (sigGty.length() != 2) {
                continue;
            }



            //re-code: A->0; B->1;
            switch (sigGty.charAt(0)) {
                case 'B':
                    indi.existence.putQuick(snp.order, true);
                    indi.paternalChrom.putQuick(snp.order, true);
                    break;
                case 'A':
                    indi.existence.putQuick(snp.order, true);
                    break;
                default:
                    //throw new Exception("Unexpected genotype " + sigGty + " in " + filePath + " for " + snp.getProbeSetID());
                    String unexpectInfo = "Unexpected genotype " + sigGty + " in " + filePath + " for " + snp.getProbeSetID();
                    GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                    GlobalVariables.addInforLog(unexpectInfo);
            }
            switch (sigGty.charAt(1)) {
                case 'B':
                    indi.maternalChrom.putQuick(snp.order, true);
                    break;
                case 'A':
                    break;
                default:
                    //throw new Exception("Unexpected genotype " + sigGty + " in " + filePath + " for " + snp.getProbeSetID());
                    String unexpectInfo = "Unexpected genotype " + sigGty + " in " + filePath + " for " + snp.getProbeSetID();
                    GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                    GlobalVariables.addInforLog(unexpectInfo);
            }

        } while ((line = br.readLine()) != null);
        br.close();

        runningInfo = filePath + " encoded!";
        GlobalVariables.mainView.setBriefRunningInfor(runningInfo, true);
        GlobalVariables.addInforLog(runningInfo);
    }
}
