// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.
// Permission is granted for you to use this file to compile IGG.
// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.controller;

import cern.colt.list.LongArrayList;
import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import org.cobi.igg3.entity.AnnotSNP;
import org.cobi.igg3.entity.ChromSNPGtySet;
import org.cobi.igg3.entity.PedFile;
import org.cobi.igg3.entity.StatusGtySet;
import org.cobi.igg3.genetformator.BIMBAMFiles;
import org.cobi.igg3.genetformator.HapMapFiles;
import org.cobi.igg3.genetformator.HaploviewFiles;
import org.cobi.igg3.genetformator.LinkageFiles;
import org.cobi.igg3.genetformator.Mega2Files;
import org.cobi.igg3.genetformator.MerlinFiles;
import org.cobi.igg3.genetformator.PlinkFiles;
import org.cobi.igg3.genetformator.SolarFiles;
import org.cobi.igg3.GlobalVariables;

/**
 *
 * @author Miaoxin Li
 */
public class RoutineGenetExporter extends BasicGenetExporter {

    public String exportPlink() throws Exception {
        List<String> chroms = integration.getChromResult();
        ChromSNPGtySet chromSet = new ChromSNPGtySet();
        String infor;
        List<String> chromArray = new ArrayList<String>();

        String path = exportSetting.getExportPath();
        String prefix = exportSetting.getPrefixFileName();
        BufferedWriter bwGeneticMaps = new BufferedWriter(new FileWriter(path + prefix + ".map"));

        PedFile hapPed = null;
        if (integration.isHapMapSet()) {
            hapPed = new PedFile();
            hapPed.parseHapMapPed(IGG_RESOURCE_HAPMAP_FOLDER + File.separator + "pedinfor" + File.separator + "hap3Sample.txt");
        }

        PlinkFiles inputer = new PlinkFiles();
        String msg;
        int index = 0;

        File a2NFile = new File(exportSetting.getExportPath() + File.separator + exportSetting.getPrefixFileName() + ".allele2number.txt");
        BufferedWriter bwA2N = null;
        if (exportSetting.isNumberAllele()) {
            bwA2N = new BufferedWriter(new FileWriter(a2NFile));
            bwA2N.write("Chromosome Position SNPID 1Allele 2Allele");
            bwA2N.newLine();
        }

        Map<String, LongArrayList> selectedRegions = exportSetting.getSelectedRegions();
        if (selectedRegions != null && selectedRegions.size() > 0) {
            for (int k = 0; k < chroms.size(); k++) {
                String chrom = chroms.get(k);
                chrom = chrom.substring(0, chrom.indexOf(' '));
                LongArrayList regions = selectedRegions.get(chrom);
                if (regions == null) {
                    continue;
                }
                chromSet.setChromName(chrom);
                chromSet.readAllObjects(integration.getResultPath());

                List<AnnotSNP> mainSnpMap = chromSet.getSnpInfoList();
                Map<String, StatusGtySet> indivGtyMap = chromSet.getIndiviGty();
                filterSNPsbyRegion(mainSnpMap, regions);
                msg = mainSnpMap.size() + " SNPs within the SELECTED region on Chromosome " + chrom;
                msg = msg + "\nChecking Genotype Quality on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);
                preprocessSNPGty(mainSnpMap, indivGtyMap, chrom);
                inputer.writeMapFile(bwGeneticMaps, mainSnpMap);

                msg = "Transforming Genotypes on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);
                if (exportSetting.isNumberAllele()) {
                    generalLinkPedGenotypeNumberAllele(bwA2N, mainSnpMap, indivGtyMap, chrom);
                } else {
                    generalLinkPedGenotypeCharAllele(mainSnpMap, indivGtyMap, chrom);
                }
                chromArray.add(chrom);
                index++;
            }
        } else {

            for (int k = 0; k < chroms.size(); k++) {
                String chrom = chroms.get(k);
                chrom = chrom.substring(0, chrom.indexOf(' '));
                chromSet.setChromName(chrom);
                chromSet.readAllObjects(integration.getResultPath());

                List<AnnotSNP> mainSnpMap = chromSet.getSnpInfoList();
                Map<String, StatusGtySet> indivGtyMap = chromSet.getIndiviGty();
                msg = "Checking Genotype Quality on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);
                preprocessSNPGty(mainSnpMap, indivGtyMap, chrom);
                inputer.writeMapFile(bwGeneticMaps, mainSnpMap);

                msg = "Transforming Genotypes on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);
                if (exportSetting.isNumberAllele()) {
                    generalLinkPedGenotypeNumberAllele(bwA2N, mainSnpMap, indivGtyMap, chrom);
                } else {
                    generalLinkPedGenotypeCharAllele(mainSnpMap, indivGtyMap, chrom);
                }
                chromArray.add(chrom);
            }
        }
        bwGeneticMaps.close();

        if (exportSetting.isNumberAllele()) {
            msg = "The original allele names and numbered allele names are saved in " + a2NFile.getCanonicalPath();
            GlobalVariables.mainView.setBriefRunningInfor(msg);
            GlobalVariables.addInforLog(msg);
            bwA2N.close();
        }

        msg = "Generating Pedigree File";
        GlobalVariables.mainView.setBriefRunningInfor(msg);
        GlobalVariables.addInforLog(msg);
        mergeGeneralLinkPedGenotypeFiles(chromArray, path + File.separator + prefix + ".ped", hapPed);
        infor = "Files are expored into " + path + "\n " + prefix + ".ped, " + prefix + ".map.";
        return infor;
    }

    public String exportPlinkBinaryGenotypeFormat() throws Exception {
        String infor;

        PedFile hapPed = null;
        if (integration.isHapMapSet()) {
            hapPed = new PedFile();
            hapPed.parseHapMapPed(IGG_RESOURCE_HAPMAP_FOLDER + File.separator + "pedinfor" + File.separator + "hap3Sample.txt");
        }

        PlinkFiles inputer = new PlinkFiles();
        String msg;

        String path = exportSetting.getExportPath();
        String prefix = exportSetting.getPrefixFileName();
        List<String[]> phenotypeClasses = exportSetting.getTraitProper();
        BufferedWriter bwPedFile = new BufferedWriter(new FileWriter(path + prefix + ".fam"));
        inputer.writePedigreeFileBinaryFormat(bwPedFile, integration, hapPed, phenotypeClasses);
        bwPedFile.close();


        msg = "Generating Pedigree file and Binary genotype files";
        GlobalVariables.mainView.setBriefRunningInfor(msg);
        GlobalVariables.addInforLog(msg);


        List<String> chroms = integration.getChromResult();
        ChromSNPGtySet chromSet = new ChromSNPGtySet();
        boolean bfile_SNP_major = true;
        DataOutputStream bedOut = new DataOutputStream(new BufferedOutputStream(new FileOutputStream(path + prefix + ".bed")));
        inputer.writeBinaryGenotypesHead(bedOut);

        BufferedWriter bwGeneticMaps = new BufferedWriter(new FileWriter(path + prefix + ".bim"));
        Map<String, LongArrayList> selectedRegions = exportSetting.getSelectedRegions();
        if (selectedRegions != null && selectedRegions.size() > 0) {
            for (int k = 0; k < chroms.size(); k++) {
                String chrom = chroms.get(k);
                chrom = chrom.substring(0, chrom.indexOf(' '));
                LongArrayList regions = selectedRegions.get(chrom);
                if (regions == null) {
                    continue;
                }
                chromSet.setChromName(chrom);
                chromSet.readAllObjects(integration.getResultPath());

                List<AnnotSNP> mainSnpMap = chromSet.getSnpInfoList();
                Map<String, StatusGtySet> indivGtyMap = chromSet.getIndiviGty();
                filterSNPsbyRegion(mainSnpMap, regions);
                msg = mainSnpMap.size() + " SNPs within the SELECTED region on Chromosome " + chrom;
                msg = msg + "\nChecking Genotype Quality on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);
                preprocessSNPGty(mainSnpMap, indivGtyMap, chrom);
                inputer.writeMapFileBinaryFormat(bwGeneticMaps, mainSnpMap);
                inputer.writeBinaryGenotypesBody(bedOut, mainSnpMap, indivGtyMap, exportSetting, integration);
            }
        } else {
            for (int k = 0; k < chroms.size(); k++) {
                String chrom = chroms.get(k);
                chrom = chrom.substring(0, chrom.indexOf(' '));
                chromSet.setChromName(chrom);
                chromSet.readAllObjects(integration.getResultPath());

                List<AnnotSNP> mainSnpMap = chromSet.getSnpInfoList();
                Map<String, StatusGtySet> indivGtyMap = chromSet.getIndiviGty();
                msg = "Checking Genotype Quality on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);
                preprocessSNPGty(mainSnpMap, indivGtyMap, chrom);
                inputer.writeMapFileBinaryFormat(bwGeneticMaps, mainSnpMap);
                inputer.writeBinaryGenotypesBody(bedOut, mainSnpMap, indivGtyMap, exportSetting, integration);
            }
        }
        bwGeneticMaps.close();
        bedOut.close();


        infor = "Files are expored into " + path + "\n " + prefix + ".fam, " + prefix + ".bed, " + prefix + ".bim.";
        return infor;
    }

    public String exportMerlinByChromsome() throws Exception {
        List<String> chroms = integration.getChromResult();
        ChromSNPGtySet chromSet = new ChromSNPGtySet();
        String infor;
        String path = exportSetting.getExportPath();
        String prefix = exportSetting.getPrefixFileName();

        PedFile hapPed = null;
        if (integration.isHapMapSet()) {
            hapPed = new PedFile();
            hapPed.parseHapMapPed(IGG_RESOURCE_HAPMAP_FOLDER + File.separator + "pedinfor" + File.separator + "hap3Sample.txt");
        }
        File a2NFile = new File(exportSetting.getExportPath() + File.separator + exportSetting.getPrefixFileName() + ".allele2number.txt");
        BufferedWriter bwA2N = null;
        if (exportSetting.isNumberAllele()) {
            bwA2N = new BufferedWriter(new FileWriter(a2NFile));
            bwA2N.write("Chromosome Position SNPID 1Allele 2Allele");
            bwA2N.newLine();
        }

        MerlinFiles inputer = new MerlinFiles();
        String msg;
        Map<String, LongArrayList> selectedRegions = exportSetting.getSelectedRegions();
        if (selectedRegions != null && selectedRegions.size() > 0) {
            for (int k = 0; k < chroms.size(); k++) {
                String chrom = chroms.get(k);
                chrom = chrom.substring(0, chrom.indexOf(' '));
                LongArrayList regions = selectedRegions.get(chrom);
                if (regions == null) {
                    continue;
                }
                chromSet.setChromName(chrom);
                List<String[]> phenotypeClasses = exportSetting.getTraitProper();
                BufferedWriter bwPedFileDescription = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".dat"));
                BufferedWriter bwGeneticMaps = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".map"));
                BufferedWriter bwAlleleFrequencies = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".freq"));

                //write the description file 
                for (int i = 0; i < phenotypeClasses.size(); i++) {
                    //Phentype types:  A - affection status, T - Quantitative Trait and C - Covariate)
                    if (phenotypeClasses.get(i)[1].startsWith("Affection Status")) {
                        bwPedFileDescription.write("A ");
                    } else if (phenotypeClasses.get(i)[1].startsWith("Quantitative Trait")) {
                        bwPedFileDescription.write("T ");
                    } else if (phenotypeClasses.get(i)[1].startsWith("Covariate")) {
                        bwPedFileDescription.write("C ");
                    }
                    bwPedFileDescription.write(phenotypeClasses.get(i)[0]);
                    bwPedFileDescription.write("\n");
                }
                bwGeneticMaps.write("CHROMOSOME  MARKER  POSITION\n");
                chromSet.readAllObjects(integration.getResultPath());

                List<AnnotSNP> mainSnpMap = chromSet.getSnpInfoList();
                Map<String, StatusGtySet> mainIndivGtyMap = chromSet.getIndiviGty();
                filterSNPsbyRegion(mainSnpMap, regions);
                msg = mainSnpMap.size() + " SNPs within the SELECTED region on Chromosome " + chrom;
                msg = msg + "\nChecking Genotype Quality on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);

                preprocessSNPGty(mainSnpMap, mainIndivGtyMap, chrom);

                inputer.writePedFileDescriptionMerlin(bwPedFileDescription, mainSnpMap);
                inputer.writeGeneticMapsMerlin(bwGeneticMaps, mainSnpMap);
                inputer.writeAlleleFrequenciesMerlin(bwAlleleFrequencies, mainSnpMap);

                bwPedFileDescription.close();
                bwGeneticMaps.close();
                bwAlleleFrequencies.close();

                msg = "Transforming Genotypes on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);
                generalLinkPedNumberAllele(bwA2N, mainSnpMap, mainIndivGtyMap, chrom, hapPed);
            }
        } else {
            for (int k = 0; k < chroms.size(); k++) {
                String chrom = chroms.get(k);
                chrom = chrom.substring(0, chrom.indexOf(' '));
                chromSet.setChromName(chrom);
                List<String[]> phenotypeClasses = exportSetting.getTraitProper();
                BufferedWriter bwPedFileDescription = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".dat"));
                BufferedWriter bwGeneticMaps = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".map"));
                BufferedWriter bwAlleleFrequencies = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".freq"));

                //write the description file 
                for (int i = 0; i < phenotypeClasses.size(); i++) {
                    //Phentype types:  A - affection status, T - Quantitative Trait and C - Covariate)
                    if (phenotypeClasses.get(i)[1].startsWith("Affection Status")) {
                        bwPedFileDescription.write("A ");
                    } else if (phenotypeClasses.get(i)[1].startsWith("Quantitative Trait")) {
                        bwPedFileDescription.write("T ");
                    } else if (phenotypeClasses.get(i)[1].startsWith("Covariate")) {
                        bwPedFileDescription.write("C ");
                    }
                    bwPedFileDescription.write(phenotypeClasses.get(i)[0]);
                    bwPedFileDescription.write("\n");
                }
                bwGeneticMaps.write("CHROMOSOME  MARKER  POSITION\n");

                chromSet.readAllObjects(integration.getResultPath());

                List<AnnotSNP> mainSnpMap = chromSet.getSnpInfoList();
                Map<String, StatusGtySet> mainIndivGtyMap = chromSet.getIndiviGty();
                msg = "Checking Genotype Quality on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);

                preprocessSNPGty(mainSnpMap, mainIndivGtyMap, chrom);

                inputer.writePedFileDescriptionMerlin(bwPedFileDescription, mainSnpMap);
                inputer.writeGeneticMapsMerlin(bwGeneticMaps, mainSnpMap);
                inputer.writeAlleleFrequenciesMerlin(bwAlleleFrequencies, mainSnpMap);
                bwPedFileDescription.close();
                bwGeneticMaps.close();
                bwAlleleFrequencies.close();

                msg = "Transforming Genotypes on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);
                generalLinkPedNumberAllele(bwA2N, mainSnpMap, mainIndivGtyMap, chrom, hapPed);
            }
        }

        if (exportSetting.isNumberAllele()) {
            msg = "The original allele names and numbered allele names are saved in " + a2NFile.getCanonicalPath();
            GlobalVariables.mainView.setBriefRunningInfor(msg);
            GlobalVariables.addInforLog(msg);
            bwA2N.close();
        }

        msg = "Generating Pedigree File";
        GlobalVariables.mainView.setBriefRunningInfor(msg);
        GlobalVariables.addInforLog(msg);
        infor = "Files are expored into " + path + "(chromosome#)\n " + prefix + ".#.ped, " + prefix + ".#.dat, " + prefix + ".#.map, "
                + prefix + ".#.freq.";
        return infor;
    }

    public String exportMerlin() throws Exception {
        List<String> chroms = integration.getChromResult();
        ChromSNPGtySet chromSet = new ChromSNPGtySet();
        String infor;
        String path = exportSetting.getExportPath();
        String prefix = exportSetting.getPrefixFileName();
        List<String> chromArray = new ArrayList<String>();
        PedFile hapPed = null;
        if (integration.isHapMapSet()) {
            hapPed = new PedFile();
            hapPed.parseHapMapPed(IGG_RESOURCE_HAPMAP_FOLDER + File.separator + "pedinfor" + File.separator + "hap3Sample.txt");
        }
        List<String[]> phenotypeClasses = exportSetting.getTraitProper();
        BufferedWriter bwPedFileDescription = new BufferedWriter(new FileWriter(path + prefix + ".dat"));
        BufferedWriter bwGeneticMaps = new BufferedWriter(new FileWriter(path + prefix + ".map"));
        BufferedWriter bwAlleleFrequencies = new BufferedWriter(new FileWriter(path + prefix + ".freq"));
 
        //write the description file 
        for (int i = 0; i < phenotypeClasses.size(); i++) {
            //Phentype types:  A - affection status, T - Quantitative Trait and C - Covariate)
            if (phenotypeClasses.get(i)[1].startsWith("Affection Status")) {
                bwPedFileDescription.write("A ");
            } else if (phenotypeClasses.get(i)[1].startsWith("Quantitative Trait")) {
                bwPedFileDescription.write("T ");
            } else if (phenotypeClasses.get(i)[1].startsWith("Covariate")) {
                bwPedFileDescription.write("C ");
            }
            bwPedFileDescription.write(phenotypeClasses.get(i)[0]);
            bwPedFileDescription.write("\n");
        }
        bwGeneticMaps.write("CHROMOSOME  MARKER  POSITION\n");
        File a2NFile = new File(exportSetting.getExportPath() + File.separator + exportSetting.getPrefixFileName() + ".allele2number.txt");
        BufferedWriter bwA2N = null;
        if (exportSetting.isNumberAllele()) {
            bwA2N = new BufferedWriter(new FileWriter(a2NFile));
            bwA2N.write("Chromosome Position SNPID 1Allele 2Allele");
            bwA2N.newLine();
        }



        MerlinFiles inputer = new MerlinFiles();
        String msg;
        int index = 0;
        Map<String, LongArrayList> selectedRegions = exportSetting.getSelectedRegions();
        if (selectedRegions != null && selectedRegions.size() > 0) {
            for (int k = 0; k < chroms.size(); k++) {
                String chrom = chroms.get(k);
                chrom = chrom.substring(0, chrom.indexOf(' '));
                LongArrayList regions = selectedRegions.get(chrom);
                if (regions == null) {
                    continue;
                }
                chromSet.setChromName(chrom);

                chromSet.readAllObjects(integration.getResultPath());

                List<AnnotSNP> mainSnpMap = chromSet.getSnpInfoList();
                Map<String, StatusGtySet> mainIndivGtyMap = chromSet.getIndiviGty();
                filterSNPsbyRegion(mainSnpMap, regions);
                msg = mainSnpMap.size() + " SNPs within the SELECTED region on Chromosome " + chrom;
                msg = msg + "\nChecking Genotype Quality on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);

                preprocessSNPGty(mainSnpMap, mainIndivGtyMap, chrom);

                inputer.writePedFileDescriptionMerlin(bwPedFileDescription, mainSnpMap);
                if (geneticMapFile != null && geneticMapFile.length() > 0) {
                    inputer.writeGeneticMapsMerlin(bwGeneticMaps, mainSnpMap, geneticMapFile.replaceAll("#", chrom));
                } else {
                    inputer.writeGeneticMapsMerlin(bwGeneticMaps, mainSnpMap);
                }
                inputer.writeAlleleFrequenciesMerlin(bwAlleleFrequencies, mainSnpMap);

                msg = "Transforming Genotypes on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);
                if (exportSetting.isNumberAllele()) {
                    generalLinkPedGenotypeNumberAllele(bwA2N, mainSnpMap, mainIndivGtyMap, chrom);
                } else {
                    generalLinkPedGenotypeCharAllele(mainSnpMap, mainIndivGtyMap, chrom);
                }
                chromArray.add(chrom);
                index++;
            }
        } else {
            for (int k = 0; k < chroms.size(); k++) {
                String chrom = chroms.get(k);
                chrom = chrom.substring(0, chrom.indexOf(' '));
                chromSet.setChromName(chrom);

                chromSet.readAllObjects(integration.getResultPath());

                List<AnnotSNP> mainSnpMap = chromSet.getSnpInfoList();
                Map<String, StatusGtySet> mainIndivGtyMap = chromSet.getIndiviGty();
                msg = "Checking Genotype Quality on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);

                preprocessSNPGty(mainSnpMap, mainIndivGtyMap, chrom);

                inputer.writePedFileDescriptionMerlin(bwPedFileDescription, mainSnpMap);
                if (geneticMapFile != null && geneticMapFile.length() > 0) {
                    inputer.writeGeneticMapsMerlin(bwGeneticMaps, mainSnpMap, geneticMapFile.replaceAll("#", chrom));
                } else {
                    inputer.writeGeneticMapsMerlin(bwGeneticMaps, mainSnpMap);
                }
                inputer.writeAlleleFrequenciesMerlin(bwAlleleFrequencies, mainSnpMap);


                msg = "Transforming Genotypes on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);
                if (exportSetting.isNumberAllele()) {
                    generalLinkPedGenotypeNumberAllele(bwA2N, mainSnpMap, mainIndivGtyMap, chrom);
                } else {
                    generalLinkPedGenotypeCharAllele(mainSnpMap, mainIndivGtyMap, chrom);
                }
                chromArray.add(chrom);
            }
        }
        bwPedFileDescription.close();
        bwGeneticMaps.close();
        bwAlleleFrequencies.close();

        if (exportSetting.isNumberAllele()) {
            msg = "The original allele names and numbered allele names are saved in " + a2NFile.getCanonicalPath();
            GlobalVariables.mainView.setBriefRunningInfor(msg);
            GlobalVariables.addInforLog(msg);
            bwA2N.close();
        }

        msg = "Generating Pedigree File";
        mergeGeneralLinkPedGenotypeFiles(chromArray, path + File.separator + prefix + ".ped", hapPed);
        GlobalVariables.mainView.setBriefRunningInfor(msg);
        GlobalVariables.addInforLog(msg);
        infor = "Files are expored into " + path + "\n " + prefix + ".ped, " + prefix + ".dat, " + prefix + ".map, "
                + prefix + ".freq.";
        return infor;
    }

    public String exportHapMapGenotypeFile() throws Exception {
        List<String> chroms = integration.getChromResult();
        ChromSNPGtySet chromSet = new ChromSNPGtySet();
        String infor;

        String path = exportSetting.getExportPath();
        String prefix = exportSetting.getPrefixFileName();
        PedFile hapPed = null;
        if (integration.isHapMapSet()) {
            hapPed = new PedFile();
            hapPed.parseHapMapPed(IGG_RESOURCE_HAPMAP_FOLDER + File.separator + "pedinfor" + File.separator + "hap3Sample.txt");
        }
        HapMapFiles inputer = new HapMapFiles();
        Map<String, LongArrayList> selectedRegions = exportSetting.getSelectedRegions();
        if (selectedRegions != null && selectedRegions.size() > 0) {
            for (int k = 0; k < chroms.size(); k++) {
                String chrom = chroms.get(k);
                chrom = chrom.substring(0, chrom.indexOf(' '));
                LongArrayList regions = selectedRegions.get(chrom);
                if (regions == null) {
                    continue;
                }
                chromSet.setChromName(chrom);
                chromSet.readAllObjects(integration.getResultPath());

                List<AnnotSNP> mainSnpMap = chromSet.getSnpInfoList();
                Map<String, StatusGtySet> indivGtyMap = chromSet.getIndiviGty();

                preprocessSNPGty(mainSnpMap, indivGtyMap, chrom);
                inputer.writeHapMapGenotypeFile(path + prefix + "." + chrom + ".gty", chrom, mainSnpMap, indivGtyMap, hapPed);
            }
        } else {
            for (int k = 0; k < chroms.size(); k++) {
                String chrom = chroms.get(k);
                chrom = chrom.substring(0, chrom.indexOf(' '));
                chromSet.setChromName(chrom);
                chromSet.readAllObjects(integration.getResultPath());

                List<AnnotSNP> mainSnpMap = chromSet.getSnpInfoList();
                Map<String, StatusGtySet> indivGtyMap = chromSet.getIndiviGty();

                preprocessSNPGty(mainSnpMap, indivGtyMap, chrom);
                inputer.writeHapMapGenotypeFile(path + prefix + "." + chrom + ".gty", chrom, mainSnpMap, indivGtyMap, hapPed);

            }
        }
        infor = "Files are expored into " + path + prefix + ".#.gty(chromosome#)";
        return infor;
    }

    public String exportMega2() throws Exception {
        List<String> chroms = integration.getChromResult();
        ChromSNPGtySet chromSet = new ChromSNPGtySet();
        String infor;

        String path = exportSetting.getExportPath();
        String prefix = exportSetting.getPrefixFileName();
        List<String[]> phenotypeClasses = exportSetting.getTraitProper();
        String msg;
        PedFile hapPed = null;
        if (integration.isHapMapSet()) {
            hapPed = new PedFile();
            hapPed.parseHapMapPed(IGG_RESOURCE_HAPMAP_FOLDER + File.separator
                    + "pedinfor" + File.separator + "hap3Sample.txt");
        }
        File a2NFile = new File(exportSetting.getExportPath() + File.separator + exportSetting.getPrefixFileName() + ".allele2number.txt");
        BufferedWriter bwA2N = null;
        if (exportSetting.isNumberAllele()) {
            bwA2N = new BufferedWriter(new FileWriter(a2NFile));
            bwA2N.write("Chromosome Position SNPID 1Allele 2Allele");
            bwA2N.newLine();
        }
        Mega2Files inputer = new Mega2Files();
        Map<String, LongArrayList> selectedRegions = exportSetting.getSelectedRegions();
        if (selectedRegions != null && selectedRegions.size() > 0) {
            for (int k = 0; k < chroms.size(); k++) {
                String chrom = chroms.get(k);
                chrom = chrom.substring(0, chrom.indexOf(' '));
                LongArrayList regions = selectedRegions.get(chrom);
                if (regions == null) {
                    continue;
                }
                chromSet.setChromName(chrom);
                chromSet.readAllObjects(integration.getResultPath());

                List<AnnotSNP> mainSnpMap = chromSet.getSnpInfoList();
                Map<String, StatusGtySet> indivGtyMap = chromSet.getIndiviGty();
                filterSNPsbyRegion(mainSnpMap, regions);
                msg = mainSnpMap.size() + " SNPs within the SELECTED region on Chromosome " + chrom;
                msg = msg + "\nChecking Genotype Quality on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);

                preprocessSNPGty(mainSnpMap, indivGtyMap, chrom);

                BufferedWriter bwGeneticMaps = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".map"));
                BufferedWriter bwDataFile = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".dat"));
                inputer.orderIndivIDbyNum(hapPed, integration);
                if (geneticMapFile != null && geneticMapFile.length() > 0) {
                    inputer.writeDataFilesMega2(bwDataFile, phenotypeClasses, mainSnpMap, geneticMapFile.replaceAll("#", chrom));
                } else {
                    inputer.writeDataFilesMega2(bwDataFile, phenotypeClasses, mainSnpMap);
                }
                bwGeneticMaps.write("CHROMOSOME    cM   NAME\n");
                if (geneticMapFile != null && geneticMapFile.length() > 0) {
                    inputer.writeGeneticMapsMega2(bwGeneticMaps, mainSnpMap, geneticMapFile.replaceAll("#", chrom));
                } else {
                    inputer.writeGeneticMapsMega2(bwGeneticMaps, mainSnpMap);
                }
                msg = "Transforming Genotypes on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);
                if (exportSetting.isNumberAllele()) {
                    generalLinkPedNumberAllele(bwA2N, mainSnpMap, indivGtyMap, chrom, hapPed);
                } else {
                    generalLinkPedCharAllele(mainSnpMap, indivGtyMap, chrom, hapPed);
                }
                bwGeneticMaps.close();
                bwDataFile.close();
            }
        } else {
            for (int k = 0; k < chroms.size(); k++) {
                String chrom = chroms.get(k);
                chrom = chrom.substring(0, chrom.indexOf(' '));
                chromSet.setChromName(chrom);
                chromSet.readAllObjects(integration.getResultPath());

                List<AnnotSNP> mainSnpMap = chromSet.getSnpInfoList();
                Map<String, StatusGtySet> indivGtyMap = chromSet.getIndiviGty();
                msg = "Checking Genotype Quality on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);
                
                preprocessSNPGty(mainSnpMap, indivGtyMap, chrom);


                BufferedWriter bwGeneticMaps = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".map"));
                BufferedWriter bwDataFile = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".dat"));
                inputer.orderIndivIDbyNum(hapPed, integration);
                if (geneticMapFile != null && geneticMapFile.length() > 0) {
                    inputer.writeDataFilesMega2(bwDataFile, phenotypeClasses, mainSnpMap, geneticMapFile.replaceAll("#", chrom));
                } else {
                    inputer.writeDataFilesMega2(bwDataFile, phenotypeClasses, mainSnpMap);
                }
                bwGeneticMaps.write("CHROMOSOME cM NAME\n");
                if (geneticMapFile != null && geneticMapFile.length() > 0) {
                    inputer.writeGeneticMapsMega2(bwGeneticMaps, mainSnpMap, geneticMapFile.replaceAll("#", chrom));
                } else {
                    inputer.writeGeneticMapsMega2(bwGeneticMaps, mainSnpMap);
                }
                msg = "Transforming Genotypes on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);
                if (exportSetting.isNumberAllele()) {
                    generalLinkPedNumberAllele(bwA2N, mainSnpMap, indivGtyMap, chrom, hapPed);
                } else {
                    generalLinkPedCharAllele(mainSnpMap, indivGtyMap, chrom, hapPed);
                }
                bwGeneticMaps.close();
                bwDataFile.close();
            }
        }
        infor = "Files are expored into " + path + "(chromosome#)\n" + prefix + ".#.map " + prefix + ".#.dat " + prefix + ".#.ped";


        if (exportSetting.isNumberAllele()) {
            msg = "The original allele names and numbered allele names are saved in " + a2NFile.getCanonicalPath();
            GlobalVariables.mainView.setBriefRunningInfor(msg);
            GlobalVariables.addInforLog(msg);
            bwA2N.close();
        }
        return infor;
    }

    public String exportLinkage() throws Exception {
        List<String> chroms = integration.getChromResult();
        ChromSNPGtySet chromSet = new ChromSNPGtySet();
        String infor;

        String path = exportSetting.getExportPath();
        String prefix = exportSetting.getPrefixFileName();
        List<String[]> phenotypeClasses = exportSetting.getTraitProper();
        String msg;
        PedFile hapPed = null;
        if (integration.isHapMapSet()) {
            hapPed = new PedFile();
            hapPed.parseHapMapPed(IGG_RESOURCE_HAPMAP_FOLDER + File.separator + "pedinfor" + File.separator + "hap3Sample.txt");
        }
        File a2NFile = new File(exportSetting.getExportPath() + File.separator + exportSetting.getPrefixFileName() + ".allele2number.txt");
        BufferedWriter bwA2N = null;
        if (exportSetting.isNumberAllele()) {
            bwA2N = new BufferedWriter(new FileWriter(a2NFile));
            bwA2N.write("Chromosome Position SNPID 1Allele 2Allele");
            bwA2N.newLine();
        }

        LinkageFiles inputer = new LinkageFiles();
        Map<String, LongArrayList> selectedRegions = exportSetting.getSelectedRegions();
        if (selectedRegions != null && selectedRegions.size() > 0) {
            for (int k = 0; k < chroms.size(); k++) {
                String chrom = chroms.get(k);
                chrom = chrom.substring(0, chrom.indexOf(' '));
                LongArrayList regions = selectedRegions.get(chrom);
                if (regions == null) {
                    continue;
                }
                chromSet.setChromName(chrom);
                chromSet.readAllObjects(integration.getResultPath());

                List<AnnotSNP> mainSnpMap = chromSet.getSnpInfoList();
                Map<String, StatusGtySet> indivGtyMap = chromSet.getIndiviGty();
                filterSNPsbyRegion(mainSnpMap, regions);
                msg = mainSnpMap.size() + " SNPs within the SELECTED region on Chromosome " + chrom;
                msg = msg + "\nChecking Genotype Quality on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);

                preprocessSNPGty(mainSnpMap, indivGtyMap, chrom);


                BufferedWriter bwDataFile = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".dat"));
                if (geneticMapFile != null && geneticMapFile.length() > 0) {
                    inputer.writeDataFileLinkage(bwDataFile, phenotypeClasses, mainSnpMap, geneticMapFile.replaceAll("#", chrom));
                } else {
                    inputer.writeDataFileLinkage(bwDataFile, phenotypeClasses, mainSnpMap);
                }

                msg = "Transforming Genotypes on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);
                if (exportSetting.isNumberAllele()) {
                    generalLinkPedNumberAllele(bwA2N, mainSnpMap, indivGtyMap, chrom, hapPed);
                } else {
                    generalLinkPedCharAllele(mainSnpMap, indivGtyMap, chrom, hapPed);
                }
                bwDataFile.close();
            }
        } else {
            for (int k = 0; k < chroms.size(); k++) {
                String chrom = chroms.get(k);
                chrom = chrom.substring(0, chrom.indexOf(' '));
                chromSet.setChromName(chrom);
                chromSet.readAllObjects(integration.getResultPath());

                List<AnnotSNP> mainSnpMap = chromSet.getSnpInfoList();
                Map<String, StatusGtySet> indivGtyMap = chromSet.getIndiviGty();
                msg = "Checking Genotype Quality on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);
                preprocessSNPGty(mainSnpMap, indivGtyMap, chrom);


                BufferedWriter bwDataFile = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".dat"));
                if (geneticMapFile != null && geneticMapFile.length() > 0) {
                    inputer.writeDataFileLinkage(bwDataFile, phenotypeClasses, mainSnpMap, geneticMapFile.replaceAll("#", chrom));
                } else {
                    inputer.writeDataFileLinkage(bwDataFile, phenotypeClasses, mainSnpMap);
                }

                msg = "Transforming Genotypes on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);
                if (exportSetting.isNumberAllele()) {
                    generalLinkPedNumberAllele(bwA2N, mainSnpMap, indivGtyMap, chrom, hapPed);
                } else {
                    generalLinkPedCharAllele(mainSnpMap, indivGtyMap, chrom, hapPed);
                }
                bwDataFile.close();
            }
        }
        if (exportSetting.isNumberAllele()) {
            msg = "The original allele names and numbered allele names are saved in " + a2NFile.getCanonicalPath();
            GlobalVariables.mainView.setBriefRunningInfor(msg);
            GlobalVariables.addInforLog(msg);
            bwA2N.close();
        }
        infor = "Files are expored into " + path + "(chromosome#)\n" + prefix + ".#.dat " + prefix + ".#.ped";
        return infor;
    }

    public String exportSolar() throws Exception {
        List<String> chroms = integration.getChromResult();
        ChromSNPGtySet chromSet = new ChromSNPGtySet();
        String infor;

        String path = exportSetting.getExportPath();
        String prefix = exportSetting.getPrefixFileName();
        List<String[]> phenotypeClasses = exportSetting.getTraitProper();
        String msg;
        PedFile hapPed = null;
        if (integration.isHapMapSet()) {
            hapPed = new PedFile();
            hapPed.parseHapMapPed(IGG_RESOURCE_HAPMAP_FOLDER + File.separator + "pedinfor" + File.separator + "hap3Sample.txt");
        }
        SolarFiles inputer = new SolarFiles();
        BufferedWriter bwPedFile = new BufferedWriter(new FileWriter(path + prefix + ".ped"));
        BufferedWriter bwPhenoFile = new BufferedWriter(new FileWriter(path + prefix + ".phen"));
        inputer.writePedigreeFileSolar(bwPedFile, integration, hapPed);
        inputer.writePhenotypeFileSolar(bwPhenoFile, integration, hapPed, phenotypeClasses);
        Map<String, LongArrayList> selectedRegions = exportSetting.getSelectedRegions();
        bwPedFile.close();
        bwPhenoFile.close();

        if (selectedRegions != null && selectedRegions.size() > 0) {
            for (int k = 0; k < chroms.size(); k++) {
                String chrom = chroms.get(k);
                chrom = chrom.substring(0, chrom.indexOf(' '));
                LongArrayList regions = selectedRegions.get(chrom);
                if (regions == null) {
                    continue;
                }
                chromSet.setChromName(chrom);
                chromSet.readAllObjects(integration.getResultPath());

                List<AnnotSNP> mainSnpMap = chromSet.getSnpInfoList();
                Map<String, StatusGtySet> indivGtyMap = chromSet.getIndiviGty();
                filterSNPsbyRegion(mainSnpMap, regions);
                msg = mainSnpMap.size() + " SNPs within the SELECTED region on Chromosome " + chrom;
                msg = msg + "\nChecking Genotype Quality on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);

                preprocessSNPGty(mainSnpMap, indivGtyMap, chrom);


                BufferedWriter bwMarkerFile = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".mark"));
                BufferedWriter bwGeneticMapFile = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".map"));
                BufferedWriter bwAlleleFrequencyFile = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".freq"));

                inputer.writeGeneticMapFileSolar(bwGeneticMapFile, chrom, mainSnpMap);
                inputer.writeAlleleFrequencyFileSolar(bwAlleleFrequencyFile, mainSnpMap);
                msg = "Transforming Genotypes on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                inputer.writeMarkerFileSolar(bwMarkerFile, mainSnpMap, indivGtyMap, integration, chrom, exportSetting, hapPed);
                bwMarkerFile.close();
                bwGeneticMapFile.close();
                bwAlleleFrequencyFile.close();
            }
        } else {
            for (int k = 0; k < chroms.size(); k++) {
                String chrom = chroms.get(k);
                chrom = chrom.substring(0, chrom.indexOf(' '));
                chromSet.setChromName(chrom);
                chromSet.readAllObjects(integration.getResultPath());

                List<AnnotSNP> mainSnpMap = chromSet.getSnpInfoList();
                Map<String, StatusGtySet> indivGtyMap = chromSet.getIndiviGty();
                msg = "Checking Genotype Quality on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);
                preprocessSNPGty(mainSnpMap, indivGtyMap, chrom);

                BufferedWriter bwMarkerFile = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".mark"));
                BufferedWriter bwGeneticMapFile = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".map"));
                BufferedWriter bwAlleleFrequencyFile = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".freq"));

                inputer.writeGeneticMapFileSolar(bwGeneticMapFile, chrom, mainSnpMap);
                inputer.writeAlleleFrequencyFileSolar(bwAlleleFrequencyFile, mainSnpMap);
                msg = "Transforming Genotypes on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                inputer.writeMarkerFileSolar(bwMarkerFile, mainSnpMap, indivGtyMap, integration, chrom, exportSetting, hapPed);
                bwMarkerFile.close();
                bwGeneticMapFile.close();
                bwAlleleFrequencyFile.close();
            }
        }
        infor = "Files are expored into " + path + "(chromosome#)\n" + prefix + ".#.ped " + prefix + ".#.info.";
        return infor;
    }

    public String exportBIMBAM() throws Exception {
        List<String> chroms = integration.getChromResult();
        ChromSNPGtySet chromSet = new ChromSNPGtySet();
        String infor;

        String path = exportSetting.getExportPath();
        String prefix = exportSetting.getPrefixFileName();

        String msg;
        PedFile hapPed = null;
        BIMBAMFiles inputer = new BIMBAMFiles();
        if (integration.isHapMapSet()) {
            hapPed = new PedFile();
            hapPed.parseHapMapPed(IGG_RESOURCE_HAPMAP_FOLDER + File.separator + "pedinfor" + File.separator + "hap3Sample.txt");
            inputer.writeHapMapPhenotypeFileBIMBAM(integration, exportSetting);
        }

        inputer.writeStudyPhenotypeFileBIMBAM(exportSetting);
        Map<String, LongArrayList> selectedRegions = exportSetting.getSelectedRegions();

        if (selectedRegions != null && selectedRegions.size() > 0) {
            for (int k = 0; k < chroms.size(); k++) {
                String chrom = chroms.get(k);
                chrom = chrom.substring(0, chrom.indexOf(' '));
                LongArrayList regions = selectedRegions.get(chrom);
                if (regions == null) {
                    continue;
                }
                chromSet.setChromName(chrom);
                chromSet.readAllObjects(integration.getResultPath());

                List<AnnotSNP> mainSnpMap = chromSet.getSnpInfoList();
                Map<String, StatusGtySet> indivGtyMap = chromSet.getIndiviGty();
                filterSNPsbyRegion(mainSnpMap, regions);
                msg = mainSnpMap.size() + " SNPs within the SELECTED region on Chromosome " + chrom;
                msg = msg + "\nChecking Genotype Quality on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);
                preprocessSNPGty(mainSnpMap, indivGtyMap, chrom);

                msg = "Transforming Genotypes on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);

                if (hapPed != null) {
                    BufferedWriter bwHapMarkerFile = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".hapmap.geno"));
                    inputer.writeHapMapGenotypeFileBIMBAM(bwHapMarkerFile, mainSnpMap, indivGtyMap, integration, exportSetting, hapPed);
                    bwHapMarkerFile.flush();
                    bwHapMarkerFile.close();
                }

                BufferedWriter bwStudyMarkerFile = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".geno"));
                inputer.writeStudyGenotypeFileBIMBAM(bwStudyMarkerFile, mainSnpMap, indivGtyMap, integration, exportSetting);
                bwStudyMarkerFile.close();
                BufferedWriter bwPosFile = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".pos"));
                inputer.writePositionMapFileBIMBAM(bwPosFile, mainSnpMap);
                bwPosFile.close();
            }
        } else {
            for (int k = 0; k < chroms.size(); k++) {
                String chrom = chroms.get(k);
                chrom = chrom.substring(0, chrom.indexOf(' '));
                chromSet.setChromName(chrom);
                chromSet.readAllObjects(integration.getResultPath());

                List<AnnotSNP> mainSnpMap = chromSet.getSnpInfoList();
                Map<String, StatusGtySet> indivGtyMap = chromSet.getIndiviGty();
                msg = "Checking Genotype Quality on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);
                preprocessSNPGty(mainSnpMap, indivGtyMap, chrom);

                msg = "Transforming Genotypes on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);

                if (hapPed != null) {
                    BufferedWriter bwHapMarkerFile = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".hapmap.geno"));
                    inputer.writeHapMapGenotypeFileBIMBAM(bwHapMarkerFile, mainSnpMap, indivGtyMap, integration, exportSetting, hapPed);
                    bwHapMarkerFile.flush();
                    bwHapMarkerFile.close();
                }
                BufferedWriter bwStudyMarkerFile = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".geno"));
                inputer.writeStudyGenotypeFileBIMBAM(bwStudyMarkerFile, mainSnpMap, indivGtyMap, integration, exportSetting);
                bwStudyMarkerFile.close();

                BufferedWriter bwPosFile = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".pos"));
                inputer.writePositionMapFileBIMBAM(bwPosFile, mainSnpMap);
                bwPosFile.close();
            }
        }
        infor = "Files are expored into " + path + "(chromosome#)\n" + prefix + ".#.geno " + prefix + ".#.pos " + prefix + ".(trait).pheno ";
        if (hapPed != null) {
            infor += (prefix + ".hapmap.#.geno ");
            infor += (prefix + ".(trait).hapmap.pheno");
        }
        return infor;
    }

    public String exportHaploview() throws Exception {
        List<String> chroms = integration.getChromResult();
        ChromSNPGtySet chromSet = new ChromSNPGtySet();
        String infor;

        String path = exportSetting.getExportPath();
        String prefix = exportSetting.getPrefixFileName();

        String msg;
        PedFile hapPed = null;
        if (integration.isHapMapSet()) {
            hapPed = new PedFile();
            hapPed.parseHapMapPed(IGG_RESOURCE_HAPMAP_FOLDER + File.separator + "pedinfor" + File.separator + "hap3Sample.txt");
        }
        HaploviewFiles inputer = new HaploviewFiles();
        Map<String, LongArrayList> selectedRegions = exportSetting.getSelectedRegions();
        File a2NFile = new File(exportSetting.getExportPath() + File.separator + exportSetting.getPrefixFileName() + ".allele2number.txt");
        BufferedWriter bwA2N = null;
        if (exportSetting.isNumberAllele()) {
            bwA2N = new BufferedWriter(new FileWriter(a2NFile));
            bwA2N.write("Chromosome Position SNPID 1Allele 2Allele");
            bwA2N.newLine();
        }

        if (selectedRegions != null && selectedRegions.size() > 0) {
            for (int k = 0; k < chroms.size(); k++) {
                String chrom = chroms.get(k);
                chrom = chrom.substring(0, chrom.indexOf(' '));
                LongArrayList regions = selectedRegions.get(chrom);
                if (regions == null) {
                    continue;
                }
                chromSet.setChromName(chrom);
                chromSet.readAllObjects(integration.getResultPath());

                List<AnnotSNP> mainSnpMap = chromSet.getSnpInfoList();
                Map<String, StatusGtySet> indivGtyMap = chromSet.getIndiviGty();
                filterSNPsbyRegion(mainSnpMap, regions);
                msg = mainSnpMap.size() + " SNPs within the SELECTED region on Chromosome " + chrom;
                msg = msg + "\nChecking Genotype Quality on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);
                preprocessSNPGty(mainSnpMap, indivGtyMap, chrom);

                BufferedWriter bwGeneticMapFile = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".info"));
                inputer.writeMarkerInformationHaploview(bwGeneticMapFile, mainSnpMap);
                msg = "Transforming Genotypes on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                if (exportSetting.isNumberAllele()) {
                    generalLinkPedNumberAllele(bwA2N, mainSnpMap, indivGtyMap, chrom, hapPed);
                } else {
                    generalLinkPedCharAllele(mainSnpMap, indivGtyMap, chrom, hapPed);
                }
                bwGeneticMapFile.close();

            }
        } else {
            for (int k = 0; k < chroms.size(); k++) {
                String chrom = chroms.get(k);
                chrom = chrom.substring(0, chrom.indexOf(' '));
                chromSet.setChromName(chrom);
                chromSet.readAllObjects(integration.getResultPath());

                List<AnnotSNP> mainSnpMap = chromSet.getSnpInfoList();
                Map<String, StatusGtySet> indivGtyMap = chromSet.getIndiviGty();
                msg = "Checking Genotype Quality on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                GlobalVariables.addInforLog(msg);
                preprocessSNPGty(mainSnpMap, indivGtyMap, chrom);



                BufferedWriter bwGeneticMapFile = new BufferedWriter(new FileWriter(path + prefix + "." + chrom + ".info"));
                inputer.writeMarkerInformationHaploview(bwGeneticMapFile, mainSnpMap);
                msg = "Transforming Genotypes on Chromosome " + chrom;
                GlobalVariables.mainView.setBriefRunningInfor(msg);
                if (exportSetting.isNumberAllele()) {
                    generalLinkPedNumberAllele(bwA2N, mainSnpMap, indivGtyMap, chrom, hapPed);
                } else {
                    generalLinkPedCharAllele(mainSnpMap, indivGtyMap, chrom, hapPed);
                }
                bwGeneticMapFile.close();
            }
        }
        if (exportSetting.isNumberAllele()) {
            msg = "The original allele names and numbered allele names are saved in " + a2NFile.getCanonicalPath();
            GlobalVariables.mainView.setBriefRunningInfor(msg);
            GlobalVariables.addInforLog(msg);
            bwA2N.close();
        }
        infor = "Files are expored into " + path + "(chromosome#)\n" + prefix + ".ped " + prefix + ".phen " + prefix + ".#.mark " + prefix + ".#.map " + prefix + ".#.freq.";
        return infor;
    }
}
