// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.
// Permission is granted for you to use this file to compile IGG.
// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.controller;

import cern.colt.bitvector.BitVector;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.TreeSet;
import org.apache.commons.collections.map.MultiValueMap;
import org.cobi.igg3.entity.AnnotSNP;
import org.cobi.igg3.entity.ChromSNPGtySet;
import org.cobi.igg3.entity.EncodedChipFileSet;
import org.cobi.igg3.entity.Integration;
import org.cobi.igg3.entity.Project;
import org.cobi.igg3.entity.SNPPosiComparator;
import org.cobi.igg3.entity.StatusGtySet;
import org.cobi.igg3.GlobalVariables;

/**
 *
 * @author mxli
 */
public class WholeGenomeChipIntegrator extends BasicIntegrator {

    int bufferSize = DEFAULT_BUF_SIZE;

    public WholeGenomeChipIntegrator() {
        this.bufferSize = DEFAULT_BUF_SIZE;
    }

    public WholeGenomeChipIntegrator(int bufferSize) {
        this.bufferSize = bufferSize;
    }

    private ArrayList<MultiValueMap> spliteSubjects(MultiValueMap chipFileMap) throws Exception {
        Iterator chipTypes = chipFileMap.keySet().iterator();
        MultiValueMap chipIndivFile = new MultiValueMap();

        //statistic individual numbers
        while (chipTypes.hasNext()) {
            String chipName = chipTypes.next().toString();
            Iterator iter = chipFileMap.getCollection(chipName).iterator();
            while (iter.hasNext()) {
                String chipFile = (String) iter.next();
                if (chipFile.endsWith("*.*")) {
                    String filePath = chipFile.substring(0, chipFile.lastIndexOf(File.separator));
                    File chipFiles = new File(filePath);
                    String[] chipFileList = chipFiles.list();
                    for (int i = 0; i < chipFileList.length; i++) {
                        if (chipName.startsWith("Affy")) {
                            getIndivChipLabelAffy(chipName, filePath + File.separator + chipFileList[i], chipIndivFile);
                        } else {
                            getIndivChipLabelIllum(chipName, filePath + File.separator + chipFileList[i], chipIndivFile);
                        }
                    }
                } else {
                    if (chipName.startsWith("Affy")) {
                        getIndivChipLabelAffy(chipName, chipFile, chipIndivFile);
                    } else {
                        getIndivChipLabelIllum(chipName, chipFile, chipIndivFile);
                    }
                }
            }
        }

        //split individuals
        int indivCounter = 0;
        MultiValueMap splitedChipIndivFile = new MultiValueMap();
        chipTypes = chipIndivFile.keySet().iterator();
        ArrayList<MultiValueMap> allIndivFiles = new ArrayList<MultiValueMap>();
        for (int k = ALL_CHIP_InSizeOrder.length - 1; k >= 0; k--) {
            Collection fileCollection = chipIndivFile.getCollection(ALL_CHIP_InSizeOrder[k]);
            if (fileCollection == null) {
                continue;
            }
            Iterator iter = fileCollection.iterator();
            while (iter.hasNext()) {
                indivCounter++;
                if (indivCounter == bufferSize) {
                    splitedChipIndivFile.put(ALL_CHIP_InSizeOrder[k], iter.next());
                    allIndivFiles.add(splitedChipIndivFile);
                    splitedChipIndivFile = new MultiValueMap();
                    indivCounter = 0;
                } else {
                    splitedChipIndivFile.put(ALL_CHIP_InSizeOrder[k], iter.next());
                }
            }
        }

        if (indivCounter != 0) {
            allIndivFiles.add(splitedChipIndivFile);
        }
        return allIndivFiles;
    }

    private void getIndivChipLabelAffy(String chipName, String filePath, MultiValueMap chipIndivFile) throws Exception {
        BufferedReader br = new BufferedReader(new FileReader(filePath), 1024);
        String line;
        String indivLabel;
        //read chipLabel 
        if ((line = br.readLine()) != null) {
            indivLabel = line.split(":")[1].trim();
            if (indivLabel.length() == 0) {
                throw new Exception("Cannot read chip label in chip file, " + filePath);
            }
            String[] item = new String[2];
            item[0] = indivLabel;
            item[1] = filePath;
            chipIndivFile.put(chipName, item);
        }
        br.close();
    }

    private void getIndivChipLabelIllum(String chipName, String filePath, MultiValueMap chipIndivFile) throws Exception {
        String line = null;
        StringTokenizer tokenizer = null;
        String delmilit = "\t ,";
        String startLabel = "Sample Names:";

        TreeSet<String> indivLabelSet = new TreeSet<String>();
        String indivLabel;
        BufferedReader br = new BufferedReader(new FileReader(filePath), FILE_READER_BUFFER_SIZE);
        StringBuilder tmpBuffer = new StringBuilder();
        if ((line = br.readLine()) != null) {
            if (line.trim().startsWith(startLabel)) {
                line = line.trim();
                line = line.substring(line.indexOf("s:") + 2).trim();
                tokenizer = new StringTokenizer(line, delmilit);
                while (tokenizer.hasMoreTokens()) {
                    tmpBuffer.delete(0, tmpBuffer.length());
                    tmpBuffer.append(tokenizer.nextToken().trim());
                    indivLabel = tmpBuffer.toString();
                    if (!indivLabelSet.contains(indivLabel)) {
                        indivLabelSet.add(indivLabel);
                        String[] item = new String[2];
                        item[0] = indivLabel;
                        item[1] = filePath;
                        chipIndivFile.put(chipName, item);
                    }
                }
                return;
            }
        }

        //if no smaple names line
        startLabel = "rs";
        while ((line = br.readLine()) != null) {
            if (line.trim().startsWith(startLabel)) {
                break;
            }
        }

        do {
            if (line.trim().length() == 0) {
                continue;
            }
            tokenizer = new StringTokenizer(line, delmilit);
            tokenizer.nextToken();
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            indivLabel = tmpBuffer.toString();
            if (!indivLabelSet.contains(indivLabel)) {
                indivLabelSet.add(indivLabel);
                String[] item = new String[2];
                item[0] = indivLabel;
                item[1] = filePath;
                chipIndivFile.put(chipName, item);
            }
        } while ((line = br.readLine()) != null);
        br.close();
    }

    public boolean integrateChipFiles(Integration intgration) throws Exception {
        boolean succeed = false;
        Project prg = GlobalVariables.currentProject;
        MultiValueMap chipFileMaps = prg.getChipFilePathList();
        String msg = "Counting chip number...";
        GlobalVariables.mainView.setBriefRunningInfor(msg, true);
        GlobalVariables.addInforLog(msg);
        ArrayList<MultiValueMap> splittedChipFileMaps = spliteSubjects(chipFileMaps);
        TreeSet<String> indivChipLabelInPed = GlobalVariables.currentPedFile.extractIndivChipLabels();
        int blockNum = splittedChipFileMaps.size();
        msg = "These files will be integrated in " + blockNum + " buffers sequentially.";
        GlobalVariables.mainView.setBriefRunningInfor(msg, true);
        GlobalVariables.addInforLog(msg);
        //integrate the chip files
        for (int blockN = 0; blockN < blockNum; blockN++) {
            msg = "Buffer " + (blockN + 1);
            GlobalVariables.mainView.setBriefRunningInfor(msg, true);
            GlobalVariables.addInforLog(msg);
            //encode the genotypes    
            Iterator chipTypes = splittedChipFileMaps.get(blockN).keySet().iterator();
            Map<String, EncodedChipFileSet> encodedChipFileMap = new HashMap<String, EncodedChipFileSet>();
            while (chipTypes.hasNext()) {
                String chipName = chipTypes.next().toString();
                EncodedChipFileSet encodeSet = new EncodedChipFileSet(GlobalVariables.initialChipBuff.get(chipName));
                if (chipName.startsWith("Affy")) {
                    encodeSet.readEncodeAffyChipFiles(splittedChipFileMaps.get(blockN).getCollection(chipName), indivChipLabelInPed);
                } else {
                    encodeSet.readEncodeIllumChipFiles(splittedChipFileMaps.get(blockN).getCollection(chipName), indivChipLabelInPed);
                }
                encodedChipFileMap.put(chipName, encodeSet);
            }
            mergeEnodedGenotypesbyList(intgration, encodedChipFileMap, blockN, blockNum);
            encodedChipFileMap.clear();
            System.gc();
        }

        //merge the chip files       
        if (blockNum > 1) {
            msg = "Merging genotypes in " + blockNum + " buffers ...";
            GlobalVariables.mainView.setBriefRunningInfor(msg, true);
            GlobalVariables.addInforLog(msg);
            File resultPath = new File(intgration.getResultPath() + File.separator);
            for (int i = 0; i < CHROM_NAME.length; i++) {
                String chrom = CHROM_NAME[i];
                if (intgration.getPhysicalRegionMap().get(chrom) == null &&
                        intgration.getGenetRegionMap().get(chrom) == null) {
                    continue;
                }
                ChromSNPGtySet chromSet0 = new ChromSNPGtySet();
                chromSet0.setChromName(chrom);
                chromSet0.readAllBufObjectsAndDelete(resultPath.getCanonicalPath(), 0);

                List<AnnotSNP> mergedSnpMap = chromSet0.getSnpInfoList();
                Map<String, StatusGtySet> mergedIndivGty = chromSet0.getIndiviGty();

                for (int blockN = 1; blockN < blockNum; blockN++) {
                    ChromSNPGtySet chromSet = new ChromSNPGtySet();
                    chromSet.setChromName(chrom);
                    chromSet.readAllBufObjectsAndDelete(resultPath.getCanonicalPath(), blockN);

                    List<AnnotSNP> tmpSnpMap2 = chromSet.getSnpInfoList();
                    Map<String, StatusGtySet> tmpIndivGty = chromSet.getIndiviGty();
                    mergeSNPGtybyListNoGtyExtend(mergedSnpMap, tmpSnpMap2, mergedIndivGty, tmpIndivGty);

                    tmpSnpMap2.clear();
                    tmpIndivGty.clear();
                    // GlobalVariables.mainView.setBriefRunningInfor(runningInfo, true);
                    // GlobalVariables.addInforLog(runningInfo);
                }

                msg = "Chromosome " + chrom + " is finished!";
                intgration.addAChromResult(chrom + " SNP#" + mergedSnpMap.size());
                GlobalVariables.mainView.setBriefRunningInfor(msg, true);
                GlobalVariables.addInforLog(msg);
                chromSet0.saveAllObjectsAndClear(resultPath.getCanonicalPath(), DEFAULT_GTY_BLOCKNUM);
                mergedSnpMap.clear();
                mergedIndivGty.clear();
                System.gc();
            }
        }
        return succeed;
    }

    public void mergeEnodedGenotypesbyList(Integration intgration, Map<String, EncodedChipFileSet> encodedChipFileMap, int blockN, int blockNum) throws Exception {
        Map<String, AnnotSNP> tmpSnpMap = new HashMap<String, AnnotSNP>();
        List<AnnotSNP> tmpSnpList = new ArrayList<AnnotSNP>();
        List<AnnotSNP> mergedSnpList = new ArrayList<AnnotSNP>();
        Map<String, StatusGtySet> tmpIndivGty = new HashMap<String, StatusGtySet>();
        Map<String, StatusGtySet> mergedIndivGty = new HashMap<String, StatusGtySet>();

        ChromSNPGtySet chromSet = new ChromSNPGtySet();
        String runningInfo;
        int populationID = 9;
        if ("Caucasian Population".equals(intgration.getFreqPopulation())) {
            populationID = 11;
        } else if ("African Population".equals(intgration.getFreqPopulation())) {
            populationID = 10;
        }

        File resultPath = new File(intgration.getResultPath() + File.separator);
        if (!resultPath.exists()) {
            resultPath.mkdir();
        }

        //read genotypes accoding chromsome
        for (int i = 0; i < CHROM_NAME.length; i++) {
            String chrom = CHROM_NAME[i];
            if (intgration.getPhysicalRegionMap().get(chrom) == null &&
                    intgration.getGenetRegionMap().get(chrom) == null) {
                continue;
            }
            chromSet.setChromName(chrom);
            for (int j = 0; j < AFFY_CHIPS.length; j++) {
                EncodedChipFileSet chipSet = encodedChipFileMap.get(AFFY_CHIPS[j]);
                if (chipSet == null) {
                    continue;
                }
                if (intgration.getGenetRegionMap().isEmpty()) {
                    retrieveChipSNPAnnotationByPysic(AFFY_CHIPS[j], tmpSnpMap, 0,
                            populationID, chrom, intgration.getPhysicalRegionMap().get(chrom));
                } else {
                    retrieveChipSNPAnnotationByGenet(AFFY_CHIPS[j], tmpSnpMap, 0,
                            populationID, chrom, intgration.getGenetRegionMap().get(chrom));
                }
                if (tmpSnpMap.isEmpty()) {
                    continue;
                }
                //System.out.print("Chrom. "+CHROM_NAME[i]+" "+AFFY_CHIPS[j]+" ");
                selectChipGenotypes(chipSet, tmpSnpMap, tmpIndivGty);
                tmpSnpList.clear();
                tmpSnpList.addAll(tmpSnpMap.values());
                tmpSnpMap.clear();
                Collections.sort(tmpSnpList, new SNPPosiComparator());

                if (mergedSnpList.isEmpty()) {
                    mergedSnpList.addAll(tmpSnpList);
                    //copy the genotypes
                    mergedIndivGty.putAll(tmpIndivGty);
                    tmpSnpList.clear();
                    tmpIndivGty.clear();
                } else {
                    runningInfo = mergeSNPGtybyListNoGtyExtend(mergedSnpList, tmpSnpList, mergedIndivGty, tmpIndivGty);
                    tmpSnpList.clear();
                    tmpIndivGty.clear();
                    GlobalVariables.mainView.setBriefRunningInfor(runningInfo, true);
                    GlobalVariables.addInforLog(runningInfo);
                }
            }

            for (int j = 0; j < ILLUMINA_CHIPS.length; j++) {
                EncodedChipFileSet chipSet = encodedChipFileMap.get(ILLUMINA_CHIPS[j]);
                if (chipSet == null) {
                    continue;
                }

                if (intgration.getGenetRegionMap().isEmpty()) {
                    retrieveChipSNPAnnotationByPysic(ILLUMINA_CHIPS[j], tmpSnpMap, 1, populationID, chrom,
                            intgration.getPhysicalRegionMap().get(chrom));
                } else {
                    retrieveChipSNPAnnotationByGenet(ILLUMINA_CHIPS[j], tmpSnpMap, 1, populationID, chrom,
                            intgration.getGenetRegionMap().get(chrom));
                }
                if (tmpSnpMap.isEmpty()) {
                    continue;
                }
                //System.out.print("Chrom. "+CHROM_NAME[i]+" "+ILLUMINA_CHIPS[j]+" ");
                selectChipGenotypes(chipSet, tmpSnpMap, tmpIndivGty);
                tmpSnpList.clear();
                tmpSnpList.addAll(tmpSnpMap.values());
                tmpSnpMap.clear();
                Collections.sort(tmpSnpList, new SNPPosiComparator());
                if (mergedSnpList.isEmpty()) {
                    mergedSnpList.addAll(tmpSnpList);
                    //copy the genotypes
                    mergedIndivGty.putAll(tmpIndivGty);
                    tmpSnpList.clear();
                    tmpIndivGty.clear();
                } else {
                    runningInfo = mergeSNPGtybyListNoGtyExtend(mergedSnpList, tmpSnpList, mergedIndivGty, tmpIndivGty);
                    tmpSnpList.clear();
                    tmpIndivGty.clear();
                    GlobalVariables.mainView.setBriefRunningInfor(runningInfo, true);
                    GlobalVariables.addInforLog(runningInfo);
                }
            }

            chromSet.setSnpInfoList(mergedSnpList);
            chromSet.setIndiviGty(mergedIndivGty);

            if (blockNum == 1) {
                intgration.addAChromResult(chrom + " SNP#" + mergedSnpList.size());
                chromSet.saveAllObjectsAndClear(resultPath.getCanonicalPath(), DEFAULT_GTY_BLOCKNUM);
            } else {
                chromSet.saveAllBufObjectsAndClear(resultPath.getCanonicalPath(), blockN);
            }
            mergedSnpList.clear();
            mergedIndivGty.clear();
        }

    }

    public void selectChipGenotypes(EncodedChipFileSet chipSet, Map<String, AnnotSNP> snpMapList, Map<String, StatusGtySet> selectedIndivGty) throws Exception {
        int gtyNum = snpMapList.size();
        int smallSetSnpGtyP, largeSetSNPGtyP;
        Map<String, Integer> largeSNPSet = chipSet.getSnpMap();

        //System.out.println( gtyNum+" "+chipSet.indiviGtyMap.size());
        for (Map.Entry<String, StatusGtySet> m : chipSet.indiviGtyMap.entrySet()) {
            String indivLabel = m.getKey();
            //get individuals
            StatusGtySet indi = selectedIndivGty.get(indivLabel);
            if (indi == null) {
                indi = new StatusGtySet();
                indi.paternalChrom = new BitVector(gtyNum);
                indi.maternalChrom = new BitVector(gtyNum);
                indi.existence = new BitVector(gtyNum);
                selectedIndivGty.put(indivLabel, indi);
            }
            StatusGtySet largeIndi = m.getValue();
            for (Map.Entry<String, AnnotSNP> snp : snpMapList.entrySet()) {
                Integer snpP = largeSNPSet.get(snp.getKey());
                if (snpP == null) {
                    continue;
                }
                smallSetSnpGtyP = snp.getValue().order;
                largeSetSNPGtyP = snpP;


                if (largeIndi.existence.getQuick(largeSetSNPGtyP)) {
                    indi.paternalChrom.putQuick(smallSetSnpGtyP, largeIndi.paternalChrom.getQuick(largeSetSNPGtyP));
                    indi.maternalChrom.putQuick(smallSetSnpGtyP, largeIndi.maternalChrom.getQuick(largeSetSNPGtyP));
                    indi.existence.putQuick(smallSetSnpGtyP, true);
                }
            }
        }
    }

    public void mergeBuffers(String rootPath) throws Exception {
        int blockNum = 10;
        String msg = "Merging genotypes in " + blockNum + " buffers ...";
        //rootPath = "D:\\home\\mxli\\MyJava\\IGG3\\InputSamples\\Integrate1";
        File resultPath = new File(rootPath + File.separator);
        for (int i = 0; i < CHROM_NAME.length; i++) {
            String chrom = CHROM_NAME[i];
            File path = new File(resultPath.getCanonicalPath() + File.separator + chrom + ".snp.buf0");
            if (!path.exists()) {
                continue;
            }
            msg = "Chromosome " + chrom;
            System.out.println(msg);

            ChromSNPGtySet chromSet0 = new ChromSNPGtySet();
            chromSet0.setChromName(chrom);
            chromSet0.readAllBufObjectsAndDelete(resultPath.getCanonicalPath(), 0);

            List<AnnotSNP> mergedSnpMap = chromSet0.getSnpInfoList();
            Map<String, StatusGtySet> mergedIndivGty = chromSet0.getIndiviGty();

            for (int blockN = 1; blockN < blockNum; blockN++) {
                ChromSNPGtySet chromSet = new ChromSNPGtySet();
                chromSet.setChromName(chrom);
                chromSet.readAllBufObjectsAndDelete(resultPath.getCanonicalPath(), blockN);

                List<AnnotSNP> tmpSnpMap2 = chromSet.getSnpInfoList();
                Map<String, StatusGtySet> tmpIndivGty = chromSet.getIndiviGty();
                mergeSNPGtybyListNoGtyExtend(mergedSnpMap, tmpSnpMap2, mergedIndivGty, tmpIndivGty);

                tmpSnpMap2.clear();
                tmpIndivGty.clear();
            }
            msg = "Chromosome " + chrom + " is finished!";


            System.out.println(msg + " And saved in " + resultPath.getCanonicalPath());

            chromSet0.saveAllObjectsAndClear(resultPath.getCanonicalPath(), DEFAULT_GTY_BLOCKNUM);
            mergedSnpMap.clear();
            mergedIndivGty.clear();
            System.gc();
        }
    }

    public void furtherSplitObjs(String rootPath) throws Exception {
        int blockNum = 4;
        String msg = "spliting genotypes in " + blockNum + " buffers ...";
        //rootPath = "D:\\home\\mxli\\MyJava\\IGG3\\InputSamples";
        File resultPath = new File(rootPath + File.separator);
        for (int i = 0; i < CHROM_NAME.length; i++) {
            String chrom = CHROM_NAME[i];
            ChromSNPGtySet chromSet0 = new ChromSNPGtySet();
            File path = new File(resultPath.getCanonicalPath() + File.separator + chrom + ".obj");
            if (!path.exists()) {
                continue;
            }
            chromSet0.readAllObjects(path.getCanonicalPath());


            chromSet0.saveAllObjectsAndClear(resultPath.getCanonicalPath() + File.separator, blockNum);
            msg = "Chromosome " + chrom + " is finished!";
            System.out.println(msg + " And saved in " + resultPath.getCanonicalPath());
        }
    }

    public static void main(String[] args) {
        WholeGenomeChipIntegrator wholeGIntor = new WholeGenomeChipIntegrator(12);
        try {
            System.out.println("Working path: " + args[0]);
            wholeGIntor.mergeBuffers(args[0]);
            //wholeGIntor.furtherSplitObjs(args[0]);
        } catch (Exception ex) {
        }

    }
}

