// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.
// Permission is granted for you to use this file to compile IGG.
// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.entity;

import cern.colt.bitvector.BitVector;
import java.io.BufferedReader;
import java.io.FileReader;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.TreeSet;
import org.apache.commons.collections.map.MultiValueMap;
import org.cobi.igg3.Constants;
import org.cobi.igg3.gui.IGG3View;
import org.cobi.igg3.GlobalVariables;

/**
 *
 * @author mxli
 */
public class EncodedChipFileSet implements Constants {

    private Map<String, Integer> snpMap;
    public Map<String, StatusGtySet> indiviGtyMap;
    int curSNPNum = 0;
    int availabeSNPBuffer = 100000;

    public EncodedChipFileSet(int buffer) {
        this.snpMap = new HashMap<String, Integer>();
        this.indiviGtyMap = new HashMap<String, StatusGtySet>();
        this.availabeSNPBuffer = buffer;
    }

    public Map<String, Integer> getSnpMap() {
        return snpMap;
    }

    public int getSnpMapNum() {
        return snpMap.size();
    }

    public void readEncodeAffyChipFiles(Collection filePaths, TreeSet<String> indivLabelInPed) throws Exception {
        String[] chipFile;
        Iterator iter = filePaths.iterator();
        //Clear genotypes
        indiviGtyMap.clear();
        snpMap.clear();
        while (iter.hasNext()) {
            chipFile = (String[]) iter.next();
            encodeAffyGenotypeFile(chipFile[1], indivLabelInPed);
        }
    }

    public void readEncodeIllumChipFiles(Collection filePaths, TreeSet<String> indivLabelInPed) throws Exception {
        String[] chipFile;
        Iterator iter = filePaths.iterator();
        //Clear genotypes
        indiviGtyMap.clear();
        snpMap.clear();
        String label;
        MultiValueMap pathIndiv = new MultiValueMap();
        while (iter.hasNext()) {
            chipFile = (String[]) iter.next();
            pathIndiv.put(chipFile[1], chipFile[0]);
        }

        iter = pathIndiv.keySet().iterator();
        TreeSet<String> indivs = new TreeSet<String>();
        while (iter.hasNext()) {
            String path = iter.next().toString();
            indivs.clear();
            Collection indivLabel = pathIndiv.getCollection(path);
            Iterator indiIterate = indivLabel.iterator();
            while (indiIterate.hasNext()) {
                label = (String) indiIterate.next();
                if (indivLabelInPed.contains(label)) {
                    indivs.add(label);
                }
            }
            readIllumGenotypesInFile(path, indivs);
        }

    }

    private void readIllumGenotypesInFile(String filePath, TreeSet<String> indivs) throws Exception {
        String line = null;
        StringTokenizer tokenizer = null;
        String delmilit = "\t ,";
        String startLabel = "rs";

        String runningInfo;
        String probID;
        String indivLabel;
        StringBuilder sigGty = new StringBuilder();
        BufferedReader br = new BufferedReader(new FileReader(filePath), FILE_READER_BUFFER_SIZE);
        long lineCounter = 0;
        StringBuilder tmpBuffer = new StringBuilder();
        while ((line = br.readLine()) != null) {
            lineCounter++;
            if (line.trim().startsWith(startLabel)) {
                break;
            }
        }
        lineCounter--;
        //stat to retreive the data
        do {
            lineCounter++;
            if (line.trim().length() == 0) {
                continue;
            }

            tokenizer = new StringTokenizer(line, delmilit);
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            probID = tmpBuffer.toString();

            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            indivLabel = tmpBuffer.toString();
            if (!indivs.contains(indivLabel)) {
                continue;
            }
            StatusGtySet sGty = indiviGtyMap.get(indivLabel);
            if (sGty == null) {
                sGty = new StatusGtySet();
                sGty.paternalChrom = new BitVector(availabeSNPBuffer);
                sGty.maternalChrom = new BitVector(availabeSNPBuffer);
                sGty.existence = new BitVector(availabeSNPBuffer);
                indiviGtyMap.put(indivLabel, sGty);
            }

            Integer gtyPos = snpMap.get(probID);
            if (gtyPos == null) {
                gtyPos = new Integer(curSNPNum);
                snpMap.put(probID, gtyPos);
                curSNPNum++;
                if (curSNPNum > availabeSNPBuffer) {
                    //adjust the size of bitvectors
                    availabeSNPBuffer += CHIP_GENOTYPE_LOAD_FACTOR;
                    sGty.paternalChrom.setSize(availabeSNPBuffer);
                    sGty.maternalChrom.setSize(availabeSNPBuffer);
                    sGty.existence.setSize(availabeSNPBuffer);
                    for (Map.Entry<String, StatusGtySet> m1 : indiviGtyMap.entrySet()) {
                        m1.getValue().paternalChrom.setSize(availabeSNPBuffer);
                        m1.getValue().maternalChrom.setSize(availabeSNPBuffer);
                        m1.getValue().existence.setSize(availabeSNPBuffer);
                    }
                }
            }
            sigGty.delete(0, sigGty.length());
            if (tokenizer.hasMoreTokens()) {
                sigGty.append(tokenizer.nextToken().trim());
            } else {
                String unexpectInfo = "Error format \'" + line + "\' in " + filePath + " at line " + lineCounter + "!!!";
                GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                GlobalVariables.addInforLog(unexpectInfo);
                continue;
            }
            if (tokenizer.hasMoreTokens()) {
                sigGty.append(tokenizer.nextToken().trim());
            } else {
                String unexpectInfo = "Error format \'" + line + "\' in " + filePath + " at line " + lineCounter + "!!!";
                GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                GlobalVariables.addInforLog(unexpectInfo);
                continue;
            }


            // make sure the genotype is made up of two alleles. otherwise regarded it as a missing data
            if (sigGty.length() != 2) {
                continue;
            }

            //re-code: A->0; B->1;
            switch (sigGty.charAt(0)) {
                case 'B':
                    sGty.existence.putQuick(gtyPos, true);
                    sGty.paternalChrom.putQuick(gtyPos, true);
                    break;
                case 'A':
                    sGty.existence.putQuick(gtyPos, true);
                    break;
                case '*':
                    break;
                case '-':
                    break;
                default:
                    //throw new Exception("Unexpected genotype " + sigGty + " in " + filePath + " for " + indivLabel + " at " + probID);
                    String unexpectInfo = "Unexpected genotype " + sigGty + " in " + filePath + " for " + indivLabel + " at " + probID + " at line " + lineCounter + "!!!";
                    GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                    GlobalVariables.addInforLog(unexpectInfo);
            }
            switch (sigGty.charAt(1)) {
                case 'B':
                    sGty.maternalChrom.putQuick(gtyPos, true);
                    break;
                case 'A':
                    break;
                case '*':
                case '-':
                    break;
                default:
                    //throw new Exception("Unexpected genotype " + sigGty + " in " + filePath + " for " + indivLabel + " at " + probID);
                    String unexpectInfo = "Unexpected genotype " + sigGty + " in " + filePath + " for " + indivLabel + " at " + probID + " at line " + lineCounter + "!!!";
                    GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                    GlobalVariables.addInforLog(unexpectInfo);
            }

        } while ((line = br.readLine()) != null);
        br.close();
        runningInfo = filePath + " encoded!";
        GlobalVariables.mainView.setBriefRunningInfor(runningInfo, true);
        GlobalVariables.addInforLog(runningInfo);
    }

    /**
     * Enocde affymetrix genotype files by the four-bit variables
     *
     * @param filePath the full genotype file path
     * @param indivLabelInPed subject ids in the pedigree files
     */
    private void encodeAffyGenotypeFile(String filePath, TreeSet<String> indivLabelInPed) throws Exception {
        String line = null;
        StringTokenizer tokenizer = null;
        String probID;
        String delmilit = "\t ,";
        String startLabel1 = "SNP_";
        String startLabel2 = "AX-";

        String runningInfo;
        String sigGty;
        String indivLabel = "";
        long lineCounter = 0;
        BufferedReader br = new BufferedReader(new FileReader(filePath), FILE_READER_BUFFER_SIZE);
        //read chipLabel 
        if ((line = br.readLine()) != null) {
            lineCounter++;
            indivLabel = line.split(":")[1].trim();
        }
        if (indivLabel.length() == 0) {
            throw new Exception("Cannot read chip label in chip file, " + filePath);
        }
        if (!indivLabelInPed.contains(indivLabel)) {
            return;
        }
        StatusGtySet sGty = indiviGtyMap.get(indivLabel);
        if (sGty == null) {
            sGty = new StatusGtySet();
            sGty.paternalChrom = new BitVector(availabeSNPBuffer);
            sGty.maternalChrom = new BitVector(availabeSNPBuffer);
            sGty.existence = new BitVector(availabeSNPBuffer);
        }

        while ((line = br.readLine()) != null) {
            lineCounter++;
            if (line.trim().startsWith(startLabel1)||line.trim().startsWith(startLabel2)) {
                break;
            }
        }
        lineCounter--;
        //stat to retreive the data
        StringBuilder tmpBuffer = new StringBuilder();
        do {
            lineCounter++;
            if (line.trim().length() == 0) {
                continue;
            }
            tokenizer = new StringTokenizer(line, delmilit);
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            probID = tmpBuffer.toString();
            Integer gtyPos = snpMap.get(probID);
            if (gtyPos == null) {
                gtyPos = new Integer(curSNPNum);
                snpMap.put(probID, gtyPos);
                curSNPNum++;
                if (curSNPNum > availabeSNPBuffer) {
                    //adjust the size of bitvectors
                    availabeSNPBuffer += CHIP_GENOTYPE_LOAD_FACTOR;
                    sGty.paternalChrom.setSize(availabeSNPBuffer);
                    sGty.maternalChrom.setSize(availabeSNPBuffer);
                    sGty.existence.setSize(availabeSNPBuffer);
                    for (Map.Entry<String, StatusGtySet> m1 : indiviGtyMap.entrySet()) {
                        m1.getValue().paternalChrom.setSize(availabeSNPBuffer);
                        m1.getValue().maternalChrom.setSize(availabeSNPBuffer);
                        m1.getValue().existence.setSize(availabeSNPBuffer);
                    }
                }
            }

            if (tokenizer.hasMoreTokens()) {
                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                sigGty = tmpBuffer.toString();
            } else {
                String unexpectInfo = "Error format \'" + line + "\' in " + filePath + " at line " + lineCounter + "!!!";
                GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                GlobalVariables.addInforLog(unexpectInfo);
                continue;
            }

            // make sure the genotype is made up of two alleles. otherwise regarded it as a missing data
            if (sigGty.length() != 2) {
                continue;
            }

            //re-code: A->0; B->1;
            switch (sigGty.charAt(0)) {
                case 'B':
                    sGty.existence.putQuick(gtyPos, true);
                    sGty.paternalChrom.putQuick(gtyPos, true);
                    break;
                case 'A':
                    sGty.existence.putQuick(gtyPos, true);
                    break;
                default:
                    //throw new Exception("Unexpected genotype in " + filePath + " for " + probID);
                    String unexpectInfo = "Unexpected genotype " + sigGty + " in " + filePath + " for " + probID + " at line " + lineCounter + "!!!";
                    GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                    GlobalVariables.addInforLog(unexpectInfo);
            }
            switch (sigGty.charAt(1)) {
                case 'B':
                    sGty.maternalChrom.putQuick(gtyPos, true);
                    break;
                case 'A':
                    break;
                default:
                    //throw new Exception("Unexpected genotype " + sigGty + " " + sigGty + " in " + filePath + " for " + probID);
                    String unexpectInfo = "Unexpected genotype " + sigGty + " in " + filePath + " for " + probID + " at line " + lineCounter + "!!!";
                    GlobalVariables.mainView.setBriefRunningInfor(unexpectInfo);
                    GlobalVariables.addInforLog(unexpectInfo);
            }
        } while ((line = br.readLine()) != null);
        br.close();
        indiviGtyMap.put(indivLabel, sGty);

        runningInfo = filePath + " encoded!";
        GlobalVariables.mainView.setBriefRunningInfor(runningInfo, true);
        GlobalVariables.addInforLog(runningInfo);
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        try {
            BufferedReader br = new BufferedReader(new FileReader("./resources/chip_annotation/Illumina_HumanHap1M.txt"));
            String line = null;
            String[] cells;
            while ((line = br.readLine()) != null) {
                cells = line.split("\t");
                if (cells[7].compareTo(cells[8]) > 0) {
                    System.out.println(line);
                }
            }
            br.close();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}
