// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.

// Permission is granted for you to use this file to compile IGG.

// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.entity;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.TreeSet;

/**
 *
 * @author Miaoxin Li
 */
public class Integration implements Cloneable {

    private String name;
    //Format of region List: chromsome:start:end #-9 indicates no limit
    private Map<String, long[]> physicalRegionMap;
    private String freqPopulation;
    private String resultPath;
    private List<String> chromResult;
    private Map<String, double[]> genetRegionMap;
    private boolean hapMapSet;
    private boolean hapMapPhased;
    private TreeSet<String> hapIndivLabels;

    public boolean isHapMapPhased() {
        return hapMapPhased;
    }

    public void setHapMapPhased(boolean hapMapPhased) {
        this.hapMapPhased = hapMapPhased;
    }

    public TreeSet<String> getHapIndivLabels() {
        return hapIndivLabels;
    }

    public void setHapIndivLabels(TreeSet<String> hapIndivLabels) {
        this.hapIndivLabels = hapIndivLabels;
    }

    public void addHapIndivLabel(String indiv) {
        if (!this.hapIndivLabels.contains(indiv)) {
            this.hapIndivLabels.add(indiv);
        }
    }

    public boolean isHapMapSet() {
        return hapMapSet;
    }

    public void setHapMapSet(boolean hapMapSet) {
        this.hapMapSet = hapMapSet;
    }

    @Override
    public Object clone() throws CloneNotSupportedException {
        Integration o = null;
        o = (Integration) super.clone();
        o.physicalRegionMap = new HashMap<String, long[]>();
        Iterator iter = this.physicalRegionMap.keySet().iterator();
        while (iter.hasNext()) {
            String temp = (String) iter.next();
            long[] tmpLong = this.physicalRegionMap.get(temp);
            long[] newLong = Arrays.copyOf(tmpLong, tmpLong.length);
            o.physicalRegionMap.put(temp, newLong);
        }
        o.genetRegionMap = new HashMap<String, double[]>();
        iter = this.genetRegionMap.keySet().iterator();
        while (iter.hasNext()) {
            String temp = (String) iter.next();
            double[] tmpLong = this.genetRegionMap.get(temp);
            double[] newLong = Arrays.copyOf(tmpLong, tmpLong.length);
            o.genetRegionMap.put(temp, newLong);
        }
        o.chromResult = new ArrayList<String>();
        o.chromResult.addAll(this.chromResult);
        o.hapIndivLabels = new TreeSet<String>();
        o.hapIndivLabels.addAll(this.hapIndivLabels);
        return o;
    }

    /**
     * Get the value of genetRegionMap
     *
     * @return the value of genetRegionMap
     */
    public Map<String, double[]> getGenetRegionMap() {
        return genetRegionMap;
    }

    @Override
    public String toString() {
        return this.name;
    }

    /**
     * Set the value of genetRegionMap
     *
     * @param genetRegionMap new value of genetRegionMap
     */
    public void setGenetRegionMap(HashMap<String, double[]> genetRegionMap) {
        this.genetRegionMap = genetRegionMap;
    }

    public List<String> getChromResult() {
        return chromResult;
    }

    public void setChromResult(List<String> chromReslut) {
        this.chromResult = chromReslut;
    }

    public void addAChromResult(String chrom) {
        this.chromResult.add(chrom);
    }

    public void updateChromResult(String chrom, int num) {
        boolean hasGot = false;
        for (int i = 0; i < chromResult.size(); i++) {
            StringTokenizer tokenizer = new StringTokenizer(chromResult.get(i));
            if (tokenizer.nextToken().equals(chrom)) {
                chromResult.set(i, chrom + " SNP#" + num);
                hasGot = true;
                break;
            }
        }
        if (!hasGot) {
            chromResult.add(chrom + " SNP#" + num);
        }

    }

    /**
     * Get the value of resutltPath
     *
     * @return the value of resutltPath
     */
    public String getResultPath() {
        return resultPath;
    }

    /**
     * Set the value of resutltPath
     *
     * @param resutltPath new value of resutltPath
     */
    public void setResultPath(String resutltPath) {
        this.resultPath = resutltPath;
    }

    public Integration(String name, String freqPopulation, String resutltPath) {
        this.name = name;
        this.freqPopulation = freqPopulation;
        this.resultPath = resutltPath;
        this.physicalRegionMap = new HashMap<String, long[]>();
        this.genetRegionMap = new HashMap<String, double[]>();
        this.chromResult = new ArrayList<String>();
        //default does not have hapmap set
        this.hapMapSet = false;
        this.hapIndivLabels = new TreeSet<String>();
        this.hapMapPhased = false;
    }

    /**
     * Add the value of physicalRegionMap
     *
     * @param physicalRegionMap new value of physicalRegionMap
     */
    public void addPhysicalRegionMap(String key, long[] values) {
        this.physicalRegionMap.put(key, values);
    }

    /**
     * Add the value of physicalRegionMap
     *
     * @param physicalRegionMap new value of physicalRegionMap
     */
    public void addGenetRegionMap(String key, double[] values) {
        this.genetRegionMap.put(key, values);
    }

    public Map<String, long[]> getPhysicalRegionMap() {
        return physicalRegionMap;
    }

    public void setPhysicalRegionMap(HashMap<String, long[]> regionList) {
        this.physicalRegionMap = regionList;
    }

    /**
     * Get the value of freqPopulation
     *
     * @return the value of freqPopulation
     */
    public String getFreqPopulation() {
        return freqPopulation;
    }

    /**
     * Set the value of freqPopulation
     *
     * @param freqPopulation new value of freqPopulation
     */
    public void setFreqPopulation(String freqPopulation) {
        this.freqPopulation = freqPopulation;
    }

    /**
     * Get the value of name
     *
     * @return the value of name
     */
    public String getName() {
        return name;
    }

    /**
     * Set the value of name
     *
     * @param name new value of name
     */
    public void setName(String name) {
        this.name = name;
    }
}
