// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed out any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.
// Permission is granted for you to use this file to compile IGG.
// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * To change this template, choose Tools | Templates
 * and open the template out the editor.
 */
package org.cobi.igg3.entity;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.TreeSet;

/**
 *
 * @author Miaoxin Li
 */
public class PedFile implements Cloneable, Serializable {

    private Map<String, Individual> allIndividuals;    //stores the individuals chosen by pedparser
    private List<String> traitNames;
    //keep the order of individuals out the pedfile
    private List<String> individualLabel;

    @Override
    public Object clone() throws CloneNotSupportedException {
        PedFile o = (PedFile) super.clone();
        o.allIndividuals = new HashMap<String, Individual>();
        o.allIndividuals.putAll(this.allIndividuals);
        o.traitNames = new ArrayList<String>();
        o.traitNames.addAll(traitNames);
        o.individualLabel = new ArrayList<String>();
        o.individualLabel.addAll(individualLabel);
        return o;
    }

    public Map<String, Individual> getAllIndividuals() {
        return allIndividuals;
    }

    public void setAllIndividuals(HashMap<String, Individual> allIndividuals) {
        this.allIndividuals = allIndividuals;
    }

    public List<String> getTraitNames() {
        return traitNames;
    }

    public void setTraitNames(List<String> traitNames) {
        this.traitNames = traitNames;
    }

    public PedFile() {
        allIndividuals = new HashMap<String, Individual>();
        individualLabel = new ArrayList<String>();
    }

    public List<String> getIndividualLabel() {
        return individualLabel;
    }

    public void setIndividualLabel(ArrayList<String> individualLabel) {
        this.individualLabel = individualLabel;
    }

    /**
     * takes out a pedigree file.
     * data is stored out families out the member hashtable families
     */
    public String parseHapMapPed(String fileAddress) throws Exception {
        Individual ind;
        this.allIndividuals = new HashMap<String, Individual>();
        this.traitNames = new ArrayList<String>();
        this.individualLabel = new ArrayList<String>();

        String line;
        int numLines = 0;
        String indivLabel;
        String info = "";
        BufferedReader br = new BufferedReader(new FileReader(fileAddress));
        //skip one line
        br.readLine();
        StringBuilder tmpBuffer = new StringBuilder();

        while ((line = br.readLine()) != null) {
            if (line.length() == 0) {
                continue;
            }
            StringTokenizer tokenizer = new StringTokenizer(line, "\t ,");
            //for big dataset countTokens() is very time-consuming
            ind = new Individual();
            if (tokenizer.hasMoreTokens()) {
                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                ind.setFamilyID(tmpBuffer.toString());

                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                ind.setIndividualID(tmpBuffer.toString());

                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                ind.setDadID(tmpBuffer.toString());

                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                ind.setMomID(tmpBuffer.toString());

                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                ind.setGender(Integer.parseInt(tmpBuffer.toString()));

                indivLabel = ind.getIndividualID();
                //check whether the individual is avaible or not
                if (allIndividuals.containsKey(indivLabel)) {
                    throw new Exception("Individual " + indivLabel + "  is duplicate");
                }
                ind.setLabelInChip(indivLabel);
                this.allIndividuals.put(indivLabel, ind);
                individualLabel.add(indivLabel);
            }
            numLines++;
            //System.out.println("Line " + numLines);
        }
        br.close();
        if (numLines == 0) {
            throw new Exception("Data format error: empty file");
        }
        return info;
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        // TODO code application logic here
        PedFile pf = new PedFile();
        try {
            //  pf.openPlinkPedFormat("D:/home/statistical tools/plink-1.07-dos/mybintest.bed");
            DataOutputStream out = new DataOutputStream(new BufferedOutputStream(new FileOutputStream("test.bed")));
            boolean[] bits = new boolean[8];
            //"BED files should always start 01101100 00011011" +00000001 (model)
            bits[2] = true;
            bits[3] = true;
            bits[5] = true;
            bits[6] = true;
            boolean bfile_SNP_major = true;
            byte b = 0;
            int[] bases = new int[8];
            int base0 = 1;
            for (int i = 0; i < 8; i++) {
                bases[i] = base0;
                base0 *= 2;

            }
            for (int i = 7; i >= 0; i--) {
                if (bits[i]) {
                    b += bases[i];
                }
            }
            out.write(b);

            Arrays.fill(bits, false);
            bits[0] = true;
            bits[1] = true;
            bits[3] = true;
            bits[4] = true;
            b = 0;
            for (int i = 7; i >= 0; i--) {
                if (bits[i]) {
                    b += bases[i];
                }
            }
            out.write(b);

            Arrays.fill(bits, false);
            bits[0] = true;
            b = 0;
            for (int i = 7; i >= 0; i--) {
                if (bits[i]) {
                    b += bases[i];
                }
            }
            out.write(b);

            out.close();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * takes out a pedigree file.
     * data is stored out families out the member hashtable families
     */
    public String parsePlinkBed(String fileAddress) throws Exception {
        Individual ind;
        this.allIndividuals = new HashMap<String, Individual>();
        this.traitNames = new ArrayList<String>();
        this.individualLabel = new ArrayList<String>();

        String line;
        int numLines = 0;
        String indivLabel;
        String info = "";





        BufferedReader br = new BufferedReader(new FileReader(fileAddress));











        //skip one line
        br.readLine();
        StringBuilder tmpBuffer = new StringBuilder();

        while ((line = br.readLine()) != null) {
            if (line.length() == 0) {
                continue;
            }
            StringTokenizer tokenizer = new StringTokenizer(line, "\t ,");
            //for big dataset countTokens() is very time-consuming
            ind = new Individual();
            if (tokenizer.hasMoreTokens()) {
                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                ind.setFamilyID(tmpBuffer.toString());

                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                ind.setIndividualID(tmpBuffer.toString());

                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                ind.setDadID(tmpBuffer.toString());

                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                ind.setMomID(tmpBuffer.toString());

                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                ind.setGender(Integer.parseInt(tmpBuffer.toString()));

                indivLabel = ind.getIndividualID();
                //check whether the individual is avaible or not
                if (allIndividuals.containsKey(indivLabel)) {
                    throw new Exception("Individual " + indivLabel + "  is duplicate");
                }
                ind.setLabelInChip(indivLabel);
                this.allIndividuals.put(indivLabel, ind);
                individualLabel.add(indivLabel);
            }
            numLines++;
            //System.out.println("Line " + numLines);
        }
        br.close();
        if (numLines == 0) {
            throw new Exception("Data format error: empty file");
        }
        return info;
    }

    public void readCommonPedigrees(String fileAddress) throws Exception {
        FileInputStream objFIn = new FileInputStream(fileAddress);
        BufferedInputStream objOBfs = new BufferedInputStream(objFIn);
        ObjectInputStream localObjIn = new ObjectInputStream(objOBfs);
        List<Individual> indList = (List<Individual>) localObjIn.readObject();
        localObjIn.close();
        objOBfs.close();
        objFIn.close();
        int indSize = indList.size();

        for (int i = 0; i < indSize; i++) {
            if (allIndividuals.get(indList.get(i).getLabelInChip()) == null) {
                allIndividuals.put(indList.get(i).getLabelInChip(), indList.get(i));
                this.individualLabel.add(indList.get(i).getLabelInChip());
            } else {
                //check whether the individual is avaible or not
                throw new Exception("Family " + indList.get(i).getFamilyID() + " Individual " + indList.get(i).getIndividualID() + "  is duplicate!!");
            }
        }
    }

    public void unloadCommonPedigrees() throws Exception {
        int indSize = allIndividuals.size() - 1;
        for (int i = indSize; i >= 0; i--) {
            if (individualLabel.get(i).indexOf("@*@") >= 0) {
                String label = individualLabel.remove(i);
                allIndividuals.remove(label);
            }
        }
    }

    /**
     * takes out a pedigree file.
     * data is stored out families out the member hashtable families
     */
    public String parseIGGPed(String fileAddress) throws Exception {
        int colNum = -1;
        Individual ind;
        this.allIndividuals = new HashMap<String, Individual>();
        List<String> localTraitNames = new ArrayList<String>();
        this.individualLabel = new ArrayList<String>();

        String line;
        int numLines = 0;
        int numTokens = 0;
        int noLabelNum = 0;
        String indivLabel;
        String info = "";
        BufferedReader br = new BufferedReader(new FileReader(fileAddress));
        StringBuilder tmpBuffer = new StringBuilder();

        //long t3 = System.currentTimeMillis();
        // read column titles
        if ((line = br.readLine()) != null) {
            StringTokenizer tokenizer = new StringTokenizer(line, "\t ,");
            colNum = tokenizer.countTokens();
            if (colNum < 6) {
                throw new Exception("Incorrect number of fields on line " + (numLines + 1));
            }
            // there must be trati(s)
            if (colNum > 6) {
                for (int i = 0; i < 5; i++) {
                    tokenizer.nextToken();
                }
                //ignore the last column--the individual label out chip files
                colNum--;
                for (int i = 5; i < colNum; i++) {
                    tmpBuffer.delete(0, tmpBuffer.length());
                    tmpBuffer.append(tokenizer.nextToken().trim());
                    localTraitNames.add(tmpBuffer.toString());
                }
                colNum++;
            }
        }
        if (traitNames.isEmpty()) {
            //if no phenotypes defined out the project....
            traitNames.addAll(localTraitNames);
        } else if (localTraitNames.size() != traitNames.size()) {
            throw new Exception("You defined " + traitNames.size() + " phenotype(s) in this project. But there are "
                    + localTraitNames.size() + " phoneoype(s) in the chip pedigree file!");
        } else {
            StringBuilder inconsistentNames = new StringBuilder();
            for (int i = 0; i < localTraitNames.size(); i++) {
                if (!traitNames.get(i).equals(localTraitNames.get(i))) {
                    inconsistentNames.append(traitNames.get(i));
                    inconsistentNames.append(" <> ");
                    inconsistentNames.append(localTraitNames.get(i));
                    inconsistentNames.append('\n');
                }
            }
            if (inconsistentNames.length() > 0) {
                inconsistentNames.insert(0, "Some inconsistent phenotype names between project phenotypes <> chip pedigree file phenotypes\n");
                throw new Exception(inconsistentNames.toString());
            }
        }

        while ((line = br.readLine()) != null) {
            if (line.length() == 0) {
                continue;
            }
            StringTokenizer tokenizer = new StringTokenizer(line, "\t ,");
            //for big dataset countTokens() is very time-consuming
            numTokens = tokenizer.countTokens();
            if (numTokens < 6) {
                throw new Exception("Incorrect number of fields on line " + (numLines + 1));
            }

            if (colNum != numTokens) {
                //this line has a different number of columns
                //should send some sort of error message
                throw new Exception("Column number mismatch in pedfile. line " + (numLines + 1)
                        + "Note Missing \"Individual Label\" should be 0");
            }

            ind = new Individual();
            if (tokenizer.hasMoreTokens()) {
                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                ind.setFamilyID(tmpBuffer.toString());

                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                ind.setIndividualID(tmpBuffer.toString());

                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                ind.setDadID(tmpBuffer.toString());

                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                ind.setMomID(tmpBuffer.toString());

                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                ind.setGender(Integer.parseInt(tmpBuffer.toString()));
                //indicate no traits
                if (numTokens == 6) {
                    tmpBuffer.delete(0, tmpBuffer.length());
                    tmpBuffer.append(tokenizer.nextToken().trim());
                    indivLabel = tmpBuffer.toString();
                } else {
                    numTokens--;
                    for (int i = 5; i < numTokens; i++) {
                        tmpBuffer.delete(0, tmpBuffer.length());
                        tmpBuffer.append(tokenizer.nextToken().trim());
                        ind.addTrait(tmpBuffer.toString());
                    }
                    tmpBuffer.delete(0, tmpBuffer.length());
                    tmpBuffer.append(tokenizer.nextToken().trim());
                    indivLabel = tmpBuffer.toString();
                }
                if (indivLabel.length() == 0 || indivLabel.equals("0")) {
                    noLabelNum++;
                    indivLabel = "-" + noLabelNum;
                } else {
                    //check whether the individual is avaible or not
                    if (allIndividuals.containsKey(indivLabel)) {
                        throw new Exception("Individual " + indivLabel + "  is duplicate");
                    }
                }
                ind.setLabelInChip(indivLabel);
                this.allIndividuals.put(indivLabel, ind);
                individualLabel.add(indivLabel);
            }
            numLines++;
            //System.out.println("Line " + numLines);
        }
        br.close();
        info = numLines + " individuals read; " + noLabelNum + " have no chip label.";
        if (numLines == 0) {
            //throw new Exception("Data format error: empty file");
            //System.out.println("Data  empty file");
        }
        return info;
    }

    public Individual getIndiviByChipLabel(String label) {
        if (allIndividuals == null) {
            return null;
        }
        return allIndividuals.get(label);
    }

    public HashMap<String, StatusGtySet> extractIndivNullGenotypes() {
        if (allIndividuals == null) {
            return null;
        }
        HashMap<String, StatusGtySet> indGty = new HashMap<String, StatusGtySet>();
        for (Map.Entry<String, Individual> m1 : allIndividuals.entrySet()) {
            indGty.put(m1.getKey(), new StatusGtySet());
        }
        return indGty;
    }

    public TreeSet<String> extractIndivChipLabels() {
        if (allIndividuals == null) {
            return null;
        }
        TreeSet<String> inds = new TreeSet<String>();
        for (Map.Entry<String, Individual> m1 : allIndividuals.entrySet()) {
            inds.add(m1.getKey());
        }
        return inds;
    }

    public boolean saveObjects(String path) throws Exception {
        boolean succeed = false;
        FileOutputStream objFOut = new FileOutputStream(path + "ped.obj");
        BufferedOutputStream objOBfs = new BufferedOutputStream(objFOut);
        ObjectOutputStream objOut = new ObjectOutputStream(objOBfs);
        //Serialization starts here.
        objOut.writeObject(this);
        objOut.flush();
        objOut.close();
        objOBfs.flush();
        objOBfs.close();
        objFOut.close();
        succeed = true;
        return succeed;
    }

    public boolean readObjects(String path) throws Exception {
        boolean succeed = false;
        PedFile obj;
        FileInputStream objFIn = new FileInputStream(path + File.separator + "ped.obj");
        BufferedInputStream objIBfs = new BufferedInputStream(objFIn);
        ObjectInputStream objIn = new ObjectInputStream(objIBfs);
        obj = (PedFile) objIn.readObject();     //Deserialazation starts here.

        this.allIndividuals = obj.allIndividuals;
        this.individualLabel = obj.individualLabel;
        this.traitNames = obj.traitNames;
        objIn.close();
        succeed = true;
        return succeed;
    }
}
