// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.
// Permission is granted for you to use this file to compile IGG.
// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.entity;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;


import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.commons.collections.map.MultiValueMap;
import org.apache.commons.collections.MultiMap;
import org.apache.crimson.tree.XmlDocument;

import org.cobi.util.file.LocalFileFunc;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

/**
 *
 * @author mxli
 */
public class Project {

    private String name;
    private Date createdDate;
    private String pedigreeFilePath;
    protected MultiValueMap chipFilePathList;
    private String workingPath;
    private List<Integration> integrationList;
    protected List<CommonGenetFileSet> commonGenetFileSetList;
    private PedFile pedFile;
    private List<String> phenotypeNames;

    public PedFile getPedFile() {
        return pedFile;
    }

    public void setPedFile(PedFile pedFile) {
        this.pedFile = pedFile;
    }

    public void addPhenotypeName(String phe) {
        phenotypeNames.add(phe);
    }

    public List<String> getPhenotypeNames() {
        return phenotypeNames;
    }

    public void setPhenotypeNames(List<String> phenotypeNames) {
        this.phenotypeNames = phenotypeNames;
    }

    /**
     * Get the value of commonGenetFileSetList
     *
     * @return the value of commonGenetFileSetList
     */
    public List<CommonGenetFileSet> getCommonGenetFileSetList() {
        return commonGenetFileSetList;
    }

    /**
     * Set the value of commonGenetFileSetList
     *
     * @param commonGenetFileSetList new value of commonGenetFileSetList
     */
    public void setCommonGenetFileSetList(List<CommonGenetFileSet> commonGenetFileSetList) {
        this.commonGenetFileSetList = commonGenetFileSetList;
    }

    public Project(String name, Date createdDate, String pedigreeFilePath, String workingPath) {
        this.name = name;
        this.createdDate = createdDate;
        this.pedigreeFilePath = pedigreeFilePath;
        this.workingPath = workingPath;
        this.chipFilePathList = new MultiValueMap();
        this.integrationList = new ArrayList<Integration>();
        this.commonGenetFileSetList = new ArrayList<CommonGenetFileSet>();
        this.phenotypeNames = new ArrayList<String>();
    }

    public Project(String name, String workingPath) {
        this.name = name;
        this.workingPath = workingPath;
        this.chipFilePathList = new MultiValueMap();
        this.integrationList = new ArrayList<Integration>();
        this.commonGenetFileSetList = new ArrayList<CommonGenetFileSet>();
        this.phenotypeNames = new ArrayList<String>();
    }

    public Project(String name, Date createdDate, String workingPath) {
        this.name = name;
        this.createdDate = createdDate;
        this.workingPath = workingPath;
        this.chipFilePathList = new MultiValueMap();
        this.integrationList = new ArrayList<Integration>();
        this.commonGenetFileSetList = new ArrayList<CommonGenetFileSet>();
        this.phenotypeNames = new ArrayList<String>();
    }

    public void addCommonGenetFileSet(CommonGenetFileSet set) {
        this.commonGenetFileSetList.add(set);
    }

    public List<Integration> getIntegrationList() {
        return integrationList;
    }

    public void setIntegrationList(ArrayList<Integration> integrationList) {
        this.integrationList = integrationList;
    }

    public void addAIntegration(Integration integration) throws Exception {
        if (isAvailableIntegrationName(integration.getName())) {
            throw new Exception(integration.getName() + " is a duplicated name in this project!");
        } else {
            this.integrationList.add(integration);
        }
    }

    public boolean isAvailableIntegrationName(String name) {
        for (int i = integrationList.size() - 1; i >= 0; i--) {
            if (integrationList.get(i).getName().equals(name)) {
                return true;
            }
        }
        return false;
    }

    public boolean isAvailableCommonGenetFileSetName(String name) {
        for (int i = commonGenetFileSetList.size() - 1; i >= 0; i--) {
            if (commonGenetFileSetList.get(i).getName().equals(name)) {
                return true;
            }
        }
        return false;
    }

    public void removeAIntegration(String name) throws Exception {
        for (int i = integrationList.size() - 1; i >= 0; i--) {
            if (integrationList.get(i).getName().equals(name)) {
                Integration integration = integrationList.remove(i);
                LocalFileFunc.delAll(new File(integration.getResultPath()));
            }
        }
    }

    public void removeACommonGentypeSet(String name) throws Exception {
        for (int i = commonGenetFileSetList.size() - 1; i >= 0; i--) {
            if (commonGenetFileSetList.get(i).getName().equals(name)) {
                commonGenetFileSetList.remove(i);
            }
        }

    }

    /**
     * Get the value of workingPath
     *
     * @return the value of workingPath
     */
    public String getWorkingPath() {
        return workingPath;
    }

    /**
     * Set the value of workingPath
     *
     * @param workingPath new value of workingPath
     */
    public void setWorkingPath(String workingPath) {
        this.workingPath = workingPath;
    }

    /**
     * Get the value of chipFilePathList
     *
     * @return the value of chipFilePathList
     */
    public MultiValueMap getChipFilePathList() {
        return chipFilePathList;
    }

    /**
     * Set the value of chipFilePathList
     *
     * @param chipFilePathList new value of chipFilePathList
     */
    public void setChipFilePathList(MultiValueMap chipFilePathList) {
        this.chipFilePathList = chipFilePathList;
    }

    /**
     * Add the value of chipFilePathList
     *
     * @param chipFilePathList new value of chipFilePathList
     */
    public void addChipFilePathList(MultiMap chipFilePathList) {
        this.chipFilePathList.putAll(chipFilePathList);
    }

    public void removeAChipFilePath(String chipType, String fileName) {
        chipFilePathList.remove(chipType, fileName);
        //Collection fileNames = chipFilePathList.getCollection(chipType);
        // boolean removed=fileNames.remove(fileName);

    }

    public void removeAChipType(String chipType) {
        chipFilePathList.remove(chipType);
    }

    /**
     * Get the value of pedigreeFilePaht
     *
     * @return the value of pedigreeFilePaht
     */
    public String getPedigreeFilePath() {
        return pedigreeFilePath;
    }

    /**
     * Set the value of pedigreeFilePaht
     *
     * @param pedigreeFilePaht new value of pedigreeFilePaht
     */
    public void setPedigreeFilePath(String pedigreeFilePaht) {
        this.pedigreeFilePath = pedigreeFilePaht;
    }

    /**
     * Get the value of createdDate
     *
     * @return the value of createdDate
     */
    public Date getCreatedDate() {
        return createdDate;
    }

    /**
     * Set the value of createdDate
     *
     * @param createdDate new value of createdDate
     */
    public void setCreatedDate(Date createdDate) {
        this.createdDate = createdDate;
    }

    /**
     * Get the value of name
     *
     * @return the value of name
     */
    public String getName() {
        return name;
    }

    /**
     * Set the value of name
     *
     * @param name new value of name
     */
    public void setName(String name) {
        this.name = name;
    }

    public void writeProjectVariables() throws Exception {
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder db = null;
        try {
            db = dbf.newDocumentBuilder();
        } catch (ParserConfigurationException pce) {
            throw new Exception(pce.toString());
        }

        Document doc = null;
        doc = db.newDocument();
        Element root = doc.createElement("IGGProject");
        doc.appendChild(root);
        Element attriNode = doc.createElement("name");
        root.appendChild(attriNode);
        Text value = doc.createTextNode(name);
        attriNode.appendChild(value);

        attriNode = doc.createElement("createdTime");
        root.appendChild(attriNode);
        SimpleDateFormat sFormat = new SimpleDateFormat("yyyy-MM-dd HH-mm-ss");
        value = doc.createTextNode(sFormat.format(createdDate));
        attriNode.appendChild(value);

        attriNode = doc.createElement("workingPath");
        root.appendChild(attriNode);
        value = doc.createTextNode(workingPath);
        attriNode.appendChild(value);

        value = doc.createTextNode(pedigreeFilePath);

        if (value!= null) {
            //note the pedigreeFilePath could be 0
            attriNode = doc.createElement("pedigreeFilePath");
            root.appendChild(attriNode);
            attriNode.appendChild(value);
        }
        
        if (chipFilePathList.size() > 0) {
            attriNode = doc.createElement("chipFilePathList");

            Iterator iter = chipFilePathList.keySet().iterator();
            while (iter.hasNext()) {
                Object key = iter.next();
                Element chipFile = doc.createElement("chipFile");
                chipFile.setAttribute("type", key.toString());
                attriNode.appendChild(chipFile);

                Element filePath = doc.createElement("path");
                chipFile.appendChild(filePath);
                Text size = doc.createTextNode(chipFilePathList.get(key).toString());
                filePath.appendChild(size);
            }
            root.appendChild(attriNode);
        }


        int commonGenetFileSetSize = commonGenetFileSetList.size();
        if (commonGenetFileSetSize > 0) {
            attriNode = doc.createElement("commonGenetFileSetList");
            root.appendChild(attriNode);
            for (int i = 0; i < commonGenetFileSetSize; i++) {
                CommonGenetFileSet key = commonGenetFileSetList.get(i);
                Element fileSet = doc.createElement("commonGenetFileSet");
                attriNode.appendChild(fileSet);
                fileSet.setAttribute("name", key.getName());
                fileSet.setAttribute("pedigreeFileName", key.getPedigreeFileName());
                fileSet.setAttribute("mapFileName", key.getMapFileName());
                fileSet.setAttribute("plinkBinaryFileName", key.getPlinkBinaryFileName() == null ? "" : key.getPlinkBinaryFileName());
                fileSet.setAttribute("mapFileName", key.getMapFileName());
                fileSet.setAttribute("strand", String.valueOf(key.getStrand()));
                fileSet.setAttribute("chromIndex", String.valueOf(key.getChromIndex()));
                fileSet.setAttribute("rsIndex", String.valueOf(key.getRsIndex()));
                fileSet.setAttribute("posIndex", String.valueOf(key.getPosIndex()));
            }
        }
        int phenotypeNameSize = phenotypeNames.size();
        if (phenotypeNameSize > 0) {
            attriNode = doc.createElement("phenotypeNames");
            root.appendChild(attriNode);
            StringBuilder names = new StringBuilder();
            for (int i = 0; i < phenotypeNameSize; i++) {
                names.append(phenotypeNames.get(i));
                names.append(',');
            }
            attriNode.setAttribute("names", names.toString());
        }

        int integrationSize = integrationList.size();
        if (integrationSize > 0) {
            attriNode = doc.createElement("integrationList");
            root.appendChild(attriNode);
            for (int i = 0; i < integrationSize; i++) {
                Integration key = integrationList.get(i);
                Element integration = doc.createElement("integration");
                attriNode.appendChild(integration);
                integration.setAttribute("name", key.getName());
                integration.setAttribute("freqPopulation", key.getFreqPopulation());
                integration.setAttribute("resultPath", key.getResultPath());
                integration.setAttribute("hasHapMap", String.valueOf(key.isHapMapSet()));
                integration.setAttribute("hapMapPhased", String.valueOf(key.isHapMapPhased()));

                if (key.getPhysicalRegionMap().size() > 0) {
                    Element getPhysicalRegionMap = doc.createElement("physicalRegionMap");
                    integration.appendChild(getPhysicalRegionMap);

                    Iterator<String> iter = key.getPhysicalRegionMap().keySet().iterator();
                    while (iter.hasNext()) {
                        Object regionKey = iter.next();
                        Element chromosome = doc.createElement("chromosome");
                        chromosome.setAttribute("name", regionKey.toString());
                        getPhysicalRegionMap.appendChild(chromosome);

                        Element regions = doc.createElement("regions");
                        chromosome.appendChild(regions);
                        StringBuilder regs = new StringBuilder();
                        long[] longReg = key.getPhysicalRegionMap().get(regionKey);
                        for (int j = 0; j < longReg.length; j++) {
                            regs.append(longReg[j]);
                            regs.append(' ');
                        }
                        Text numbers = doc.createTextNode(regs.toString());
                        regions.appendChild(numbers);
                    }
                }

                if (key.getChromResult().size() > 0) {
                    Element chromResult = doc.createElement("chromResult");
                    integration.appendChild(chromResult);
                    value = doc.createTextNode(key.getChromResult().toString());
                    chromResult.appendChild(value);
                }

                if (key.getHapIndivLabels().size() > 0) {
                    Element hapIndivLabels = doc.createElement("hapIndivLabels");
                    integration.appendChild(hapIndivLabels);
                    value = doc.createTextNode(key.getHapIndivLabels().toString());
                    hapIndivLabels.appendChild(value);
                }
            }
        }



        FileOutputStream outStream = new FileOutputStream(workingPath + File.separator + name + ".xml");
        OutputStreamWriter outWriter = new OutputStreamWriter(outStream);
        ((XmlDocument) doc).write(outWriter, "UTF-8");
        outWriter.close();
        outStream.close();
    }

    public void readProjectVariables() throws Exception {
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder db = null;
        File lcurFile = new File(workingPath + File.separator + name + ".xml");
        if (!lcurFile.exists()) {
            return;
        }
        try {
            db = dbf.newDocumentBuilder();
        } catch (ParserConfigurationException pce) {
            throw new Exception(pce.toString());
        }
        Document doc = null;
        try {
            doc = db.parse(lcurFile.getCanonicalPath());
        } catch (DOMException dom) {
            throw new Exception(dom.getMessage());
        } catch (IOException ioe) {
            throw new Exception(ioe.toString() + " No project in this folder!");
        }

        Element root = doc.getDocumentElement();
        NodeList nodes = root.getElementsByTagName("createdTime");
        if (nodes.getLength() == 1) {
            Element e = (Element) nodes.item(0);
            Text t = (Text) e.getFirstChild();
            if (t == null) {
                createdDate = new Date();
            } else {
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH-mm-ss", Locale.US);
                createdDate = sdf.parse(t.getNodeValue());
            }
        }

        nodes = root.getElementsByTagName("pedigreeFilePath");
        if (nodes.getLength() == 1) {
            Element e = (Element) nodes.item(0);
            Text t = (Text) e.getFirstChild();
            if (t == null) {
                pedigreeFilePath = null;
            } else {
                pedigreeFilePath = t.getNodeValue();
            }
        }

        nodes = root.getElementsByTagName("chipFile");
        for (int i = 0; i < nodes.getLength(); i++) {
            Element typeFile = (Element) nodes.item(i);
            String type = typeFile.getAttribute("type");
            NodeList subNodes = typeFile.getElementsByTagName("path");
            if (subNodes.getLength() == 1) {
                Element e = (Element) subNodes.item(0);
                Text t = (Text) e.getFirstChild();
                String fileName = t.getNodeValue();
                fileName = fileName.substring(1, fileName.length() - 1);

                String[] fileNames = fileName.split(",");
                for (int j = 0; j < fileNames.length; j++) {
                    chipFilePathList.put(type, fileNames[j].trim());
                }
            }
        }


        //add the common file sets
        nodes = root.getElementsByTagName("commonGenetFileSet");
        for (int i = 0; i < nodes.getLength(); i++) {
            Element fileSet = (Element) nodes.item(i);
            String setName = fileSet.getAttribute("name");
            String pedigreeFileName = fileSet.getAttribute("pedigreeFileName");
            String mapFileName = fileSet.getAttribute("mapFileName");
            int strand = Integer.parseInt(fileSet.getAttribute("strand"));
            int chromIndex = Integer.parseInt(fileSet.getAttribute("chromIndex"));
            int rsIndex = Integer.parseInt(fileSet.getAttribute("rsIndex"));
            int posIndex = Integer.parseInt(fileSet.getAttribute("posIndex"));

            CommonGenetFileSet set = new CommonGenetFileSet(setName, pedigreeFileName, mapFileName,
                    strand, chromIndex, rsIndex, posIndex);
            String plinkBinaryGentypeFile = fileSet.getAttribute("plinkBinaryFileName");
            if (plinkBinaryGentypeFile != null && plinkBinaryGentypeFile.length() > 0) {
                set.setPlinkBinaryFileName(plinkBinaryGentypeFile);
            }

            commonGenetFileSetList.add(set);
        }
        nodes = root.getElementsByTagName("phenotypeNames");
        if (nodes != null) {
            Element fileSet = (Element) nodes.item(0);
            if (fileSet != null) {
                String nameStr = fileSet.getAttribute("names");
                String[] names = nameStr.split(",");
                phenotypeNames.addAll(Arrays.asList(names));
            }
        }


        //add the integration setting
        nodes = root.getElementsByTagName("integration");
        for (int i = 0; i < nodes.getLength(); i++) {
            Element integration = (Element) nodes.item(i);
            String integrationName = integration.getAttribute("name");
            String freqPopulation = integration.getAttribute("freqPopulation");
            String resultPath = integration.getAttribute("resultPath");
            boolean hasHapMap = Boolean.parseBoolean(integration.getAttribute("hasHapMap"));
            boolean hapMapPhased = Boolean.parseBoolean(integration.getAttribute("hapMapPhased"));
            Integration newInteg = new Integration(integrationName, freqPopulation, resultPath);
            newInteg.setHapMapSet(hasHapMap);
            newInteg.setHapMapPhased(hapMapPhased);
            NodeList subNodes = integration.getElementsByTagName("chromResult");
            if (subNodes.getLength() == 1) {
                Element e = (Element) subNodes.item(0);
                Text t = (Text) e.getFirstChild();
                String fileName = t.getNodeValue();
                fileName = fileName.substring(1, fileName.length() - 1);

                String[] fileNames = fileName.split(",");
                for (int j = 0; j < fileNames.length; j++) {
                    newInteg.addAChromResult(fileNames[j].trim());
                }
            }

            subNodes = integration.getElementsByTagName("hapIndivLabels");
            if (subNodes.getLength() == 1) {
                Element e = (Element) subNodes.item(0);
                Text t = (Text) e.getFirstChild();
                String fileName = t.getNodeValue();
                fileName = fileName.substring(1, fileName.length() - 1);

                String[] fileNames = fileName.split(",");
                for (int j = 0; j < fileNames.length; j++) {
                    newInteg.addHapIndivLabel(fileNames[j].trim());
                }
            }

            subNodes = integration.getElementsByTagName("physicalRegionMap");
            if (subNodes != null) {
                Element physiMap = (Element) subNodes.item(0);
                subNodes = physiMap.getElementsByTagName("chromosome");
                for (int k = 0; k < subNodes.getLength(); k++) {
                    Element chromosome = (Element) subNodes.item(k);
                    String chromosomeName = chromosome.getAttribute("name");
                    NodeList tmpSubNodes = chromosome.getElementsByTagName("regions");
                    if (tmpSubNodes.getLength() == 1) {
                        Element e = (Element) tmpSubNodes.item(0);
                        Text t = (Text) e.getFirstChild();
                        String regionNames = t.getNodeValue();
                        String[] regs = regionNames.split(" ");
                        long[] longRegs = new long[regs.length];
                        for (int j = 0; j < regs.length; j++) {
                            longRegs[j] = Long.parseLong(regs[j]);
                        }
                        newInteg.addPhysicalRegionMap(chromosomeName, longRegs);
                    }
                }
            }
            this.addAIntegration(newInteg);
        }
    }
}
