/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.genetformator;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.cobi.igg3.entity.AnnotSNP;
import org.cobi.igg3.entity.ExportSetting;
import org.cobi.igg3.entity.Individual;
import org.cobi.igg3.entity.Integration;
import org.cobi.igg3.entity.PedFile;
import org.cobi.igg3.entity.StatusGtySet;
import org.cobi.igg3.GlobalVariables;

/**
 *
 * @author MX Li
 */
public class BIMBAMFiles {

    /**This function can generate genetic map for solar
     *
     *
     */
    public void writePositionMapFileBIMBAM(BufferedWriter bw, List<AnnotSNP> snpInfo) throws
            Exception {
        int snpInfoSize = snpInfo.size();
        for (int i = 0; i < snpInfoSize; i++) {
            AnnotSNP snp = snpInfo.get(i);
            bw.write(snp.getRSID());
            bw.write(",");
            bw.write(String.valueOf(snp.getPhysicalPosition()));
            bw.write("\n");
        }
    }

    public void writeStudyPhenotypeFileBIMBAM(ExportSetting exportSetting) throws Exception {
        String path = exportSetting.getExportPath();
        String prefix = exportSetting.getPrefixFileName();
        List<String[]> outTraits = exportSetting.getTraitProper();
        List<String> avaiTrais = GlobalVariables.currentPedFile.getTraitNames();
        int traitSize = outTraits.size();
        int allTraitSize = avaiTrais.size();
        String[] cell;

        Map<String, Individual> prjIndivMap = GlobalVariables.currentPedFile.getAllIndividuals();
        List<String> prjIndiLabels = GlobalVariables.currentPedFile.getIndividualLabel();
        int indiviSize = prjIndiLabels.size();

        //determine the output trait
        for (int i = 0; i < traitSize; i++) {
            cell = outTraits.get(i);
            for (int j = 0; j < allTraitSize; j++) {
                if (cell[0].equals(avaiTrais.get(j))) {
                    BufferedWriter bw = new BufferedWriter(new FileWriter(path + prefix + "." + cell[0] + ".pheno"));
                    for (int k = 0; k < indiviSize; k++) {
                        Individual indiv = prjIndivMap.get(prjIndiLabels.get(k));
                        //write trait value
                        bw.write(indiv.getTraits().get(j));
                        bw.write("\n");
                    }
                    bw.close();
                }
            }
        }

    }

    public void writeHapMapPhenotypeFileBIMBAM(Integration integration, ExportSetting exportSetting) throws Exception {
        String path = exportSetting.getExportPath();
        String prefix = exportSetting.getPrefixFileName();
        List<String[]> outTraits = exportSetting.getTraitProper();

        List<String> avaiTrais = GlobalVariables.currentPedFile.getTraitNames();
        int traitSize = outTraits.size();
        int allTraitSize = avaiTrais.size();
        String[] cell;

        //determine the output trait
        for (int i = 0; i < traitSize; i++) {
            cell = outTraits.get(i);
            for (int j = 0; j < allTraitSize; j++) {
                if (cell[0].equals(avaiTrais.get(j))) {
                    BufferedWriter bw = new BufferedWriter(new FileWriter(path + prefix + "." + cell[0] + ".hapmap.pheno"));
                    Iterator hapIndiLabelIter = integration.getHapIndivLabels().iterator();
                    while (hapIndiLabelIter.hasNext()) {
                        hapIndiLabelIter.next();
                        bw.write(outTraits.get(j)[2]);
                        bw.write("\n");
                    }
                    bw.close();
                }
            }
        }
    }

    public void writeStudyGenotypeFileBIMBAM(BufferedWriter bw, List<AnnotSNP> entries, Map<String, StatusGtySet> indivGtyMap,
            Integration integration, ExportSetting exportSetting) throws Exception {
        int snpSize = entries.size();
        String missingGty = "?";
 

        List<String> prjIndiLabels = GlobalVariables.currentPedFile.getIndividualLabel();
        int writenIndivNum = prjIndiLabels.size();

        //the number of individuals
        bw.write(String.valueOf(writenIndivNum));
        bw.write("\n");
        //number of SNPs
        bw.write(String.valueOf(snpSize));
        bw.write("\n");

        int indiviSize = prjIndiLabels.size();
        //individual identifiers for each individual
        bw.write("IND");
        for (int k = 0; k < indiviSize; k++) {
            bw.write(",");
            bw.write(prjIndiLabels.get(k));
        }
        bw.write("\n");

        for (int j = 0; j < snpSize; j++) {
            AnnotSNP snp = entries.get(j);
            bw.write(snp.getRSID());
            int snpOrder = snp.order;
            String aAllele = String.valueOf(snp.getAAllele());
            String bAllele = String.valueOf(snp.getBAllele());

            //project individuals
            for (int k = 0; k < indiviSize; k++) {
                StatusGtySet indivGty = indivGtyMap.get(prjIndiLabels.get(k));
                bw.write(",");
                if (indivGty != null) {
                    if (indivGty.existence.size() > snpOrder) {
                        if (indivGty.existence.getQuick(snpOrder)) {
                            if (indivGty.paternalChrom.getQuick(snpOrder)) {
                                bw.write(bAllele);
                            } else {
                                bw.write(aAllele);
                            }
                        } else {
                            bw.write(missingGty);
                        }

                        if (indivGty.existence.getQuick(snpOrder)) {
                            if (indivGty.maternalChrom.getQuick(snpOrder)) {
                                bw.write(bAllele);
                            } else {
                                bw.write(aAllele);
                            }
                        } else {
                            bw.write(missingGty);
                        }
                    } else {
                        bw.write(missingGty);
                        bw.write(missingGty);
                    }
                } else {
                    bw.write(missingGty);
                    bw.write(missingGty);
                }
            }
            bw.write("\n");
        }

    }

    public void writeHapMapGenotypeFileBIMBAM(BufferedWriter bw, List<AnnotSNP> entries, Map<String, StatusGtySet> indivGtyMap,
            Integration integration, ExportSetting exportSetting, PedFile hapPed) throws Exception {
        int snpSize = entries.size();
        String missingGty = "?";

        Map<String, Individual> hapIndivMap = null;
        int writenIndivNum = integration.getHapIndivLabels().size();
        //the number of individuals
        if (integration.isHapMapPhased()) {
            bw.write(String.valueOf(writenIndivNum));
            bw.write(" =");
        } else {
            bw.write(String.valueOf(writenIndivNum));
        }
        bw.write("\n");
        //number of SNPs
        bw.write(String.valueOf(snpSize));
        bw.write("\n");

        //individual identifiers for each individual
        bw.write("IND");
        Iterator<String> hapIndiLabelIter = integration.getHapIndivLabels().iterator();
        while (hapIndiLabelIter.hasNext()) {
            bw.write(",");
            bw.write(hapIndiLabelIter.next());
        }
        bw.write("\n");

        for (int j = 0; j < snpSize; j++) {
            AnnotSNP snp = entries.get(j);
            bw.write(snp.getRSID());
            int snpOrder = snp.order;
            String aAllele = String.valueOf(snp.getAAllele());
            String bAllele = String.valueOf(snp.getBAllele());

            hapIndivMap = hapPed.getAllIndividuals();
            hapIndiLabelIter = integration.getHapIndivLabels().iterator();
            while (hapIndiLabelIter.hasNext()) {
                Individual indiv = hapIndivMap.get(hapIndiLabelIter.next());
                StatusGtySet indivGty = indivGtyMap.get(indiv.getLabelInChip());
                bw.write(",");
                if (indivGty != null) {
                    if (indivGty.existence.size() > snpOrder) {
                        if (indivGty.existence.getQuick(snpOrder)) {
                            if (indivGty.paternalChrom.getQuick(snpOrder)) {
                                bw.write(bAllele);
                            } else {
                                bw.write(aAllele);
                            }
                        } else {
                            bw.write(missingGty);
                        }

                        if (indivGty.existence.getQuick(snpOrder)) {
                            if (indivGty.maternalChrom.getQuick(snpOrder)) {
                                bw.write(bAllele);
                            } else {
                                bw.write(aAllele);
                            }
                        } else {
                            bw.write(missingGty);
                        }
                    } else {
                        bw.write(missingGty);
                        bw.write(missingGty);
                    }
                } else {
                    bw.write(missingGty);
                    bw.write(missingGty);
                }
            }

            bw.write("\n");
        }

    }
}
