// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.

// Permission is granted for you to use this file to compile IGG.

// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.genetformator;

import java.io.BufferedWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import org.cobi.igg3.entity.AnnotSNP;
import org.cobi.igg3.entity.Individual;
import org.cobi.igg3.entity.PedFile;
import org.cobi.igg3.entity.StatusGtySet;
import org.cobi.igg3.GlobalVariables;

/**
 *
 * @author mxli
 */
public class BeagleFiles {

    public boolean writeBeagleFile(BufferedWriter prjBw, BufferedWriter hapBw, List<AnnotSNP> entries,
            Map<String, StatusGtySet> indivGtyMap, PedFile hapPed, List<String[]> outTraits) throws
            Exception {
        int snpSize = entries.size();
        int[] orders = new int[snpSize];
        String missingGty = "?";
        char[] AAlleles = new char[snpSize];
        char[] BAlleles = new char[snpSize];
        for (int i = 0; i < snpSize; i++) {
            AnnotSNP snp = entries.get(i);
            orders[i] = snp.order;
            AAlleles[i] = snp.getAAllele();
            BAlleles[i] = snp.getBAllele();
        }

        Map<String, Individual> prjIndivMap = GlobalVariables.currentPedFile.getAllIndividuals();
        List<String> prjIndiLabels = GlobalVariables.currentPedFile.getIndividualLabel();
        //------------------------------write project data
        prjBw.write("I id");
        int prjIndiviSize = prjIndiLabels.size();
        for (int i = 0; i < prjIndiviSize; i++) {
            prjBw.write(" ");
            prjBw.write(prjIndiLabels.get(i));
            prjBw.write(" ");
            prjBw.write(prjIndiLabels.get(i));
        }
        prjBw.write("\n");

        int traitSize = outTraits.size();
        List<String> avaiTrais = GlobalVariables.currentPedFile.getTraitNames();
        int allTraitSize = avaiTrais.size();
        int[] traitIndex = new int[traitSize];
        String[] cell = null;
        //determine the output trait
        for (int i = 0; i < traitSize; i++) {
            cell = outTraits.get(i);
            for (int j = 0; j < allTraitSize; j++) {
                if (cell[0].equals(avaiTrais.get(j))) {
                    traitIndex[i] = j;
                    break;
                }
            }
        }

        for (int i = 0; i < traitSize; i++) {
            cell = outTraits.get(i);
            if (cell[1].startsWith("Affection Status")) {
                prjBw.write("A ");
            } else if (cell[1].startsWith("Quantitative Trait")) {
                prjBw.write("T ");
            } else if (cell[1].startsWith("Covariate")) {
                prjBw.write("C ");
            }
            prjBw.write(cell[0]);
            for (int j = 0; j < prjIndiviSize; j++) {
                Individual indiv = prjIndivMap.get(prjIndiLabels.get(j));
                prjBw.write(" ");
                prjBw.write(indiv.getTraits().get(traitIndex[i]));
                prjBw.write(" ");
                prjBw.write(indiv.getTraits().get(traitIndex[i]));
            }
            prjBw.write("\n");
        }

        for (int j = 0; j < snpSize; j++) {
            AnnotSNP snp = entries.get(j);
            prjBw.write("M ");
            prjBw.write(snp.getRSID());

            for (int i = 0; i < prjIndiviSize; i++) {
                StatusGtySet indivGty = indivGtyMap.get(prjIndiLabels.get(i));
                prjBw.write(" ");
                if (indivGty.existence.size() > orders[j] && indivGty.existence.getQuick(orders[j])) {
                    if (indivGty.paternalChrom.getQuick(orders[j])) {
                        prjBw.write(String.valueOf(BAlleles[j]));
                    } else {
                        prjBw.write(String.valueOf(AAlleles[j]));
                    }
                } else {
                    prjBw.write(missingGty);
                }
                prjBw.write(" ");
                if (indivGty.existence.size() > orders[j] && indivGty.existence.getQuick(orders[j])) {
                    if (indivGty.maternalChrom.getQuick(orders[j])) {
                        prjBw.write(String.valueOf(BAlleles[j]));
                    } else {
                        prjBw.write(String.valueOf(AAlleles[j]));
                    }
                } else {
                    prjBw.write(missingGty);
                }
            }
            prjBw.write("\n");
        }

        //-------------------write hapmap data
        List<String> filteredHapIndiLabels = new ArrayList<String>();
        List<String> hapIndiLabels = new ArrayList<String>();
        if (hapPed != null) {
            hapIndiLabels = hapPed.getIndividualLabel();
        }
        hapBw.write("I id");
        int hapIndiviSize = hapIndiLabels.size();
        for (int i = 0; i < hapIndiviSize; i++) {
            if (indivGtyMap.get(hapIndiLabels.get(i)) != null) {
                hapBw.write(" ");
                hapBw.write(hapIndiLabels.get(i));
                hapBw.write(" ");
                hapBw.write(hapIndiLabels.get(i));
                filteredHapIndiLabels.add(hapIndiLabels.get(i));
            }
        }
        hapBw.write("\n");
        hapIndiviSize = filteredHapIndiLabels.size();
        for (int i = 0; i < traitSize; i++) {
            cell = outTraits.get(i);
            if (cell[1].startsWith("Affection Status")) {
                hapBw.write("A ");
            } else if (cell[1].startsWith("Quantitative Trait")) {
                hapBw.write("T ");
            } else if (cell[1].startsWith("Covariate")) {
                hapBw.write("C ");
            }
            hapBw.write(cell[0]);
            for (int j = 0; j < hapIndiviSize; j++) {
                hapBw.write(" ");
                hapBw.write(cell[2]);
                hapBw.write(" ");
                hapBw.write(cell[2]);
            }
            hapBw.write("\n");
        }
        for (int j = 0; j < snpSize; j++) {
            AnnotSNP snp = entries.get(j);
            hapBw.write("M ");
            hapBw.write(snp.getRSID());
            for (int i = 0; i < hapIndiviSize; i++) {
                StatusGtySet indivGty = indivGtyMap.get(filteredHapIndiLabels.get(i));
                hapBw.write(" ");
                if (indivGty.existence.size() > orders[j] && indivGty.existence.getQuick(orders[j])) {
                    if (indivGty.paternalChrom.getQuick(orders[j])) {
                        hapBw.write(String.valueOf(BAlleles[j]));
                    } else {
                        hapBw.write(String.valueOf(AAlleles[j]));
                    }
                } else {
                    hapBw.write(missingGty);
                }
                hapBw.write(" ");
                if (indivGty.existence.size() > orders[j] && indivGty.existence.getQuick(orders[j])) {
                    if (indivGty.maternalChrom.getQuick(orders[j])) {
                        hapBw.write(String.valueOf(BAlleles[j]));
                    } else {
                        hapBw.write(String.valueOf(AAlleles[j]));
                    }
                } else {
                    hapBw.write(missingGty);
                }
            }
            hapBw.write("\n");
        }
        return true;
    }

    public void writeMarkerFile(BufferedWriter br, List<AnnotSNP> entries) throws
            Exception {
        int snpSize = entries.size();
        for (int i = 0; i < snpSize; i++) {
            AnnotSNP snp = entries.get(i);
            br.write(snp.getRSID());
            br.write(" ");
            br.write(String.valueOf(snp.getPhysicalPosition()));
            br.write(" ");
            br.write(String.valueOf(snp.getAAllele()));
            br.write(" ");
            br.write(String.valueOf(snp.getBAllele()));
            br.write("\n");
        }
    }
}
