// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.

// Permission is granted for you to use this file to compile IGG.

// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.genetformator;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import org.cobi.igg3.entity.AnnotSNP;
import org.cobi.igg3.entity.PedFile;
import org.cobi.igg3.entity.StatusGtySet;
import org.cobi.igg3.GlobalVariables;

/**
 *
 * @author mxli
 */
public class HapMapFiles {

    public boolean writeHapMapGenotypeFile(String path, String chrom, List<AnnotSNP> entries,
            Map<String, StatusGtySet> indivGtyMap, PedFile hapPed) throws
            Exception {
        int snpSize = entries.size();
        int[] orders = new int[snpSize];
        String missingGty = "nn";
        char[] AAlleles = new char[snpSize];
        char[] BAlleles = new char[snpSize];
        for (int i = 0; i < snpSize; i++) {
            AnnotSNP snp = entries.get(i);
            orders[i] = snp.order;
            AAlleles[i] = snp.getAAllele();
            BAlleles[i] = snp.getBAllele();
        }
        BufferedWriter bw = new BufferedWriter(new FileWriter(path));
        List<String> prjIndiLabels = GlobalVariables.currentPedFile.getIndividualLabel();
        List<String> filteredHapIndiLabels = new ArrayList<String>();
        List<String> hapIndiLabels = new ArrayList<String>();
        if (hapPed != null) {
            hapIndiLabels = hapPed.getIndividualLabel();
        }

        bw.write("rs# alleles chrom pos");
        int prjIndiviSize = prjIndiLabels.size();
        int hapIndiviSize = hapIndiLabels.size();
        for (int i = 0; i < prjIndiviSize; i++) {
            bw.write(" ");
            bw.write(prjIndiLabels.get(i));
        }
        for (int i = 0; i < hapIndiviSize; i++) {
            if (indivGtyMap.get(hapIndiLabels.get(i)) != null) {
                bw.write(" ");
                bw.write(hapIndiLabels.get(i));
                filteredHapIndiLabels.add(hapIndiLabels.get(i));
            }
        }
        hapIndiviSize = filteredHapIndiLabels.size();
        bw.write("\n");
        for (int j = 0; j < snpSize; j++) {
            AnnotSNP snp = entries.get(j);
            bw.write(snp.getRSID());
            bw.write(" ");
            bw.write(snp.getAAllele() + "/" + snp.getBAllele());
            bw.write(" ");
            bw.write(chrom);
            bw.write(" ");
            bw.write(String.valueOf(snp.getPhysicalPosition()));

            for (int i = 0; i < prjIndiviSize; i++) {
                StatusGtySet indivGty = indivGtyMap.get(prjIndiLabels.get(i));
                bw.write(" ");
                if (indivGty != null) {                    
                    if (indivGty.existence.size()>orders[j] && indivGty.existence.getQuick(orders[j])) {
                        if (indivGty.paternalChrom.getQuick(orders[j])) {
                            bw.write(String.valueOf(BAlleles[j]));
                        } else {
                            bw.write(String.valueOf(AAlleles[j]));
                        }
                        if (indivGty.maternalChrom.getQuick(orders[j])) {
                            bw.write(String.valueOf(BAlleles[j]));
                        } else {
                            bw.write(String.valueOf(AAlleles[j]));
                        }
                    } else {
                        bw.write(missingGty);
                    }
                } else {
                    bw.write(missingGty);
                }
            }

            for (int i = 0; i < hapIndiviSize; i++) {
                StatusGtySet indivGty = indivGtyMap.get(filteredHapIndiLabels.get(i));
                bw.write(" ");
                if (indivGty == null) {
                    bw.write(missingGty);
                } else {
                    if (indivGty.existence.size()>orders[j] && indivGty.existence.getQuick(orders[j])) {
                        if (indivGty.paternalChrom.getQuick(orders[j])) {
                            bw.write(String.valueOf(BAlleles[j]));
                        } else {
                            bw.write(String.valueOf(AAlleles[j]));
                        }
                        if (indivGty.maternalChrom.getQuick(orders[j])) {
                            bw.write(String.valueOf(BAlleles[j]));
                        } else {
                            bw.write(String.valueOf(AAlleles[j]));
                        }
                    } else {
                        bw.write(missingGty);
                    }
                }
            }
            bw.write("\n");
        }
        bw.close();
        return true;
    }
}
