// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.
// Permission is granted for you to use this file to compile IGG.
// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.genetformator;

import cern.colt.bitvector.BitVector;
import cern.colt.list.DoubleArrayList;
import cern.jet.stat.Descriptive;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;
import org.cobi.igg3.Constants;
import org.cobi.igg3.entity.AnnotSNP;
import org.cobi.igg3.entity.Individual;
import org.cobi.igg3.entity.Integration;
import org.cobi.igg3.entity.StatusGtySet;
import org.cobi.igg3.GlobalVariables;
import org.cobi.igg3.entity.ChromSNPGtySet;
import org.cobi.igg3.entity.PedFile;

/**
 *
 * @author Miaoxin Li
 */
public class ImputeFiles implements Constants {

    public void writeGenotypeFilesIMPUTE(BufferedWriter bw,
            List<AnnotSNP> entries, Map<String, StatusGtySet> indivGtyMap, List<String> prjIndiLabels) throws
            Exception {

        int snpSize = entries.size();
        int[] orders = new int[snpSize];
        String missingGty = "0 0 0";
        for (int i = 0; i < snpSize; i++) {
            AnnotSNP snp = entries.get(i);
            orders[i] = snp.order;
        }

        int prjIndiviSize = prjIndiLabels.size();

        //the new version imput format do not need individual labels
        /*
        for (int j = 0; j < prjIndiviSize; j++) {
        bw.write(" ");
        bw.write(prjIndiLabels.get(j));
        }
        bw.write("\n");
         */

        for (int j = 0; j < snpSize; j++) {
            AnnotSNP snp = entries.get(j);
            bw.write("SNP");
            bw.write(String.valueOf(j + 1));
            bw.write(" ");
            bw.write(snp.getRSID());
            bw.write(" ");
            bw.write(String.valueOf(snp.getPhysicalPosition()));
            bw.write(" ");
            bw.write(String.valueOf(snp.getAAllele()));
            bw.write(" ");
            bw.write(String.valueOf(snp.getBAllele()));

            for (int i = 0; i < prjIndiviSize; i++) {
                StatusGtySet indivGty = indivGtyMap.get(prjIndiLabels.get(i));
                bw.write(" ");
                if (indivGty != null && indivGty.existence.size() > orders[j] && indivGty.existence.getQuick(orders[j])) {
                    if (indivGty.paternalChrom.getQuick(orders[j]) && indivGty.maternalChrom.getQuick(orders[j])) {
                        bw.write("0 0 1");
                    } else if (!indivGty.paternalChrom.getQuick(orders[j]) && indivGty.maternalChrom.getQuick(orders[j])) {
                        bw.write("0 1 0");
                    } else if (indivGty.paternalChrom.getQuick(orders[j]) && !indivGty.maternalChrom.getQuick(orders[j])) {
                        bw.write("0 1 0");
                    } else {
                        bw.write("1 0 0");
                    }
                } else {
                    bw.write(missingGty);
                }
            }
            bw.write("\n");
        }
    }

    public void writeStrandFilesIMPUTE(BufferedWriter bw, List<AnnotSNP> entries) throws
            Exception {
        int snpSize = entries.size();
        for (int i = 0; i < snpSize; i++) {
            AnnotSNP snp = entries.get(i);
            bw.write(String.valueOf(snp.getPhysicalPosition()));
            bw.write(" ");
            //note: here means the genotypes in genotype files are always consistant with the haplotypes
            //bw.write(String.valueOf(snp.getStrand()));
            bw.write(String.valueOf("+"));
            bw.write("\n");
        }
    }

    /*
     * for SNP test 2.0
     * D	Discrete covariate (coded using positive integers)
C	Continuous covariates
P	Continuous Phenotype
B	Binary Phenotype (0 = Controls, 1 = Cases)
     */
    public void writeSampleFilesIMPUTE(BufferedWriter bw, List<String[]> outTraits) throws Exception {
        Map<String, Individual> prjIndivMap = GlobalVariables.currentPedFile.getAllIndividuals();
        List<String> prjIndiLabels = GlobalVariables.currentPedFile.getIndividualLabel();
        List<String> avaiTrais = GlobalVariables.currentPedFile.getTraitNames();
        int traitSize = outTraits.size();
        int[] traitIndex = new int[traitSize];
        int allTraitSize = avaiTrais.size();
        String[] cell;
        traitSize--;
        bw.write("ID_1 ID_2 missing");
        //determine the output trait
        for (int i = 0; i <= traitSize; i++) {
            cell = outTraits.get(i);
            for (int j = 0; j < allTraitSize; j++) {
                if (cell[0].equals(avaiTrais.get(j))) {
                    traitIndex[i] = j;
                    break;
                }
            }
            bw.write(" ");
            bw.write(cell[0]);
        }
        bw.write("\n");
        bw.write("0 0 0");
        //determine the output trait
        for (int i = 0; i <= traitSize; i++) {
            cell = outTraits.get(i);
            if (cell[1].startsWith("Affection Status")) {
                bw.write(" B");
            } else if (cell[1].startsWith("Quantitative Trait")) {
                bw.write(" P");
            } else if (cell[1].startsWith("Covariate")) {
                bw.write(" C");
            }
        }
        bw.write("\n");
        int indiviSize = prjIndiLabels.size();
        for (int k = 0; k < indiviSize; k++) {
            Individual indiv = prjIndivMap.get(prjIndiLabels.get(k));
            bw.write(indiv.getFamilyID());
            bw.write(" ");
            bw.write(indiv.getIndividualID());
            bw.write(" ");
            bw.write("0.0");
            if (traitSize >= 0) {
                bw.write(" ");
                //write trait value
                for (int j = 0; j < traitSize; j++) {
                    bw.write(indiv.getTraits().get(traitIndex[j]));
                    bw.write(" ");
                }
                bw.write(indiv.getTraits().get(traitIndex[traitSize]));
            }
            bw.write("\n");
        }
    }

    public String writeLegendHaploFilesIMPUTE(BufferedWriter snpBw, BufferedWriter hapBw,
            List<AnnotSNP> entries, Map<String, StatusGtySet> indivGtyMap,
            Integration integration, String chrNum, String prefix) throws
            Exception {

        int snpSize = entries.size();
        int[] orders = new int[snpSize];
        snpBw.write("rs position X0 X1\n");
        long minPos = Long.MAX_VALUE;
        long maxPos = Long.MIN_VALUE;
        long segLen = 5000000;
        for (int i = 0; i < snpSize; i++) {
            AnnotSNP snp = entries.get(i);
            orders[i] = snp.order;
            snpBw.write(snp.getRSID());
            snpBw.write(" ");
            snpBw.write(String.valueOf(snp.getPhysicalPosition()));
            snpBw.write(" ");
            snpBw.write(String.valueOf(snp.getAAllele()));
            snpBw.write(" ");
            snpBw.write(String.valueOf(snp.getBAllele()));
            snpBw.write("\n");
            if (minPos > snp.getPhysicalPosition()) {
                minPos = snp.getPhysicalPosition();
            }
            if (maxPos < snp.getPhysicalPosition()) {
                maxPos = snp.getPhysicalPosition();
            }
        }
        minPos = minPos - 100000;
        if (minPos < 0) {
            minPos = 1;
        }
        maxPos = maxPos + 100000;

        int segNum = (int) ((maxPos - minPos) / segLen);
        if ((maxPos - minPos) % segLen != 0) {
            segNum = segNum + 1;
        }
        //./impute2 -m ./genetic_map_chr5_combined_b36.txt -h ./hap2.5.haplo hap3.5.haplo -l ./hap2.5.legend  hap3.5.legend  -g ./study.5.geno -strand_g ./study.5.strand -int 70000 5070000 -Ne 11418 -o ./study.5.phased.1
        StringBuilder commandsStr = new StringBuilder();
        StringBuilder catCommandStr = new StringBuilder();
        catCommandStr.append("cat ");
        long accumulatedLen = minPos;

        for (int i = 0; i < segNum; i++) {
            commandsStr.append("./impute2 -m ./genetic_map_chr");
            commandsStr.append(chrNum);
            commandsStr.append("_combined_b36.txt -h ./");

            commandsStr.append(prefix);
            commandsStr.append(".");
            commandsStr.append(chrNum);
            commandsStr.append(".hapmap.haps -l ./");
           // commandsStr.append(".haplo northhapIII.").append(chrNum).append(".haplo -l ./");
            commandsStr.append(prefix);
            commandsStr.append(".");
            commandsStr.append(chrNum);
            commandsStr.append(".hapmap.legend -g ./");
            //commandsStr.append(".legend northhapIII.").append(chrNum).append(".legend -g ./");
            commandsStr.append(prefix);
            //commandsStr.append("study");
            commandsStr.append(".");
            commandsStr.append(chrNum);
            commandsStr.append(".study.gens -strand_g ./");
            commandsStr.append(prefix);
            //commandsStr.append("north");
            commandsStr.append(".");
            commandsStr.append(chrNum);
            commandsStr.append(".study.strand  -int ");
            commandsStr.append(accumulatedLen);
            commandsStr.append(" ");
            accumulatedLen += segLen;
            commandsStr.append(accumulatedLen);
            accumulatedLen += 1;
            commandsStr.append(" -Ne 11418 -o ./result/");
            commandsStr.append(prefix);
            commandsStr.append(".");
            commandsStr.append(chrNum);
            commandsStr.append(".phased.");
            commandsStr.append(i + 1);
            commandsStr.append("\n");

            catCommandStr.append(" ./result/");
            catCommandStr.append(prefix);
            catCommandStr.append(".");
            catCommandStr.append(chrNum);
            catCommandStr.append(".phased.");
            catCommandStr.append(i + 1);
        }
        catCommandStr.append(" > ./result/");
        catCommandStr.append(prefix);
        catCommandStr.append(".");
        catCommandStr.append(chrNum);
        catCommandStr.append(".phased\n");

        commandsStr.append(catCommandStr);

        Iterator<String> hapIndiLabelIter = null;
        String chipLabel;
        StringBuilder phasedPoolforOneSNP = new StringBuilder();

        for (int j = 0; j < snpSize; j++) {
            hapIndiLabelIter = integration.getHapIndivLabels().iterator();
            phasedPoolforOneSNP.delete(0, phasedPoolforOneSNP.length());

            while (hapIndiLabelIter.hasNext()) {
                chipLabel = hapIndiLabelIter.next();
                StatusGtySet indivGty = indivGtyMap.get(chipLabel);

                if (indivGty != null) {
                    /*
                    //in the impute version 2 it allowed missing data
                    if (indivGty.existence.getQuick(orders[j])) {
                    if (indivGty.paternalChrom.getQuick(orders[j])) {
                    phasedPoolforOneSNP.append("1 ");
                    } else {
                    phasedPoolforOneSNP.append("0 ");
                    }
                    } else {
                    throw new Exception("Missing data is not allowed in the IMPUTE haplo file!");
                    }

                    if (indivGty.existence.getQuick(orders[j])) {
                    if (indivGty.maternalChrom.getQuick(orders[j])) {
                    phasedPoolforOneSNP.append("1 ");
                    } else {
                    phasedPoolforOneSNP.append("0 ");
                    }
                    } else {
                    throw new Exception("Missing data is not allowed in the IMPUTE haplo file!");
                    }
                     *
                     */
                    if (indivGty.paternalChrom.getQuick(orders[j])) {
                        phasedPoolforOneSNP.append("1 ");
                    } else {
                        phasedPoolforOneSNP.append("0 ");
                    }
                    if (indivGty.maternalChrom.getQuick(orders[j])) {
                        phasedPoolforOneSNP.append("1 ");
                    } else {
                        phasedPoolforOneSNP.append("0 ");
                    }
                } else {
                    throw new Exception("Cannot find phased genotypes for " + chipLabel + " haplo file!");
                }
            }
            hapBw.write(phasedPoolforOneSNP.substring(0, phasedPoolforOneSNP.length() - 1));
            hapBw.write("\n");
        }
        return commandsStr.toString();
    }

    public void readImputProbGenotypes(String fileName, double probThresh,
            Map<String, AnnotSNP> snpMap, List<StatusGtySet> indivGtyList, Set<Integer> availableIDs) throws Exception {
        String line1 = null;
        String rsID = null;
        char alleleA;
        char alleleB;
        double prob1, prob2, prob3;
        StringTokenizer tokenizer;
        BufferedReader br1 = new BufferedReader(new FileReader(fileName));
        StringBuilder tmpBuffer = new StringBuilder();
        int indiviSize = indivGtyList.size();
        int foundID = 0;

        while ((line1 = br1.readLine()) != null) {
            tokenizer = new StringTokenizer(line1);
            //skip first column
            tokenizer.nextToken();
            tmpBuffer.delete(0, tmpBuffer.length());
            tmpBuffer.append(tokenizer.nextToken().trim());
            rsID = tmpBuffer.toString();
            AnnotSNP snp = snpMap.get(rsID);
            if (snp == null) {
                continue;
            }
            foundID++;
            // System.out.println(foundID + " " + rsID);
            tmpBuffer.delete(0, tmpBuffer.length());
            //skip postion column
            tokenizer.nextToken();
            alleleA = tokenizer.nextToken().charAt(0);
            alleleB = tokenizer.nextToken().charAt(0);
            boolean reverseAllele = false;
            if (alleleA == snp.getAAllele() && alleleB == snp.getBAllele()) {
            } else if (alleleA == snp.getBAllele() && alleleB == snp.getAAllele()) {
                reverseAllele = true;
            } else {
                System.err.println("Miss matched allele names at SNP " + rsID);
                return;
            }
            availableIDs.add(snp.order);
            for (int i = 0; i < indiviSize; i++) {
                StatusGtySet indivGty = indivGtyList.get(i);
                prob1 = Double.parseDouble(tokenizer.nextToken());
                prob2 = Double.parseDouble(tokenizer.nextToken());
                prob3 = Double.parseDouble(tokenizer.nextToken());
                if (reverseAllele) {
                    if (prob1 > probThresh) {
                        indivGty.existence.putQuick(snp.order, true);
                        indivGty.paternalChrom.putQuick(snp.order, true);
                        indivGty.maternalChrom.putQuick(snp.order, true);
                    } else if (prob2 > probThresh) {
                        indivGty.existence.putQuick(snp.order, true);
                        indivGty.paternalChrom.putQuick(snp.order, true);
                    } else if (prob3 > probThresh) {
                        indivGty.existence.putQuick(snp.order, true);
                    }
                } else {
                    if (prob1 > probThresh) {
                        indivGty.existence.putQuick(snp.order, true);
                    } else if (prob2 > probThresh) {
                        indivGty.existence.putQuick(snp.order, true);
                        indivGty.paternalChrom.putQuick(snp.order, true);
                    } else if (prob3 > probThresh) {
                        indivGty.existence.putQuick(snp.order, true);
                        indivGty.paternalChrom.putQuick(snp.order, true);
                        indivGty.maternalChrom.putQuick(snp.order, true);
                    }
                }
            }
        }
        System.out.println(availableIDs.size() + "  SNPs are read!");
        br1.close();
    }

    public static void main(String[] args) {
        MergeImputeDatasets main = new MergeImputeDatasets();
        String folder = "D:\\home\\mxli\\MyProjects\\imput2\\";
        String pedFile1 = folder + "1K_South.chr21.II+III.phased";
        String pedFile2 = folder + "testout";
        String outPath = folder + "re.txt";
        try {
            ImputeFiles inputer = new ImputeFiles();
            HashMap<String, List<String>> chromFiles;
            Integration oldIntgration = new Integration("Integrate1", "Asian Population",
                    "D:\\home\\mxli\\MyProjects\\imput2\\genotype_files\\Integrate1\\");

            List<AnnotSNP> projSnpList;
            Map<String, StatusGtySet> projIndivGty;
            ChromSNPGtySet chromSet = new ChromSNPGtySet();


            List<double[]> result = new ArrayList<double[]>();
            Map<String, AnnotSNP> snpMap = new HashMap<String, AnnotSNP>();
            List<StatusGtySet> indivGtyList = new ArrayList<StatusGtySet>();

            File commPedFile = new File(oldIntgration.getResultPath() + File.separator + "ped");
            PedFile pedFile = new PedFile();

            if (commPedFile.exists()) {
                pedFile.readCommonPedigrees(commPedFile.getCanonicalPath());
            }
            List<String> indivList = pedFile.getIndividualLabel();
            Set<Integer> snpIDs = new HashSet<Integer>();
            int indivNum = indivList.size();


            int snpNum = 0;
            for (int i = 0; i < CHROM_NAME.length; i++) {
                chromSet.setChromName(CHROM_NAME[i]);
                chromSet.readAllObjects(oldIntgration.getResultPath());

                projSnpList = chromSet.getSnpInfoList();
                projIndivGty = chromSet.getIndiviGty();

                snpNum = projSnpList.size();
                if (snpNum == 0) {
                    continue;
                }
                for (int j = 0; j < snpNum; j++) {
                    snpMap.put(projSnpList.get(j).getRSID(), projSnpList.get(j));
                }
                double t = 0.34;
                snpIDs.clear();
                while (t <= 1) {
                    indivGtyList.clear();
                    for (int j = 0; j < indivNum; j++) {
                        StatusGtySet gtySet = new StatusGtySet();
                        gtySet.existence = new BitVector(snpNum);
                        gtySet.paternalChrom = new BitVector(snpNum);
                        gtySet.maternalChrom = new BitVector(snpNum);
                        indivGtyList.add(gtySet);
                    }
                    inputer.readImputProbGenotypes(pedFile1, t, snpMap, indivGtyList, snpIDs);
                    result.add(inputer.compareGtySets(indivList, projIndivGty, indivGtyList, snpIDs));
                    t += 0.01;
                }

            }

            int resultSize = result.size();
            for (int k = 0; k < resultSize; k++) {
                double[] items = result.get(k);
                for (int j = 0; j < items.length; j++) {
                    System.out.print(items[j] == 0 ? "0" : String.valueOf(items[j] * 100));
                    System.out.print(" ");
                }
                System.out.println();
            }

        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public double[] compareGtySets(List<String> indivList, Map<String, StatusGtySet> gtyList1,
            List<StatusGtySet> gtyList2, Set<Integer> snpIDs) {
        int indivNum = indivList.size();
        DoubleArrayList missRates = new DoubleArrayList();
        DoubleArrayList discorRates = new DoubleArrayList();

        for (int i = 0; i < indivNum; i++) {
            StatusGtySet gtySet1 = gtyList1.get(indivList.get(i));
            StatusGtySet gtySet2 = gtyList2.get(i);
            int snpNum = gtySet1.existence.size();
            double missRate = 0;
            double discorRate = 0;
            Iterator<Integer> iter = snpIDs.iterator();
            while (iter.hasNext()) {
                int k = iter.next();
                if (gtySet2.existence.getQuick(k)) {
                    if (gtySet1.paternalChrom.getQuick(k) && gtySet2.paternalChrom.getQuick(k)
                            && gtySet1.maternalChrom.getQuick(k) && gtySet2.maternalChrom.getQuick(k)) {
                    } else if (!gtySet1.paternalChrom.getQuick(k) && !gtySet2.paternalChrom.getQuick(k)
                            && !gtySet1.maternalChrom.getQuick(k) && !gtySet2.maternalChrom.getQuick(k)) {
                    } else if ((gtySet1.paternalChrom.getQuick(k) || gtySet1.maternalChrom.getQuick(k))
                            && (gtySet2.paternalChrom.getQuick(k) || gtySet2.maternalChrom.getQuick(k))) {
                    } else {
                        discorRate++;
                        //System.err.println("Indiv: " + i + " " + indivList.get(i) + " " + k);
                    }
                } else {
                    missRate++;
                }
            }

            missRates.add(missRate / indivNum);
            discorRates.add(discorRate / indivNum);
        }
        double[] results = new double[4];
        results[0] = Descriptive.mean(missRates);
        results[1] = Descriptive.sampleVariance(missRates, results[0]);
        results[2] = Descriptive.mean(discorRates);
        results[3] = Descriptive.sampleVariance(discorRates, results[2]);
        return results;
    }
}
