// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.
// Permission is granted for you to use this file to compile IGG.
// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.genetformator;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import org.cobi.igg3.entity.AnnotSNP;

/**
 *
 * @author mxli
 */
public class LinkageFiles {

    /**This function can  generate pedigree description file for merlin
     *
     *
     */
    public boolean writeDataFileLinkage(BufferedWriter bw, List<String[]> phenotypes, List<AnnotSNP> snpInfo) throws
            Exception {

        int snpInfoSize = snpInfo.size();
        int phenotypeNum = phenotypes.size();
        int totalLoci = snpInfoSize + phenotypeNum;
        AnnotSNP snpInfoRow = null;
        String[] cells = null;
        DecimalFormat df = new DecimalFormat();
        df.setMaximumFractionDigits(6);

        bw.write(totalLoci + " 0 0 5  << NO. OF LOCI, RISK LOCUS, SEXLINKED (IF 1) PROGRAM\n");
        bw.write("0 0.0 0.0 0 << MUT LOCUS, MUT MALE, MUT FEM, HAP FREQ (IF 1)\n");
        for (int k = 1; k < totalLoci; k++) {
            bw.write(" " + k);
        }
        bw.write(" " + totalLoci);
        bw.write("\n");
        for (int i = 0; i < phenotypeNum; i++) {
            //Phentype types:  A - affection status, T - Quantitative Trait and C - Covariate)
            cells = phenotypes.get(i);
            if (cells[1].equals("Affection Status")) {
                bw.write("1   2 # " + cells[0] + " << AFFECTION, NO. OF ALLELES\n");
                bw.write("0.99000  0.01000 << GENE FREQUENCIES\n");
                bw.write("1 << NO. OF LIABILITY CLASSES\n");
                bw.write("0.00100000  0.9990  0.9990 << PENETRANCES\n");
            } else if (cells[1].indexOf("Quantitative Trait") >= 0) {
                bw.write("0   2 # " + cells[0] + " << TRAIT, NO. OF ALLELES\n");
                bw.write("0.99000  0.01000 << GENE FREQUENCIES\n");
                bw.write("1 << NO. OF TRAITS\n");
                bw.write("1.000   10.000 20.000 << GENOTYPE MEANS\n");
                bw.write("1.000   << VARIANCE - COVARIANCE MATRIX\n");
                bw.write("1.000   << MULTIPLIER FOR VARIANCE IN HETEROZYGOTES\n");
            }
        }

 
        for (int k = 0; k < snpInfoSize; k++) {
            snpInfoRow = snpInfo.get(k);
            bw.write("3   2 # " + snpInfoRow.getRSID() + " << ALLELE NUMBERS, NO. OF ALLELES\n");
            //if the frequencies do exitst
            if (snpInfoRow.getAAlleleFreq() >= 0) {
                double freq0 = snpInfoRow.getAAlleleFreq();
                double freq1 = 1 - freq0;
                if (freq0 == 0) {
                    freq0 = 0.001;
                    freq1 = 0.999;
                } else if (freq1 == 1) {
                    freq0 = 0.999;
                    freq1 = 0.001;
                }
                bw.write(df.format(freq0) + "  " + df.format(freq1) + " << GENE FREQUENCIES\n");
            } else {
                bw.write("0.5  0.5 << GENE FREQUENCIES\n");
            }
        }
        bw.write("0 0  << SEX DIFFERENCE, INTERFERENCE (IF 1 OR 2)\n");
        bw.write("0.1000 ");

        AnnotSNP tmpSnpInfoRow = null;

        if (snpInfoSize > 0) {
            double lastPosotion = snpInfo.get(0).getDecodeGeneticMap();
            double currentPostion = -1.0;
            double incTime = 1.0;
            for (int k = 1; k < snpInfoSize; k++) {
                snpInfoRow = snpInfo.get(k);
                if (snpInfoRow.getDecodeGeneticMap() < 0) {
                    bw.write("0.0 ");
                    continue;
                }
                currentPostion = snpInfoRow.getDecodeGeneticMap();
                //make sure the genetic map of the markers in ascending order
                if (lastPosotion > currentPostion) {
                    int s = k + 2;
                    while (true) {
                        if (s >= snpInfoSize) {
                            currentPostion = lastPosotion + 0.00001 * incTime;
                            break;
                        }
                        tmpSnpInfoRow = snpInfo.get(s);
                        if (tmpSnpInfoRow.getDecodeGeneticMap() < 0) {
                            s++;
                            tmpSnpInfoRow = snpInfo.get(s);
                            continue;
                        }

                        currentPostion = tmpSnpInfoRow.getDecodeGeneticMap();
                        if (lastPosotion < currentPostion) {
                            currentPostion = lastPosotion + (currentPostion - lastPosotion) / (s - k);
                            break;
                        } else {
                            s++;
                        }
                    }
                }
                //make sure the genetic map of the markers are unique
                if (lastPosotion == currentPostion) {
                    bw.write(df.format(converRecombinationKosambiMap(0.00001)) + " ");
                    //System.out.println(snpInfoRow[1]+" : "+incTime);
                    incTime++;
                } else {
                    bw.write(df.format(converRecombinationKosambiMap(currentPostion - lastPosotion)) + " ");
                }
                lastPosotion = currentPostion;
            }
        }
        bw.write("<< RECOMBINATION VALUES\n");
        bw.write("1 0.10000 0.45000 << REC VARIED, INCREMENT, FINISHING VALUE\n");
        bw.close();
        return true;
    }

    public boolean writeDataFileLinkage(BufferedWriter bw, List<String[]> phenotypes, List<AnnotSNP> snpInfo, String hapMapRecombFilePath) throws
            Exception {
        Map<Integer, String[]> freqMap = null;
        String line = null;
        /*
         * the frequency format will be like this
         * Chromosome	Position	ReferenceAlternativeAllele	dbAltAF
        17	26873612	C/T	0.33
        17	27649318	G/A	0.432203
        17	28343137	A/C	0.746
        17	28348190	C/A	0.435
        
         */

        File freqFile = new File("as.frq.txt");
        if (freqFile.exists()) {
            freqMap = new HashMap<Integer, String[]>();
            BufferedReader br1 = new BufferedReader(new FileReader(freqFile));

            br1.readLine();
            while ((line = br1.readLine()) != null) {
                String[] cells = line.split("\t");
                freqMap.put(Integer.parseInt(cells[1]), cells);
            }
            br1.close();
        }



        File recomFile = new File(hapMapRecombFilePath);
        if (!recomFile.exists()) {
            writeDataFileLinkage(bw, phenotypes, snpInfo);
            System.err.println("Warning! " + hapMapRecombFilePath + " does not exist!");
            return true;
        }
        BufferedReader br = new BufferedReader(new FileReader(recomFile));

        br.readLine();
        //read map
        List<String[]> postionMapList = new ArrayList<String[]>();
        StringBuilder tmpBuffer = new StringBuilder();
        while ((line = br.readLine()) != null) {
            StringTokenizer tokenizer = new StringTokenizer(line);
            String[] cells = new String[3];

            for (int i = 0; i < 3; i++) {
                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                cells[i] = tmpBuffer.toString();
            }
            postionMapList.add(cells);
        }
        br.close();

        int snpInfoSize = snpInfo.size();
        int phenotypeNum = phenotypes.size();
        int totalLoci = snpInfoSize + phenotypeNum;
        AnnotSNP snpInfoRow = null;
        String[] cells = null;
        DecimalFormat df = new DecimalFormat();
        df.setMaximumFractionDigits(6);

        bw.write(totalLoci + " 0 0 5  << NO. OF LOCI, RISK LOCUS, SEXLINKED (IF 1) PROGRAM\n");
        bw.write("0 0.0 0.0 0 << MUT LOCUS, MUT MALE, MUT FEM, HAP FREQ (IF 1)\n");
        for (int k = 1; k < totalLoci; k++) {
            bw.write(" " + k);
        }
        bw.write(" " + totalLoci);
        bw.write("\n");
        for (int i = 0; i < phenotypeNum; i++) {
            //Phentype types:  A - affection status, T - Quantitative Trait and C - Covariate)
            cells = phenotypes.get(i);
            if (cells[1].equals("Affection Status")) {
                bw.write("1   2 # " + cells[0] + " << AFFECTION, NO. OF ALLELES\n");
                bw.write("0.99000  0.01000 << GENE FREQUENCIES\n");
                bw.write("1 << NO. OF LIABILITY CLASSES\n");
                bw.write("0.00100000  0.9990  0.9990 << PENETRANCES\n");
            } else if (cells[1].indexOf("Quantitative Trait") >= 0) {
                bw.write("0   2 # " + cells[0] + " << TRAIT, NO. OF ALLELES\n");
                bw.write("0.99000  0.01000 << GENE FREQUENCIES\n");
                bw.write("1 << NO. OF TRAITS\n");
                bw.write("1.000   10.000 20.000 << GENOTYPE MEANS\n");
                bw.write("1.000   << VARIANCE - COVARIANCE MATRIX\n");
                bw.write("1.000   << MULTIPLIER FOR VARIANCE IN HETEROZYGOTES\n");
            }
        }


        for (int k = 0; k < snpInfoSize; k++) {
            snpInfoRow = snpInfo.get(k);
            bw.write("3   2 # " + snpInfoRow.getRSID() + " << ALLELE NUMBERS, NO. OF ALLELES\n");
            //if the frequencies do exitst
            double freq0 = 0.01;
            double freq1 = 0.99;
            
            if (freqMap != null) {
                String[] cells1 = freqMap.get((int) snpInfoRow.getPhysicalPosition());
                char altAllele = cells1[2].charAt(2);
                if (snpInfoRow.getAAllele() == altAllele) {
                    freq0 = Double.parseDouble(cells1[3]);
                } else {
                    freq0 = 1 - Double.parseDouble(cells1[3]);
                }
                if (freq0 == 0) {
                    freq0 = 0.001;
                } else if (freq0 == 1) {
                    freq0 = 0.999;
                }
                freq1 = 1 - freq0;
                //bw.write(df.format(freq0) + "  " + df.format(freq1)  + " << GENE FREQUENCIES\n");
                bw.write(df.format(freq0) + "  " + df.format(freq1) + " << GENE FREQUENCIES " + snpInfoRow.getAAllele() + " " + snpInfoRow.getBAllele() + "\n");

            } else 
                if (snpInfoRow.getAAlleleFreq() >= 0) {
                freq0 = snpInfoRow.getAAlleleFreq();
                freq1 = 1 - freq0;

                if (freq0 == 0) {
                    freq0 = 0.001;
                    freq1 = 0.999;
                } else if (freq1 == 1) {
                    freq0 = 0.999;
                    freq1 = 0.001;
                }
                //bw.write(df.format(freq0) + "  " + df.format(freq1)  + " << GENE FREQUENCIES\n");
                bw.write(df.format(freq0) + "  " + df.format(freq1) + " << GENE FREQUENCIES " + snpInfoRow.getAAllele() + " " + snpInfoRow.getBAllele() + "\n");
            } else {
                bw.write("0.5  0.5 << GENE FREQUENCIES\n");
            }
        }
        bw.write("0 0  << SEX DIFFERENCE, INTERFERENCE (IF 1 OR 2)\n");
        bw.write("0.1000 ");


        if (snpInfoSize > 0) {
            int currentPhysicalPos = (int) snpInfo.get(0).getPhysicalPosition();
            double lastGenetMap = Double.parseDouble(interpolateLinkagePostion(postionMapList, currentPhysicalPos));
            double currentGenetMap = Double.parseDouble(interpolateLinkagePostion(postionMapList, currentPhysicalPos));

            for (int k = 1; k < snpInfoSize; k++) {
                snpInfoRow = snpInfo.get(k);

                currentPhysicalPos = (int) snpInfoRow.getPhysicalPosition();
                currentGenetMap = Double.parseDouble(interpolateLinkagePostion(postionMapList, currentPhysicalPos));

                //make sure the genetic map of the markers are unique
                if (lastGenetMap == currentGenetMap) {
                    bw.write(df.format(converRecombinationKosambiMap(0.00001)) + " ");
                    //System.out.println(snpInfoRow[1]+" : "+incTime);
                } else {
                    bw.write(df.format(converRecombinationKosambiMap(currentGenetMap - lastGenetMap)) + " ");
                }
                lastGenetMap = currentGenetMap;
            }
        }
        bw.write("<< RECOMBINATION VALUES\n");
        bw.write("1 0.10000 0.45000 << REC VARIED, INCREMENT, FINISHING VALUE\n");
        bw.close();
        return true;
    }

    public String interpolateLinkagePostion(List<String[]> postionMapList, int postion) {
        int size = postionMapList.size();
        for (int i = 1; i < size; i++) {
            int pos1 = Integer.parseInt(postionMapList.get(i - 1)[0]);
            int pos2 = Integer.parseInt(postionMapList.get(i)[0]);

            if (postion <= pos1) {
                return postionMapList.get(i - 1)[2];
            } else if (pos2 == postion) {
                return postionMapList.get(i)[2];
            } else if (postion > pos1 && postion < pos2) {
                if (Math.abs(postion - pos1) < Math.abs(postion - pos2)) {
                    return postionMapList.get(i - 1)[2];
                } else {
                    return postionMapList.get(i)[2];
                }
            }
        }
        return "-1";
    }

    public static double converRecombinationHaldaneMap(double cm) {
        return (0.5 * (1 - Math.exp(-2 * cm / 100)));
    }

    public static double converRecombinationKosambiMap(double cm) {
        return (0.5 * Math.tanh(2 * cm / 100));
    }
}
