// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.
// Permission is granted for you to use this file to compile IGG.
// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.genetformator;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import org.apache.commons.collections.map.MultiValueMap;
import org.cobi.igg3.entity.AnnotSNP;
import org.cobi.igg3.entity.Individual;
import org.cobi.igg3.entity.Integration;
import org.cobi.igg3.entity.PedFile;
import org.cobi.igg3.GlobalVariables;

/**
 *
 * @author mxli
 */
public class Mega2Files {

    LinkageFiles linkageFiles = new LinkageFiles();

    public void writeDataFilesMega2(BufferedWriter bw, List<String[]> phenotypes, List<AnnotSNP> snpInfo, String hapMapRecombFilePath) throws
            Exception {
        linkageFiles.writeDataFileLinkage(bw, phenotypes, snpInfo, hapMapRecombFilePath);
    }
    public void writeDataFilesMega2(BufferedWriter bw, List<String[]> phenotypes, List<AnnotSNP> snpInfo) throws
            Exception {
        linkageFiles.writeDataFileLinkage(bw, phenotypes, snpInfo);
    }

    public boolean writeGeneticMapsMega2(BufferedWriter bw, List<AnnotSNP> snpInfo) throws
            Exception {
        DecimalFormat df = new DecimalFormat();
        df.setMaximumFractionDigits(6);
        int snpInfoSize = 0;
        snpInfoSize = snpInfo.size();
        AnnotSNP snpInfoRow = null;
        AnnotSNP tmpSnpInfoRow = null;
        double lastPosotion = -1.0;
        double currentPostion = -1.0;
        double incTime = 1.0;
        for (int k = 0; k < snpInfoSize; k++) {
            snpInfoRow = snpInfo.get(k);
            bw.write(String.valueOf(snpInfoRow.getChromosomeNum()));
            bw.write(" ");
            if (snpInfoRow.getDecodeGeneticMap() < 0) {
                bw.write("0.0 ");
                bw.write(snpInfoRow.getRSID());
                bw.write("\n");
                continue;
            } else {
                currentPostion = snpInfoRow.getDecodeGeneticMap();
                //make sure the genetic map of the markers in ascending order
                if (lastPosotion > currentPostion) {
                    int s = k + 2;
                    while (true) {
                        if (s >= snpInfoSize) {
                            currentPostion = lastPosotion + 0.00001 * incTime;
                            break;
                        }
                        tmpSnpInfoRow = snpInfo.get(s);
                        if (tmpSnpInfoRow.getDecodeGeneticMap() < 0) {
                            s++;
                            continue;
                        }
                        currentPostion = tmpSnpInfoRow.getDecodeGeneticMap();
                        if (lastPosotion < currentPostion) {
                            currentPostion = lastPosotion + (currentPostion - lastPosotion) / (s - k);
                            break;
                        } else {
                            s++;
                        }
                    }
                }
            }
            //make sure the genetic map of the markers are unique
            if (lastPosotion == currentPostion) {
                bw.write(df.format(lastPosotion + 0.00001 * incTime));
                //System.out.println(snpInfoRow[1]+" : "+incTime);
                incTime++;
            } else {
                bw.write(df.format(currentPostion));
            }
            bw.write(" ");
            bw.write(snpInfoRow.getRSID());
            bw.write("\n");
            lastPosotion = currentPostion;
        }
        return true;
    }

    public boolean writeGeneticMapsMega2(BufferedWriter bw, List<AnnotSNP> snpInfo, String hapMapRecombFilePath) throws
            Exception {
        DecimalFormat df = new DecimalFormat();
        df.setMaximumFractionDigits(6);
        int snpInfoSize = 0;
        snpInfoSize = snpInfo.size();
        AnnotSNP snpInfoRow = null;

        int currentPostion = 0;

        File recomFile = new File(hapMapRecombFilePath);
        if (!recomFile.exists()) {
            writeGeneticMapsMega2(bw, snpInfo);
            System.err.println("Warning! " + hapMapRecombFilePath + " does not exist!");
            return true;
        }
        BufferedReader br = new BufferedReader(new FileReader(recomFile));
        String line = null;

        br.readLine();
        //read map
        List<String[]> postionMapList = new ArrayList<String[]>();
        StringBuilder tmpBuffer = new StringBuilder();
        while ((line = br.readLine()) != null) {
            StringTokenizer tokenizer = new StringTokenizer(line);
            String[] cells = new String[3];

            for (int i = 0; i < 3; i++) {
                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                cells[i] = tmpBuffer.toString();
            }
            postionMapList.add(cells);
        }
        br.close();

        for (int k = 0; k < snpInfoSize; k++) {
            snpInfoRow = snpInfo.get(k);
            bw.write(String.valueOf(snpInfoRow.getChromosomeNum()));
            bw.write(" ");
            currentPostion = (int) snpInfoRow.getPhysicalPosition();
            String strPost = interpolateLinkagePostion(postionMapList, currentPostion);
            bw.write(strPost);
            bw.write(" ");
            bw.write(snpInfoRow.getRSID());
            bw.write("\n");
            //make sure the genetic map of the markers are unique

        }
        return true;
    }
    /*This funciton will change the value of original variable
     * 
     */

    public void orderIndivIDbyNum(PedFile hapPed, Integration integration) throws Exception {
        Map<String, Individual> prjIndivMap = GlobalVariables.currentPedFile.getAllIndividuals();
        int famCounter = 1, indivCounter = 1;
        MultiValueMap fams = new MultiValueMap();
        Map<String, Integer> famNumberMap = new HashMap<String, Integer>();
        for (Map.Entry<String, Individual> m : prjIndivMap.entrySet()) {
            fams.put(m.getValue().getFamilyID(), m.getValue().getFamilyID()
                    + m.getValue().getIndividualID() + "&" + m.getValue().getLabelInChip());

            if (famNumberMap.get(m.getValue().getFamilyID()) == null) {
                famNumberMap.put(m.getValue().getFamilyID(), famCounter);
                famCounter++;
            }
        }
        Iterator famIter = fams.keySet().iterator();
        Map<String, Integer> indivNumberMap = new HashMap<String, Integer>();
        List<String> newLabelOrder = new ArrayList<String>();
        //decide the order
        while (famIter.hasNext()) {
            String famID = famIter.next().toString();
            Collection memebers = fams.getCollection(famID);
            Iterator membIter = memebers.iterator();
            indivCounter = 1;
            while (membIter.hasNext()) {
                String indivLabel = membIter.next().toString();
                indivNumberMap.put(indivLabel.substring(0, indivLabel.indexOf('&')), indivCounter);
                indivCounter++;
                newLabelOrder.add(indivLabel.substring(indivLabel.indexOf('&') + 1));
            }
        }
        //replace the order
        for (Map.Entry<String, Individual> m : prjIndivMap.entrySet()) {
            Individual indiv = m.getValue();
            indiv.setIndividualID(indivNumberMap.get(indiv.getFamilyID() + indiv.getIndividualID()).toString());
            if (!indiv.getDadID().equals("0")) {
                indiv.setDadID(indivNumberMap.get(indiv.getFamilyID() + indiv.getDadID()).toString());
            }
            if (!indiv.getMomID().equals("0")) {
                indiv.setMomID(indivNumberMap.get(indiv.getFamilyID() + indiv.getMomID()).toString());
            }
            indiv.setFamilyID(famNumberMap.get(indiv.getFamilyID()).toString());
        }
        GlobalVariables.currentPedFile.getIndividualLabel().clear();
        GlobalVariables.currentPedFile.getIndividualLabel().addAll(newLabelOrder);

        if (hapPed != null) {
            Iterator hapIndiLabelIter = integration.getHapIndivLabels().iterator();
            Map<String, Individual> hapIndivMap = hapPed.getAllIndividuals();
            famNumberMap.clear();
            indivNumberMap.clear();
            newLabelOrder.clear();
            fams.clear();
            String hapIndiv;
            while (hapIndiLabelIter.hasNext()) {
                //this is awful format of hapmap like NA18594.dup
                hapIndiv = (String) hapIndiLabelIter.next();
                int awfulIndex = hapIndiv.indexOf(".");
                if (awfulIndex >= 0) {
                    hapIndiv = hapIndiv.substring(0, awfulIndex);
                    GlobalVariables.mainView.setBriefRunningInfor("Warning!!! a duplicate HapMap sample " + hapIndiv + " is ignored!");
                    GlobalVariables.addInforLog("Warning!!! a duplicate HapMap sample " + hapIndiv + " is ignored!");
                }
                Individual indiv = hapIndivMap.get(hapIndiv);
                fams.put(indiv.getFamilyID(), indiv.getFamilyID()
                        + indiv.getIndividualID() + "&" + indiv.getLabelInChip());

                //if (indiv.getLabelInChip())
                if (famNumberMap.get(indiv.getFamilyID()) == null) {
                    famNumberMap.put(indiv.getFamilyID(), famCounter);
                    famCounter++;
                }
            }

            famIter = fams.keySet().iterator();
            //decide the order
            while (famIter.hasNext()) {
                String famID = famIter.next().toString();
                Collection memebers = fams.getCollection(famID);
                Iterator membIter = memebers.iterator();
                indivCounter = 1;

                while (membIter.hasNext()) {
                    String indivLabel = membIter.next().toString();
                    indivNumberMap.put(indivLabel.substring(0, indivLabel.indexOf('&')), indivCounter);
                    indivCounter++;
                    newLabelOrder.add(indivLabel.substring(indivLabel.indexOf('&') + 1));
                }
            }
            //replace the order
            hapIndiLabelIter = integration.getHapIndivLabels().iterator();
            while (hapIndiLabelIter.hasNext()) {
                hapIndiv = (String) hapIndiLabelIter.next();
                int awfulIndex = hapIndiv.indexOf(".");
                if (awfulIndex >= 0) {
                    hapIndiv = hapIndiv.substring(0, awfulIndex);
                }
                Individual indiv = hapIndivMap.get(hapIndiv);

                indiv.setIndividualID(indivNumberMap.get(indiv.getFamilyID() + indiv.getIndividualID()).toString());
                if (!indiv.getDadID().equals("0")) {
                    Integer tmpInt = indivNumberMap.get(indiv.getFamilyID() + indiv.getDadID());
                    if (tmpInt != null) {
                        indiv.setDadID(tmpInt.toString());
                    } else {
                        indiv.setDadID("0");
                    }
                }
                if (!indiv.getMomID().equals("0")) {
                    Integer tmpInt = indivNumberMap.get(indiv.getFamilyID() + indiv.getMomID());
                    if (tmpInt != null) {
                        indiv.setMomID(tmpInt.toString());
                    } else {
                        indiv.setMomID("0");
                    }

                }
                indiv.setFamilyID(famNumberMap.get(indiv.getFamilyID()).toString());
            }

            integration.getHapIndivLabels().clear();
            integration.getHapIndivLabels().addAll(newLabelOrder);
        }
    }

    public void interpolateLinkageMap(String hapMapRecombFilePath, String varaintsFilePath, String outName) throws Exception {
        BufferedReader br = new BufferedReader(new FileReader(hapMapRecombFilePath));
        String line = null;
        String delmilit = "\t";
        br.readLine();
        //read map
        List<String[]> postionMapList = new ArrayList<String[]>();
        StringBuilder tmpBuffer = new StringBuilder();
        while ((line = br.readLine()) != null) {
            StringTokenizer tokenizer = new StringTokenizer(line);
            String[] cells = new String[3];

            for (int i = 0; i < 3; i++) {
                tmpBuffer.delete(0, tmpBuffer.length());
                tmpBuffer.append(tokenizer.nextToken().trim());
                cells[i] = tmpBuffer.toString();
            }
            postionMapList.add(cells);
        }
        br.close();
        int pos = 0;
        BufferedWriter bw = new BufferedWriter(new FileWriter(outName));
        BufferedReader br1 = new BufferedReader(new FileReader(varaintsFilePath));
        line = br1.readLine();
        bw.write(line + "\t genetPos");
        bw.newLine();
        while ((line = br1.readLine()) != null) {
            StringTokenizer tokenizer = new StringTokenizer(line);
            tokenizer.nextToken();
            pos = Integer.parseInt(tokenizer.nextToken().trim());
            String strPost = interpolateLinkagePostion(postionMapList, pos);
            bw.write(line + "\t" + strPost);
            bw.newLine();
        }
        bw.close();
        br1.close();
    }

    public String interpolateLinkagePostion(List<String[]> postionMapList, int postion) {
        int size = postionMapList.size();
        for (int i = 1; i < size; i++) {
            int pos1 = Integer.parseInt(postionMapList.get(i - 1)[0]);
            int pos2 = Integer.parseInt(postionMapList.get(i)[0]);

            if (postion <= pos1) {
                return postionMapList.get(i - 1)[2];
            } else if (pos2 == postion) {
                return postionMapList.get(i)[2];
            } else if (postion > pos1 && postion < pos2) {
                if (Math.abs(postion - pos1) < Math.abs(postion - pos2)) {
                    return postionMapList.get(i - 1)[2];
                } else {
                    return postionMapList.get(i)[2];
                }
            }
        }
        return "-1";
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        // TODO code application logic here
        Mega2Files converter = new Mega2Files();
        try {
            converter.interpolateLinkageMap("genetic_map_chr17_b36.txt", "atrial.flt.txt", "atrial.filter.genet.txt");
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}
