// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.

// Permission is granted for you to use this file to compile IGG.

// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.genetformator;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import org.cobi.util.file.LocalFile;

/**
 *
 * @author mxli
 */
public class MergeMerlinDatasets {

    private Map<String, int[]> matchMarkers(String markerFile1, String markerFile2) throws
            Exception {
        List<String> markerList1 = new ArrayList<String>();
        List<String> markerList2 = new ArrayList<String>();
        Map<String, int[]> markerMap = new HashMap<String, int[]>();
        LocalFile.retrieveData(markerFile1, markerList1, 1, null);
        LocalFile.retrieveData(markerFile2, markerList2, 1, null);
        int sizeList = markerList1.size() - 1;

        markerList1.remove(0);
        markerList2.remove(0);
        //append the index id to marker list 2
        for (int i = 0; i < sizeList; i++) {
            int[] pos = new int[2];
            pos[0] = i;
            pos[1] = -1;
            markerMap.put(markerList1.get(i), pos);
        }
        sizeList = markerList2.size();
        int matchedNum=0;
        for (int i = 0; i < sizeList; i++) {
            int[] pos = markerMap.get(markerList2.get(i));
            if (pos == null) {
                pos = new int[2];
                pos[0] = -1;
                pos[1] = i;
                markerMap.put(markerList2.get(i), pos);
            } else {
                pos[1] = i;
                matchedNum++;
            }
        }
        System.out.println("There "+matchedNum+" matched markers!");
        return markerMap;
    }

    private void comparePedigreeFiles(String pedFile1, String pedFile2, Map<String, int[]> markerMap, String outputFile, int offsetCol) throws
            Exception {
        BufferedWriter bw = new BufferedWriter(new FileWriter(outputFile));
        BufferedReader br1 = new BufferedReader(new FileReader(pedFile1));
        BufferedReader br2 = new BufferedReader(new FileReader(pedFile2));
        String line1 = null;
        String line2 = null;

        List<String> lineCell1 = new ArrayList<String>();
        List<String> lineCell2 = new ArrayList<String>();
        StringTokenizer tokenizer;
        int lineCounter = 0;
        while ((line1 = br1.readLine()) != null) {
            lineCell1.clear();
            lineCell2.clear();
            tokenizer = new StringTokenizer(line1);
            while (tokenizer.hasMoreTokens()) {
                lineCell1.add(tokenizer.nextToken().trim());
            }
            line2 = br2.readLine();
            tokenizer = new StringTokenizer(line2);
            while (tokenizer.hasMoreTokens()) {
                lineCell2.add(tokenizer.nextToken().trim());
            }
            lineCounter++;
            if (!lineCell1.get(0).equals(lineCell2.get(0)) || !lineCell1.get(1).equals(lineCell2.get(1))) {
                throw new Exception(lineCounter + " lines have different subjects!");
            }
            for (Map.Entry<String, int[]> m : markerMap.entrySet()) {
                int[] pos = m.getValue();
                if (pos[0] >= 0 && pos[1] >= 0) {
                    String gty1 = lineCell1.get(offsetCol + pos[0] * 2) + lineCell1.get(offsetCol + pos[0] * 2 + 1);
                    String gty2 = lineCell2.get(offsetCol + pos[1] * 2) + lineCell2.get(offsetCol + pos[1] * 2 + 1);
                    if (!gty1.equals(gty2)) {
                        System.out.println("Idiv " + lineCell1.get(0) + "->" + lineCell1.get(1) + " has inconsistant genotypes at SNP " + m.getKey() + " " + gty1 + " (File 1)" + gty2 + " (File 2)");
                    }
                }
            }
        }
        br1.close();
        br2.close();
        bw.close();
    }

    public static void main(String[] args) {
        MergeMerlinDatasets main = new MergeMerlinDatasets();
        /*
        pedFile1="D:/inputSamples/plink.ped";
        pedFile2="D:/inputSamples/plink.ped";
        mapFile1="D:/inputSamples/plink.map";
        mapFile2="D:/inputSamples/plink.map";
         */
       // String folder = "C:/cygwin/home/mxli/IGG/inputSamples/";
        //String folder="D:/home/mxli/MyProjects/CMT2_Disease/genotypesmore90rate/all/";
        String folder="D:/copy/";
        String pedFile1 = folder + "test1.ped";
        String pedFile2 = folder + "test2.ped";
        String markerFile1 = folder + "chx.dat";
        String markerFile2 = folder + "chx.dat";
        String freqFile1 = folder + "chx.freq";
        String freqFile2 = folder + "chx.freq";
        String mapFile1 = folder + "test1.map";
        String mapFile2 = folder + "test2.map";
        String outPath = folder + "re.txt";
        try {
            Map<String, int[]> markerMap = main.matchMarkers(mapFile1, mapFile2);
            main.comparePedigreeFiles(pedFile1, pedFile2, markerMap, outPath, 6);
        } catch (Exception ex) {
            ex.printStackTrace();
        }

    }
}
