// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.
// Permission is granted for you to use this file to compile IGG.
// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.genetformator;

import cern.colt.list.IntArrayList;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import org.cobi.igg3.entity.AnnotSNP;
import org.cobi.igg3.entity.StatusGtySet;
import org.cobi.igg3.entity.Integration;

/**
 *
 * @author mxli
 */
public class MerlinFiles {

    /**This function can  generate pedigree description file for merlin
     *
     *
     */
    public boolean writePedFileDescriptionMerlin(BufferedWriter bw, List<AnnotSNP> snpInfo) throws
            Exception {
        try {
            int snpInfoSize = 0;
            snpInfoSize = snpInfo.size();
            for (int k = 0; k < snpInfoSize; k++) {
                bw.write("M  ");
                bw.write(String.valueOf(snpInfo.get(k).getRSID()));
                bw.write("\n");
            }
            return true;
        } catch (Exception e) {
            throw e;
        }
    }

    /**This function can generate genetic map for merlin
     *
     *
     */
    public boolean writeGeneticMapsMerlin(BufferedWriter bw, List<AnnotSNP> snpInfo) throws
            Exception {
        DecimalFormat df = new DecimalFormat();
        df.setMaximumFractionDigits(6);
        int snpInfoSize = 0;
        snpInfoSize = snpInfo.size();
        AnnotSNP snpInfoRow = null;
        AnnotSNP tmpSnpInfoRow = null;
        double lastPosotion = -1.0;
        double currentPostion = -1.0;
        double incTime = 1.0;
        for (int k = 0; k < snpInfoSize; k++) {
            snpInfoRow = snpInfo.get(k);

            bw.write(String.valueOf(snpInfoRow.getChromosomeNum()));
            bw.write("   ");
            bw.write(snpInfoRow.getRSID());
            bw.write("  ");
            if (snpInfoRow.getDecodeGeneticMap() < 0) {
                bw.write("0.0\n");
                continue;
            }
            currentPostion = snpInfoRow.getDecodeGeneticMap();
            //make sure the genetic map of the markers in ascending order
            if (lastPosotion > currentPostion) {
                int s = k + 2;
                while (true) {
                    if (s >= snpInfoSize) {
                        currentPostion = lastPosotion + 0.00001 * incTime;
                        break;
                    }

                    tmpSnpInfoRow = snpInfo.get(s);
                    if (tmpSnpInfoRow.getDecodeGeneticMap() < 0) {
                        s++;
                        continue;
                    }
                    currentPostion = tmpSnpInfoRow.getDecodeGeneticMap();
                    if (lastPosotion < currentPostion) {
                        currentPostion = lastPosotion + (currentPostion - lastPosotion) / (s - k);
                        break;
                    } else {
                        s++;
                    }
                }
            }
            //make sure the genetic map of the markers are unique
            if (lastPosotion == currentPostion) {
                bw.write(df.format(lastPosotion + 0.00001 * incTime));
                //System.out.println(snpInfoRow[1]+" : "+incTime);
                incTime++;
            } else {
                bw.write(df.format(currentPostion));
            }
            bw.write("\n");
            lastPosotion = currentPostion;
        }
        return true;
    }

    /**This function can generate genetic map for merlin
     *
     *
     */
    public boolean writeGeneticMapsMerlin(BufferedWriter bw, List<AnnotSNP> snpInfo, String hapMapRecombFilePath) throws
            Exception {
        File mapFile = new File(hapMapRecombFilePath);
        List<String[]> postionMapList = new ArrayList<String[]>();
        IntArrayList posList = new IntArrayList();
        int snpMapInfoSize = 0;
        boolean hasMapFile = false;
        if (mapFile.exists()) {
            BufferedReader br = new BufferedReader(new FileReader(mapFile));
            String line = null;
            br.readLine();
            //read map      
            StringBuilder tmpBuffer = new StringBuilder();
            while ((line = br.readLine()) != null) {
                StringTokenizer tokenizer = new StringTokenizer(line);
                String[] cells = new String[3];

                for (int i = 0; i < 3; i++) {
                    tmpBuffer.delete(0, tmpBuffer.length());
                    tmpBuffer.append(tokenizer.nextToken().trim());
                    cells[i] = tmpBuffer.toString();
                }
                postionMapList.add(cells);
            }
            br.close();

            snpMapInfoSize = postionMapList.size();
            //assume the positions are in order
            for (int k = 0; k < snpMapInfoSize; k++) {
                posList.add(Integer.parseInt(postionMapList.get(k)[0]));
            }
            hasMapFile = true;
        }



        DecimalFormat df = new DecimalFormat();
        df.setMaximumFractionDigits(6);

        AnnotSNP snpInfoRow = null;
        int index = 0, lastIndex = 0;

        int snpSize = snpInfo.size();
        snpMapInfoSize--;
        for (int k = 0; k < snpSize; k++) {
            snpInfoRow = snpInfo.get(k);
            bw.write(String.valueOf(snpInfoRow.getChromosomeNum()));
            bw.write("   ");
            bw.write(snpInfoRow.getRSID());
            bw.write("  ");
            int pos = (int) snpInfoRow.getPhysicalPosition();
            if (hasMapFile) {
                index = posList.binarySearchFromTo(pos, lastIndex, snpMapInfoSize);
                if (index < 0) {
                    index = -index - 1;
                }
                if (index >= snpMapInfoSize) {
                    index = snpMapInfoSize;
                }

                //String strPost = interpolateLinkagePostion(postionMapList, pos);
                //as the hapmap file has very large density, we just use the nearest pos to approximate the position
                bw.write(postionMapList.get(index)[2]);
            } else {
                bw.write(String.valueOf(pos/1000000.0));
            }

            bw.write("\n");
            lastIndex = index;
        }
        return true;
    }

    public String interpolateLinkagePostion(List<String[]> postionMapList, int postion) {
        int size = postionMapList.size();
        for (int i = 1; i < size; i++) {
            int pos1 = Integer.parseInt(postionMapList.get(i - 1)[0]);
            int pos2 = Integer.parseInt(postionMapList.get(i)[0]);

            if (postion <= pos1) {
                return postionMapList.get(i - 1)[2];
            } else if (pos2 == postion) {
                return postionMapList.get(i)[2];
            } else if (postion > pos1 && postion < pos2) {
                if (Math.abs(postion - pos1) < Math.abs(postion - pos2)) {
                    return postionMapList.get(i - 1)[2];
                } else {
                    return postionMapList.get(i)[2];
                }
            }
        }
        return "-1";
    }

    /**This function can generate allele frequency files for  merlin
     *
     */
    public boolean writeAlleleFrequenciesMerlin(BufferedWriter bw, List<AnnotSNP> snpInfo) throws
            Exception {
        DecimalFormat df = new DecimalFormat();
        df.setMaximumFractionDigits(6);
        int snpInfoSize = 0;
        snpInfoSize = snpInfo.size();
        AnnotSNP snpInfoRow = null;
        for (int k = 0; k < snpInfoSize; k++) {
            snpInfoRow = snpInfo.get(k);
            bw.write("M  ");
            bw.write(snpInfoRow.getRSID());
            bw.write("\n");
            bw.write("F  ");

            double freq0 = snpInfoRow.getAAlleleFreq();
            double freq1 = 1 - freq0;
            /*
            if(Double.parseDouble(freqInfo[0])==1)
            bw.write("0.9999");
            else  if(Double.parseDouble(freqInfo[0])==0)
            bw.write("0.0001");
            else   bw.write(freqInfo[0]);
             */
            bw.write(df.format(freq0));
            bw.write(" ");
            /*
            if(Double.parseDouble(freqInfo[1])==1)
            bw.write("0.9999");
            else  if(Double.parseDouble(freqInfo[1])==0)
            bw.write("0.0001");
            else   bw.write(freqInfo[1]);
             */
            bw.write(df.format(freq1));
            bw.write("\n");
        }
        return true;
    }

    public boolean writePhaseFilesMACH(BufferedWriter snpBw, BufferedWriter hapBw,
            List<AnnotSNP> entries, Map<String, StatusGtySet> indivGtyMap,
            Integration integration) throws
            Exception {
        int snpSize = entries.size();
        int[] orders = new int[snpSize];
        String[] alleleAs = new String[snpSize];
        String[] alleleBs = new String[snpSize];
        String missingGty = "0";
        for (int i = 0; i < snpSize; i++) {
            AnnotSNP snp = entries.get(i);
            orders[i] = snp.order;
            alleleAs[i] = String.valueOf(snp.getAAllele());
            alleleBs[i] = String.valueOf(snp.getBAllele());
            /*
            //defined by Mach Alleles can be coded as 1, 2, 3, 4 or A, C, G, T.
            switch (snp.getAAllele()) {
            case 'A':
            alleleAs[i] = "1";
            break;
            case 'C':
            alleleAs[i] = "2";
            break;
            case 'G':
            alleleAs[i] = "3";
            break;
            case 'T':
            alleleAs[i] = "4";
            break;
            }
            switch (snp.getBAllele()) {
            case 'A':
            alleleBs[i] = "1";
            break;
            case 'C':
            alleleBs[i] = "2";
            break;
            case 'G':
            alleleBs[i] = "3";
            break;
            case 'T':
            alleleBs[i] = "4";
            break;
            }
             */
            snpBw.write(snp.getRSID());
            snpBw.write("\n");
        }

        Iterator hapIndiLabelIter = integration.getHapIndivLabels().iterator();
        String chipLabel;
        while (hapIndiLabelIter.hasNext()) {
            chipLabel = hapIndiLabelIter.next().toString();
            StatusGtySet indivGty = indivGtyMap.get(chipLabel);
            if (indivGty != null) {
                hapBw.write(chipLabel);
                hapBw.write("->");
                hapBw.write(chipLabel);
                hapBw.write(" HAPLO1 ");
                for (int j = 0; j < snpSize; j++) {
                    if (indivGty.existence.getQuick(orders[j])) {
                        if (indivGty.paternalChrom.getQuick(orders[j])) {
                            hapBw.write(alleleBs[j]);
                        } else {
                            hapBw.write(alleleAs[j]);
                        }
                    } else {
                        hapBw.write(missingGty);
                    }
                }
                hapBw.write("\n");
                hapBw.write(chipLabel);
                hapBw.write("->");
                hapBw.write(chipLabel);
                hapBw.write(" HAPLO2 ");
                for (int j = 0; j < snpSize; j++) {
                    if (indivGty.existence.getQuick(orders[j])) {
                        if (indivGty.maternalChrom.getQuick(orders[j])) {
                            hapBw.write(alleleBs[j]);
                        } else {
                            hapBw.write(alleleAs[j]);
                        }
                    } else {
                        hapBw.write(missingGty);
                    }
                }
                hapBw.write("\n");
            } else {
                hapBw.write(chipLabel);
                hapBw.write("->");
                hapBw.write(chipLabel);
                hapBw.write(" HAPLO1 ");
                for (int j = 0; j < snpSize; j++) {
                    hapBw.write(missingGty);
                }
                hapBw.write("\n");
                hapBw.write(chipLabel);
                hapBw.write("->");
                hapBw.write(chipLabel);
                hapBw.write(" HAPLO2 ");
                for (int j = 0; j < snpSize; j++) {
                    hapBw.write(missingGty);
                }
                hapBw.write("\n");
            }

        }

        return true;
    }
}
