// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.

// Permission is granted for you to use this file to compile IGG.

// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.genetformator;

import java.io.BufferedWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.cobi.igg3.entity.AnnotSNP;
import org.cobi.igg3.entity.Individual;
import org.cobi.igg3.entity.Integration;
import org.cobi.igg3.entity.PedFile;
import org.cobi.igg3.entity.StatusGtySet;
import org.cobi.igg3.GlobalVariables;

/**
 *
 * @author mxli
 */
public class PhaseFiles {

    public void writeInputFilePhase(BufferedWriter prjBw, BufferedWriter hapBw, List<AnnotSNP> entries,
            Map<String, StatusGtySet> indivGtyMap, PedFile hapPed, List<String[]> outTraits,
            Integration integration) throws
            Exception {
        int snpSize = entries.size();
        String missingGty = "?";

        Map<String, Individual> prjIndivMap = GlobalVariables.currentPedFile.getAllIndividuals();
        List<String> prjIndiLabels = GlobalVariables.currentPedFile.getIndividualLabel();
        List<String> avaiTrais = GlobalVariables.currentPedFile.getTraitNames();
        int traitSize = outTraits.size();
        int[] traitIndex = new int[traitSize];
        int allTraitSize = avaiTrais.size();
        String[] cell;
        traitSize--;

        prjBw.write(String.valueOf(prjIndiLabels.size()));  //NumberOfIndividuals
        prjBw.write("\n");
        prjBw.write(String.valueOf(snpSize));  //numberOfLoci

        prjBw.write("\n");
        prjBw.write("P");
        int[] orders = new int[snpSize];

        for (int i = 0; i < snpSize; i++) {
            AnnotSNP snp = entries.get(i);
            prjBw.write(" ");
            prjBw.write(String.valueOf(snp.getPhysicalPosition()));
            orders[i] = snp.order;
        }

        prjBw.write("\n");
        for (int i = 0; i < snpSize; i++) {
            prjBw.write("S");
        }
        prjBw.write("\n");


        //determine the output trait
        for (int i = 0; i < traitSize; i++) {
            cell = outTraits.get(i);
            for (int j = 0; j < allTraitSize; j++) {
                if (cell[0].equals(avaiTrais.get(j))) {
                    traitIndex[i] = j;
                    break;
                }
            }
        }

        //write genotype data
        int indiviSize = prjIndiLabels.size();
        for (int k = 0; k < indiviSize; k++) {
            StatusGtySet indivGty = indivGtyMap.get(prjIndiLabels.get(k));
            if (indivGty != null) {
                Individual indiv = prjIndivMap.get(prjIndiLabels.get(k));

                if (traitSize >= 0) {
                    //write trait value
                    for (int j = 0; j < traitSize; j++) {
                        prjBw.write(indiv.getTraits().get(traitIndex[j]));
                        prjBw.write(" ");
                    }
                    prjBw.write(indiv.getTraits().get(traitIndex[traitSize]));
                }

                prjBw.write("# ");
                prjBw.write(indiv.getFamilyID() + "_" + indiv.getIndividualID());
                prjBw.write("\n");
                for (int j = 0; j < snpSize; j++) {
                    if (indivGty.existence.size()>orders[j]&& 
                            indivGty.existence.getQuick(orders[j])) {
                        if (indivGty.paternalChrom.getQuick(orders[j])) {
                            prjBw.write("1");
                        } else {
                            prjBw.write("0");
                        }
                    } else {
                        prjBw.write(missingGty);
                    }
                }
                prjBw.write("\n");
                for (int j = 0; j < snpSize; j++) {
                    if (indivGty.existence.size()>orders[j]&& 
                            indivGty.existence.getQuick(orders[j])) {

                        if (indivGty.maternalChrom.getQuick(orders[j])) {
                            prjBw.write("1");
                        } else {
                            prjBw.write("0");
                        }
                    } else {
                        prjBw.write(missingGty);
                    }
                }
                prjBw.write("\n");
            } else {
                hapBw.write("# ");
                hapBw.write(prjIndiLabels.get(k));
                hapBw.write("\n");
                for (int j = 0; j < snpSize; j++) {
                    prjBw.write(missingGty);
                }
                prjBw.write("\n");
                for (int j = 0; j < snpSize; j++) {
                    prjBw.write(missingGty);
                }
                prjBw.write("\n");
            }
        }

        String indivLabel = null;
        if (hapPed != null) {
            Iterator hapIndiLabelIter = integration.getHapIndivLabels().iterator();
            Map<String, Individual> hapIndivMap = hapPed.getAllIndividuals();

            hapBw.write(String.valueOf(integration.getHapIndivLabels().size() * 2));  //NumberOfIndividuals
            hapBw.write("\n");

            while (hapIndiLabelIter.hasNext()) {
                indivLabel = hapIndiLabelIter.next().toString();
                StatusGtySet indivGty = indivGtyMap.get(indivLabel);
                if (indivGty != null) {
                    Individual indiv = hapIndivMap.get(indivLabel);
                    if (traitSize >= 0) {
                        //write trait value                        
                        for (int j = 0; j < traitSize; j++) {
                            cell = outTraits.get(j);
                            hapBw.write(cell[2]);
                            hapBw.write(" ");
                        }
                        hapBw.write(outTraits.get(traitSize)[2]);
                    }
                    hapBw.write("# ");
                    hapBw.write(indiv.getFamilyID() + "_" + indiv.getIndividualID() + "_1");
                    hapBw.write("\n");
                    for (int j = 0; j < snpSize; j++) {
                        if (indivGty.existence.size()>orders[j]&& 
                                indivGty.existence.getQuick(orders[j])) {
                            if (indivGty.paternalChrom.getQuick(orders[j])) {
                                hapBw.write("1");
                            } else {
                                hapBw.write("0");
                            }
                        } else {
                            hapBw.write(missingGty);
                        }
                    }
                    hapBw.write("\n");
                    if (traitSize >= 0) {
                        //write trait value                        
                        for (int j = 0; j < traitSize; j++) {
                            cell = outTraits.get(j);
                            hapBw.write(cell[2]);
                            hapBw.write(" ");
                        }
                        hapBw.write(outTraits.get(traitSize)[2]);
                    }
                    hapBw.write("# ");
                    hapBw.write(indiv.getFamilyID() + "_" + indiv.getIndividualID() + "_2");

                    hapBw.write("\n");
                    for (int j = 0; j < snpSize; j++) {
                        if (indivGty.existence.size()>orders[j]&&
                                indivGty.existence.getQuick(orders[j])) {
                            if (indivGty.maternalChrom.getQuick(orders[j])) {
                                hapBw.write("1");
                            } else {
                                hapBw.write("0");
                            }
                        } else {
                            hapBw.write(missingGty);
                        }
                    }
                } else {
                    hapBw.write("# ");
                    hapBw.write(indivLabel + "_1");
                    hapBw.write("\n");
                    for (int j = 0; j < snpSize; j++) {
                        hapBw.write(missingGty);
                    }
                    hapBw.write("\n");
                    hapBw.write("# ");
                    hapBw.write(indivLabel + "_2");
                    hapBw.write("\n");
                    for (int j = 0; j < snpSize; j++) {
                        hapBw.write(missingGty);
                    }
                }
                hapBw.write("\n");
            }
        }
    }

    public void writeInputFileUnPhase(BufferedWriter prjBw, List<AnnotSNP> entries,
            Map<String, StatusGtySet> indivGtyMap, PedFile hapPed, List<String[]> outTraits,
            Integration integration) throws
            Exception {
        int snpSize = entries.size();
        String missingGty = "?";

        Map<String, Individual> prjIndivMap = GlobalVariables.currentPedFile.getAllIndividuals();
        List<String> prjIndiLabels = GlobalVariables.currentPedFile.getIndividualLabel();
        List<String> avaiTrais = GlobalVariables.currentPedFile.getTraitNames();
        int traitSize = outTraits.size();
        int[] traitIndex = new int[traitSize];
        int allTraitSize = avaiTrais.size();
        String[] cell;
        traitSize--;

        prjBw.write(String.valueOf(prjIndiLabels.size()));  //NumberOfIndividuals
        prjBw.write("\n");
        prjBw.write(String.valueOf(snpSize));  //numberOfLoci

        prjBw.write("\n");
        prjBw.write("P");
        int[] orders = new int[snpSize];

        for (int i = 0; i < snpSize; i++) {
            AnnotSNP snp = entries.get(i);
            prjBw.write(" ");
            prjBw.write(String.valueOf(snp.getPhysicalPosition()));
            orders[i] = snp.order ;
        }

        prjBw.write("\n");
        for (int i = 0; i < snpSize; i++) {
            prjBw.write("S");
        }
        prjBw.write("\n");


        //determine the output trait
        for (int i = 0; i < traitSize; i++) {
            cell = outTraits.get(i);
            for (int j = 0; j < allTraitSize; j++) {
                if (cell[0].equals(avaiTrais.get(j))) {
                    traitIndex[i] = j;
                    break;
                }
            }
        }

        //write genotype data
        int indiviSize = prjIndiLabels.size();
        for (int k = 0; k < indiviSize; k++) {
            StatusGtySet indivGty = indivGtyMap.get(prjIndiLabels.get(k));
            if (indivGty != null) {
                Individual indiv = prjIndivMap.get(prjIndiLabels.get(k));
                if (traitSize >= 0) {
                    //write trait value
                    for (int j = 0; j < traitSize; j++) {
                        prjBw.write(indiv.getTraits().get(traitIndex[j]));
                        prjBw.write(" ");
                    }
                    prjBw.write(indiv.getTraits().get(traitIndex[traitSize]));
                }

                prjBw.write("#");
                prjBw.write(indiv.getFamilyID() + "_" + indiv.getIndividualID() + " ");
                prjBw.write("\n");
                for (int j = 0; j < snpSize; j++) {
                    if (indivGty.existence.getQuick(orders[j])) {
                        if (indivGty.paternalChrom.getQuick(orders[j])) {
                            prjBw.write("1");
                        } else {
                            prjBw.write("0");
                        }
                    } else {
                        prjBw.write(missingGty);
                    }
                }
                prjBw.write("\n");
                for (int j = 0; j < snpSize; j++) {
                    if (indivGty.existence.getQuick(orders[j])) {
                        if (indivGty.maternalChrom.getQuick(orders[j])) {
                            prjBw.write("1");
                        } else {
                            prjBw.write("0");
                        }
                    } else {
                        prjBw.write(missingGty);
                    }
                }
                prjBw.write("\n");
            } else {
                for (int j = 0; j < snpSize; j++) {
                    prjBw.write(missingGty);
                }
                prjBw.write("\n");
                for (int j = 0; j < snpSize; j++) {
                    prjBw.write(missingGty);
                }
                prjBw.write("\n");
            }
        }

        String indivLabel = null;
        if (hapPed != null) {

            Iterator hapIndiLabelIter = integration.getHapIndivLabels().iterator();
            Map<String, Individual> hapIndivMap = hapPed.getAllIndividuals();
            while (hapIndiLabelIter.hasNext()) {
                indivLabel = hapIndiLabelIter.next().toString();
                StatusGtySet indivGty = indivGtyMap.get(indivLabel);
                if (indivGty != null) {
                    Individual indiv = hapIndivMap.get(indivLabel);
                    if (traitSize >= 0) {
                        //write trait value                        
                        for (int j = 0; j < traitSize; j++) {
                            cell = outTraits.get(j);
                            prjBw.write(cell[2]);
                            prjBw.write(" ");
                        }
                        prjBw.write(outTraits.get(traitSize)[2]);
                    }

                    prjBw.write("#");
                    prjBw.write(indiv.getFamilyID() + "_" + indiv.getIndividualID() + " ");
                    prjBw.write("\n");
                    for (int j = 0; j < snpSize; j++) {
                        if (indivGty.existence.getQuick(orders[j])) {
                            if (indivGty.paternalChrom.getQuick(orders[j])) {
                                prjBw.write("1");
                            } else {
                                prjBw.write("0");
                            }
                        } else {
                            prjBw.write(missingGty);
                        }
                    }
                    prjBw.write("\n");

                    for (int j = 0; j < snpSize; j++) {
                        if (indivGty.existence.getQuick(orders[j])) {
                            if (indivGty.maternalChrom.getQuick(orders[j])) {
                                prjBw.write("1");
                            } else {
                                prjBw.write("0");
                            }
                        } else {
                            prjBw.write(missingGty);
                        }
                    }
                } else {
                    for (int j = 0; j < snpSize; j++) {
                        prjBw.write(missingGty);
                    }
                    prjBw.write("\n");
                    for (int j = 0; j < snpSize; j++) {
                        prjBw.write(missingGty);
                    }
                    prjBw.write("\n");
                }
                prjBw.write("\n");
            }
        }
    }
}
