// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.
// Permission is granted for you to use this file to compile IGG.
// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.genetformator;

import java.io.BufferedWriter;
import java.io.DataOutputStream;
import java.text.DecimalFormat;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.cobi.igg3.GlobalVariables;
import org.cobi.igg3.entity.AnnotSNP;
import org.cobi.igg3.entity.ExportSetting;
import org.cobi.igg3.entity.Individual;
import org.cobi.igg3.entity.Integration;
import org.cobi.igg3.entity.PedFile;
import org.cobi.igg3.entity.StatusGtySet;

/**
 *
 * @author mxli
 */
public class PlinkFiles {

    /**
     *This function can generate genetic map for merlin
     *
     */
    public boolean writeMapFile(BufferedWriter bw, List<AnnotSNP> snpInfo) throws
            Exception {
        DecimalFormat df = new DecimalFormat();
        df.setMaximumFractionDigits(6);
        int snpInfoSize = 0;
        snpInfoSize = snpInfo.size();
        AnnotSNP snp = null;
        AnnotSNP tmpSnpInfoRow = null;
        double lastPosotion = -1.0;
        double currentPostion = -1.0;
        double incTime = 1.0;
        for (int k = 0; k < snpInfoSize; k++) {
            snp = snpInfo.get(k);
            bw.write(String.valueOf(snp.getChromosomeNum()));
            bw.write(" ");
            bw.write(snp.getRSID());
            bw.write(" ");
            if (snp.getDecodeGeneticMap() < 0) {
                bw.write("0.0");
            } else {
                currentPostion = snp.getDecodeGeneticMap();
                //make sure the genetic map of the markers in ascending order
                if (lastPosotion > currentPostion) {
                    int s = k + 2;
                    while (true) {
                        if (s >= snpInfoSize) {
                            currentPostion = lastPosotion + 0.00001 * incTime;
                            break;
                        }

                        tmpSnpInfoRow = snpInfo.get(s);
                        if (tmpSnpInfoRow.getDecodeGeneticMap() < 0) {
                            s++;
                            continue;
                        }
                        currentPostion = tmpSnpInfoRow.getDecodeGeneticMap();
                        if (lastPosotion < currentPostion) {
                            currentPostion = lastPosotion + (currentPostion - lastPosotion) / (s - k);
                            break;
                        } else {
                            s++;
                        }
                    }
                }
                //make sure the genetic map of the markers are unique
                if (lastPosotion == currentPostion) {
                    bw.write(df.format(lastPosotion + 0.00001 * incTime));
                    //System.bedOut.println(snp[1]+" : "+incTime);
                    incTime++;
                } else {
                    bw.write(df.format(currentPostion));
                }
            }
            bw.write(" ");
            bw.write(String.valueOf(snp.getPhysicalPosition()));
            bw.write("\n");
            lastPosotion = currentPostion;
        }
        return true;
    }

    public boolean writeMapFileBinaryFormat(BufferedWriter bw, List<AnnotSNP> snpInfo) throws
            Exception {
        DecimalFormat df = new DecimalFormat();
        df.setMaximumFractionDigits(6);
        int snpInfoSize = 0;
        snpInfoSize = snpInfo.size();
        AnnotSNP snp = null;
        AnnotSNP tmpSnpInfoRow = null;
        double lastPosotion = -1.0;
        double currentPostion = -1.0;
        double incTime = 1.0;
        for (int k = 0; k < snpInfoSize; k++) {
            snp = snpInfo.get(k);
            bw.write(String.valueOf(snp.getChromosomeNum()));
            bw.write(" ");
            bw.write(snp.getRSID());
            bw.write(" ");
            if (snp.getDecodeGeneticMap() < 0) {
                bw.write("0.0");
            } else {
                currentPostion = snp.getDecodeGeneticMap();
                //make sure the genetic map of the markers in ascending order
                if (lastPosotion > currentPostion) {
                    int s = k + 2;
                    while (true) {
                        if (s >= snpInfoSize) {
                            currentPostion = lastPosotion + 0.00001 * incTime;
                            break;
                        }

                        tmpSnpInfoRow = snpInfo.get(s);
                        if (tmpSnpInfoRow.getDecodeGeneticMap() < 0) {
                            s++;
                            continue;
                        }
                        currentPostion = tmpSnpInfoRow.getDecodeGeneticMap();
                        if (lastPosotion < currentPostion) {
                            currentPostion = lastPosotion + (currentPostion - lastPosotion) / (s - k);
                            break;
                        } else {
                            s++;
                        }
                    }
                }
                //make sure the genetic map of the markers are unique
                if (lastPosotion == currentPostion) {
                    bw.write(df.format(lastPosotion + 0.00001 * incTime));
                    //System.bedOut.println(snp[1]+" : "+incTime);
                    incTime++;
                } else {
                    bw.write(df.format(currentPostion));
                }
            }
            bw.write(" ");
            bw.write(String.valueOf(snp.getPhysicalPosition()));
            bw.write(" ");
            bw.write(String.valueOf(snp.getAAllele()));
            bw.write(" ");
            bw.write(String.valueOf(snp.getBAllele()));
            bw.write("\n");
            lastPosotion = currentPostion;
        }
        return true;
    }


    public void writeBinaryGenotypesHead(DataOutputStream bedOut) throws Exception {
        boolean[] bits = new boolean[8];

        byte b = 0;
        int[] bases = new int[8];
        int base0 = 1;
        for (int i = 0; i < 8; i++) {
            bases[i] = base0;
            base0 *= 2;

        }

        //"BED files should always start 01101100 00011011" +00000001 (model)
        bits[2] = true;
        bits[3] = true;
        bits[5] = true;
        bits[6] = true;


        for (int i = 7; i >= 0; i--) {
            if (bits[i]) {
                b += bases[i];
            }
        }
        bedOut.write(b);

        Arrays.fill(bits, false);
        bits[0] = true;
        bits[1] = true;
        bits[3] = true;
        bits[4] = true;
        b = 0;
        for (int i = 7; i >= 0; i--) {
            if (bits[i]) {
                b += bases[i];
            }
        }
        bedOut.write(b);

        Arrays.fill(bits, false);
        bits[0] = true;
        b = 0;
        for (int i = 7; i >= 0; i--) {
            if (bits[i]) {
                b += bases[i];
            }
        }
        bedOut.write(b);

    }

    public void writeBinaryGenotypesBody(DataOutputStream bedOut, List<AnnotSNP> entries, Map<String, StatusGtySet> indivGtyMap,
            ExportSetting exportSetting, Integration integration) throws Exception {
        int snpSize = entries.size();
        int[] orders = new int[snpSize];
        for (int i = 0; i < snpSize; i++) {
            orders[i] = entries.get(i).order;
        }

        //start to write the genotype data
        List<String> prjIndiLabels = GlobalVariables.currentPedFile.getIndividualLabel();
        int prjIndiviSize = prjIndiLabels.size();
        boolean[] bits = new boolean[8];


        byte b = 0;
        int[] bases = new int[8];
        int base0 = 1;
        for (int i = 0; i < 8; i++) {
            bases[i] = base0;
            base0 *= 2;

        }

        /*
         *  01101100
        HGFEDCBA

        AB   00  -- homozygote (first)
        CD     11  -- other homozygote (second)
        EF       01  -- heterozygote (third)
        GH         10  -- missing genotype (fourth)

         *
         */
        int indivCount = 0;
        int currentPoint = 0;
        for (int j = 0; j < snpSize; j++) {
            indivCount = 0;
            for (int i = 0; i < prjIndiviSize; i++) {
                StatusGtySet indivGty = indivGtyMap.get(prjIndiLabels.get(i));
                currentPoint = (indivCount % 4) * 2;
                if (indivGty != null) {
                    int gtySize = indivGty.existence.size();
                    if (gtySize > orders[j] && indivGty.existence.getQuick(orders[j])) {
                        if (indivGty.paternalChrom.getQuick(orders[j]) && indivGty.maternalChrom.getQuick(orders[j])) {
                            bits[currentPoint] = true;
                            bits[currentPoint + 1] = true;
                        } else if (!indivGty.paternalChrom.getQuick(orders[j]) && indivGty.maternalChrom.getQuick(orders[j])) {
                            // bits[currentPoint]=true;
                            bits[currentPoint + 1] = true;
                        } else if (indivGty.paternalChrom.getQuick(orders[j]) && !indivGty.maternalChrom.getQuick(orders[j])) {
                            // bits[currentPoint]=true;
                            bits[currentPoint + 1] = true;
                        }

                    } else {
                        bits[currentPoint] = true;
                        //    bits[currentPoint+1]=true;
                    }
                } else {
                    bits[currentPoint] = true;
                    //    bits[currentPoint+1]=true;
                }
                indivCount++;
                if (indivCount % 4 == 0) {
                    b = 0;
                    for (int k = 7; k >= 0; k--) {
                        if (bits[k]) {
                            b += bases[k];
                        }
                    }
                    bedOut.write(b);
                    //refresh
                    Arrays.fill(bits, false);
                }
            }

            Iterator hapIndiLabelIter = integration.getHapIndivLabels().iterator();
            String hapmapLabel = null;
            while (hapIndiLabelIter.hasNext()) {
                hapmapLabel = hapIndiLabelIter.next().toString();
                StatusGtySet indivGty = indivGtyMap.get(hapmapLabel);
                currentPoint = (indivCount % 4) * 2;
                if (indivGty != null) {
                    int gtySize = indivGty.existence.size();
                    if (gtySize > orders[j] && indivGty.existence.getQuick(orders[j])) {
                        if (indivGty.paternalChrom.getQuick(orders[j]) && indivGty.maternalChrom.getQuick(orders[j])) {
                            bits[currentPoint] = true;
                            bits[currentPoint + 1] = true;
                        } else if (!indivGty.paternalChrom.getQuick(orders[j]) && indivGty.maternalChrom.getQuick(orders[j])) {
                            // bits[currentPoint]=true;
                            bits[currentPoint + 1] = true;
                        } else if (indivGty.paternalChrom.getQuick(orders[j]) && !indivGty.maternalChrom.getQuick(orders[j])) {
                            // bits[currentPoint]=true;
                            bits[currentPoint + 1] = true;
                        }
                    } else {
                        bits[currentPoint] = true;
                        //    bits[currentPoint+1]=true;
                    }
                } else {
                    bits[currentPoint] = true;
                    //    bits[currentPoint+1]=true;
                }
                indivCount++;
                if (indivCount % 4 == 0) {
                    b = 0;
                    for (int k = 7; k >= 0; k--) {
                        if (bits[k]) {
                            b += bases[k];
                        }
                    }
                    bedOut.write(b);
                    //refresh
                    Arrays.fill(bits, false);
                }
            }

            if (indivCount % 4 != 0) {
                b = 0;
                for (int k = 7; k >= 0; k--) {
                    if (bits[k]) {
                        b += bases[k];
                    }
                }
                bedOut.write(b);
                //refresh
                Arrays.fill(bits, false);
            }
        }
    }

    public void writePedigreeFileBinaryFormat(BufferedWriter bw, Integration integration, PedFile hapPed, List<String[]> outTraits) throws
            Exception {
        Map<String, Individual> prjIndivMap = GlobalVariables.currentPedFile.getAllIndividuals();
        List<String> prjIndiLabels = GlobalVariables.currentPedFile.getIndividualLabel();

        List<String> avaiTrais = GlobalVariables.currentPedFile.getTraitNames();
        int traitSize = outTraits.size();
        int[] traitIndex = new int[traitSize];
        int allTraitSize = avaiTrais.size();
        String[] cell;
        traitSize--;
        //determine the output trait
        for (int i = 0; i <= traitSize; i++) {
            cell = outTraits.get(i);
            for (int j = 0; j < allTraitSize; j++) {
                if (cell[0].equals(avaiTrais.get(j))) {
                    traitIndex[i] = j;
                    break;
                }
            }
        }

        int indiviSize = prjIndiLabels.size();
        for (int k = 0; k < indiviSize; k++) {
            Individual indiv = prjIndivMap.get(prjIndiLabels.get(k));
            bw.write(indiv.getFamilyID());
            bw.write(" ");
            bw.write(indiv.getIndividualID());
            bw.write(" ");
            bw.write(indiv.getDadID());
            bw.write(" ");
            bw.write(indiv.getMomID());
            bw.write(" ");
            bw.write(String.valueOf(indiv.getGender()));
            if (traitSize >= 0) {
                bw.write(" ");
                //write trait value
                for (int j = 0; j < traitSize; j++) {
                    bw.write(indiv.getTraits().get(traitIndex[j]));
                    bw.write(" ");
                }
                bw.write(indiv.getTraits().get(traitIndex[traitSize]));
            }
            bw.write("\n");
        }




        if (hapPed != null) {
            Iterator<String> hapIndiLabelIter = integration.getHapIndivLabels().iterator();
            Map<String, Individual> hapIndivMap = hapPed.getAllIndividuals();
            while (hapIndiLabelIter.hasNext()) {
                Individual indiv = hapIndivMap.get(hapIndiLabelIter.next());
                bw.write(indiv.getFamilyID());
                bw.write(" ");
                bw.write(indiv.getIndividualID());
                bw.write(" ");
                bw.write(indiv.getDadID());
                bw.write(" ");
                bw.write(indiv.getMomID());
                bw.write(" ");
                bw.write(String.valueOf(indiv.getGender()));
                //write trait value
                if (traitSize >= 0) {
                    bw.write(" ");
                    for (int j = 0; j < traitSize; j++) {
                        bw.write(outTraits.get(j)[2]);
                        bw.write(" ");
                    }
                    bw.write(outTraits.get(traitSize)[2]);
                }
                bw.write("\n");
            }
        }
    }
}
