// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.

// Permission is granted for you to use this file to compile IGG.

// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.genetformator;

import java.io.BufferedWriter;
import java.text.DecimalFormat;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.cobi.igg3.entity.AnnotSNP;
import org.cobi.igg3.entity.ExportSetting;
import org.cobi.igg3.entity.Individual;
import org.cobi.igg3.entity.Integration;
import org.cobi.igg3.entity.PedFile;
import org.cobi.igg3.entity.StatusGtySet;
import org.cobi.igg3.GlobalVariables;

/**
 *
 * @author mxli
 */
public class SolarFiles {

    public void writePedigreeFileSolar(BufferedWriter bw, Integration integration, PedFile hapPed) throws
            Exception {
        bw.write("FAMID,ID,FA,MO,SEX\n");
        Map<String, Individual> prjIndivMap = GlobalVariables.currentPedFile.getAllIndividuals();
        List<String> prjIndiLabels = GlobalVariables.currentPedFile.getIndividualLabel();
        int indiviSize = prjIndiLabels.size();
        for (int k = 0; k < indiviSize; k++) {
            Individual indiv = prjIndivMap.get(prjIndiLabels.get(k));
            bw.write(indiv.getFamilyID());
            bw.write(",");
            bw.write(indiv.getIndividualID());
            bw.write(",");
            bw.write(indiv.getDadID());
            bw.write(",");
            bw.write(indiv.getMomID());
            bw.write(",");
            bw.write(String.valueOf(indiv.getGender()));
            bw.write("\n");
        }

        if (hapPed != null) {
            Iterator hapIndiLabelIter = integration.getHapIndivLabels().iterator();
            Map<String, Individual> hapIndivMap = hapPed.getAllIndividuals();
            while (hapIndiLabelIter.hasNext()) {
                Individual indiv = hapIndivMap.get(hapIndiLabelIter.next());
                bw.write(indiv.getFamilyID());
                bw.write(",");
                bw.write(indiv.getIndividualID());
                bw.write(",");
                bw.write(indiv.getDadID());
                bw.write(",");
                bw.write(indiv.getMomID());
                bw.write(",");
                bw.write(String.valueOf(indiv.getGender()));
                bw.write("\n");
            }
        }
    }

    public void writePhenotypeFileSolar(BufferedWriter bw, Integration integration, PedFile hapPed, List<String[]> outTraits) throws
            Exception {
        // write column names
        bw.write("FAMID,ID");
        List<String> avaiTrais = GlobalVariables.currentPedFile.getTraitNames();
        int traitSize = outTraits.size();
        int[] traitIndex = new int[traitSize];
        int allTraitSize = avaiTrais.size();
        String[] cell;
        traitSize--;
        //determine the output trait
        for (int i = 0; i <= traitSize; i++) {
            cell = outTraits.get(i);
            for (int j = 0; j < allTraitSize; j++) {
                if (cell[0].equals(avaiTrais.get(j))) {
                    traitIndex[i] = j;
                    break;
                }
            }
            bw.write(",");
            bw.write(cell[0]);
        }
        bw.write("/n");

        Map<String, Individual> prjIndivMap = GlobalVariables.currentPedFile.getAllIndividuals();
        List<String> prjIndiLabels = GlobalVariables.currentPedFile.getIndividualLabel();
        int indiviSize = prjIndiLabels.size();
        for (int k = 0; k < indiviSize; k++) {
            Individual indiv = prjIndivMap.get(prjIndiLabels.get(k));
            bw.write(indiv.getFamilyID());
            bw.write(",");
            bw.write(indiv.getIndividualID());

            if (traitSize >= 0) {
                bw.write(",");
                //write trait value
                for (int j = 0; j < traitSize; j++) {
                    bw.write(indiv.getTraits().get(traitIndex[j]));
                    bw.write(",");
                }
                bw.write(indiv.getTraits().get(traitIndex[traitSize]));
            }
            bw.write("\n");
        }

        if (hapPed != null) {
            Iterator<String> hapIndiLabelIter = integration.getHapIndivLabels().iterator();
            Map<String, Individual> hapIndivMap = hapPed.getAllIndividuals();
            while (hapIndiLabelIter.hasNext()) {
                Individual indiv = hapIndivMap.get(hapIndiLabelIter.next());
                bw.write(indiv.getFamilyID());
                bw.write(",");
                bw.write(indiv.getIndividualID());
                //write trait value
                if (traitSize >= 0) {
                    bw.write(",");
                    for (int j = 0; j < traitSize; j++) {
                        bw.write(outTraits.get(j)[2]);
                        bw.write(",");
                    }
                    bw.write(outTraits.get(traitSize)[2]);
                }
                bw.write("\n");
            }
        }
    }

    /**This function can generate genetic map for solar
     *
     *
     */
    public void writeGeneticMapFileSolar(BufferedWriter bw, String chr, List<AnnotSNP> snpInfo) throws
            Exception {
        DecimalFormat df = new DecimalFormat();
        df.setMaximumFractionDigits(6);

        int snpInfoSize = 0;
        snpInfoSize = snpInfo.size();
        AnnotSNP snpInfoRow = null;
        AnnotSNP tmpSnpInfoRow = null;
        double lastPosotion = -1.0;
        double currentPostion = -1.0;
        double incTime = 1.0;
        bw.write(chr);
        bw.write("\n");
        for (int k = 0; k < snpInfoSize; k++) {
            snpInfoRow = snpInfo.get(k);
            bw.write(snpInfoRow.getRSID());
            bw.write("\t");
            if (snpInfoRow.getDecodeGeneticMap() < 0) {
                bw.write("0.0\n");
                continue;
            }
            currentPostion = snpInfoRow.getDecodeGeneticMap();
            //make sure the genetic map of the markers in ascending order
            if (lastPosotion > currentPostion) {
                int s = k + 2;
                while (true) {
                    if (s >= snpInfoSize) {
                        currentPostion = lastPosotion + 0.00001 * incTime;
                        break;
                    }

                    tmpSnpInfoRow = snpInfo.get(s);
                    if (tmpSnpInfoRow.getDecodeGeneticMap() < 0) {
                        s++;
                        tmpSnpInfoRow = snpInfo.get(s);
                        continue;
                    }
                    currentPostion = tmpSnpInfoRow.getDecodeGeneticMap();
                    if (lastPosotion < currentPostion) {
                        currentPostion = lastPosotion + (currentPostion - lastPosotion) / (s - k);
                        break;
                    } else {
                        s++;
                    }
                }
            }
            //make sure the genetic map of the markers are unique
            if (lastPosotion == currentPostion) {
                bw.write(df.format(lastPosotion + 0.00001 * incTime));
                //System.out.println(snpInfoRow[1]+" : "+incTime);
                incTime++;
            } else {
                bw.write(df.format(currentPostion));
            }
            bw.write("\n");
            lastPosotion = currentPostion;
        }
        bw.close();
    }

    /**This function can generate allele frequency files for  solar
     *
     */
    public void writeAlleleFrequencyFileSolar(BufferedWriter bw, List<AnnotSNP> snpInfo) throws
            Exception {
        int snpInfoSize = 0;
        snpInfoSize = snpInfo.size();
        AnnotSNP snpInfoRow = null;
        DecimalFormat df = new DecimalFormat();
        df.setMaximumFractionDigits(6);
        for (int k = 0; k < snpInfoSize; k++) {
            snpInfoRow = snpInfo.get(k);
            bw.write(snpInfoRow.getRSID());
            bw.write(", ");
            bw.write("1, ");

            /*
            if(Double.parseDouble(freqInfo[0])==1)
            bw.write("0.9999");
            else  if(Double.parseDouble(freqInfo[0])==0)
            bw.write("0.0001");
            else   bw.write(freqInfo[0]);
             */
            double freq0 = snpInfoRow.getAAlleleFreq();
            double freq1 = 1 - freq0;

            bw.write(df.format(freq0));
            bw.write(", 2, ");
            /*
            if(Double.parseDouble(freqInfo[1])==1)
            bw.write("0.9999");
            else  if(Double.parseDouble(freqInfo[1])==0)
            bw.write("0.0001");
            else   bw.write(freqInfo[1]);
             */
            bw.write(df.format(freq1));
            bw.write("\n");
        }
        bw.close();
    }

    public void writeMarkerFileSolar(BufferedWriter bw, List<AnnotSNP> entries, Map<String, StatusGtySet> indivGtyMap,
            Integration integration, String chrom, ExportSetting exportSetting, PedFile hapPed) throws Exception {
        int snpSize = entries.size();
        int[] orders = new int[snpSize];
        String missingGty = "0/0";
        for (int i = 0; i < snpSize; i++) {
            orders[i] = entries.get(i).order;
        }
        Map<String, Individual> prjIndivMap = GlobalVariables.currentPedFile.getAllIndividuals();
        List<String> prjIndiLabels = GlobalVariables.currentPedFile.getIndividualLabel();
        List<String[]> outTraits = exportSetting.getTraitProper();
        List<String> avaiTrais = GlobalVariables.currentPedFile.getTraitNames();
        int traitSize = outTraits.size();
        int[] traitIndex = new int[traitSize];
        int allTraitSize = avaiTrais.size();
        String[] cell;
        traitSize--;
        //determine the output trait
        for (int i = 0; i <= traitSize; i++) {
            cell = outTraits.get(i);
            for (int j = 0; j < allTraitSize; j++) {
                if (cell[0].equals(avaiTrais.get(j))) {
                    traitIndex[i] = j;
                    break;
                }
            }
        }

        int indiviSize = prjIndiLabels.size();
        for (int k = 0; k < indiviSize; k++) {
            Individual indiv = prjIndivMap.get(prjIndiLabels.get(k));
            bw.write(indiv.getFamilyID());
            bw.write(",");
            bw.write(indiv.getIndividualID());

            StatusGtySet indivGty = indivGtyMap.get(prjIndiLabels.get(k));
            if (indivGty != null) {
                int gtySize = indivGty.existence.size();
                for (int j = 0; j < snpSize; j++) {
                    bw.write(",");
                    if (gtySize > orders[j] && indivGty.existence.getQuick(orders[j])) {
                        if (indivGty.paternalChrom.getQuick(orders[j])) {
                            bw.write("2");
                        } else {
                            bw.write("1");
                        }
                        bw.write("/");
                        if (indivGty.maternalChrom.getQuick(orders[j])) {
                            bw.write("2");
                        } else {
                            bw.write("1");
                        }
                    } else {
                        bw.write(missingGty);
                    }
                }
            } else {
                for (int j = 0; j < snpSize; j++) {
                    bw.write(",");
                    bw.write(missingGty);
                }
            }
            bw.write("\n");
        }

        if (hapPed != null) {
            Iterator hapIndiLabelIter = integration.getHapIndivLabels().iterator();
            Map<String, Individual> hapIndivMap = hapPed.getAllIndividuals();
            while (hapIndiLabelIter.hasNext()) {
                Individual indiv = hapIndivMap.get(hapIndiLabelIter.next());
                bw.write(indiv.getFamilyID());
                bw.write(",");
                bw.write(indiv.getIndividualID());
                //write trait value
                StatusGtySet indivGty = indivGtyMap.get(indiv.getLabelInChip());
                if (indivGty != null) {
                    int gtySize = indivGty.existence.size();
                    for (int j = 0; j < snpSize; j++) {
                        bw.write(",");
                        if (gtySize > orders[j] && indivGty.existence.getQuick(orders[j])) {
                            if (indivGty.paternalChrom.getQuick(orders[j])) {
                                bw.write("2");
                            } else {
                                bw.write("1");
                            }
                            bw.write("/");
                            if (indivGty.maternalChrom.getQuick(orders[j])) {
                                bw.write("2");
                            } else {
                                bw.write("1");
                            }
                        } else {
                            bw.write(missingGty);
                        }
                    }
                } else {
                    for (int j = 0; j < snpSize; j++) {
                        bw.write(",");
                        bw.write(missingGty);
                    }
                }
                bw.write("\n");
            }
        }
    }
}
