// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.
// Permission is granted for you to use this file to compile IGG.
// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * IGG3App.java
 */
package org.cobi.igg3.gui;

import java.awt.Color;
import java.awt.Toolkit;
import java.util.Locale;
import java.util.ResourceBundle;
import javax.swing.JOptionPane;
import javax.swing.UIManager;
import javax.swing.plaf.ColorUIResource;
import org.cobi.igg3.Constants;
import org.cobi.util.download.Utils;
import org.jdesktop.application.Application;
import org.jdesktop.application.SingleFrameApplication;

/**
 * The main class of the application.
 */
public class IGG3App extends SingleFrameApplication implements Constants {

    /**
     * At startup create and show the main frame of the application.
     */
    @Override
    protected void startup() {
        show(new IGG3View(this));
    }

    /**
     * This method is to initialize the specified window by injecting resources.
     * Windows shown in our application come fully initialized from the GUI
     * builder, so this additional configuration is not needed.
     */
    @Override
    protected void configureWindow(java.awt.Window root) {
        ResourceBundle rb = ResourceBundle.getBundle("LocalStrings", Locale.US);
        UIManager.put("OptionPane.cancelButtonText", rb.getString("OptionPane.cancelButtonText"));
        UIManager.put("OptionPane.noButtonText", rb.getString("OptionPane.noButtonText"));
        UIManager.put("OptionPane.okButtonText", rb.getString("OptionPane.okButtonText"));
        UIManager.put("OptionPane.yesButtonText", rb.getString("OptionPane.yesButtonText"));

        UIManager.put("FileChooser.lookInLabelMnemonic", new Integer('d'));
        UIManager.put("FileChooser.lookInLabelText", rb.getString("FileChooser.lookInLabelText"));
        UIManager.put("FileChooser.filesOfTypeLabelText", rb.getString("FileChooser.filesOfTypeLabelText"));
        UIManager.put("FileChooser.upFolderToolTipText", rb.getString("FileChooser.upFolderToolTipText"));
        UIManager.put("FileChooser.fileNameLabelText", rb.getString("FileChooser.fileNameLabelText"));
        UIManager.put("FileChooser.homeFolderToolTipText", rb.getString("FileChooser.homeFolderToolTipText"));
        UIManager.put("FileChooser.newFolderToolTipText", rb.getString("FileChooser.newFolderToolTipText"));
        UIManager.put("FileChooser.listViewButtonToolTipText", rb.getString("FileChooser.listViewButtonToolTipText"));
        UIManager.put("FileChooser.detailsViewButtonToolTipText", rb.getString("FileChooser.detailsViewButtonToolTipText"));
        UIManager.put("FileChooser.saveButtonText", rb.getString("FileChooser.saveButtonText"));
        UIManager.put("FileChooser.openButtonText", rb.getString("FileChooser.openButtonText"));
        UIManager.put("FileChooser.cancelButtonText", rb.getString("FileChooser.cancelButtonText"));
        UIManager.put("FileChooser.updateButtonText", rb.getString("FileChooser.updateButtonText"));
        UIManager.put("FileChooser.helpButtonText", rb.getString("FileChooser.helpButtonText"));
        UIManager.put("FileChooser.saveButtonToolTipText", rb.getString("FileChooser.saveButtonToolTipText"));
        UIManager.put("FileChooser.openButtonToolTipText", rb.getString("FileChooser.openButtonToolTipText"));
        UIManager.put("FileChooser.cancelButtonToolTipText", rb.getString("FileChooser.cancelButtonToolTipText"));
        UIManager.put("FileChooser.updateButtonToolTipText", rb.getString("FileChooser.updateButtonToolTipText"));
        UIManager.put("FileChooser.helpButtonToolTipText", rb.getString("FileChooser.helpButtonToolTipText"));
        UIManager.put("FileChooser.saveInLabelText", rb.getString("FileChooser.saveInLabelText"));
        UIManager.put("FileChooser.acceptAllFileFilterText", rb.getString("FileChooser.acceptAllFileFilterText"));

        UIManager.put("Tree.hash", new ColorUIResource(Color.BLUE));

    }

    /**
     * A convenient static getter for the application instance.
     * @return the instance of IGG3App
     */
    public static IGG3App getApplication() {
        return Application.getInstance(IGG3App.class);
    }

    /**
     * Main method launching the application.
     */
    public static void main(String[] args) {

        try {
            if (!Utils.isConnected()) {
                String msg = "Sorry, I cannot connect to website to update its resources!\n Please check your local network configurations!";
                JOptionPane.showMessageDialog(null, msg, "Warning", JOptionPane.WARNING_MESSAGE);
                launch(IGG3App.class, args);
            } else {
                if (Utils.needUpdate()) 
                {
                    java.awt.EventQueue.invokeLater(new Runnable() {

                        public void run() {
                            UpdateItselfDialog dialog = new UpdateItselfDialog(new javax.swing.JFrame(), true);
                            dialog.addWindowListener(new java.awt.event.WindowAdapter() {

                                @Override
                                public void windowClosing(java.awt.event.WindowEvent e) {
                                    System.exit(0);
                                }
                            });

                            //Get x and y by geometry relationship
                            double x = 0.5 * dialog.getWidth();
                            double y = 0.5 * dialog.getHeight();


                            int screenWidth = (int) Toolkit.getDefaultToolkit().getScreenSize().width;
                            int screenHeight = (int) Toolkit.getDefaultToolkit().getScreenSize().height;
                            dialog.setLocation((int) (screenWidth / 2 - x), (int) (screenHeight / 2 - y));
                            dialog.setVisible(true);
                        }
                    });
                } else {
                    launch(IGG3App.class, args);
                }
            }

        } catch (NoClassDefFoundError ex) {
            ex.printStackTrace();
            // System.out.println("The library of KGG has been updated! Please re-initiate KGG!");
        } catch (Exception ex) {
            ex.printStackTrace();
        }


    }
}
