// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.
// Permission is granted for you to use this file to compile IGG.
// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * IGG3View.java
 */
package org.cobi.igg3.gui;

import java.awt.Color;
import java.awt.Font;
import javax.swing.JProgressBar;
import javax.swing.JTextPane;
import org.jdesktop.application.Action;
import org.jdesktop.application.ResourceMap;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;
import org.jdesktop.application.TaskMonitor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.EventObject;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import javax.swing.text.AttributeSet;
import javax.swing.Timer;
import javax.swing.Icon;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;
import javax.swing.table.TableColumn;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;

import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;
import org.apache.commons.collections.MultiMap;
import org.apache.http.client.HttpClient;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.tsccm.ThreadSafeClientConnManager;
import org.cobi.igg3.Constants;
import org.cobi.igg3.entity.CommonGenetFileSet;
import org.cobi.igg3.entity.Integration;
import org.cobi.igg3.entity.PedFile;
import org.cobi.igg3.entity.Project;
import org.cobi.igg3.GlobalVariables;
import org.cobi.util.download.HttpClient4API;
import org.cobi.util.download.HttpClient4DownloadTaskProgressBar;
import org.cobi.util.download.HttpClient4TaskBeanProgressBar;
import org.cobi.util.file.LocalFile;

import org.cobi.util.thread.TaskQueue;
import org.cobi.util.thread.ThreadPool;
import org.jdesktop.application.Application;

/**
 * The application's main frame.
 */
public class IGG3View extends FrameView implements Constants {

    private DefaultTreeModel projectsTreeModel = null;
    private DefaultMutableTreeNode resourceRoot = null;
    private DefaultMutableTreeNode projectsRoot = null;
    private DefaultMutableTreeNode annotation = null;
    private DefaultMutableTreeNode dbSNPAnno = null;
    private DefaultMutableTreeNode illumina = null;
    private DefaultMutableTreeNode affymetrix = null;
    private DefaultTreeModel resourceTreeModel = null;
    public DefaultMutableTreeNode illuminaFiles = null;
    public DefaultMutableTreeNode chipFiles = null;
    private ArrayList<String[]> listTableData = null;
    private ArrayListStringArrayTableModel listTableModel = null;

    public IGG3View(SingleFrameApplication app) {
        super(app);
        this.getFrame().setIconImage(GUIUtil.readImageIcon("igg.png").getImage());
        this.getFrame().setTitle("IGG - A tool to Integrate GeneChips for Genetic studies");
        listTableData = new ArrayList<String[]>();
        listTableModel = new ArrayListStringArrayTableModel();
        initComponents();
        initializeTrees();
        // status bar initialization - message timeout, idle icon and busy animation, etc
        ResourceMap resourceMap = getResourceMap();
        int messageTimeout = resourceMap.getInteger("StatusBar.messageTimeout");
        messageTimer = new Timer(messageTimeout, new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                statusMessageLabel.setText("");
            }
        });
        messageTimer.setRepeats(false);
        int busyAnimationRate = resourceMap.getInteger("StatusBar.busyAnimationRate");
        for (int i = 0; i < busyIcons.length; i++) {
            busyIcons[i] = resourceMap.getIcon("StatusBar.busyIcons[" + i + "]");
        }
        busyIconTimer = new Timer(busyAnimationRate, new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                busyIconIndex = (busyIconIndex + 1) % busyIcons.length;
                statusAnimationLabel.setIcon(busyIcons[busyIconIndex]);
            }
        });

        idleIcon = resourceMap.getIcon("StatusBar.idleIcon");
        statusAnimationLabel.setIcon(idleIcon);
        progressBar.setVisible(false);

        // connecting action tasks to status bar via TaskMonitor
        TaskMonitor taskMonitor = new TaskMonitor(getApplication().getContext());
        taskMonitor.addPropertyChangeListener(new java.beans.PropertyChangeListener() {

            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                String propertyName = evt.getPropertyName();
                if ("started".equals(propertyName)) {
                    if (!busyIconTimer.isRunning()) {
                        statusAnimationLabel.setIcon(busyIcons[0]);
                        busyIconIndex = 0;
                        busyIconTimer.start();
                    }
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(true);
                } else if ("done".equals(propertyName)) {
                    busyIconTimer.stop();
                    statusAnimationLabel.setIcon(idleIcon);
                    progressBar.setVisible(false);
                    progressBar.setValue(0);
                } else if ("message".equals(propertyName)) {
                    String text = (String) (evt.getNewValue());
                    statusMessageLabel.setText((text == null) ? "" : text);
                    messageTimer.restart();
                } else if ("progress".equals(propertyName)) {
                    int value = (Integer) (evt.getNewValue());
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(false);
                    progressBar.setValue(value);
                }
            }
        });
        setDownloadVisible(false);
        MemoryProgressThread pThread = new MemoryProgressThread(this.memoryProgressBar, this.memoryLabel);
        pThread.start();
    }

    public void setDownloadVisible(boolean show) {
        // downloadingLabel.setVisible(show);
        downloadProgressBar.setVisible(show);
    }

    public JTextPane getBreifTextPane() {
        return breifTextPane;
    }

    public void setBreifTextPane(JTextPane breifTextPane) {
        this.breifTextPane = breifTextPane;
    }

    public JProgressBar getDownloadProgressBar() {
        return downloadProgressBar;
    }

    public void setDownloadProgressBar(JProgressBar downloadProgressBar) {
        this.downloadProgressBar = downloadProgressBar;
    }

    private void initializeTrees() {
        projectsRoot = new DefaultMutableTreeNode("Projects");
        projectsTreeModel = new DefaultTreeModel(projectsRoot);
        projectsTree.setModel(projectsTreeModel);
        projectsTree.getSelectionModel().setSelectionMode(TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
        projectsTree.putClientProperty("JTree.lineStyle", "Angled");
        projectsTree.setRowHeight(20);

        TreeIconRender projectsTreeRender = new TreeIconRender("open.gif", "close.gif", "leaf.gif", "leaf.gif");
        projectsTreeRender.setFont(new Font("Courier New", Font.PLAIN, 12));//

        projectsTreeRender.setBackgroundNonSelectionColor(Color.white);
        projectsTreeRender.setBackgroundSelectionColor(Color.yellow);
        projectsTreeRender.setBorderSelectionColor(Color.red);
        projectsTreeRender.setTextNonSelectionColor(Color.black);
        projectsTreeRender.setTextSelectionColor(Color.blue);
        projectsTree.setCellRenderer(projectsTreeRender);
        projectsTree.setRootVisible(false);

        resourceRoot = new DefaultMutableTreeNode("Resources");
        annotation = new DefaultMutableTreeNode("Chip Annotation");
        dbSNPAnno = new DefaultMutableTreeNode("dbSNP Annotation");

        resourceTreeModel = new DefaultTreeModel(resourceRoot);
        resourceTreeModel.insertNodeInto(annotation, resourceRoot, resourceRoot.getChildCount());
        resourceTreeModel.insertNodeInto(dbSNPAnno, resourceRoot, resourceRoot.getChildCount());

        illumina = new DefaultMutableTreeNode("Illumina");
        affymetrix = new DefaultMutableTreeNode("Affymetrix");
        resourceTreeModel.insertNodeInto(illumina, annotation, annotation.getChildCount());
        resourceTreeModel.insertNodeInto(affymetrix, annotation, annotation.getChildCount());
        updateChipAnnos();
        updateHapMapAnnos();
        resourceTree.setModel(resourceTreeModel);
        resourceTree.getSelectionModel().setSelectionMode(TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
        resourceTree.putClientProperty("JTree.lineStyle", "Angled");
        resourceTree.setRowHeight(20);
        TreeIconRender resourceTreeRender = new TreeIconRender("expand.gif", "collapse.gif", "chips.jpg", "chips.jpg");
        resourceTree.setCellRenderer(resourceTreeRender);
        resourceTree.setRootVisible(false);
        resourceTree.expandPath(new TreePath(illumina.getPath()));
        resourceTree.expandPath(new TreePath(affymetrix.getPath()));
        resourceTree.expandPath(new TreePath(dbSNPAnno.getPath()));

    }

    public boolean addProject2Tree(Project prj) throws Exception {
        if (contain(prj.getName(), projectsRoot)) {
            return false;
        }
        DefaultMutableTreeNode prjNode = new DefaultMutableTreeNode(prj.getName());
        projectsTreeModel.insertNodeInto(prjNode, projectsRoot, projectsRoot.getChildCount());
        String pedFilePath = prj.getPedigreeFilePath();
        if (pedFilePath != null) {
            pedFilePath = "Chip Pedigree File: " + pedFilePath.substring(pedFilePath.lastIndexOf(File.separator) + 1);
            DefaultMutableTreeNode pedFilePathNode = new DefaultMutableTreeNode(new FileTreeNode(prj.getPedigreeFilePath(), pedFilePath));
            projectsTreeModel.insertNodeInto(pedFilePathNode, prjNode, prjNode.getChildCount());
        }
        projectsTree.expandPath(new TreePath(prjNode.getPath()));
        switchLoading(true);
        if (prj.getChipFilePathList().size() > 0 || prj.getCommonGenetFileSetList().size() > 0) {
            addChipFileNodes(prjNode, prj.getChipFilePathList());
            switchChipIntegration(true);
        }
        for (int i = 0; i < prj.getCommonGenetFileSetList().size(); i++) {
            addCommonDataSetNode(prj.getCommonGenetFileSetList().get(i));
        }

        for (int i = 0; i < prj.getIntegrationList().size(); i++) {
            addIntegrationNodes(prj.getIntegrationList().get(i));
        }
        projectsTree.expandPath(new TreePath(prjNode.getPath()[0]));
        projectsTree.setToolTipText(prj.getWorkingPath());
        return true;
    }

    private class ConfirmExit implements Application.ExitListener {

        @Override
        public boolean canExit(EventObject e) {
            if (GlobalVariables.currentProject == null) {
                return true;
            }
            int response = JOptionPane.showConfirmDialog(mainFrame,
                    "Going to exit?",
                    "Exit Confirmation", JOptionPane.YES_OPTION);
            if (response == 1) {
                return false;
            }
            return true;
        }

        @Override
        public void willExit(EventObject e) {
            try {
                GlobalVariables.writeEnvironmentalVariables();
                if (GlobalVariables.currentProject != null) {
                    GlobalVariables.currentProject.writeProjectVariables();
                }
                System.out.println("IGG Exit Successfully!");

            } catch (Exception ex) {
                System.err.println(ex.toString());
            }
        }
    }

    public void switchChipIntegration(boolean status) {
        jMenuItem1.setEnabled(status);
        jMenuItem2.setEnabled(status);
        jButton4.setEnabled(status);
        jMenuItem18.setEnabled(status);
        chooseChipFilesMenuItem.setEnabled(status);
    }

    public void switchHapIntegration(boolean status) throws Exception {
        jMenuItem6.setEnabled(status);
        jMenuItem9.setEnabled(status);
        jButton5.setEnabled(status);
        jButton6.setEnabled(status);
    }

    public void switchExort(boolean status) throws Exception {
        jMenuItem3.setEnabled(status);
        jMenuItem8.setEnabled(status);
        jButton7.setEnabled(status);
        jButton3.setEnabled(status);
    }

    public void switchLoading(boolean status) throws Exception {
        jMenuItem18.setEnabled(status);
        chooseChipFilesMenuItem.setEnabled(status);
        jMenuItem17.setEnabled(status);
        jButton2.setEnabled(status);
        jButton10.setEnabled(status);
        jMenuItem19.setEnabled(status);
    }

    private class MyActionListener implements ActionListener {

        private String name;

        MyActionListener(String n) {
            name = n;
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            String prjName = name.substring(name.lastIndexOf(File.separator) + 1);
            String workingPath = name.substring(0, name.lastIndexOf(File.separator));
            Project openProject = new Project(prjName, workingPath);
            try {
                closeCurrentProject();
                openProject.readProjectVariables();
            } catch (Exception ex) {
                JOptionPane.showMessageDialog(GlobalVariables.currentApplication.getMainFrame(), ex.toString(), "Error", JOptionPane.ERROR_MESSAGE);
            }

            PedFile newPedFile = new PedFile();
            try {
                GlobalVariables.lastAccessedPath = workingPath;
                GlobalVariables.currentPedFile = newPedFile;
                newPedFile.setTraitNames(openProject.getPhenotypeNames());
                String msg = "";
                if (openProject.getPedigreeFilePath() != null) {
                    msg = newPedFile.parseIGGPed(openProject.getPedigreeFilePath());
                    setBriefRunningInfor(msg, true);
                    GlobalVariables.addInforLog(msg);
                }

                GlobalVariables.currentProject = openProject;
                addProject2Tree(openProject);
                if (openProject.getIntegrationList().size() > 0) {
                    switchHapIntegration(true);
                    switchExort(true);
                }
            } catch (Exception ex) {
                JOptionPane.showMessageDialog(mainFrame, ex.toString(), "Error", JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    private void setTestPanelAttribute(String str) {
        SimpleAttributeSet attrset = new SimpleAttributeSet();
        StyleConstants.setForeground(attrset, Color.BLACK);

        //StyleConstants.setForeground(attrset, Color.BLUE.brighter());
        //StyleConstants.setBold(attrset, true);      
        StyleConstants.setFontFamily(attrset, "Courier New");
        StyleConstants.setFontSize(attrset, 13);
        insert(str, attrset);
    }

    private void insert(String str, AttributeSet attrset) {
        Document docs = breifTextPane.getDocument();
        str = str + "\n";
        try {
            docs.insertString(docs.getLength(), str, attrset);
        } catch (BadLocationException ble) {
            JOptionPane.showMessageDialog(this.mainFrame, "BadLocationException:" + ble, "Error", JOptionPane.ERROR_MESSAGE);
        }
    }

    public void setBriefRunningInfor(String msg, boolean toAppend) {
        if (!toAppend) {
            breifTextPane.removeAll();
        }
        setTestPanelAttribute(msg);
    }

    public void setBriefRunningInfor(String msg) {
        setTestPanelAttribute(msg);
    }

    public void setBriefRunningIcons(String iconName) {
        //move to the end of panel
        breifTextPane.setSelectionStart(breifTextPane.getDocument().getLength());
        breifTextPane.insertIcon(GUIUtil.readImageIcon(iconName));
    }

    public void setDetailRunningInfor(String msg, boolean toAppand) {
        if (toAppand) {
            detailTextArea.setText(detailTextArea.getText().concat(msg));
        } else {
            detailTextArea.setText(msg);
        }
    }

    public void setStatusMessage(String msg) {
        statusMessageLabel.setText(msg);
    }

    private boolean contain(String value, DefaultMutableTreeNode rootNode) {
        boolean hasContained = false;
        if (rootNode != null) {
            Enumeration nodeEnum = rootNode.breadthFirstEnumeration();
            while (nodeEnum.hasMoreElements()) {
                if (nodeEnum.nextElement().toString().equals(value)) {
                    hasContained = true;
                    break;
                }
            }
        }
        return hasContained;
    }

    public void updateChipAnnos() {
        FileTreeNode file;
        for (int i = 0; i < ILLUMINA_CHIPS.length; i++) {
            file = new FileTreeNode("resources/chip_annotation/" + ILLUMINA_CHIPS[i] + ".txt", ILLUMINA_CHIPS[i]);
            if (file.exists() && !contain(ILLUMINA_CHIPS[i], illumina)) {
                DefaultMutableTreeNode newNode = new DefaultMutableTreeNode(file);
                resourceTreeModel.insertNodeInto(newNode, illumina, illumina.getChildCount());
            }
        }
        resourceTree.expandPath(new TreePath(illumina.getPath()));
        for (int i = 0; i < AFFY_CHIPS.length; i++) {
            file = new FileTreeNode("resources/chip_annotation/" + AFFY_CHIPS[i] + ".txt", AFFY_CHIPS[i]);
            if (file.exists() && !contain(AFFY_CHIPS[i], affymetrix)) {
                DefaultMutableTreeNode newNode = new DefaultMutableTreeNode(file);
                resourceTreeModel.insertNodeInto(newNode, affymetrix, affymetrix.getChildCount());
            }
        }
        resourceTree.expandPath(new TreePath(affymetrix.getPath()));
    }

    public void specifyHapMapFiles(String chipName) {
        downloadHapmapAnnotationDialog.specifySelection(chipName);
    }

    public void specifyChipAnnotFiles(String chipName) {
        downloadChipAnnotationDialog.specifySelection(chipName);
    }

    public void updateHapMapAnnos() {
        FileTreeNode file;
        for (int i = 0; i < CHROM_NAME.length; i++) {
            file = new FileTreeNode("resources/hapmap/" + CHROM_NAME[i], "Chromosome " + CHROM_NAME[i]);
            if (file.exists() && !contain("Chromosome " + CHROM_NAME[i], dbSNPAnno)) {
                DefaultMutableTreeNode newNode = new DefaultMutableTreeNode(file);
                resourceTreeModel.insertNodeInto(newNode, dbSNPAnno, dbSNPAnno.getChildCount());
            }
        }
        resourceTree.expandPath(new TreePath(dbSNPAnno.getPath()));
    }

    public void addChipFileNodes(MultiMap chipFilePathList) {
        if (chipFilePathList.size() > 0) {
            DefaultMutableTreeNode prjNode = getTreeNodebyName(GlobalVariables.getCurrentProject().getName(), projectsRoot);
            Iterator iter = chipFilePathList.keySet().iterator();
            while (iter.hasNext()) {
                Object key = iter.next();
                DefaultMutableTreeNode parentNode = getTreeNodebyName(key.toString(), prjNode);
                if (parentNode == null) {
                    parentNode = new DefaultMutableTreeNode(key.toString());
                    projectsTreeModel.insertNodeInto(parentNode, prjNode, prjNode.getChildCount());
                }
                String fileName = chipFilePathList.get(key).toString();
                fileName = fileName.substring(1, fileName.length() - 1);
                String[] files = fileName.split(",");
                for (int i = 0; i < files.length; i++) {
                    fileName = files[i].trim();
                    DefaultMutableTreeNode newNode = null;
                    if (fileName.lastIndexOf("*.*") >= 0) {
                        String lastSubFolder = fileName.substring(0, fileName.lastIndexOf(File.separator));
                        lastSubFolder = lastSubFolder.substring(lastSubFolder.lastIndexOf(File.separator));
                        newNode = new DefaultMutableTreeNode(new FileTreeNode(fileName.substring(0, fileName.lastIndexOf(File.separator)),
                                lastSubFolder + File.separator + "*.*"));
                    } else {
                        newNode = new DefaultMutableTreeNode(new FileTreeNode(fileName, fileName.substring(fileName.lastIndexOf(File.separator) + 1)));
                    }

                    projectsTreeModel.insertNodeInto(newNode, parentNode, parentNode.getChildCount());
                }
                projectsTree.expandPath(new TreePath(parentNode.getPath()));
                projectsTree.setSelectionPath(new TreePath(parentNode.getPath()));
            }
        }
    }

    public void addCommonDataSetNode(CommonGenetFileSet set) {
        DefaultMutableTreeNode prjNode = getTreeNodebyName(GlobalVariables.getCurrentProject().getName(), projectsRoot);
        DefaultMutableTreeNode parentNode = new DefaultMutableTreeNode(set);
        projectsTreeModel.insertNodeInto(parentNode, prjNode, prjNode.getChildCount());
        try {
            File file = new File(set.getPedigreeFileName());
            DefaultMutableTreeNode pedFileNode = new DefaultMutableTreeNode(new FileTreeNode(file.getCanonicalPath(), file.getName()));
            projectsTreeModel.insertNodeInto(pedFileNode, parentNode, parentNode.getChildCount());
            file = new File(set.getMapFileName());
            DefaultMutableTreeNode mapFileNode = new DefaultMutableTreeNode(new FileTreeNode(file.getCanonicalPath(), file.getName()));
            projectsTreeModel.insertNodeInto(mapFileNode, parentNode, parentNode.getChildCount());
            projectsTree.expandPath(new TreePath(parentNode.getPath()));
            projectsTree.setSelectionPath(new TreePath(parentNode.getPath()));

        } catch (Exception ex) {
            JOptionPane.showMessageDialog(mainFrame, ex.toString(), "Error", JOptionPane.ERROR_MESSAGE);
        }
    }

    public void addChipPedigreeFileNode(String filePath) {
        DefaultMutableTreeNode prjNode = getTreeNodebyName(GlobalVariables.getCurrentProject().getName(), projectsRoot);
        try {
            if (filePath != null) {
                String fileName = "Chip Pedigree File: " + filePath.substring(filePath.lastIndexOf(File.separator) + 1);
                DefaultMutableTreeNode pedFilePathNode = new DefaultMutableTreeNode(new FileTreeNode(filePath, fileName));
                projectsTreeModel.insertNodeInto(pedFilePathNode, prjNode, prjNode.getChildCount());
                projectsTree.expandPath(new TreePath(pedFilePathNode.getPath()));
                projectsTree.setSelectionPath(new TreePath(pedFilePathNode.getPath()));
            }
        } catch (Exception ex) {
            JOptionPane.showMessageDialog(mainFrame, ex.toString(), "Error", JOptionPane.ERROR_MESSAGE);
        }
    }

    public void addIntegrationNodes(Integration intgr) {
        if (intgr != null) {
            DefaultMutableTreeNode prjNode = getTreeNodebyName(GlobalVariables.getCurrentProject().getName(), projectsRoot);

            DefaultMutableTreeNode integName = new DefaultMutableTreeNode(intgr);
            DefaultMutableTreeNode freqPop = new DefaultMutableTreeNode("Frequency Population: " + intgr.getFreqPopulation());
            /*
            DefaultMutableTreeNode trimInfo = new DefaultMutableTreeNode("Trimming Genotypes: " + intgr.getTrimingType() + intgr.getTrimmingInterval());
            DefaultMutableTreeNode removeMendl = new DefaultMutableTreeNode("Remove Bad Mendel: " + (intgr.isRemoveBadMendel() ? "Yes" : "No"));
            DefaultMutableTreeNode removeMissAll = new DefaultMutableTreeNode("Remove SNPs without Genotypes: " + (intgr.isRemoveMissingAll() ? "Yes" : "No"));
            DefaultMutableTreeNode correctFreq = new DefaultMutableTreeNode("Correct Inconsistant Frequency: " + (intgr.isCorrectAnnoFreq() ? "Yes" : "No"));
             */
            DefaultMutableTreeNode resultPath = new DefaultMutableTreeNode("Result Path: " + intgr.getResultPath());

            StringBuilder strRegion = new StringBuilder();
            DefaultMutableTreeNode resultRegion = new DefaultMutableTreeNode("Chr# SNP# Region");

            List<String> chrList = intgr.getChromResult();
            if (intgr.getPhysicalRegionMap().size() > 0) {
                for (int i = 0; i < chrList.size(); i++) {
                    String chroInfo = chrList.get(i);
                    strRegion.delete(0, strRegion.length());
                    strRegion.append("Chr");
                    strRegion.append(chroInfo);
                    long[] physicalRegions = intgr.getPhysicalRegionMap().get(chroInfo.substring(0, chroInfo.indexOf(' ')));
                    int regionNum = physicalRegions.length / 2;

                    for (int j = 0; j < regionNum; j++) {
                        if (physicalRegions[j * 2] != -9 || physicalRegions[j * 2 + 1] != -9) {

                            if (physicalRegions[j * 2 + 1] == -9) {
                                physicalRegions[1] = Long.MAX_VALUE;
                            }
                            strRegion.append(" [");
                            strRegion.append(physicalRegions[j * 2]);
                            strRegion.append(",");
                            strRegion.append(physicalRegions[j * 2 + 1]);
                            strRegion.append("]bp");
                        } else {
                            strRegion.append(" whole chrom.");
                        }
                    }
                    DefaultMutableTreeNode newNode = new DefaultMutableTreeNode(strRegion.toString());
                    resultRegion.insert(newNode, resultRegion.getChildCount());
                }
            } else {
                for (int i = 0; i < chrList.size(); i++) {
                    String chroInfo = chrList.get(i);
                    strRegion.delete(0, strRegion.length());
                    strRegion.append("Chr");
                    strRegion.append(chroInfo);
                    double[] genetRegions = intgr.getGenetRegionMap().get(chroInfo.substring(0, chroInfo.indexOf(' ')));
                    int regionNum = genetRegions.length / 2;

                    for (int j = 0; j < regionNum; j++) {
                        if (genetRegions[j * 2] != -9 || genetRegions[j * 2 + 1] != -9) {
                            strRegion.append(" [");
                            strRegion.append(genetRegions[0]);
                            strRegion.append(",");
                            strRegion.append(genetRegions[1]);
                            strRegion.append("]cM");
                        } else {
                            strRegion.append(" whole chrom.");
                        }
                    }
                    DefaultMutableTreeNode newNode = new DefaultMutableTreeNode(strRegion.toString());
                    resultRegion.insert(newNode, resultRegion.getChildCount());
                }

            }
            integName.insert(freqPop, integName.getChildCount());
            integName.insert(resultPath, integName.getChildCount());
            integName.insert(resultRegion, integName.getChildCount());

            projectsTreeModel.insertNodeInto(integName, prjNode, prjNode.getChildCount());
            projectsTree.expandPath(new TreePath(resultRegion.getPath()));
            projectsTree.setSelectionPath(new TreePath(resultRegion.getPath()));
            GlobalVariables.allIntegrationModel.addElement(intgr);
            if (!intgr.isHapMapSet()) {
                GlobalVariables.chipIntegrationModel.addElement(intgr);
            }
        }
    }

    public void addChipFileNodes(DefaultMutableTreeNode prjNode, MultiMap chipFilePathList) {
        if (chipFilePathList.size() > 0) {
            Iterator iter = chipFilePathList.keySet().iterator();
            while (iter.hasNext()) {
                Object key = iter.next();
                DefaultMutableTreeNode parentNode = getTreeNodebyName(key.toString(), prjNode);
                if (parentNode == null) {
                    parentNode = new DefaultMutableTreeNode(key.toString());
                }
                String fileName = chipFilePathList.get(key).toString();
                fileName = fileName.substring(1, fileName.length() - 1);
                String[] files = fileName.split(",");
                for (int i = 0; i < files.length; i++) {
                    fileName = files[i].trim();
                    DefaultMutableTreeNode newNode = null;
                    if (fileName.lastIndexOf("*.*") >= 0) {
                        String lastSubFolder = fileName.substring(0, fileName.lastIndexOf(File.separator));
                        lastSubFolder = lastSubFolder.substring(lastSubFolder.lastIndexOf(File.separator));
                        newNode = new DefaultMutableTreeNode(new FileTreeNode(fileName.substring(0, fileName.lastIndexOf(File.separator)),
                                lastSubFolder + File.separator + "*.*"));
                    } else {
                        newNode = new DefaultMutableTreeNode(new FileTreeNode(fileName, fileName.substring(fileName.lastIndexOf(File.separator) + 1)));
                    }
                    projectsTreeModel.insertNodeInto(newNode, parentNode, parentNode.getChildCount());
                }
                projectsTreeModel.insertNodeInto(parentNode, prjNode, prjNode.getChildCount());
                projectsTree.expandPath(new TreePath(parentNode.getPath()));
                projectsTree.setSelectionPath(new TreePath(parentNode.getPath()));
            }
        }
    }

    private DefaultMutableTreeNode getTreeNodebyName(String value, DefaultMutableTreeNode rootNode) {
        DefaultMutableTreeNode returnNode = null;
        if (rootNode != null) {
            Enumeration nodeEnum = rootNode.children();
            while (nodeEnum.hasMoreElements()) {
                DefaultMutableTreeNode temp = (DefaultMutableTreeNode) nodeEnum.nextElement();
                String node = temp.getUserObject().toString();
                if (node.equals(value)) {
                    returnNode = temp;
                    break;
                }
            }
        }
        return returnNode;
    }

    //check previously downloaded resources
    private void checkResourceVersion() throws Exception {

        ArrayList<String> updatedFiles = new ArrayList<String>();
        Set keys = GlobalVariables.resourceFileSize.keySet();
        if (keys == null) {
            return;
        }
        Long fileSize = GlobalVariables.resourceFileSize.get("hg17ToHg18.over.zip");
        if (fileSize == null) {
            GlobalVariables.resourceFileSize.put("hg17ToHg18.over.zip", 0L);
        }
        fileSize = GlobalVariables.resourceFileSize.get("hg19ToHg18.over.zip");
        if (fileSize == null) {
            GlobalVariables.resourceFileSize.put("hg19ToHg18.over.zip", 0L);
        }
        fileSize = GlobalVariables.resourceFileSize.get("hg18ToHg17.over.zip");
        if (fileSize == null) {
            GlobalVariables.resourceFileSize.put("hg18ToHg17.over.zip", 0L);
        }
        fileSize = GlobalVariables.resourceFileSize.get("hg18ToHg19.over.zip");
        if (fileSize == null) {
            GlobalVariables.resourceFileSize.put("hg18ToHg19.over.zip", 0L);
        }

        File localFileAddress = new File(RESOURCE_PATH);
        for (Iterator i = keys.iterator(); i.hasNext();) {
            String fileName = (String) i.next();
            fileSize = (Long) GlobalVariables.resourceFileSize.get(fileName);
            String urlFile = IGG_RESOURCE_URL + fileName;
            // throw new Exception("Cannot connect to IGG website to update your resources!");
            long urlFileSize = HttpClient4API.getContentLength(urlFile);
            if (fileSize == null || fileSize != urlFileSize) {
                updatedFiles.add(fileName);
            }

        }
        if (updatedFiles.isEmpty()) {
            JOptionPane.showMessageDialog(mainFrame, "The DOWNLOADED resources have already been the latest! ", "Message", JOptionPane.INFORMATION_MESSAGE);
            return;
        }
        StringBuilder infor = new StringBuilder();
        infor.append("The following ");
        infor.append(" data will be downloaded and updated!\n");
        for (int i = 0; i < updatedFiles.size(); i++) {
            infor.append(i + 1);
            infor.append(".  ");
            infor.append(updatedFiles.get(i));
            infor.append("\n");
            updatedFiles.set(i, IGG_RESOURCE_URL + updatedFiles.get(i));
        }
        Object[] options = {"OK", "CANCEL"};
        int response = JOptionPane.showOptionDialog(mainFrame, infor.toString(), "Message",
                JOptionPane.DEFAULT_OPTION, JOptionPane.INFORMATION_MESSAGE,
                null, options, options[0]);
        if (response == 1) {
            return;
        }
        setStatusMessage("Downloading...");
        setDownloadVisible(true);
        int filesNum = updatedFiles.size();

        ClientConnectionManager cm = new ThreadSafeClientConnManager(GlobalVariables.httpParams, GlobalVariables.connectionSchemeRegistry);
        HttpClient httpClient = new DefaultHttpClient(cm, GlobalVariables.httpParams);
        TaskQueue queue = new TaskQueue();
        for (int i = 0; i < filesNum; i++) {
            HttpClient4TaskBeanProgressBar task = new HttpClient4TaskBeanProgressBar(httpClient);
            task.setDownURL(updatedFiles.get(i));
            task.setSectionCount(FILE_SEGEMENT_NUM);
            task.setWorkerCount(FILE_SEGEMENT_NUM);
            task.setDownloadProgressBar(downloadProgressBar);
            task.setInforTextPane(breifTextPane);
            task.setBufferSize(128 * 1024);
            if (updatedFiles.get(i).lastIndexOf("hapmap/") >= 0) {
                task.setSaveFile(localFileAddress.getCanonicalPath() + File.separator + "hapmap"
                        + File.separator + updatedFiles.get(i).substring(updatedFiles.get(i).lastIndexOf("/") + 1));
            } else {
                task.setSaveFile(localFileAddress.getCanonicalPath() + File.separator + "chip_annotation"
                        + File.separator + updatedFiles.get(i).substring(updatedFiles.get(i).lastIndexOf("/") + 1));
            }
            queue.putTask(new HttpClient4DownloadTaskProgressBar(task));
        }
        ThreadPool pool = new ThreadPool(queue);

        //launtch two thread to do these task
        for (int i = 0; i < MAX_THREAD_NUM; i++) {
            pool.addWorkerThread();
        }

        this.setBriefRunningInfor("Updating resource files ...", true);
        GlobalVariables.addInforLog("Updating resource files ...");
        pool.jointAllWorkerThread();
        httpClient.getConnectionManager().shutdown();

        GlobalVariables.addInforLog(updatedFiles.size() + " file(s) have been downloaded and updated!");
        this.setBriefRunningInfor(updatedFiles.size() + " file(s) have been downloaded and updated!", true);
        JOptionPane.showMessageDialog(mainFrame, updatedFiles.size() + " file(s) have been downloaded and updated!", "Message", JOptionPane.INFORMATION_MESSAGE);
        setStatusMessage("Ready");
    }

    //check previously downloaded resources
    private void checkCoreVersion() throws Exception {
        File localFileAddress = new File("./lib/IGG3.jar");
        String urlFile = IGG_RESOURCE_URL + "lib/" + localFileAddress.getName();

        if (localFileAddress.exists()) {
            long fileSize = localFileAddress.length();
            long urlFileSize = HttpClient4API.getContentLength(urlFile);
            if (fileSize == urlFileSize) {
                JOptionPane.showMessageDialog(mainFrame, "The IGG has already been the latest! ", "Message", JOptionPane.INFORMATION_MESSAGE);
                return;
            }
        }

        StringBuilder infor = new StringBuilder();
        infor.append("Your IGG is not the latest, do you want update it?");

        Object[] options = {"OK", "CANCEL"};
        int response = JOptionPane.showOptionDialog(mainFrame, infor.toString(), "Message",
                JOptionPane.DEFAULT_OPTION, JOptionPane.INFORMATION_MESSAGE,
                null, options, options[0]);
        if (response == 1) {
            return;
        }
        setStatusMessage("Downloading...");
        setDownloadVisible(true);

        ClientConnectionManager cm = new ThreadSafeClientConnManager(GlobalVariables.httpParams, GlobalVariables.connectionSchemeRegistry);
        HttpClient httpClient = new DefaultHttpClient(cm, GlobalVariables.httpParams);
        TaskQueue queue = new TaskQueue();
        HttpClient4TaskBeanProgressBar task = new HttpClient4TaskBeanProgressBar(httpClient);
        task.setDownURL(urlFile);
        task.setSectionCount(FILE_SEGEMENT_NUM);
        task.setWorkerCount(FILE_SEGEMENT_NUM);
        task.setDownloadProgressBar(downloadProgressBar);
        task.setInforTextPane(breifTextPane);
        task.setBufferSize(128 * 1024);
        task.setSaveFile(localFileAddress.getCanonicalPath());

        queue.putTask(new HttpClient4DownloadTaskProgressBar(task));
        ThreadPool pool = new ThreadPool(queue);
        pool.addWorkerThread();

        this.setBriefRunningInfor("Updating IGG  ...", true);
        GlobalVariables.addInforLog("Updating IGG ...");
        pool.jointAllWorkerThread();
        httpClient.getConnectionManager().shutdown();

        setStatusMessage("Ready");
        GlobalVariables.hasUpdatedCore = true;
    }

    @Action
    public void showAboutBox() {
        if (aboutBox == null) {
            aboutBox = new IGG3AboutBox(mainFrame);
            aboutBox.setLocationRelativeTo(mainFrame);
        }
        IGG3App.getApplication().show(aboutBox);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mainPanel = new javax.swing.JPanel();
        jSplitPane1 = new javax.swing.JSplitPane();
        jSplitPane3 = new javax.swing.JSplitPane();
        jScrollPane2 = new javax.swing.JScrollPane();
        projectsTree = new javax.swing.JTree();
        jScrollPane3 = new javax.swing.JScrollPane();
        resourceTree = new javax.swing.JTree();
        jSplitPane2 = new javax.swing.JSplitPane();
        jTabbedPane1 = new javax.swing.JTabbedPane();
        jScrollPane4 = new javax.swing.JScrollPane();
        breifTextPane = new javax.swing.JTextPane();
        jScrollPane5 = new javax.swing.JScrollPane();
        detailTextArea = new javax.swing.JTextArea();
        jScrollPane1 = new javax.swing.JScrollPane();
        dataViewerTable = new javax.swing.JTable();
        menuBar = new javax.swing.JMenuBar();
        javax.swing.JMenu fileMenu = new javax.swing.JMenu();
        createProjectMenuItem = new javax.swing.JMenuItem();
        jSeparator6 = new javax.swing.JSeparator();
        jMenuItem19 = new javax.swing.JMenuItem();
        jSeparator8 = new javax.swing.JPopupMenu.Separator();
        jMenuItem17 = new javax.swing.JMenuItem();
        jSeparator7 = new javax.swing.JPopupMenu.Separator();
        jMenuItem18 = new javax.swing.JMenuItem();
        chooseChipFilesMenuItem = new javax.swing.JMenuItem();
        jSeparator4 = new javax.swing.JSeparator();
        jMenuItem15 = new javax.swing.JMenuItem();
        latestProjectMenu = new javax.swing.JMenu();
        jMenuItem16 = new javax.swing.JMenuItem();
        jSeparator3 = new javax.swing.JSeparator();
        javax.swing.JMenuItem exitMenuItem = new javax.swing.JMenuItem();
        integrateMenu = new javax.swing.JMenu();
        jMenuItem1 = new javax.swing.JMenuItem();
        jMenuItem2 = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JSeparator();
        jMenuItem6 = new javax.swing.JMenuItem();
        jMenuItem9 = new javax.swing.JMenuItem();
        jMenu2 = new javax.swing.JMenu();
        jMenuItem3 = new javax.swing.JMenuItem();
        jMenuItem8 = new javax.swing.JMenuItem();
        jMenu1 = new javax.swing.JMenu();
        jMenuItem10 = new javax.swing.JMenuItem();
        jMenuItem7 = new javax.swing.JMenuItem();
        jSeparator5 = new javax.swing.JSeparator();
        jMenuItem4 = new javax.swing.JMenuItem();
        javax.swing.JMenu helpMenu = new javax.swing.JMenu();
        jMenuItem11 = new javax.swing.JMenuItem();
        jMenuItem12 = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();
        statusPanel = new javax.swing.JPanel();
        javax.swing.JSeparator statusPanelSeparator = new javax.swing.JSeparator();
        statusMessageLabel = new javax.swing.JLabel();
        statusAnimationLabel = new javax.swing.JLabel();
        progressBar = new javax.swing.JProgressBar();
        memoryLabel = new javax.swing.JLabel();
        memoryProgressBar = new javax.swing.JProgressBar();
        downloadProgressBar = new javax.swing.JProgressBar();
        mainToolBar = new javax.swing.JToolBar();
        jButton1 = new javax.swing.JButton();
        jButton2 = new javax.swing.JButton();
        jButton10 = new javax.swing.JButton();
        jButton4 = new javax.swing.JButton();
        jButton5 = new javax.swing.JButton();
        jButton6 = new javax.swing.JButton();
        jButton7 = new javax.swing.JButton();
        jButton3 = new javax.swing.JButton();
        jButton8 = new javax.swing.JButton();
        jButton9 = new javax.swing.JButton();
        projectPopupMenu = new javax.swing.JPopupMenu();
        jMenuItem5 = new javax.swing.JMenuItem();
        tableViewerPopupMenu = new javax.swing.JPopupMenu();
        resourcePopupMenu = new javax.swing.JPopupMenu();
        jMenuItem13 = new javax.swing.JMenuItem();
        jMenuItem14 = new javax.swing.JMenuItem();

        mainPanel.setName("mainPanel"); // NOI18N
        mainPanel.addAncestorListener(new javax.swing.event.AncestorListener() {
            public void ancestorMoved(javax.swing.event.AncestorEvent evt) {
            }
            public void ancestorAdded(javax.swing.event.AncestorEvent evt) {
                mainPanelAncestorAdded(evt);
            }
            public void ancestorRemoved(javax.swing.event.AncestorEvent evt) {
            }
        });

        jSplitPane1.setDividerLocation(150);
        jSplitPane1.setName("jSplitPane1"); // NOI18N

        jSplitPane3.setDividerLocation(150);
        jSplitPane3.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        jSplitPane3.setName("jSplitPane3"); // NOI18N

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(org.cobi.igg3.gui.IGG3App.class).getContext().getResourceMap(IGG3View.class);
        jScrollPane2.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("jScrollPane2.border.title"))); // NOI18N
        jScrollPane2.setName("jScrollPane2"); // NOI18N

        projectsTree.setName("projectsTree"); // NOI18N
        projectsTree.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                projectsTreeMouseClicked(evt);
            }
        });
        projectsTree.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() {
            public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
                projectsTreeValueChanged(evt);
            }
        });
        projectsTree.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                projectsTreeKeyReleased(evt);
            }
        });
        jScrollPane2.setViewportView(projectsTree);

        jSplitPane3.setTopComponent(jScrollPane2);

        jScrollPane3.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("jScrollPane3.border.title"))); // NOI18N
        jScrollPane3.setName("jScrollPane3"); // NOI18N

        resourceTree.setName("resourceTree"); // NOI18N
        resourceTree.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                resourceTreeMouseClicked(evt);
            }
        });
        resourceTree.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() {
            public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
                resourceTreeValueChanged(evt);
            }
        });
        jScrollPane3.setViewportView(resourceTree);

        jSplitPane3.setRightComponent(jScrollPane3);

        jSplitPane1.setLeftComponent(jSplitPane3);

        jSplitPane2.setDividerLocation(150);
        jSplitPane2.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        jSplitPane2.setName("jSplitPane2"); // NOI18N

        jTabbedPane1.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("jTabbedPane1.border.title"))); // NOI18N
        jTabbedPane1.setName("jTabbedPane1"); // NOI18N

        jScrollPane4.setName("jScrollPane4"); // NOI18N

        breifTextPane.setEditable(false);
        breifTextPane.setName("breifTextPane"); // NOI18N
        jScrollPane4.setViewportView(breifTextPane);

        jTabbedPane1.addTab(resourceMap.getString("jScrollPane4.TabConstraints.tabTitle"), jScrollPane4); // NOI18N

        jScrollPane5.setName("jScrollPane5"); // NOI18N

        detailTextArea.setColumns(20);
        detailTextArea.setEditable(false);
        detailTextArea.setRows(5);
        detailTextArea.setName("detailTextArea"); // NOI18N
        detailTextArea.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseEntered(java.awt.event.MouseEvent evt) {
                detailTextAreaMouseEntered(evt);
            }
        });
        jScrollPane5.setViewportView(detailTextArea);

        jTabbedPane1.addTab(resourceMap.getString("jScrollPane5.TabConstraints.tabTitle"), jScrollPane5); // NOI18N

        jSplitPane2.setRightComponent(jTabbedPane1);

        jScrollPane1.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("jScrollPane1.border.title"))); // NOI18N
        jScrollPane1.setName("jScrollPane1"); // NOI18N

        dataViewerTable.setModel(listTableModel);
        dataViewerTable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF);
        dataViewerTable.setName("dataViewerTable"); // NOI18N
        dataViewerTable.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                dataViewerTableMouseClicked(evt);
            }
        });
        jScrollPane1.setViewportView(dataViewerTable);

        jSplitPane2.setLeftComponent(jScrollPane1);

        jSplitPane1.setRightComponent(jSplitPane2);

        javax.swing.GroupLayout mainPanelLayout = new javax.swing.GroupLayout(mainPanel);
        mainPanel.setLayout(mainPanelLayout);
        mainPanelLayout.setHorizontalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jSplitPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 760, Short.MAX_VALUE)
        );
        mainPanelLayout.setVerticalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jSplitPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 383, Short.MAX_VALUE)
        );

        menuBar.setName("menuBar"); // NOI18N

        fileMenu.setText(resourceMap.getString("fileMenu.text")); // NOI18N
        fileMenu.setName("fileMenu"); // NOI18N

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(org.cobi.igg3.gui.IGG3App.class).getContext().getActionMap(IGG3View.class, this);
        createProjectMenuItem.setAction(actionMap.get("showCreateProjectDialog")); // NOI18N
        createProjectMenuItem.setText(resourceMap.getString("createProjectMenuItem.text")); // NOI18N
        createProjectMenuItem.setName("createProjectMenuItem"); // NOI18N
        fileMenu.add(createProjectMenuItem);

        jSeparator6.setName("jSeparator6"); // NOI18N
        fileMenu.add(jSeparator6);

        jMenuItem19.setAction(actionMap.get("openLoadPlinkBinaryFormat")); // NOI18N
        jMenuItem19.setText(resourceMap.getString("jMenuItem19.text")); // NOI18N
        jMenuItem19.setName("jMenuItem19"); // NOI18N
        fileMenu.add(jMenuItem19);

        jSeparator8.setName("jSeparator8"); // NOI18N
        fileMenu.add(jSeparator8);

        jMenuItem17.setAction(actionMap.get("openLoadGenotypewithCommonFormatDialog")); // NOI18N
        jMenuItem17.setText(resourceMap.getString("jMenuItem17.text")); // NOI18N
        jMenuItem17.setName("jMenuItem17"); // NOI18N
        fileMenu.add(jMenuItem17);

        jSeparator7.setName("jSeparator7"); // NOI18N
        fileMenu.add(jSeparator7);

        jMenuItem18.setAction(actionMap.get("openLoadChipPedigreeDialog")); // NOI18N
        jMenuItem18.setName("jMenuItem18"); // NOI18N
        fileMenu.add(jMenuItem18);

        chooseChipFilesMenuItem.setAction(actionMap.get("loadSNPChipFiles")); // NOI18N
        chooseChipFilesMenuItem.setText(resourceMap.getString("chooseChipFilesMenuItem.text")); // NOI18N
        chooseChipFilesMenuItem.setName("chooseChipFilesMenuItem"); // NOI18N
        fileMenu.add(chooseChipFilesMenuItem);

        jSeparator4.setName("jSeparator4"); // NOI18N
        fileMenu.add(jSeparator4);

        jMenuItem15.setAction(actionMap.get("openProject")); // NOI18N
        jMenuItem15.setName("jMenuItem15"); // NOI18N
        fileMenu.add(jMenuItem15);

        latestProjectMenu.setText(resourceMap.getString("latestProjectMenu.text")); // NOI18N
        latestProjectMenu.setName("latestProjectMenu"); // NOI18N
        fileMenu.add(latestProjectMenu);

        jMenuItem16.setAction(actionMap.get("closeCurrentProject")); // NOI18N
        jMenuItem16.setName("jMenuItem16"); // NOI18N
        fileMenu.add(jMenuItem16);

        jSeparator3.setName("jSeparator3"); // NOI18N
        fileMenu.add(jSeparator3);

        exitMenuItem.setAction(actionMap.get("quit")); // NOI18N
        exitMenuItem.setName("exitMenuItem"); // NOI18N
        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        integrateMenu.setText(resourceMap.getString("integrateMenu.text")); // NOI18N
        integrateMenu.setName("integrateMenu"); // NOI18N

        jMenuItem1.setAction(actionMap.get("openChipFileWholeGenomeDialog")); // NOI18N
        jMenuItem1.setText(resourceMap.getString("jMenuItem1.text")); // NOI18N
        jMenuItem1.setName("jMenuItem1"); // NOI18N
        integrateMenu.add(jMenuItem1);

        jMenuItem2.setAction(actionMap.get("openIntegrateChipGenomeRegion")); // NOI18N
        jMenuItem2.setText(resourceMap.getString("jMenuItem2.text")); // NOI18N
        jMenuItem2.setName("jMenuItem2"); // NOI18N
        integrateMenu.add(jMenuItem2);

        jSeparator2.setName("jSeparator2"); // NOI18N
        integrateMenu.add(jSeparator2);

        jMenuItem6.setAction(actionMap.get("openIntegrateHapMapDialog")); // NOI18N
        jMenuItem6.setText(resourceMap.getString("jMenuItem6.text")); // NOI18N
        jMenuItem6.setActionCommand(resourceMap.getString("jMenuItem6.actionCommand")); // NOI18N
        jMenuItem6.setName("jMenuItem6"); // NOI18N
        integrateMenu.add(jMenuItem6);

        jMenuItem9.setAction(actionMap.get("openIntegrateHapMapPhase")); // NOI18N
        jMenuItem9.setText(resourceMap.getString("jMenuItem9.text")); // NOI18N
        jMenuItem9.setName("jMenuItem9"); // NOI18N
        integrateMenu.add(jMenuItem9);

        menuBar.add(integrateMenu);

        jMenu2.setText(resourceMap.getString("jMenu2.text")); // NOI18N
        jMenu2.setName("jMenu2"); // NOI18N

        jMenuItem3.setAction(actionMap.get("openExportDialog")); // NOI18N
        jMenuItem3.setText(resourceMap.getString("jMenuItem3.text")); // NOI18N
        jMenuItem3.setName("jMenuItem3"); // NOI18N
        jMenu2.add(jMenuItem3);

        jMenuItem8.setAction(actionMap.get("openImputationDialog")); // NOI18N
        jMenuItem8.setText(resourceMap.getString("jMenuItem8.text")); // NOI18N
        jMenuItem8.setName("jMenuItem8"); // NOI18N
        jMenu2.add(jMenuItem8);

        menuBar.add(jMenu2);

        jMenu1.setText(resourceMap.getString("jMenu1.text")); // NOI18N
        jMenu1.setName("jMenu1"); // NOI18N

        jMenuItem10.setAction(actionMap.get("openDonloadChipAnnot")); // NOI18N
        jMenuItem10.setName("jMenuItem10"); // NOI18N
        jMenu1.add(jMenuItem10);

        jMenuItem7.setAction(actionMap.get("openDownloadHapDialog")); // NOI18N
        jMenuItem7.setText(resourceMap.getString("jMenuItem7.text")); // NOI18N
        jMenuItem7.setName("jMenuItem7"); // NOI18N
        jMenu1.add(jMenuItem7);

        jSeparator5.setName("jSeparator5"); // NOI18N
        jMenu1.add(jSeparator5);

        jMenuItem4.setAction(actionMap.get("openGenerateSampleDialog")); // NOI18N
        jMenuItem4.setName("jMenuItem4"); // NOI18N
        jMenu1.add(jMenuItem4);

        menuBar.add(jMenu1);

        helpMenu.setText(resourceMap.getString("helpMenu.text")); // NOI18N
        helpMenu.setName("helpMenu"); // NOI18N

        jMenuItem11.setAction(actionMap.get("openUserManual")); // NOI18N
        jMenuItem11.setText(resourceMap.getString("jMenuItem11.text")); // NOI18N
        jMenuItem11.setName("jMenuItem11"); // NOI18N
        helpMenu.add(jMenuItem11);

        jMenuItem12.setAction(actionMap.get("iggHomepage")); // NOI18N
        jMenuItem12.setText(resourceMap.getString("jMenuItem12.text")); // NOI18N
        jMenuItem12.setName("jMenuItem12"); // NOI18N
        helpMenu.add(jMenuItem12);

        jSeparator1.setName("jSeparator1"); // NOI18N
        helpMenu.add(jSeparator1);

        aboutMenuItem.setAction(actionMap.get("showAboutBox")); // NOI18N
        aboutMenuItem.setName("aboutMenuItem"); // NOI18N
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        statusPanel.setName("statusPanel"); // NOI18N
        statusPanel.setPreferredSize(new java.awt.Dimension(100, 26));

        statusPanelSeparator.setName("statusPanelSeparator"); // NOI18N

        statusMessageLabel.setName("statusMessageLabel"); // NOI18N

        statusAnimationLabel.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        statusAnimationLabel.setName("statusAnimationLabel"); // NOI18N

        progressBar.setName("progressBar"); // NOI18N

        memoryLabel.setText(resourceMap.getString("memoryLabel.text")); // NOI18N
        memoryLabel.setName("memoryLabel"); // NOI18N

        memoryProgressBar.setBackground(resourceMap.getColor("memoryProgressBar.background")); // NOI18N
        memoryProgressBar.setFont(resourceMap.getFont("memoryProgressBar.font")); // NOI18N
        memoryProgressBar.setForeground(resourceMap.getColor("memoryProgressBar.foreground")); // NOI18N
        memoryProgressBar.setName("memoryProgressBar"); // NOI18N
        memoryProgressBar.setStringPainted(true);

        downloadProgressBar.setBackground(resourceMap.getColor("downloadProgressBar.background")); // NOI18N
        downloadProgressBar.setFont(resourceMap.getFont("downloadProgressBar.font")); // NOI18N
        downloadProgressBar.setForeground(resourceMap.getColor("downloadProgressBar.foreground")); // NOI18N
        downloadProgressBar.setName("downloadProgressBar"); // NOI18N
        downloadProgressBar.setStringPainted(true);

        javax.swing.GroupLayout statusPanelLayout = new javax.swing.GroupLayout(statusPanel);
        statusPanel.setLayout(statusPanelLayout);
        statusPanelLayout.setHorizontalGroup(
            statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(statusPanelSeparator, javax.swing.GroupLayout.DEFAULT_SIZE, 760, Short.MAX_VALUE)
            .addGroup(statusPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(statusMessageLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 740, Short.MAX_VALUE)
                .addComponent(statusAnimationLabel)
                .addContainerGap())
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, statusPanelLayout.createSequentialGroup()
                .addGap(183, 183, 183)
                .addComponent(downloadProgressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 20, Short.MAX_VALUE)
                .addComponent(memoryLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(memoryProgressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(59, 59, 59))
        );

        statusPanelLayout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {downloadProgressBar, memoryProgressBar, progressBar});

        statusPanelLayout.setVerticalGroup(
            statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(statusPanelLayout.createSequentialGroup()
                .addGap(8, 8, 8)
                .addComponent(statusAnimationLabel)
                .addGap(30, 30, 30))
            .addGroup(statusPanelLayout.createSequentialGroup()
                .addComponent(statusPanelSeparator, javax.swing.GroupLayout.PREFERRED_SIZE, 2, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(memoryProgressBar, javax.swing.GroupLayout.Alignment.CENTER, 0, 0, Short.MAX_VALUE)
                    .addComponent(memoryLabel, javax.swing.GroupLayout.Alignment.CENTER)
                    .addComponent(downloadProgressBar, javax.swing.GroupLayout.Alignment.CENTER, javax.swing.GroupLayout.PREFERRED_SIZE, 19, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
            .addGroup(statusPanelLayout.createSequentialGroup()
                .addGap(8, 8, 8)
                .addComponent(statusMessageLabel)
                .addGap(30, 30, 30))
        );

        statusPanelLayout.linkSize(javax.swing.SwingConstants.VERTICAL, new java.awt.Component[] {downloadProgressBar, memoryLabel, memoryProgressBar, progressBar});

        mainToolBar.setRollover(true);
        mainToolBar.setName("mainToolBar"); // NOI18N

        jButton1.setAction(actionMap.get("showCreateProjectDialog")); // NOI18N
        jButton1.setIcon(resourceMap.getIcon("jButton1.icon")); // NOI18N
        jButton1.setFocusable(false);
        jButton1.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton1.setName("jButton1"); // NOI18N
        jButton1.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        mainToolBar.add(jButton1);

        jButton2.setAction(actionMap.get("loadSNPChipFiles")); // NOI18N
        jButton2.setIcon(resourceMap.getIcon("jButton2.icon")); // NOI18N
        jButton2.setToolTipText(resourceMap.getString("jButton2.toolTipText")); // NOI18N
        jButton2.setFocusable(false);
        jButton2.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton2.setName("jButton2"); // NOI18N
        jButton2.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        mainToolBar.add(jButton2);

        jButton10.setAction(actionMap.get("openLoadGenotypewithCommonFormatDialog")); // NOI18N
        jButton10.setIcon(resourceMap.getIcon("jButton10.icon")); // NOI18N
        jButton10.setToolTipText(resourceMap.getString("jButton10.toolTipText")); // NOI18N
        jButton10.setFocusable(false);
        jButton10.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton10.setName("jButton10"); // NOI18N
        jButton10.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        mainToolBar.add(jButton10);

        jButton4.setAction(actionMap.get("openChipFileWholeGenomeDialog")); // NOI18N
        jButton4.setIcon(resourceMap.getIcon("jButton4.icon")); // NOI18N
        jButton4.setToolTipText(resourceMap.getString("jButton4.toolTipText")); // NOI18N
        jButton4.setFocusable(false);
        jButton4.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton4.setName("jButton4"); // NOI18N
        jButton4.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        mainToolBar.add(jButton4);

        jButton5.setAction(actionMap.get("openIntegrateHapMapDialog")); // NOI18N
        jButton5.setIcon(resourceMap.getIcon("jButton5.icon")); // NOI18N
        jButton5.setToolTipText(resourceMap.getString("jButton5.toolTipText")); // NOI18N
        jButton5.setFocusable(false);
        jButton5.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton5.setName("jButton5"); // NOI18N
        jButton5.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        mainToolBar.add(jButton5);

        jButton6.setAction(actionMap.get("openIntegrateHapMapPhase")); // NOI18N
        jButton6.setIcon(resourceMap.getIcon("jButton6.icon")); // NOI18N
        jButton6.setToolTipText(resourceMap.getString("jButton6.toolTipText")); // NOI18N
        jButton6.setFocusable(false);
        jButton6.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton6.setName("jButton6"); // NOI18N
        jButton6.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        mainToolBar.add(jButton6);

        jButton7.setAction(actionMap.get("openExportDialog")); // NOI18N
        jButton7.setIcon(resourceMap.getIcon("jButton7.icon")); // NOI18N
        jButton7.setToolTipText(resourceMap.getString("jButton7.toolTipText")); // NOI18N
        jButton7.setFocusable(false);
        jButton7.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton7.setName("jButton7"); // NOI18N
        jButton7.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        mainToolBar.add(jButton7);

        jButton3.setAction(actionMap.get("openImputationDialog")); // NOI18N
        jButton3.setIcon(resourceMap.getIcon("jButton3.icon")); // NOI18N
        jButton3.setToolTipText(resourceMap.getString("jButton3.toolTipText")); // NOI18N
        jButton3.setFocusable(false);
        jButton3.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton3.setName("jButton3"); // NOI18N
        jButton3.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        mainToolBar.add(jButton3);

        jButton8.setAction(actionMap.get("openUserManual")); // NOI18N
        jButton8.setIcon(resourceMap.getIcon("jButton8.icon")); // NOI18N
        jButton8.setToolTipText(resourceMap.getString("jButton8.toolTipText")); // NOI18N
        jButton8.setFocusable(false);
        jButton8.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton8.setName("jButton8"); // NOI18N
        jButton8.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        mainToolBar.add(jButton8);

        jButton9.setAction(actionMap.get("iggHomepage")); // NOI18N
        jButton9.setIcon(resourceMap.getIcon("jButton9.icon")); // NOI18N
        jButton9.setToolTipText(resourceMap.getString("jButton9.toolTipText")); // NOI18N
        jButton9.setFocusable(false);
        jButton9.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton9.setName("jButton9"); // NOI18N
        jButton9.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        mainToolBar.add(jButton9);

        projectPopupMenu.setName("projectPopupMenu"); // NOI18N

        jMenuItem5.setAction(actionMap.get("removeProjectComponent")); // NOI18N
        jMenuItem5.setName("jMenuItem5"); // NOI18N
        projectPopupMenu.add(jMenuItem5);

        tableViewerPopupMenu.setName("tableViewerPopupMenu"); // NOI18N

        resourcePopupMenu.setName("resourcePopupMenu"); // NOI18N

        jMenuItem13.setAction(actionMap.get("dowloadResource")); // NOI18N
        jMenuItem13.setName("jMenuItem13"); // NOI18N
        resourcePopupMenu.add(jMenuItem13);

        jMenuItem14.setAction(actionMap.get("deleteSelected")); // NOI18N
        jMenuItem14.setName("jMenuItem14"); // NOI18N
        resourcePopupMenu.add(jMenuItem14);

        setComponent(mainPanel);
        setMenuBar(menuBar);
        setStatusBar(statusPanel);
        setToolBar(mainToolBar);
    }// </editor-fold>//GEN-END:initComponents

private void mainPanelAncestorAdded(javax.swing.event.AncestorEvent evt) {//GEN-FIRST:event_mainPanelAncestorAdded
    // TODO add your handling code here:
    mainFrame = IGG3App.getApplication().getMainFrame();
    GlobalVariables.mainView = this;
    try {
        GlobalVariables.readEnvironmentalVariables();
        switchChipIntegration(false);
        switchHapIntegration(false);
        switchExort(false);
        switchLoading(false);

        // MainFrame Exit Listener
        GlobalVariables.currentApplication.addExitListener(new ConfirmExit());
        String path;
        JMenuItem pathMenuItem;
        for (int i = 0; i < GlobalVariables.latestAccessedProjects.size(); i++) {
            path = GlobalVariables.latestAccessedProjects.get(i);
            pathMenuItem = new JMenuItem(path);
            MyActionListener listener = new MyActionListener(path);
            pathMenuItem.addActionListener(listener);
            latestProjectMenu.add(pathMenuItem);
        }

        if (GlobalVariables.needStartHint.equals("Yes")) {
            if (startingHintJDialog == null) {
                startingHintJDialog = new StartingHintJDialog(mainFrame, true);
                startingHintJDialog.setLocationRelativeTo(mainFrame);
            }
            IGG3App.getApplication().show(startingHintJDialog);
        }

        GlobalVariables.initialGlobalVariables();
        SwingWorker worker = new SwingWorker() {

            @Override
            protected Object doInBackground() {
                try {
                    /*
                    GlobalVariables.mainView.setBriefRunningInfor("Checking IGG Version...", true);
                    GlobalVariables.addInforLog("Checking IGG Version...");
                    checkCoreVersion();
                    if (GlobalVariables.hasUpdatedCore) {
                    File iggTmpFile = new File("./IGG.jar.r_save");
                    File iggFile = new File("./IGG.jar");
                    String infor = "IGG has been downloaded and updated.\n1. Please exit IGG now and remove " + iggFile.getCanonicalPath() +
                    "\n2. Rename \'" + iggTmpFile.getCanonicalPath() + "\' as \'" + iggFile.getCanonicalPath() + "\'" +
                    "\'\nDo you want to exit now?";
                    Object[] options = {"OK", "CANCEL"};
                    int response = JOptionPane.showOptionDialog(mainFrame, infor.toString(), "Message",
                    JOptionPane.DEFAULT_OPTION, JOptionPane.INFORMATION_MESSAGE,
                    null, options, options[0]);
                    if (response == 0) {
                    System.exit(0);
                    }
                    }
                     */

                    GlobalVariables.buildInternetConnection();
                    if (!GlobalVariables.canConnect2Website) {
                        String msg = "IGG cannot connect to website to update its resources! Please check your local network configurations!";
                        GlobalVariables.addInforLog(msg);
                        GlobalVariables.mainView.setBriefRunningInfor(msg, true);
                    } else {
                        GlobalVariables.mainView.setBriefRunningInfor("Checking Resources Version...", true);
                        GlobalVariables.addInforLog("Checking Resources Version...");
                        checkResourceVersion();
                        if (downloadHapmapAnnotationDialog == null) {
                            downloadHapmapAnnotationDialog = new DownloaddbSNPAnnotationDialog(mainFrame, true);
                            downloadHapmapAnnotationDialog.pack();
                            downloadHapmapAnnotationDialog.setLocationRelativeTo(mainFrame);
                        }

                        if (downloadChipAnnotationDialog == null) {
                            downloadChipAnnotationDialog = new DownloadChipAnnotationDialog(mainFrame, true);
                            downloadChipAnnotationDialog.pack();
                            downloadChipAnnotationDialog.setLocationRelativeTo(mainFrame);
                        }
                    }

                } catch (Exception ex) {
                    JOptionPane.showMessageDialog(mainFrame, ex.toString(), "Error", JOptionPane.ERROR_MESSAGE);
                }
                return null;
            }

            @Override
            protected void done() {
                try {
                    setDownloadVisible(false);
                    GlobalVariables.mainView.setBriefRunningInfor("Done.", true);
                    GlobalVariables.addInforLog("Done.");
                } catch (Exception ex) {
                    JOptionPane.showMessageDialog(mainFrame, ex.toString(), "Error", JOptionPane.ERROR_MESSAGE);
                }
            }
        };
        worker.execute();

        GlobalVariables.hasNewLog = true;
    } catch (Exception ex) {
        JOptionPane.showMessageDialog(GlobalVariables.currentApplication.getMainFrame(), ex.toString(), "Error", JOptionPane.ERROR_MESSAGE);
    }

}//GEN-LAST:event_mainPanelAncestorAdded

private void projectsTreeMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_projectsTreeMouseClicked
// TODO add your handling code here:
    try {
        //SwingUtilities
        if (evt.getClickCount() == 1 && SwingUtilities.isRightMouseButton(evt)) {
            TreePath path = projectsTree.getPathForLocation(evt.getX(), evt.getY());
            if (path == null) {
                return;
            }
            //pedAndGeneTree.setSelectionPath(path);
            projectPopupMenu.show(projectsTree, evt.getX(), evt.getY());
        }
    } catch (Exception ex) {
        ex.printStackTrace();
        JOptionPane.showMessageDialog(mainFrame, ex.toString(), "Error", JOptionPane.ERROR_MESSAGE);
    }

}//GEN-LAST:event_projectsTreeMouseClicked

private void dataViewerTableMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_dataViewerTableMouseClicked
// TODO add your handling code here:
    try {
        if (evt.getClickCount() == 1 && SwingUtilities.isRightMouseButton(evt)) { //SwingUtilities 

            tableViewerPopupMenu.show(dataViewerTable, evt.getX(), evt.getY());
        }
        return;
    } catch (Exception ex) {
        JOptionPane.showMessageDialog(mainFrame, ex.toString(), "Error", JOptionPane.ERROR_MESSAGE);
    }
}//GEN-LAST:event_dataViewerTableMouseClicked

private void projectsTreeValueChanged(javax.swing.event.TreeSelectionEvent evt) {//GEN-FIRST:event_projectsTreeValueChanged
// TODO add your handling code here:
    // DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) evt.getPath().getParentPath().getLastPathComponent();
    DefaultMutableTreeNode node = (DefaultMutableTreeNode) evt.getPath().getLastPathComponent();
    Object obj = node.getUserObject();
    if (obj instanceof File) {
        File f = (File) obj;
        if (f.isFile()) {
            try {
                listTableData.clear();
                if (node.toString().startsWith("Chip Pedigree File:")) {
                    LocalFile.retrieveData(f.getAbsolutePath(), listTableData, 500, null, null, true);
                    if (listTableData.size() == 0) {
                        throw new Exception("The file has no data!");
                    }
                    String[] tmpArray = new String[listTableData.get(0).length];
                    Arrays.fill(tmpArray, "");
                    listTableModel.setTitle(tmpArray);
                    if (listTableData.size() == 0) {
                        tmpArray = new String[listTableModel.getColumnCount()];
                        Arrays.fill(tmpArray, "");
                        listTableData.add(tmpArray);
                    }
                    listTableModel.setDataList(listTableData);
                    listTableModel.fireTableStructureChanged();
                    GlobalVariables.mainView.setStatusMessage("Note: Only present 500 items!");
                } else {
                    LocalFile.retrieveAsOneColumn(f.getAbsolutePath(), listTableData, 500);
                    if (listTableData.size() == 0) {
                        throw new Exception("The file has no data!");
                    }
                    String[] tmpArray = new String[listTableData.get(0).length];
                    Arrays.fill(tmpArray, "");
                    listTableModel.setTitle(tmpArray);
                    if (listTableData.size() == 0) {
                        tmpArray = new String[listTableModel.getColumnCount()];
                        Arrays.fill(tmpArray, "");
                        listTableData.add(tmpArray);
                    }
                    listTableModel.setDataList(listTableData);
                    listTableModel.fireTableStructureChanged();
                    TableColumn firsetColumn = dataViewerTable.getColumnModel().getColumn(0);
                    firsetColumn.setPreferredWidth(400);
                    firsetColumn.sizeWidthToFit();
                    GlobalVariables.mainView.setStatusMessage("Note: Only present 500 items!");
                }
            } catch (Exception err) {
                err.printStackTrace();
                JOptionPane.showMessageDialog(mainFrame, err.toString());
            }
        }
    }

}//GEN-LAST:event_projectsTreeValueChanged

private void resourceTreeValueChanged(javax.swing.event.TreeSelectionEvent evt) {//GEN-FIRST:event_resourceTreeValueChanged
// TODO add your handling code here:
    DefaultMutableTreeNode node = (DefaultMutableTreeNode) evt.getPath().getLastPathComponent();
    Object obj = node.getUserObject();
    if (obj instanceof File) {
        File f = (File) obj;
        if (f.isFile()) {
            try {
                listTableData.clear();
                LocalFile.retrieveData(f.getAbsolutePath(), listTableData, 500, "\t", null, true);
                if (listTableData.size() == 0) {
                    throw new Exception("The file has no data!");
                }
                String[] tmpArray = new String[listTableData.get(0).length];
                Arrays.fill(tmpArray, "");
                listTableModel.setTitle(tmpArray);
                if (listTableData.size() == 0) {
                    tmpArray = new String[listTableModel.getColumnCount()];
                    Arrays.fill(tmpArray, "");
                    listTableData.add(tmpArray);
                }
                listTableModel.setDataList(listTableData);
                listTableModel.fireTableStructureChanged();
                GlobalVariables.mainView.setStatusMessage("Note: Only present 500 items!");

            } catch (Exception err) {
                err.printStackTrace();
                JOptionPane.showMessageDialog(mainFrame, err.toString());
            }
        }
    }
}//GEN-LAST:event_resourceTreeValueChanged

private void resourceTreeMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_resourceTreeMouseClicked
// TODO add your handling code here:
    try {
        //SwingUtilities
        if (evt.getClickCount() == 1 && SwingUtilities.isRightMouseButton(evt)) {
            TreePath path = resourceTree.getPathForLocation(evt.getX(), evt.getY());
            if (path == null) {
                return;
            }
            //pedAndGeneTree.setSelectionPath(path);
            resourcePopupMenu.show(resourceTree, evt.getX(), evt.getY());
        }
    } catch (Exception ex) {
        ex.printStackTrace();
        JOptionPane.showMessageDialog(mainFrame, ex.toString(), "Error", JOptionPane.ERROR_MESSAGE);
    }
}//GEN-LAST:event_resourceTreeMouseClicked

private void detailTextAreaMouseEntered(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_detailTextAreaMouseEntered
// TODO add your handling code here:
    StringBuffer strBu = new StringBuffer();
    try {
        if (GlobalVariables.hasNewLog) {
            LocalFile.retrieveData("./igg.log", strBu);
            detailTextArea.setText("");
            detailTextArea.append(strBu.toString());
            GlobalVariables.hasNewLog = false;
        }
    } catch (Exception ex) {
        JOptionPane.showMessageDialog(mainFrame, ex.toString(), "Error", JOptionPane.ERROR_MESSAGE);
    }
}//GEN-LAST:event_detailTextAreaMouseEntered

private void projectsTreeKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_projectsTreeKeyReleased
    // TODO add your handling code here:
    if (evt.getKeyCode() == KeyEvent.VK_DELETE) {
        removeProjectComponent();
    }
}//GEN-LAST:event_projectsTreeKeyReleased

    @Action
    public void showCreateProjectDialog() {
        if (createProjectJDialog == null) {
            createProjectJDialog = new CreateProjectJDialog(mainFrame);
            createProjectJDialog.setLocationRelativeTo(mainFrame);
        }
        IGG3App.getApplication().show(createProjectJDialog);
    }

    @Action(selectedProperty = "toLoadSNPChipFiles")
    public void loadSNPChipFiles() {
        if (loadGeneChipFilesDialog == null) {
            loadGeneChipFilesDialog = new LoadGeneChipFilesDialog(mainFrame, true);
            loadGeneChipFilesDialog.setLocationRelativeTo(mainFrame);
        }
        IGG3App.getApplication().show(loadGeneChipFilesDialog);
    }
    private boolean toLoadSNPChipFiles = true;

    public boolean isToLoadSNPChipFiles() {
        return toLoadSNPChipFiles;
    }

    public void setToLoadSNPChipFiles(boolean b) {
        boolean old = isToLoadSNPChipFiles();
        this.toLoadSNPChipFiles = b;
        firePropertyChange("toLoadSNPChipFiles", old, isToLoadSNPChipFiles());
    }

    @Action
    public void openChipFileWholeGenomeDialog() {
        if (integrateWholeGenome == null) {
            integrateWholeGenome = new IntegrateWholeGenomeDialog(mainFrame, true);
            integrateWholeGenome.setLocationRelativeTo(mainFrame);
        }
        IGG3App.getApplication().show(integrateWholeGenome);
    }

    @Action
    public void openIntegrateChipGenomeRegion() {
        if (integrateGenomeRegion == null) {
            integrateGenomeRegion = new IntegrateGenomeRegionDialog(mainFrame, true);
            integrateGenomeRegion.setLocationRelativeTo(mainFrame);
        }
        IGG3App.getApplication().show(integrateGenomeRegion);
    }

    @Action
    public void openExportDialog() {
        if (exportIntegrateDataJDialog == null) {
            exportIntegrateDataJDialog = new ExportIntegrateDataJDialog(mainFrame, true);
            exportIntegrateDataJDialog.setLocationRelativeTo(mainFrame);
        }
        IGG3App.getApplication().show(exportIntegrateDataJDialog);
    }

    @Action
    public void openGenerateSampleDialog() {
        if (generateSampleDialog == null) {
            generateSampleDialog = new GenerateSampleDialog(mainFrame, true);
            generateSampleDialog.setLocationRelativeTo(mainFrame);
        }
        IGG3App.getApplication().show(generateSampleDialog);

    }

    @Action
    public void removeProjectComponent() {
        TreePath[] nodePaths = projectsTree.getSelectionPaths();
        String msg = "Not Remove Node(s)!";
        try {
            if (nodePaths != null) {
                for (int i = 0; i < nodePaths.length; i++) {
                    String pareNodeName = nodePaths[i].getParentPath().getLastPathComponent().toString();
                    DefaultMutableTreeNode selectionNode = (DefaultMutableTreeNode) nodePaths[i].getLastPathComponent();
                    String nodeName = selectionNode.toString();
                    Object obj = selectionNode.getUserObject();

                    if (nodeName.startsWith("Chip Pedigree File:")) {
                        msg = "Sorry, you cannot remove this node!";
                    } else if (obj instanceof File) {
                        projectsTreeModel.removeNodeFromParent(selectionNode);
                        String path = ((File) obj).getCanonicalPath();
                        if (nodeName.lastIndexOf("*.*") >= 0) {
                            path = path + File.separator + "*.*";
                        }
                        GlobalVariables.currentProject.removeAChipFilePath(pareNodeName, path);
                        msg = "Node(s) Removed Successfull";
                    } else if (obj instanceof Integration) {
                        int result = JOptionPane.showConfirmDialog(mainFrame, "Do you really want remove \"" + nodeName + "\"?"
                                + " All data (including the INTEGRATION RESULTS) of integration will be"
                                + "deleted!!");
                        if (result == 0) {
                            projectsTreeModel.removeNodeFromParent(selectionNode);
                            GlobalVariables.allIntegrationModel.removeElement(obj);
                            GlobalVariables.chipIntegrationModel.removeElement(obj);
                            GlobalVariables.currentProject.removeAIntegration(nodeName);
                            msg = "Node(s) Removed Successfull";
                        }
                    } else if (nodeName.startsWith("Affymetrix_") || nodeName.startsWith("Illumina_")) {
                        projectsTreeModel.removeNodeFromParent(selectionNode);
                        GlobalVariables.currentProject.removeAChipType(nodeName);
                        msg = "Node(s) Removed Successfull";
                    } else if (obj instanceof CommonGenetFileSet) {
                        projectsTreeModel.removeNodeFromParent(selectionNode);
                        GlobalVariables.currentProject.removeACommonGentypeSet(nodeName);
                        msg = "Node(s) Removed Successfull";
                    } else {
                        msg = "Sorry, you cannot remove this node!";
                    }
                }
                JOptionPane.showMessageDialog(mainFrame, msg);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            JOptionPane.showMessageDialog(mainFrame, ex.toString(), "Error", JOptionPane.ERROR_MESSAGE);
        }
    }

    @Action
    public void openIntegrateHapMapDialog() {
        if (integrateHapMapDialog == null) {
            integrateHapMapDialog = new IntegrateHapMapDialog(mainFrame, true);
            integrateHapMapDialog.setLocationRelativeTo(mainFrame);
        }
        IGG3App.getApplication().show(integrateHapMapDialog);
    }

    @Action
    public void openDownloadHapDialog() {
        IGG3App.getApplication().show(downloadHapmapAnnotationDialog);
    }

    @Action
    public void openImputationDialog() {
        if (imputationExportDialog == null) {
            imputationExportDialog = new ImputationExportDialog(mainFrame, true);
            imputationExportDialog.setLocationRelativeTo(mainFrame);
        }

        IGG3App.getApplication().show(imputationExportDialog);
    }

    @Action
    public void openIntegrateHapMapPhase() {
        if (integrateHapMapPhaseDialog == null) {
            integrateHapMapPhaseDialog = new IntegrateHapMapPhaseDialog(mainFrame, true);
            integrateHapMapPhaseDialog.setLocationRelativeTo(mainFrame);
        }
        IGG3App.getApplication().show(integrateHapMapPhaseDialog);
    }

    @Action
    public void openDonloadChipAnnot() {
        IGG3App.getApplication().show(downloadChipAnnotationDialog);
    }

    @Action
    public void openUserManual() {
        String osName = System.getProperty("os.name");
        try {
            if (osName.startsWith("Windows")) {
//FOR WINDOWS 95 AND 98 USE COMMAND.COM
                if (osName.equals("Windows 95") || osName.equals("Windows 98")) {
                    Runtime.getRuntime().exec("command /K start UserManual.pdf");
                } //FOR WINDOWS NT/XP/2000 USE CMD.EXE
                else {
                    Runtime.getRuntime().exec("cmd /K start UserManual.pdf");
                }

            } else {
                JOptionPane.showMessageDialog(mainFrame, "Sorry, I cannot directly open UserManual.pdf in your OS.\n Please find it in the installation folder.", "Information", JOptionPane.ERROR_MESSAGE);
            }

        } catch (Exception ex) {
            JOptionPane.showMessageDialog(mainFrame, ex.toString(), "Error", JOptionPane.ERROR_MESSAGE);
        }

    }

    @Action
    public void iggHomepage() {
        URLOpener.openURL("http://bioinfo.hku.hk/iggweb");
    }

    @Action
    public void dowloadResource() {
        TreePath[] nodePaths = resourceTree.getSelectionPaths();
        String msg = "";
        try {
            if (nodePaths != null) {
                for (int i = 0; i < nodePaths.length; i++) {
                    Object[] path = nodePaths[i].getPath();
                    for (int j = 0; j < path.length; j++) {
                        if (path[j].toString().startsWith("Chip Anno")) {
                            IGG3App.getApplication().show(downloadChipAnnotationDialog);
                            break;
                        } else if (path[j].toString().startsWith("HapMap ")) {
                            IGG3App.getApplication().show(downloadHapmapAnnotationDialog);
                            break;
                        }
                    }

                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            JOptionPane.showMessageDialog(mainFrame, ex.toString(), "Error", JOptionPane.ERROR_MESSAGE);
        }

    }

    @Action
    public void deleteSelected() {
        TreePath[] nodePaths = resourceTree.getSelectionPaths();
        String msg = "";
        try {
            if (nodePaths != null) {
                for (int i = 0; i < nodePaths.length; i++) {
                    DefaultMutableTreeNode selectionNode = (DefaultMutableTreeNode) nodePaths[i].getLastPathComponent();
                    String nodeName = selectionNode.toString();
                    Object obj = selectionNode.getUserObject();
                    if (obj instanceof File) {
                        int result = JOptionPane.showConfirmDialog(mainFrame, "Do you really want remove \"" + nodeName + "\"?"
                                + " This file will be removed from your disk!!");
                        if (result == 0) {
                            File file = (File) obj;
                            resourceTreeModel.removeNodeFromParent(selectionNode);
                            if (file.getCanonicalPath().indexOf("hapmap") >= 0) {
                                String fullName = "hapmap/" + file.getName() + ".zip";
                                GlobalVariables.resourceFileSize.remove(fullName);
                            } else {
                                GlobalVariables.resourceFileSize.remove(selectionNode + ".txt.zip");
                            }
                            file.delete();
                            msg = "Node(s) Removed Successfull";
                        } else {
                            msg = "Cancel a deletion!";
                        }
                    } else {
                        msg = "Sorry, you cannot remove this node!";
                    }
                }
            }
            JOptionPane.showMessageDialog(mainFrame, msg);
        } catch (Exception ex) {
            ex.printStackTrace();
            JOptionPane.showMessageDialog(mainFrame, ex.toString(), "Error", JOptionPane.ERROR_MESSAGE);
        }
    }

    @Action
    public void openProject() {
        JFileChooser fDialog = new MyJFileChooser("xml", "IGG Project Files");
        fDialog.setDialogTitle("Open IGG Project");

        int result = fDialog.showOpenDialog(mainFrame);
        if (result == JFileChooser.APPROVE_OPTION) {
            String filePath = fDialog.getSelectedFile().getPath();
            String projectPath = filePath.substring(0, filePath.length() - 4);

            if (!GlobalVariables.latestAccessedProjects.contains(projectPath)) {
                GlobalVariables.latestAccessedProjects.add(projectPath);
            }
            try {
                String prjName = filePath.substring(filePath.lastIndexOf(File.separator) + 1, filePath.length() - 4);
                String workingPath = filePath.substring(0, filePath.lastIndexOf(File.separator));
                Project openProject = new Project(prjName, workingPath);

                openProject.readProjectVariables();
                PedFile newPedFile = new PedFile();
                String msg = "";
                newPedFile.setTraitNames(openProject.getPhenotypeNames());
                if (openProject.getPedigreeFilePath() != null) {
                    msg = newPedFile.parseIGGPed(openProject.getPedigreeFilePath());
                    setBriefRunningInfor(msg, true);
                    GlobalVariables.addInforLog(msg);
                }

                GlobalVariables.currentProject = openProject;
                GlobalVariables.lastAccessedPath = workingPath;
                GlobalVariables.currentPedFile = newPedFile;

                addProject2Tree(openProject);
                if (openProject.getIntegrationList().size() > 0) {
                    switchHapIntegration(true);
                    switchExort(true);
                }


            } catch (Exception ex) {
                JOptionPane.showMessageDialog(mainFrame, ex.toString(), "Error", JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    @Action
    public void closeCurrentProject() throws Exception {
        GlobalVariables.currentProject = null;
        GlobalVariables.currentPedFile = null;
        projectsRoot.removeAllChildren();
        projectsTreeModel.reload();
        GlobalVariables.allIntegrationModel.removeAllElements();
        GlobalVariables.chipIntegrationModel.removeAllElements();
        switchChipIntegration(false);
        switchHapIntegration(false);
        switchExort(false);
        switchLoading(false);
    }

    @Action
    public void openLoadGenotypewithCommonFormatDialog() {
        if (loadGenotypewithCommonFormatDialog == null) {
            loadGenotypewithCommonFormatDialog = new LoadCommonGenotypeSetDialog(mainFrame, true);

            loadGenotypewithCommonFormatDialog.setLocationRelativeTo(mainFrame);
        }
        IGG3App.getApplication().show(loadGenotypewithCommonFormatDialog);
    }

    @Action
    public void openAppendSNPAnnoationDialog() {
        return;
        /*
        if (appendSNPAnnotationJDialog == null) {
        appendSNPAnnotationJDialog = new AppendSNPAnnotationJDialog(mainFrame, true);
        appendSNPAnnotationJDialog.setLocationRelativeTo(mainFrame);
        }
        IGG3App.getApplication().show(appendSNPAnnotationJDialog);
         */
    }

    @Action
    public void openLoadChipPedigreeDialog() {
        if (loadPedigreeInformationForChipFileDialog == null) {
            loadPedigreeInformationForChipFileDialog = new LoadPedigreeInformationForChipFileDialog(mainFrame, true);
            loadPedigreeInformationForChipFileDialog.setLocationRelativeTo(mainFrame);
        }
        IGG3App.getApplication().show(loadPedigreeInformationForChipFileDialog);

    }

    @Action
    public void openLoadPlinkBinaryFormat() {
        if (loadPlinkBinaryFormatDialog == null) {
            loadPlinkBinaryFormatDialog = new LoadPlinkBinaryGenotypeSetDialog(mainFrame, true);
            loadPlinkBinaryFormatDialog.setLocationRelativeTo(mainFrame);
        }
        IGG3App.getApplication().show(loadPlinkBinaryFormatDialog);
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextPane breifTextPane;
    private javax.swing.JMenuItem chooseChipFilesMenuItem;
    private javax.swing.JMenuItem createProjectMenuItem;
    private javax.swing.JTable dataViewerTable;
    private javax.swing.JTextArea detailTextArea;
    private javax.swing.JProgressBar downloadProgressBar;
    private javax.swing.JMenu integrateMenu;
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton10;
    private javax.swing.JButton jButton2;
    private javax.swing.JButton jButton3;
    private javax.swing.JButton jButton4;
    private javax.swing.JButton jButton5;
    private javax.swing.JButton jButton6;
    private javax.swing.JButton jButton7;
    private javax.swing.JButton jButton8;
    private javax.swing.JButton jButton9;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JMenu jMenu2;
    private javax.swing.JMenuItem jMenuItem1;
    private javax.swing.JMenuItem jMenuItem10;
    private javax.swing.JMenuItem jMenuItem11;
    private javax.swing.JMenuItem jMenuItem12;
    private javax.swing.JMenuItem jMenuItem13;
    private javax.swing.JMenuItem jMenuItem14;
    private javax.swing.JMenuItem jMenuItem15;
    private javax.swing.JMenuItem jMenuItem16;
    private javax.swing.JMenuItem jMenuItem17;
    private javax.swing.JMenuItem jMenuItem18;
    private javax.swing.JMenuItem jMenuItem19;
    private javax.swing.JMenuItem jMenuItem2;
    private javax.swing.JMenuItem jMenuItem3;
    private javax.swing.JMenuItem jMenuItem4;
    private javax.swing.JMenuItem jMenuItem5;
    private javax.swing.JMenuItem jMenuItem6;
    private javax.swing.JMenuItem jMenuItem7;
    private javax.swing.JMenuItem jMenuItem8;
    private javax.swing.JMenuItem jMenuItem9;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JScrollPane jScrollPane4;
    private javax.swing.JScrollPane jScrollPane5;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JSeparator jSeparator3;
    private javax.swing.JSeparator jSeparator4;
    private javax.swing.JSeparator jSeparator5;
    private javax.swing.JSeparator jSeparator6;
    private javax.swing.JPopupMenu.Separator jSeparator7;
    private javax.swing.JPopupMenu.Separator jSeparator8;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JSplitPane jSplitPane2;
    private javax.swing.JSplitPane jSplitPane3;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JMenu latestProjectMenu;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JToolBar mainToolBar;
    private javax.swing.JLabel memoryLabel;
    private javax.swing.JProgressBar memoryProgressBar;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JProgressBar progressBar;
    private javax.swing.JPopupMenu projectPopupMenu;
    private javax.swing.JTree projectsTree;
    private javax.swing.JPopupMenu resourcePopupMenu;
    private javax.swing.JTree resourceTree;
    private javax.swing.JLabel statusAnimationLabel;
    private javax.swing.JLabel statusMessageLabel;
    private javax.swing.JPanel statusPanel;
    private javax.swing.JPopupMenu tableViewerPopupMenu;
    // End of variables declaration//GEN-END:variables
    private final Timer messageTimer;
    private final Timer busyIconTimer;
    private final Icon idleIcon;
    private final Icon[] busyIcons = new Icon[15];
    private int busyIconIndex = 0;
    private JDialog aboutBox;
    private JDialog createProjectJDialog;
    private JDialog loadGeneChipFilesDialog;
    private JDialog integrateWholeGenome;
    private JDialog integrateGenomeRegion;
    private JDialog exportIntegrateDataJDialog;
    private JDialog generateSampleDialog;
    private JDialog integrateHapMapDialog;
    private JDialog imputationExportDialog;
    private JDialog integrateHapMapPhaseDialog;
    private JDialog loadGenotypewithCommonFormatDialog;
    private DownloaddbSNPAnnotationDialog downloadHapmapAnnotationDialog;
    private DownloadChipAnnotationDialog downloadChipAnnotationDialog;
    private JDialog loadPedigreeInformationForChipFileDialog;
    private JDialog loadPlinkBinaryFormatDialog;
    private JDialog startingHintJDialog;
    private JFrame mainFrame;
}
