// (c) 2008-2009 Miaoxin Li
// This file is distributed as part of the IGG source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.

// Permission is granted for you to use this file to compile IGG.

// All computer programs have bugs. Use this file at your own risk.
// Saturday, January 17, 2009
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.cobi.igg3.util;

import org.cobi.igg3.GlobalVariables;
import cern.colt.list.FloatArrayList;
import cern.jet.random.engine.MersenneTwister;
import cern.jet.random.engine.RandomEngine;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;
import org.apache.commons.collections.map.MultiValueMap;
import org.cobi.igg3.Constants;
import org.cobi.igg3.gui.IGG3View;

/**
 *
 * @author mxli
 */
public class IGGTestDataGenerator implements Constants {

    MultiValueMap chipSubjectMap = new MultiValueMap();
    ArrayList<String[]> chipTypes = new ArrayList<String[]>();

    public IGGTestDataGenerator(ArrayList<String[]> chipTypes) {
        this.chipTypes = chipTypes;
    }

    public void generateSimplePedFile(String outPathStr) throws Exception {
        String[] titles = {"PedID", "IndivID", "FatherID", "MotherID", "Sex", "Disease", "Trait", "SampleName"};
        int nCol = titles.length;
        int nSubject = 0;
        String chipType;
        int counter = 1;
        String label = null;
        File outPath = new File(outPathStr);
        if (!outPath.exists()) {
            outPath.mkdir();
        }
        BufferedWriter bw = new BufferedWriter(new FileWriter(outPath.getCanonicalPath() + File.separator + "pedigree.txt"));
        nCol--;
        for (int j = 0; j < nCol; j++) {
            bw.write(titles[j]);
            bw.write("  ");
        }
        bw.write(titles[nCol]);
        bw.write("\n");
        for (int j = 0; j < chipTypes.size(); j++) {
            chipType = chipTypes.get(j)[0];
            nSubject = Integer.parseInt(chipTypes.get(j)[1]);
            for (int i = 0; i < nSubject; i++) {
                bw.write(String.valueOf(counter));
                bw.write("  1  0  0  1  1  1.5  ");
                label = "Label" + counter;
                bw.write(label);
                bw.write("\n");
                chipSubjectMap.put(chipType, label);
                counter++;
            }
        }
        bw.close();
        GlobalVariables.mainView.setBriefRunningInfor("pedigree.txt created!", true);
    }

    public void generateChipFiles(String outPathStr, boolean overWrite) throws Exception {
        String chipType;
        List<String> snpList = new ArrayList<String>();
        FloatArrayList snpFreqList = new FloatArrayList();
        int probIndex = 0;
        int freqIndex = 9;
        Iterator iters = chipSubjectMap.keySet().iterator();
        while (iters.hasNext()) {
            chipType = iters.next().toString();

            Collection labels = chipSubjectMap.getCollection(chipType);
            if (chipType.startsWith("Affy")) {
                probIndex = 0;
                readSNPInfo(IGG_RESOURCE_CHIP_FOLDER + chipType + ".txt", snpList, snpFreqList, probIndex, freqIndex);
                File outPath = new File(outPathStr + File.separator + chipType);
                if (!outPath.exists()) {
                    outPath.mkdir();
                }
                generateAffyChipFiles(snpList, snpFreqList, labels, outPath, overWrite);
            } else if (chipType.startsWith("Illu")) {
                probIndex = 1;
                readSNPInfo(IGG_RESOURCE_CHIP_FOLDER + chipType + ".txt", snpList, snpFreqList, probIndex, freqIndex);
                File outPath = new File(outPathStr + File.separator + chipType);
                generateIllumChipFiles(snpList, snpFreqList, labels, outPath, overWrite);
            }

            snpList.clear();
            snpFreqList.clear();
        }
    }

    public void generateAffyChipFiles(List<String> snpList, FloatArrayList snpFreqList, Collection labels, File outPath, boolean overWrite) throws Exception {
        RandomEngine twister = new MersenneTwister(new java.util.Date());
        int arrySize = snpList.size();
        Iterator labeIter = labels.iterator();
        while (labeIter.hasNext()) {
            String label = labeIter.next().toString();
            File chipFile = new File(outPath.getCanonicalPath() + File.separator + label + ".txt");
            if (!overWrite) {
                if (chipFile.exists()) {
                    continue;
                }
            }
            BufferedWriter bw = new BufferedWriter(new FileWriter(chipFile));
            bw.write("Sample Name: ");
            bw.write(label);
            bw.write("\n##Call Rate Filter Threshold=95.000000\nProbe_Set_ID	Call\n");
            double prob = 0;
            double homoFeq = 0;
            for (int j = 1; j < arrySize; j++) {
                bw.write(snpList.get(j));
                prob = twister.raw();
                homoFeq = snpFreqList.get(j) * snpFreqList.get(j);
                if (prob <= homoFeq) {
                    bw.write("	AA\n");
                } else if (prob <= (2 * snpFreqList.get(j) - homoFeq)) {
                    bw.write("	AB\n");
                } else {
                    bw.write("	BB\n");
                }
            }
            bw.close();
            GlobalVariables.mainView.setBriefRunningInfor(label + ".txt created!", true);
        }
    }

    public void generateIllumChipFiles(List<String> snpList, FloatArrayList snpFreqList, Collection labels, File outPath, boolean overWrite) throws Exception {
        RandomEngine twister = new MersenneTwister(new java.util.Date());
        int arrySize = snpList.size();
        Iterator labeIter = labels.iterator();
        File chipFile = new File(outPath.getCanonicalPath() + ".csv");
        if (!overWrite) {
            if (chipFile.exists()) {
                return;
            }
        }
        String label = null;
        BufferedWriter bw = new BufferedWriter(new FileWriter(chipFile));
        bw.write("Sample Names: ");

        Object[] labelObjs = labels.toArray();
        int labelNum = labelObjs.length;
        labelNum--;
        for (int i = 0; i < labelNum; i++) {
            label = labelObjs[i].toString();
            bw.write(label);
            bw.write(", ");
        }
        label = labelObjs[labelNum].toString();
        bw.write(label);
        bw.write("\n");

        bw.write("Locus_Name,Sample_Name,Allele1,Allele2\n");
        labeIter = labels.iterator();
        while (labeIter.hasNext()) {
            label = labeIter.next().toString();
            double prob = 0;
            double homoFeq = 0;
            for (int j = 1; j < arrySize; j++) {
                bw.write(snpList.get(j));
                bw.write(",");
                bw.write(label);
                prob = twister.raw();
                homoFeq = snpFreqList.get(j) * snpFreqList.get(j);
                if (prob <= homoFeq) {
                    bw.write(",A,A\n");
                } else if (prob <= (2 * snpFreqList.get(j) - homoFeq)) {
                    bw.write(",A,B\n");
                } else {
                    bw.write(",B,B\n");
                }
            }
            GlobalVariables.mainView.setBriefRunningInfor(label + ".txt created!", true);
        }
        bw.close();
    }

    private boolean readSNPInfo(String fileName, List<String> snpList, FloatArrayList snpFreqList, int probIndex, int freqIndex) throws Exception {
        BufferedReader br = new BufferedReader(new FileReader(fileName));
        String line = null;
        int posIndex;
        String delmilit = "\t";
        int index = 0;
        float freqA;
        String tmpStr;

        br.readLine();
        while ((line = br.readLine()) != null) {
            StringTokenizer tokenizer = new StringTokenizer(line, delmilit);
            index = 0;
            while (tokenizer.hasMoreTokens()) {
                tmpStr = tokenizer.nextToken();
      
                if (index == probIndex) {
                    snpList.add(tmpStr);
                } else if (index == freqIndex) {
                    posIndex = tmpStr.indexOf("//");
                    tmpStr = tmpStr.substring(0, posIndex).trim();
                    if (LocalString.isNumeric(tmpStr)) {
                        freqA = Float.parseFloat(tmpStr);
                        if (freqA < 0) {
                            freqA = -freqA;
                        }
                    } else {
                        freqA = 0.5f;
                    }
                    snpFreqList.add(freqA);
                }
                index++;
                if (index > freqIndex) {
                    break;
                }
            }
        }
        br.close();

        if (snpList.size() != snpFreqList.size()) {
            System.err.println("Frequencies and SNP donot match!!!");
        }
        return true;
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        // TODO code application logic here
        // IGGTestDataGenerator generator = new IGGTestDataGenerator();
        int nSubject = 2500;
        String outPath = "InputSamples";
        try {
            // generator.generateChipFiles("Affymetrix_GenomeWide_Human_SNP_Array_5.0", outPath); 
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}
